@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,10 +9,10 @@ from __future__ import annotations
 from abc import ABC, abstractmethod, abstractproperty
 from datetime import datetime, timedelta
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional
+from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING
 
-from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
+from ax.core.data import Data
 from ax.core.generator_run import GeneratorRun
 from ax.core.metric import Metric
 from ax.core.runner import Runner
@@ -32,19 +32,40 @@ class TrialStatus(int, Enum):
     General lifecycle of a trial is:::
 
         CANDIDATE --> STAGED --> RUNNING --> COMPLETED
-                  ------------->         --> FAILED (machine failure)
+                  ------------->         --> FAILED (retryable)
                                          --> EARLY_STOPPED (deemed unpromising)
-                  -------------------------> ABANDONED (human-initiated action)
+                  -------------------------> ABANDONED (non-retryable)
 
-    Trials may be abandoned at any time prior to completion or failure
-    via human intervention. The difference between abandonment and failure
-    is that the former is human-directed, while the latter is an internal
-    failure state. Early-stopped refers to trials that were deemed
+    Trial is marked as a ``CANDIDATE`` immediately upon its creation.
+
+    Trials may be abandoned at any time prior to completion or failure.
+    The difference between abandonment and failure is that the ``FAILED`` state
+    is meant to express a possibly transient or retryable error, so trials in
+    that state may be re-run and arm(s) in them may be resuggested by Ax models
+    to be added to new trials.
+
+    ``ABANDONED`` trials on the other end, indicate
+    that the trial (and arms(s) in it) should not be rerun or added to new
+    trials. A trial might be marked ``ABANDONED`` as a result of human-initiated
+    action (if some trial in experiment is poorly-performing, deterministically
+    failing etc., and should not be run again in the experiment). It might also
+    be marked ``ABANDONED`` in an automated way if the trial's execution
+    encounters an error that indicates that the arm(s) in the trial should bot
+    be evaluated in the experiment again (e.g. the parameterization in a given
+    arm deterministically causes trial evaluation to fail). Note that it's also
+    possible to abandon a single arm in a `BatchTrial` via
+    ``batch.mark_arm_abandoned``.
+
+    Early-stopped refers to trials that were deemed
     unpromising by an early-stopping strategy and therefore terminated.
 
     Additionally, when trials are deployed, they may be in an intermediate
     staged state (e.g. scheduled but waiting for resources) or immediately
-    transition to running.
+    transition to running. Note that ``STAGED`` trial status is not always
+    applicable and depends on the ``Runner`` trials are deployed with
+    (and whether a ``Runner`` is present at all; for example, in Ax Service
+    API, trials are marked as ``RUNNING`` immediately when generated from
+    ``get_next_trial``, skipping the ``STAGED`` status).
 
     NOTE: Data for abandoned trials (or abandoned arms in batch trials) is
     not passed to the model as part of training data, unless ``fit_abandoned``
@@ -114,7 +135,6 @@ class TrialStatus(int, Enum):
         """True if this trial is an early stopped one."""
         return self == TrialStatus.EARLY_STOPPED
 
-    # pyre-fixme[14]: `__format__` overrides method defined in `object` inconsistently.
     def __format__(self, fmt: str) -> str:
         """Define `__format__` to avoid pulling the `__format__` from the `int`
         mixin (since its better for statuses to show up as `RUNNING` than as
@@ -128,12 +148,23 @@ class TrialStatus(int, Enum):
         return f"{self!s}"
 
 
+DEFAULT_STATUSES_TO_WARM_START: List[TrialStatus] = [
+    TrialStatus.RUNNING,
+    TrialStatus.COMPLETED,
+    TrialStatus.ABANDONED,
+    TrialStatus.EARLY_STOPPED,
+]
+
+
+# pyre-fixme[24]: Generic type `Callable` expects 2 type parameters.
 def immutable_once_run(func: Callable) -> Callable:
     """Decorator for methods that should throw Error when
     trial is running or has ever run and immutable.
     """
 
     # no type annotation for now; breaks sphinx-autodoc-typehints
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def _immutable_once_run(self, *args, **kwargs):
         if self._status != TrialStatus.CANDIDATE:
             raise ValueError(
@@ -184,6 +215,7 @@ class BaseTrial(ABC, SortableBase):
         if ttl_seconds is not None and ttl_seconds <= 0:
             raise ValueError("TTL must be a positive integer (or None).")
         self._ttl_seconds: Optional[int] = ttl_seconds
+        # pyre-fixme[4]: Attribute must be annotated.
         self._index = self._experiment._attach_trial(self, index=index)
 
         if trial_type is not None:
@@ -195,6 +227,7 @@ class BaseTrial(ABC, SortableBase):
             trial_type = self._experiment.default_trial_type
         self._trial_type: Optional[str] = trial_type
 
+        # pyre-fixme[4]: Attribute must be annotated.
         self.__status = None
         # Uses `_status` setter, which updates trial statuses to trial indices
         # mapping on the experiment, with which this trial is associated.
@@ -208,6 +241,7 @@ class BaseTrial(ABC, SortableBase):
 
         self._abandoned_reason: Optional[str] = None
         self._run_metadata: Dict[str, Any] = {}
+        self._stop_metadata: Dict[str, Any] = {}
 
         self._runner: Optional[Runner] = None
 
@@ -217,7 +251,9 @@ class BaseTrial(ABC, SortableBase):
         # If generator run(s) in this trial were generated from a generation
         # strategy, this property will be set to the generation step that produced
         # the generator run(s).
+        # pyre-fixme[4]: Attribute must be annotated.
         self._generation_step_index = None
+        # pyre-fixme[4]: Attribute must be annotated.
         self._properties = {}
 
     @property
@@ -278,10 +314,6 @@ class BaseTrial(ABC, SortableBase):
     @runner.setter
     @immutable_once_run
     def runner(self, runner: Optional[Runner]) -> None:
-        if self.experiment.is_simple_experiment:
-            raise NotImplementedError(
-                "SimpleExperiment does not support addition of runners."
-            )
         self._runner = runner
 
     @property
@@ -300,6 +332,14 @@ class BaseTrial(ABC, SortableBase):
         """
         return self._run_metadata
 
+    @property
+    def stop_metadata(self) -> Dict[str, Any]:
+        """Dict containing metadata from the stopping process.
+
+        This is set implicitly during `trial.stop()`.
+        """
+        return self._stop_metadata
+
     @property
     def trial_type(self) -> Optional[str]:
         """The type of the trial.
@@ -334,6 +374,12 @@ class BaseTrial(ABC, SortableBase):
         self._run_metadata.update(metadata)
         return self._run_metadata
 
+    def update_stop_metadata(self, metadata: Dict[str, Any]) -> Dict[str, Any]:
+        """Updates the stop metadata dict stored on this trial and returns the
+        updated dict."""
+        self._stop_metadata.update(metadata)
+        return self._stop_metadata
+
     def run(self) -> BaseTrial:
         """Deploys the trial according to the behavior on the runner.
 
@@ -362,23 +408,63 @@ class BaseTrial(ABC, SortableBase):
             self.mark_running()
         return self
 
-    def complete(self) -> BaseTrial:
+    def stop(self, new_status: TrialStatus, reason: Optional[str] = None) -> BaseTrial:
+        """Stops the trial according to the behavior on the runner.
+
+        The runner returns a `stop_metadata` dict containining metadata
+        of the stopping process.
+
+        Args:
+            new_status: The new TrialStatus. Must be one of {TrialStatus.COMPLETED,
+                TrialStatus.ABANDONED, TrialStatus.EARLY_STOPPED}
+            reason: A message containing information why the trial is to be stopped.
+
+        Returns:
+            The trial instance.
+        """
+        if self.status not in {TrialStatus.STAGED, TrialStatus.RUNNING}:
+            raise ValueError("Can only stop STAGED or RUNNING trials.")
+
+        if new_status not in {
+            TrialStatus.COMPLETED,
+            TrialStatus.ABANDONED,
+            TrialStatus.EARLY_STOPPED,
+        }:
+            raise ValueError(
+                "New status of a stopped trial must either be "
+                "COMPLETED, ABANDONED or EARLY_STOPPED."
+            )
+
+        # Default to experiment runner if trial doesn't have one
+        self.assign_runner()
+        if self._runner is None:
+            raise ValueError("No runner set on trial or experiment.")
+        runner = not_none(self._runner)
+
+        self._stop_metadata = runner.stop(self, reason=reason)
+        self.mark_as(new_status)
+        return self
+
+    def complete(self, reason: Optional[str] = None) -> BaseTrial:
         """Stops the trial if functionality is defined on runner
             and marks trial completed.
 
+        Args:
+            reason: A message containing information why the trial is to be
+                completed.
+
         Returns:
             The trial instance.
         """
         if self.status != TrialStatus.RUNNING:
             raise ValueError("Can only stop a running trial.")
-
-        not_none(self._runner).stop(self)
-        self.mark_completed()
+        try:
+            self.stop(new_status=TrialStatus.COMPLETED, reason=reason)
+        except NotImplementedError:
+            self.mark_completed()
         return self
 
-    def fetch_data(
-        self, metrics: Optional[List[Metric]] = None, **kwargs: Any
-    ) -> AbstractDataFrameData:
+    def fetch_data(self, metrics: Optional[List[Metric]] = None, **kwargs: Any) -> Data:
         """Fetch data for this trial for all metrics on experiment.
 
         Args:
@@ -395,15 +481,10 @@ class BaseTrial(ABC, SortableBase):
         )
 
     def lookup_data(
-        self, merge_across_timestamps: bool = False
-    ) -> AbstractDataFrameData:
+        self,
+    ) -> Data:
         """Lookup cached data on experiment for this trial.
 
-        Args:
-            merge_across_timestamps: Whether to return all ``Data`` objects
-            for this trial, merged together. ``Data`` objects are identified
-            by trial index and timestamp in Ax.
-
         Returns:
             If not merging across timestamps, the latest ``Data`` object
             associated with the trial. If merging, all data for trial, merged.
@@ -411,14 +492,13 @@ class BaseTrial(ABC, SortableBase):
         """
         return self.experiment.lookup_data_for_trial(
             trial_index=self.index,
-            merge_across_timestamps=merge_across_timestamps,
         )[0]
 
     def _check_existing_and_name_arm(self, arm: Arm) -> None:
         """Sets name for given arm; if this arm is already in the
         experiment, uses the existing arm name.
         """
-        proposed_name = f"{self.index}_{self._num_arms_created}"
+        proposed_name = self._get_default_name()
         self.experiment._name_and_store_arm_if_not_exists(
             arm=arm, proposed_name=proposed_name
         )
@@ -426,6 +506,11 @@ class BaseTrial(ABC, SortableBase):
         if arm.name == proposed_name:
             self._num_arms_created += 1
 
+    def _get_default_name(self, arm_index: Optional[int] = None) -> str:
+        if arm_index is None:
+            arm_index = self._num_arms_created
+        return f"{self.index}_{arm_index}"
+
     def _set_generation_step_index(self, generation_step_index: Optional[int]) -> None:
         """Sets the `generation_step_index` property of the trial, to reflect which
         generation step of a given generation strategy (if any) produced the generator
@@ -509,40 +594,44 @@ class BaseTrial(ABC, SortableBase):
     def abandoned_reason(self) -> Optional[str]:
         return self._abandoned_reason
 
-    def mark_staged(self) -> BaseTrial:
+    def mark_staged(self, unsafe: bool = False) -> BaseTrial:
         """Mark the trial as being staged for running.
 
+        Args:
+            unsafe: Ignore sanity checks on state transitions.
         Returns:
             The trial instance.
         """
-        if self._status != TrialStatus.CANDIDATE:
-            raise ValueError("Can only stage a candidate trial.")
+        if not unsafe and self._status != TrialStatus.CANDIDATE:
+            raise ValueError(
+                f"Can only stage a candidate trial.  This trial is {self._status}"
+            )
         self._status = TrialStatus.STAGED
         self._time_staged = datetime.now()
         return self
 
-    def mark_running(self, no_runner_required: bool = False) -> BaseTrial:
+    def mark_running(
+        self, no_runner_required: bool = False, unsafe: bool = False
+    ) -> BaseTrial:
         """Mark trial has started running.
 
+        Args:
+            no_runner_required: Whether to skip the check for presence of a ``Runner``
+            on experiment.
+            unsafe: Ignore sanity checks on state transitions.
         Returns:
             The trial instance.
         """
-        if self.experiment.is_simple_experiment:
-            self._status = TrialStatus.RUNNING
-            self._time_run_started = datetime.now()
-            return self
-
         if self._runner is None and not no_runner_required:
             raise ValueError("Cannot mark trial running without setting runner.")
 
         prev_step = (
             TrialStatus.STAGED
-            # pyre-fixme[16]: `Optional` has no attribute `staging_required`.
             if self._runner is not None and self._runner.staging_required
             else TrialStatus.CANDIDATE
         )
         prev_step_str = "staged" if prev_step == TrialStatus.STAGED else "candidate"
-        if self._status != prev_step:
+        if not unsafe and self._status != prev_step:
             raise ValueError(
                 f"Can only mark this trial as running when {prev_step_str}."
             )
@@ -550,19 +639,23 @@ class BaseTrial(ABC, SortableBase):
         self._time_run_started = datetime.now()
         return self
 
-    def mark_completed(self) -> BaseTrial:
+    def mark_completed(self, unsafe: bool = False) -> BaseTrial:
         """Mark trial as completed.
 
+        Args:
+            unsafe: Ignore sanity checks on state transitions.
         Returns:
             The trial instance.
         """
-        if self._status != TrialStatus.RUNNING:
+        if not unsafe and self._status != TrialStatus.RUNNING:
             raise ValueError("Can only complete trial that is currently running.")
         self._status = TrialStatus.COMPLETED
         self._time_completed = datetime.now()
         return self
 
-    def mark_abandoned(self, reason: Optional[str] = None) -> BaseTrial:
+    def mark_abandoned(
+        self, reason: Optional[str] = None, unsafe: bool = False
+    ) -> BaseTrial:
         """Mark trial as abandoned.
 
         NOTE: Arms in abandoned trials are considered to be 'pending points'
@@ -573,11 +666,12 @@ class BaseTrial(ABC, SortableBase):
 
         Args:
             abandoned_reason: The reason the trial was abandoned.
+            unsafe: Ignore sanity checks on state transitions.
 
         Returns:
             The trial instance.
         """
-        if self._status.is_terminal:
+        if not unsafe and self._status.is_terminal:
             raise ValueError("Cannot abandon a trial in a terminal state.")
 
         self._abandoned_reason = reason
@@ -585,22 +679,26 @@ class BaseTrial(ABC, SortableBase):
         self._time_completed = datetime.now()
         return self
 
-    def mark_failed(self) -> BaseTrial:
+    def mark_failed(self, unsafe: bool = False) -> BaseTrial:
         """Mark trial as failed.
 
+        Args:
+            unsafe: Ignore sanity checks on state transitions.
         Returns:
             The trial instance.
         """
-        if self._status != TrialStatus.RUNNING:
+        if not unsafe and self._status != TrialStatus.RUNNING:
             raise ValueError("Can only mark failed a trial that is currently running.")
 
         self._status = TrialStatus.FAILED
         self._time_completed = datetime.now()
         return self
 
-    def mark_early_stopped(self) -> BaseTrial:
+    def mark_early_stopped(self, unsafe: bool = False) -> BaseTrial:
         """Mark trial as early stopped.
 
+        Args:
+            unsafe: Ignore sanity checks on state transitions.
         Returns:
             The trial instance.
         """
@@ -610,11 +708,14 @@ class BaseTrial(ABC, SortableBase):
         self._time_completed = datetime.now()
         return self
 
-    def mark_as(self, status: TrialStatus, **kwargs: Any) -> BaseTrial:
+    def mark_as(
+        self, status: TrialStatus, unsafe: bool = False, **kwargs: Any
+    ) -> BaseTrial:
         """Mark trial with a new TrialStatus.
 
         Args:
             status: The new status of the trial.
+            unsafe: Ignore sanity checks on state transitions.
             kwargs: Additional keyword args, as can be ued in the respective `mark_`
                 methods associated with the trial status.
 
@@ -622,18 +723,18 @@ class BaseTrial(ABC, SortableBase):
             The trial instance.
         """
         if status == TrialStatus.STAGED:
-            self.mark_staged()
+            self.mark_staged(unsafe=unsafe)
         elif status == TrialStatus.RUNNING:
             no_runner_required = kwargs.get("no_runner_required", False)
-            self.mark_running(no_runner_required=no_runner_required)
+            self.mark_running(no_runner_required=no_runner_required, unsafe=unsafe)
         elif status == TrialStatus.ABANDONED:
-            self.mark_abandoned(reason=kwargs.get("reason"))
+            self.mark_abandoned(reason=kwargs.get("reason"), unsafe=unsafe)
         elif status == TrialStatus.FAILED:
-            self.mark_failed()
+            self.mark_failed(unsafe=unsafe)
         elif status == TrialStatus.COMPLETED:
-            self.mark_completed()
+            self.mark_completed(unsafe=unsafe)
         elif status == TrialStatus.EARLY_STOPPED:
-            self.mark_early_stopped()
+            self.mark_early_stopped(unsafe=unsafe)
         else:
             raise ValueError(f"Cannot mark trial as {status}.")
         return self
