@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,15 +7,22 @@
 from __future__ import annotations
 
 import json
-from typing import Dict, Iterable, Optional, Set, Type
+from hashlib import md5
+from typing import Any, Dict, Iterable, Optional, Set, Type
 
 import numpy as np
 import pandas as pd
-from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.types import TFidelityTrialEvaluation, TTrialEvaluation
+from ax.utils.common.base import Base
+from ax.utils.common.serialization import (
+    extract_init_args,
+    SerializationMixin,
+    serialize_init_args,
+)
+from ax.utils.common.typeutils import checked_cast, not_none
 
 
-class Data(AbstractDataFrameData):
+class Data(Base, SerializationMixin):
     """Class storing data for an experiment.
 
     The dataframe is retrieved via the `df` property. The data can be stored
@@ -33,7 +40,7 @@ class Data(AbstractDataFrameData):
     # downstream models can infer missing SEMs. Simply specify NaN as the SEM value,
     # either in your Metric class or in Data explicitly.
     REQUIRED_COLUMNS = {"arm_name", "metric_name", "mean", "sem"}
-    # pyre-ignore[15]: Inconsistent override. Adds FieldExperiment-specific fields
+
     COLUMN_DATA_TYPES = {
         "arm_name": str,
         "metric_name": str,
@@ -48,6 +55,8 @@ class Data(AbstractDataFrameData):
         "fidelities": str,  # Dictionary stored as json
     }
 
+    _df: pd.DataFrame
+
     def __init__(
         self,
         df: Optional[pd.DataFrame] = None,
@@ -62,7 +71,7 @@ class Data(AbstractDataFrameData):
         """
         # Initialize with barebones DF.
         if df is None:
-            self._df = pd.DataFrame(columns=self.required_columns())
+            self._df = pd.DataFrame(columns=list(self.required_columns()))
         else:
             columns = set(df.columns)
             missing_columns = self.required_columns() - columns
@@ -79,21 +88,171 @@ class Data(AbstractDataFrameData):
             # Reorder the columns for easier viewing
             col_order = [c for c in self.column_data_types() if c in df.columns]
             self._df = df[col_order]
-        super().__init__(description=description)
+
+        self.description = description
+
+    @classmethod
+    def _safecast_df(
+        cls,
+        df: pd.DataFrame,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
+        extra_column_types: Optional[Dict[str, Type]] = None,
+    ) -> pd.DataFrame:
+        """Function for safely casting df to standard data types.
+
+        Needed because numpy does not support NaNs in integer arrays.
+
+        Allows `Any` to be specified as a type, and will skip casting for that column.
+
+        Args:
+            df: DataFrame to safe-cast.
+            extra_column_types: types of columns only specified at instantiation-time.
+
+        Returns:
+            safe_df: DataFrame cast to standard dtypes.
+
+        """
+        extra_column_types = extra_column_types or {}
+        dtype = {
+            # Pandas timestamp handlng is weird
+            col: "datetime64[ns]" if coltype is pd.Timestamp else coltype
+            for col, coltype in cls.column_data_types(
+                extra_column_types=extra_column_types
+            ).items()
+            if col in df.columns.values
+            and not (
+                cls.column_data_types(extra_column_types)[col] is np.int64
+                and df.loc[:, col].isnull().any()
+            )
+            and not (coltype is Any)
+        }
+
+        return checked_cast(pd.DataFrame, df.astype(dtype=dtype))
+
+    @classmethod
+    def required_columns(cls) -> Set[str]:
+        """Names of columns that must be present in the underlying ``DataFrame``."""
+        return cls.REQUIRED_COLUMNS
+
+    @classmethod
+    def supported_columns(
+        cls, extra_column_names: Optional[Iterable[str]] = None
+    ) -> Set[str]:
+        """Names of columns supported (but not necessarily required) by this class."""
+        extra_column_names = set(extra_column_names or [])
+        extra_column_types: Dict[str, Any] = {name: Any for name in extra_column_names}
+        return cls.REQUIRED_COLUMNS.union(
+            cls.column_data_types(extra_column_types=extra_column_types)
+        )
+
+    @classmethod
+    def column_data_types(
+        cls,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
+        extra_column_types: Optional[Dict[str, Type]] = None,
+        excluded_columns: Optional[Iterable[str]] = None,
+        # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+        #  `typing.Type` to avoid runtime subscripting errors.
+    ) -> Dict[str, Type]:
+        """Type specification for all supported columns."""
+        extra_column_types = extra_column_types or {}
+        excluded_columns = excluded_columns or []
+
+        columns = {**cls.COLUMN_DATA_TYPES, **extra_column_types}
+
+        for column in excluded_columns:
+            if column in columns:
+                del columns[column]
+
+        return columns
+
+    @classmethod
+    # pyre-fixme[2]: Parameter annotation cannot be `Any`.
+    def serialize_init_args(cls, obj: Any) -> Dict[str, Any]:
+        """Serialize the class-dependent properties needed to initialize this Data.
+        Used for storage and to help construct new similar Data.
+        """
+        data = checked_cast(Data, obj)
+        return serialize_init_args(object=data)
+
+    @classmethod
+    def deserialize_init_args(cls, args: Dict[str, Any]) -> Dict[str, Any]:
+        """Given a dictionary, extract the properties needed to initialize the metric.
+        Used for storage.
+        """
+        # Extract `df` only if present, since certain inputs to this fn, e.g.
+        # SQAData.structure_metadata_json, don't have a `df` attribute.
+        if "df" in args and not isinstance(args["df"], pd.DataFrame):
+            # NOTE: Need dtype=False, otherwise infers arm_names like
+            # "4_1" should be int 41.
+            args["df"] = pd.read_json(args["df"]["value"], dtype=False)
+        return extract_init_args(args=args, class_=cls)
+
+    @property
+    def true_df(self) -> pd.DataFrame:
+        """Return the `DataFrame` being used as the source of truth (avoid using
+        except for caching).
+        """
+
+        return self._df
+
+    @property
+    def df(self) -> pd.DataFrame:
+        return self._df
+
+    @property
+    def df_hash(self) -> str:
+        """Compute hash of pandas DataFrame.
+
+        This first serializes the DataFrame and computes the md5 hash on the
+        resulting string. Note that this may cause performance issue for very large
+        DataFrames.
+
+        Args:
+            df: The DataFrame for which to compute the hash.
+
+        Returns
+            str: The hash of the DataFrame.
+
+        """
+        return md5(not_none(self.df.to_json()).encode("utf-8")).hexdigest()
+
+    @property
+    def metric_names(self) -> Set[str]:
+        """Set of metric names that appear in the underlying dataframe of
+        this object.
+        """
+        return set() if self.df.empty else set(self.df["metric_name"].values)
+
+    def get_filtered_results(self, **filters: Dict[str, Any]) -> pd.DataFrame:
+        """Return filtered subset of data.
+
+        Args:
+            filter: Column names and values they must match.
+
+        Returns
+            df: The filtered DataFrame.
+        """
+        df = self.df.copy()
+        columns = df.columns
+        for colname, value in filters.items():
+            if colname not in columns:
+                raise ValueError(
+                    f"{colname} not in the set of columns: {columns}"
+                    f"in this data object of type: {str(type(self))}."
+                )
+            df = df[df[colname] == value]
+        return df
 
     @staticmethod
-    # pyre-ignore [14]: `Iterable[Data]` not a supertype of overridden parameter.
     def from_multiple_data(
         data: Iterable[Data], subset_metrics: Optional[Iterable[str]] = None
     ) -> Data:
         """Combines multiple data objects into one (with the concatenated
         underlying dataframe).
 
-        NOTE: if one or more data objects in the iterable is of a custom
-        subclass of `Data`, object of that class will be returned. If
-        the iterable contains multiple types of `Data`, an error will be
-        raised.
-
         Args:
             data: Iterable of Ax `Data` objects to combine.
             subset_metrics: If specified, combined `Data` will only contain
@@ -108,19 +267,7 @@ class Data(AbstractDataFrameData):
         if subset_metrics:
             dfs = [df.loc[df["metric_name"].isin(subset_metrics)] for df in dfs]
 
-        # obtain type of first elt in iterable (we know it's not empty)
-        data_type = type(data[0])
-
-        # check if all types in iterable match the first type
-        if all((type(datum) is data_type) for datum in data):
-            # if all types in iterable are subclasses of Data, return the subclass
-            if issubclass(data_type, Data):
-                return data_type(df=pd.concat(dfs, axis=0, sort=True))
-            else:
-                # if not, return the original Data object
-                return Data(df=pd.concat(dfs, axis=0, sort=True))
-        else:
-            raise ValueError("More than one custom data type found in data iterable")
+        return Data(df=pd.concat(dfs, axis=0, sort=True))
 
     @staticmethod
     def from_evaluations(
@@ -134,8 +281,9 @@ class Data(AbstractDataFrameData):
         Convert dict of evaluations to Ax data object.
 
         Args:
-            evaluations: Map from arm name to metric outcomes (itself a mapping
-                of metric names to tuples of mean and optionally a SEM).
+            evaluations: Map from arm name to metric outcomes, which itself is  a
+                mapping of metric names to means or tuples of mean and an SEM). If
+                SEM is not specified, it will be set to None and inferred from data.
             trial_index: Trial index to which this data belongs.
             sample_sizes: Number of samples collected for each arm.
             start_time: Optional start time of run of the trial that produced this
@@ -150,12 +298,12 @@ class Data(AbstractDataFrameData):
             {
                 "arm_name": name,
                 "metric_name": metric_name,
-                "mean": evaluation[metric_name][0],
-                "sem": evaluation[metric_name][1],
+                "mean": value[0] if isinstance(value, tuple) else value,
+                "sem": value[1] if isinstance(value, tuple) else None,
                 "trial_index": trial_index,
             }
             for name, evaluation in evaluations.items()
-            for metric_name in evaluation.keys()
+            for metric_name, value in evaluation.items()
         ]
         if start_time is not None or end_time is not None:
             for record in records:
@@ -178,8 +326,9 @@ class Data(AbstractDataFrameData):
 
         Args:
             evaluations: Map from arm name to list of (fidelity, metric outcomes)
-                (where metric outcomes is itself a mapping of metric names to
-                tuples of mean and SEM).
+                where metric outcomes is itself a mapping of metric names to means
+                or tuples of mean and SEM. If SEM is not specified, it will be set
+                to None and inferred from data.
             trial_index: Trial index to which this data belongs.
             sample_sizes: Number of samples collected for each arm.
             start_time: Optional start time of run of the trial that produced this
@@ -194,14 +343,14 @@ class Data(AbstractDataFrameData):
             {
                 "arm_name": name,
                 "metric_name": metric_name,
-                "mean": evaluation[metric_name][0],
-                "sem": evaluation[metric_name][1],
+                "mean": value[0] if isinstance(value, tuple) else value,
+                "sem": value[1] if isinstance(value, tuple) else None,
                 "trial_index": trial_index,
                 "fidelities": json.dumps(fidelity),
             }
             for name, fidelity_and_metrics_list in evaluations.items()
             for fidelity, evaluation in fidelity_and_metrics_list
-            for metric_name in evaluation.keys()
+            for metric_name, value in evaluation.items()
         ]
         if start_time is not None or end_time is not None:
             for record in records:
@@ -211,6 +360,14 @@ class Data(AbstractDataFrameData):
                 record["n"] = sample_sizes[str(record["arm_name"])]
         return Data(df=pd.DataFrame(records))
 
+    def copy_structure_with_df(self, df: pd.DataFrame) -> Data:
+        """Serialize the structural properties needed to initialize this Data.
+        Used for storage and to help construct new similar Data. All kwargs
+        other than ``df`` and ``description`` are considered structural.
+        """
+        cls = type(self)
+        return cls(df=df, **cls.serialize_init_args(self))
+
 
 def set_single_trial(data: Data) -> Data:
     """Returns a new Data object where we set all rows to have the same
@@ -243,6 +400,8 @@ def clone_without_metrics(data: Data, excluded_metric_names: Iterable[str]) -> D
 
 
 def custom_data_class(
+    # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+    #  `typing.Type` to avoid runtime subscripting errors.
     column_data_types: Optional[Dict[str, Type]] = None,
     required_columns: Optional[Set[str]] = None,
     time_columns: Optional[Set[str]] = None,
