@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,17 +7,17 @@
 from __future__ import annotations
 
 import logging
-from collections import OrderedDict, defaultdict
+import warnings
+from collections import defaultdict, OrderedDict
 from datetime import datetime
 from enum import Enum
 from functools import reduce
 from typing import Any, Dict, Iterable, List, Optional, Set, Tuple, Type
 
 import pandas as pd
-from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
-from ax.core.base_trial import BaseTrial, TrialStatus
-from ax.core.batch_trial import BatchTrial
+from ax.core.base_trial import BaseTrial, DEFAULT_STATUSES_TO_WARM_START, TrialStatus
+from ax.core.batch_trial import BatchTrial, LifecycleStage
 from ax.core.data import Data
 from ax.core.generator_run import GeneratorRun
 from ax.core.map_data import MapData
@@ -30,7 +30,7 @@ from ax.core.search_space import SearchSpace
 from ax.core.trial import Trial
 from ax.exceptions.core import UnsupportedError
 from ax.utils.common.base import Base
-from ax.utils.common.constants import Keys
+from ax.utils.common.constants import EXPERIMENT_IS_TEST_WARNING, Keys
 from ax.utils.common.docutils import copy_doc
 from ax.utils.common.logger import get_logger
 from ax.utils.common.timeutils import current_timestamp_in_millis
@@ -41,14 +41,18 @@ logger: logging.Logger = get_logger(__name__)
 
 class DataType(Enum):
     DATA = 1
-    MAP_DATA = 2
+    MAP_DATA = 3
 
 
+# pyre-fixme[24]: Generic type `type` expects 1 type parameter, use `typing.Type` to
+#  avoid runtime subscripting errors.
 DATA_TYPE_LOOKUP: Dict[DataType, Type] = {
     DataType.DATA: Data,
     DataType.MAP_DATA: MapData,
 }
 
+DEFAULT_OBJECTIVE_NAME = "objective"
+
 
 # pyre-fixme[13]: Attribute `_search_space` is never initialized.
 class Experiment(Base):
@@ -86,27 +90,23 @@ class Experiment(Base):
         # appease pyre
         self._search_space: SearchSpace
         self._status_quo: Optional[Arm] = None
+        self._is_test: bool
 
         self._name = name
         self.description = description
         self.runner = runner
         self.is_test = is_test
 
-        self._data_by_trial: Dict[int, OrderedDict[int, AbstractDataFrameData]] = {}
+        self._data_by_trial: Dict[int, OrderedDict[int, Data]] = {}
         self._experiment_type: Optional[str] = experiment_type
+        # pyre-fixme[4]: Attribute must be annotated.
         self._optimization_config = None
         self._tracking_metrics: Dict[str, Metric] = {}
         self._time_created: datetime = datetime.now()
         self._trials: Dict[int, BaseTrial] = {}
         self._properties: Dict[str, Any] = properties or {}
-        self._default_data_type = default_data_type or (
-            DataType.MAP_DATA
-            if (
-                optimization_config is not None
-                and isinstance(optimization_config.objective.metrics[0], MapMetric)
-            )
-            else DataType.DATA
-        )
+        # pyre-fixme[4]: Attribute must be annotated.
+        self._default_data_type = default_data_type or DataType.DATA
         # Used to keep track of whether any trials on the experiment
         # specify a TTL. Since trials need to be checked for their TTL's
         # expiration often, having this attribute helps avoid unnecessary
@@ -137,7 +137,6 @@ class Experiment(Base):
         """Get experiment name. Throws if name is None."""
         if self._name is None:
             raise ValueError("Experiment's name is None.")
-        # pyre-fixme[7]: Expected `str` but got `Optional[str]`.
         return self._name
 
     @name.setter
@@ -146,10 +145,16 @@ class Experiment(Base):
         self._name = name
 
     @property
-    def is_simple_experiment(self):
-        """Whether this experiment is a regular Experiment or the subclassing
-        `SimpleExperiment`."""
-        return False
+    def is_test(self) -> bool:
+        """Get whether the experiment is a test."""
+        return self._is_test
+
+    @is_test.setter
+    def is_test(self, is_test: bool) -> None:
+        """Set whether the experiment is a test."""
+        if is_test:
+            logger.info(EXPERIMENT_IS_TEST_WARNING)
+        self._is_test = is_test
 
     @property
     def time_created(self) -> datetime:
@@ -220,7 +225,12 @@ class Experiment(Base):
     @status_quo.setter
     def status_quo(self, status_quo: Optional[Arm]) -> None:
         if status_quo is not None:
-            self.search_space.check_types(status_quo.parameters, raise_error=True)
+            self.search_space.check_types(
+                parameterization=status_quo.parameters, raise_error=True
+            )
+            self.search_space.check_all_parameters_present(
+                parameterization=status_quo.parameters, raise_error=True
+            )
 
             # Compute a unique name if "status_quo" is taken
             name = "status_quo"
@@ -282,6 +292,13 @@ class Experiment(Base):
         """The experiment's optimization config."""
         return self._optimization_config
 
+    @property
+    def is_moo_problem(self) -> bool:
+        """Whether the experiment's optimization config contains multiple objectives."""
+        if self.optimization_config is None:
+            return False
+        return not_none(self.optimization_config).is_moo_problem
+
     @optimization_config.setter
     def optimization_config(self, optimization_config: OptimizationConfig) -> None:
         if (
@@ -299,7 +316,14 @@ class Experiment(Base):
                 self.remove_tracking_metric(metric_name)
         self._optimization_config = optimization_config
 
+        if any(
+            isinstance(metric, MapMetric)
+            for metric in optimization_config.metrics.values()
+        ):
+            self._default_data_type = DataType.MAP_DATA
+
     @property
+    # pyre-fixme[24]: Generic type `OrderedDict` expects 2 type parameters.
     def data_by_trial(self) -> Dict[int, OrderedDict]:
         """Data stored on the experiment, indexed by trial index and storage time.
 
@@ -322,7 +346,11 @@ class Experiment(Base):
         """
         return self._properties.get(Keys.IMMUTABLE_SEARCH_SPACE_AND_OPT_CONF, False)
 
-    def add_tracking_metric(self, metric: Metric) -> "Experiment":
+    @property
+    def tracking_metrics(self) -> List[Metric]:
+        return list(self._tracking_metrics.values())
+
+    def add_tracking_metric(self, metric: Metric) -> Experiment:
         """Add a new metric to the experiment.
 
         Args:
@@ -342,10 +370,13 @@ class Experiment(Base):
                 "before adding it to tracking metrics."
             )
 
+        if isinstance(metric, MapMetric):
+            self._default_data_type = DataType.MAP_DATA
+
         self._tracking_metrics[metric.name] = metric
         return self
 
-    def add_tracking_metrics(self, metrics: List[Metric]) -> "Experiment":
+    def add_tracking_metrics(self, metrics: List[Metric]) -> Experiment:
         """Add a list of new metrics to the experiment.
 
         If any of the metrics are already defined on the experiment,
@@ -360,7 +391,7 @@ class Experiment(Base):
             self.add_tracking_metric(metric)
         return self
 
-    def update_tracking_metric(self, metric: Metric) -> "Experiment":
+    def update_tracking_metric(self, metric: Metric) -> Experiment:
         """Redefine a metric that already exists on the experiment.
 
         Args:
@@ -372,7 +403,7 @@ class Experiment(Base):
         self._tracking_metrics[metric.name] = metric
         return self
 
-    def remove_tracking_metric(self, metric_name: str) -> "Experiment":
+    def remove_tracking_metric(self, metric_name: str) -> Experiment:
         """Remove a metric that already exists on the experiment.
 
         Args:
@@ -389,7 +420,6 @@ class Experiment(Base):
         """The metrics attached to the experiment."""
         optimization_config_metrics: Dict[str, Metric] = {}
         if self.optimization_config is not None:
-            # pyre-fixme[16]: `Optional` has no attribute `metrics`.
             optimization_config_metrics = self.optimization_config.metrics
         return {**self._tracking_metrics, **optimization_config_metrics}
 
@@ -407,15 +437,19 @@ class Experiment(Base):
         return metrics_by_class
 
     def fetch_data(
-        self, metrics: Optional[List[Metric]] = None, **kwargs: Any
-    ) -> AbstractDataFrameData:
+        self,
+        metrics: Optional[List[Metric]] = None,
+        combine_with_last_data: bool = False,
+        overwrite_existing_data: bool = False,
+        **kwargs: Any,
+    ) -> Data:
         """Fetches data for all trials on this experiment and for either the
         specified metrics or all metrics currently on the experiment, if `metrics`
         argument is not specified.
 
         NOTE: For metrics that are not available while trial is running, the data
         may be retrieved from cache on the experiment. Data is cached on the experiment
-        via calls to `experiment.attach_data` and whetner a given metric class is
+        via calls to `experiment.attach_data` and whether a given metric class is
         available while trial is running is determined by the boolean returned from its
         `is_available_while_running` class method.
 
@@ -428,15 +462,21 @@ class Experiment(Base):
             Data for the experiment.
         """
         return self._lookup_or_fetch_trials_data(
-            trials=list(self.trials.values()), metrics=metrics, **kwargs
+            trials=list(self.trials.values()),
+            metrics=metrics,
+            combine_with_last_data=combine_with_last_data,
+            overwrite_existing_data=overwrite_existing_data,
+            **kwargs,
         )
 
     def fetch_trials_data(
         self,
         trial_indices: Iterable[int],
         metrics: Optional[List[Metric]] = None,
+        combine_with_last_data: bool = False,
+        overwrite_existing_data: bool = False,
         **kwargs: Any,
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         """Fetches data for specific trials on the experiment.
 
         NOTE: For metrics that are not available while trial is running, the data
@@ -457,6 +497,8 @@ class Experiment(Base):
         return self._lookup_or_fetch_trials_data(
             trials=self.get_trials_by_indices(trial_indices=trial_indices),
             metrics=metrics,
+            combine_with_last_data=combine_with_last_data,
+            overwrite_existing_data=overwrite_existing_data,
             **kwargs,
         )
 
@@ -464,8 +506,10 @@ class Experiment(Base):
         self,
         trials: List[BaseTrial],
         metrics: Optional[Iterable[Metric]] = None,
+        combine_with_last_data: bool = False,
+        overwrite_existing_data: bool = False,
         **kwargs: Any,
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         if not self.metrics and not metrics:
             raise ValueError(
                 "No metrics to fetch data for, as no metrics are defined for "
@@ -477,28 +521,42 @@ class Experiment(Base):
         metrics_to_fetch = list(metrics or self.metrics.values())
         metrics_by_class = self._metrics_by_class(metrics=metrics_to_fetch)
         data_list = []
+        contains_new_data = False
         for metric_cls in metrics_by_class:
-            data_list.append(
-                metric_cls.lookup_or_fetch_experiment_data_multi(
-                    experiment=self,
-                    metrics=metrics_by_class[metric_cls],
-                    trials=trials,
-                    **kwargs,
-                )
+            (
+                metric_data,
+                metric_data_contains_new_data,
+            ) = metric_cls.lookup_or_fetch_experiment_data_multi(
+                experiment=self,
+                metrics=metrics_by_class[metric_cls],
+                trials=trials,
+                **kwargs,
             )
-        return self.default_data_constructor.from_multiple_data(data=data_list)
+            contains_new_data = contains_new_data or metric_data_contains_new_data
+            data_list.append(metric_data)
+        data = self.default_data_constructor.from_multiple_data(data=data_list)
+        if contains_new_data and not data.df.empty:
+            self.attach_data(
+                data=data,
+                combine_with_last_data=combine_with_last_data,
+                overwrite_existing_data=overwrite_existing_data,
+            )
+        return data
 
     @copy_doc(BaseTrial.fetch_data)
     def _fetch_trial_data(
         self, trial_index: int, metrics: Optional[List[Metric]] = None, **kwargs: Any
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         trial = self.trials[trial_index]
         return self._lookup_or_fetch_trials_data(
             trials=[trial], metrics=metrics, **kwargs
         )
 
     def attach_data(
-        self, data: AbstractDataFrameData, combine_with_last_data: bool = False
+        self,
+        data: Data,
+        combine_with_last_data: bool = False,
+        overwrite_existing_data: bool = False,
     ) -> int:
         """Attach data to experiment. Stores data in `experiment._data_by_trial`,
         to be looked up via `experiment.lookup_data_for_trial`.
@@ -507,29 +565,41 @@ class Experiment(Base):
             data: Data object to store.
             combine_with_last_data: By default, when attaching data, it's identified
                 by its timestamp, and `experiment.lookup_data_for_trial` returns
-                data by most recent timestamp. Sometimes, however, we attach
-                data for some metrics at one point and data for the rest of the
-                metrics later on. In this case, we actually want to combine the
-                data so that one dataframe contains data for all metrics.
-                To achieve that goal, every call to `attach_data` after
-                the initial data is attached to trials, should be set to `True`.
+                data by most recent timestamp. Sometimes, however, we want to combine
+                the data from multiple calls to `attach_data` into one dataframe.
+                This might be because:
+                    - We attached data for some metrics at one point and data for
+                    the rest of the metrics later on.
+                    - We attached data for some fidelity at one point and data for
+                    another fidelity later one.
+                To achieve that goal, set `combine_with_last_data` to `True`.
                 In this case, we will take the most recent previously attached
-                data, append the newly attached data to it, and attach a new
-                Data object with the merged result. Afterwards, calls to
-                `lookup_data_for_trial` will return this combined data object,
-                rather than just the most recently added data. This operation
-                will also validate that the newly added data does not contain
-                observations for the metrics that already have observations in
-                the most recent data stored.
+                data, append the newly attached data to it, attach a new
+                Data object with the merged result, and delete the old one.
+                Afterwards, calls to `lookup_data_for_trial` will return this
+                new combined data object. This operation will also validate that the
+                newly added data does not contain observations for metrics that
+                already have observations at the same fidelity in the most recent data.
+            overwrite_existing_data: By default, we keep around all data that has
+                ever been attached to the experiment. However, if we know that
+                the incoming data contains all the information we need for a given
+                trial, we can replace the existing data for that trial, thereby
+                reducing the amount we need to store in the database.
 
         Returns:
             Timestamp of storage in millis.
         """
+        if combine_with_last_data and overwrite_existing_data:
+            raise UnsupportedError(
+                "Cannot set both combine_with_last_data=True and "
+                "overwrite_existing_data=True. Data can either be "
+                "combined, or overwritten, or neither."
+            )
         data_type = type(data)
-        data_init_args = data.serialize_init_args(data)
+        data_init_args = data.deserialize_init_args(data.serialize_init_args(data))
         if data.df.empty:
             raise ValueError("Data to attach is empty.")
-        metrics_not_on_exp = set(data.df["metric_name"].values) - set(
+        metrics_not_on_exp = set(data.true_df["metric_name"].values) - set(
             self.metrics.keys()
         )
         if metrics_not_on_exp:
@@ -542,7 +612,9 @@ class Experiment(Base):
                 "the experiment's optimization config."
             )
         cur_time_millis = current_timestamp_in_millis()
-        for trial_index, trial_df in data.df.groupby(data.df["trial_index"]):
+        for trial_index, trial_df in data.true_df.groupby(data.true_df["trial_index"]):
+            # Overwrite `df` so that `data` only has current trial data.
+            data_init_args["df"] = trial_df
             current_trial_data = (
                 self._data_by_trial[trial_index]
                 if trial_index in self._data_by_trial
@@ -550,10 +622,14 @@ class Experiment(Base):
             )
             if combine_with_last_data and len(current_trial_data) > 0:
                 last_ts, last_data = list(current_trial_data.items())[-1]
+                last_data_type = type(last_data)
+                merge_keys = ["trial_index", "metric_name", "arm_name"] + (
+                    last_data.map_keys if issubclass(last_data_type, MapData) else []
+                )
                 merged = pd.merge(
-                    last_data.df,
+                    last_data.true_df,
                     trial_df,
-                    on=["trial_index", "metric_name", "arm_name"],
+                    on=merge_keys,
                     how="inner",
                 )
                 if not merged.empty:
@@ -561,23 +637,34 @@ class Experiment(Base):
                         f"Last data for trial {trial_index} already contained an "
                         f"observation for metric {merged.head()['metric_name']}."
                     )
-                last_data_type = type(last_data)
+                del current_trial_data[last_ts]
                 current_trial_data[cur_time_millis] = last_data_type.from_multiple_data(
                     [
                         last_data,
-                        last_data_type(trial_df, **data_init_args),
+                        last_data_type(**data_init_args),
                     ]
                 )
-            else:
-                # pyre-ignore [45]: Cannot instantiate `AbstractDataFrameData`.
-                current_trial_data[cur_time_millis] = data_type(
-                    trial_df, **data_init_args
+            elif overwrite_existing_data:
+                if len(current_trial_data) > 0:
+                    _, last_data = list(current_trial_data.items())[-1]
+                    last_data_metrics = set(last_data.df["metric_name"])
+                    new_data_metrics = set(trial_df["metric_name"])
+                    if last_data_metrics.difference(new_data_metrics):
+                        raise ValueError(
+                            "overwrite_trial_data is True, but the new data contains "
+                            "only a subset of the metrics that are present in the "
+                            "previous data."
+                        )
+                current_trial_data = OrderedDict(
+                    {cur_time_millis: data_type(**data_init_args)}
                 )
+            else:
+                current_trial_data[cur_time_millis] = data_type(**data_init_args)
             self._data_by_trial[trial_index] = current_trial_data
 
         return cur_time_millis
 
-    def lookup_data_for_ts(self, timestamp: int) -> AbstractDataFrameData:
+    def lookup_data_for_ts(self, timestamp: int) -> Data:
         """Collect data for all trials stored at this timestamp.
 
         Useful when many trials' data was fetched and stored simultaneously
@@ -600,8 +687,9 @@ class Experiment(Base):
         return self.default_data_constructor.from_multiple_data(trial_datas)
 
     def lookup_data_for_trial(
-        self, trial_index: int, merge_across_timestamps: bool = False
-    ) -> Tuple[AbstractDataFrameData, int]:
+        self,
+        trial_index: int,
+    ) -> Tuple[Data, int]:
         """Lookup stored data for a specific trial.
 
         Returns latest data object, and its storage timestamp, present for this trial.
@@ -609,9 +697,6 @@ class Experiment(Base):
 
         Args:
             trial_index: The index of the trial to lookup data for.
-            merge_across_timestamps: Whether to return Data from all timestamps instead
-                of only the latest.
-
         Returns:
             The requested data object, and its storage timestamp in milliseconds.
         """
@@ -625,26 +710,18 @@ class Experiment(Base):
 
         storage_time = max(trial_data_dict.keys())
         trial_data = trial_data_dict[storage_time]
-        if merge_across_timestamps:
-            trial_data = trial_data.from_multiple_data(
-                data=list(trial_data_dict.values())
-            )
-            trial_data = checked_cast(AbstractDataFrameData, trial_data)
         return trial_data, storage_time
 
     def lookup_data(
         self,
         trial_indices: Optional[Iterable[int]] = None,
-        merge_across_timestamps: bool = False,
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         """Lookup data for all trials on this experiment and for either the
         specified metrics or all metrics currently on the experiment, if `metrics`
         argument is not specified.
 
         Args:
             trial_indices: Indices of trials, for which to fetch data.
-            merge_across_timestamps: Whether to return data across all timestamps.
-
         Returns:
             Data for the experiment.
         """
@@ -654,14 +731,14 @@ class Experiment(Base):
             data_by_trial.append(
                 self.lookup_data_for_trial(
                     trial_index=trial_index,
-                    merge_across_timestamps=merge_across_timestamps,
                 )[0]
             )
         if not data_by_trial:
             return self.default_data_constructor()
         last_data = data_by_trial[-1]
         last_data_type = type(last_data)
-        return last_data_type.from_multiple_data(data_by_trial)
+        data = last_data_type.from_multiple_data(data_by_trial)
+        return data
 
     @property
     def num_trials(self) -> int:
@@ -703,11 +780,18 @@ class Experiment(Base):
         self._check_TTL_on_running_trials()  # Marks past-TTL trials as failed.
         return self._trial_indices_by_status
 
+    @property
+    def running_trial_indices(self) -> Set[int]:
+        """Indices of running trials, associated with the experiment."""
+        return self._trial_indices_by_status[TrialStatus.RUNNING]
+
     @property
     def default_data_type(self) -> DataType:
         return self._default_data_type
 
     @property
+    # pyre-fixme[24]: Generic type `type` expects 1 type parameter, use
+    #  `typing.Type` to avoid runtime subscripting errors.
     def default_data_constructor(self) -> Type:
         return DATA_TYPE_LOOKUP[self.default_data_type]
 
@@ -748,6 +832,7 @@ class Experiment(Base):
         trial_type: Optional[str] = None,
         optimize_for_power: Optional[bool] = False,
         ttl_seconds: Optional[int] = None,
+        lifecycle_stage: Optional[LifecycleStage] = None,
     ) -> BatchTrial:
         """Create a new batch trial associated with this experiment.
 
@@ -766,6 +851,8 @@ class Experiment(Base):
                 'dead' trials, for which the evaluation process might have
                 crashed etc., and which should be considered failed after
                 their 'time to live' has passed.
+            lifecycle_stage: The stage of the experiment lifecycle that this
+                trial represents
         """
         if ttl_seconds is not None:
             self._trials_have_ttl = True
@@ -775,6 +862,7 @@ class Experiment(Base):
             generator_run=generator_run,
             optimize_for_power=optimize_for_power,
             ttl_seconds=ttl_seconds,
+            lifecycle_stage=lifecycle_stage,
         )
 
     def get_trials_by_indices(self, trial_indices: Iterable[int]) -> List[BaseTrial]:
@@ -835,7 +923,12 @@ class Experiment(Base):
         self._trials[index] = trial
         return index
 
-    def warm_start_from_old_experiment(self, old_experiment: Experiment) -> List[Trial]:
+    def warm_start_from_old_experiment(
+        self,
+        old_experiment: Experiment,
+        copy_run_metadata_keys: Optional[List[str]] = None,
+        trial_statuses_to_copy: Optional[List[TrialStatus]] = None,
+    ) -> List[Trial]:
         """Copy all completed trials with data from an old Ax expeirment to this one.
         This function checks that the parameters of each trial are members of the
         current experiment's search_space.
@@ -845,6 +938,11 @@ class Experiment(Base):
 
         Args:
             old_experiment: The experiment from which to transfer trials and data
+            copy_run_metadata_keys: A list of keys denoting which items to copy over
+                from each trial's run_metadata.
+            trial_statuses_to_copy: All trials with a status in this list will be
+                copied. By default, copies all ``COMPLETED``, ``ABANDONED``, and
+                ``EARLY_STOPPED`` trials.
 
         Returns:
             List of trials successfully copied from old_experiment to this one
@@ -852,23 +950,33 @@ class Experiment(Base):
         if len(self.trials) > 0:
             raise ValueError(  # pragma: no cover
                 f"Can only warm-start experiments that don't yet have trials. "
-                f"Experiment {self.name} has {len(self.trials)} trials."
+                f"Experiment {self._name} has {len(self.trials)} trials."
             )
 
         old_parameter_names = set(old_experiment.search_space.parameters.keys())
         parameter_names = set(self.search_space.parameters.keys())
         if old_parameter_names.symmetric_difference(parameter_names):
             raise ValueError(  # pragma: no cover
-                f"Cannot warm-start experiment '{self.name}' from experiment "
-                f"'{old_experiment.name}' due to mismatch in search space parameters."
-                f"Parameters in '{self.name}' but not in '{old_experiment.name}': "
+                f"Cannot warm-start experiment '{self._name}' from experiment "
+                f"'{old_experiment._name}' due to mismatch in search space parameters."
+                f"Parameters in '{self._name}' but not in '{old_experiment._name}': "
                 f"{old_parameter_names - parameter_names}. Vice-versa: "
                 f"{parameter_names - old_parameter_names}."
             )
 
-        old_completed_trials = old_experiment.trials_by_status[TrialStatus.COMPLETED]
+        trial_statuses_to_copy = (
+            trial_statuses_to_copy
+            if trial_statuses_to_copy is not None
+            else DEFAULT_STATUSES_TO_WARM_START
+        )
+
+        warm_start_trials = [
+            trial
+            for trial in old_experiment.trials.values()
+            if trial.status in trial_statuses_to_copy
+        ]
         copied_trials = []
-        for trial in old_completed_trials:
+        for trial in warm_start_trials:
             if not isinstance(trial, Trial):
                 raise NotImplementedError(  # pragma: no cover
                     "Only experiments with 1-arm trials currently supported."
@@ -877,20 +985,23 @@ class Experiment(Base):
                 not_none(trial.arm).parameters, raise_error=True
             )
             dat, ts = old_experiment.lookup_data_for_trial(trial_index=trial.index)
-            if ts != -1 and not dat.df.empty:
-                # Trial has data, so we replicate it on the new experiment.
-                new_trial = self.new_trial()
-                new_trial.add_arm(not_none(trial.arm).clone(clear_name=True))
-                new_trial.mark_running(no_runner_required=True)
-                new_trial.update_run_metadata(
-                    {"run_id": trial.run_metadata.get("run_id")}
-                )
-                new_trial._properties["source"] = (
-                    f"Warm start from Experiment: `{old_experiment.name}`, "
-                    f"trial: `{trial.index}`"
-                )
-                # Set trial index and arm name to their values in new trial.
-                new_df = dat.df.copy()
+            # Set trial index and arm name to their values in new trial.
+            new_trial = self.new_trial()
+            add_arm_and_maybe_clear_name(new_trial=new_trial, old_trial=trial)
+            new_trial.mark_running(no_runner_required=True)
+            new_trial._properties["source"] = (
+                f"Warm start from Experiment: `{old_experiment._name}`, "
+                f"trial: `{trial.index}`"
+            )
+            if copy_run_metadata_keys is not None:
+                for run_metadata_field in copy_run_metadata_keys:
+                    new_trial.update_run_metadata(
+                        {run_metadata_field: trial.run_metadata.get(run_metadata_field)}
+                    )
+            # Trial has data, so we replicate it on the new experiment.
+            has_data = ts != -1 and not dat.df.empty
+            if has_data:
+                new_df = dat.true_df.copy()
                 new_df["trial_index"].replace(
                     {trial.index: new_trial.index}, inplace=True
                 )
@@ -898,21 +1009,33 @@ class Experiment(Base):
                     {not_none(trial.arm).name: not_none(new_trial.arm).name},
                     inplace=True,
                 )
-                # Attach updated data to new trial on experiment and mark trial
-                # as completed.
-                self.attach_data(data=Data(df=new_df))
-                new_trial.mark_completed()
-                copied_trials.append(new_trial)
+                # Attach updated data to new trial on experiment.
+                old_data = (
+                    old_experiment.default_data_constructor(
+                        df=new_df,
+                        map_key_infos=checked_cast(
+                            MapData, old_experiment.lookup_data()
+                        ).map_key_infos,
+                    )
+                    if old_experiment.default_data_type == DataType.MAP_DATA
+                    else old_experiment.default_data_constructor(df=new_df)
+                )
+                self.attach_data(data=old_data)
+            if trial.status == TrialStatus.ABANDONED:
+                new_trial.mark_abandoned(reason=trial.abandoned_reason)
+            elif trial.status is not TrialStatus.RUNNING:
+                new_trial.mark_as(trial.status)
+            copied_trials.append(new_trial)
 
         if self._name is not None:
             logger.info(
                 f"Copied {len(copied_trials)} completed trials and their data "
-                f"from {old_experiment.name} to {self.name}."
+                f"from {old_experiment._name} to {self._name}."
             )
         else:
             logger.info(
                 f"Copied {len(copied_trials)} completed trials and their data "
-                f"from {old_experiment.name}."
+                f"from {old_experiment._name}."
             )
 
         return copied_trials
@@ -1003,3 +1126,22 @@ class Experiment(Base):
         with multiple trial types, use the MultiTypeExperiment class.
         """
         return trial_type is None
+
+
+def add_arm_and_maybe_clear_name(new_trial: Trial, old_trial: Trial) -> None:
+    # Clear name only if it matches auto-generated format.
+    # `arm_index` is 0 since all trials are single-armed.
+    clear_name = bool(
+        not_none(old_trial.arm).name == old_trial._get_default_name(arm_index=0)
+    )
+    if clear_name:
+        new_trial.add_arm(not_none(old_trial.arm).clone(clear_name=True))
+    else:
+        try:
+            new_trial.add_arm(not_none(old_trial.arm).clone(clear_name=False))
+        except ValueError as e:
+            warnings.warn(
+                f"Attaching arm {old_trial.arm} to trial {new_trial} while preserving "
+                f"its name failed with error: {e}. Retrying with `clear_name=True`."
+            )
+            new_trial.add_arm(not_none(old_trial.arm).clone(clear_name=True))
