@@ -1,17 +1,16 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Dict, Iterable, Optional, Type
+from typing import Any, Dict, Iterable, Optional, Tuple, Type, TYPE_CHECKING
 
-from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.data import Data
 from ax.utils.common.base import SortableBase
-from ax.utils.common.serialization import extract_init_args, serialize_init_args
+from ax.utils.common.serialization import SerializationMixin
 from ax.utils.common.typeutils import checked_cast
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -19,7 +18,7 @@ if TYPE_CHECKING:  # pragma: no cover
     from ax import core  # noqa F401
 
 
-class Metric(SortableBase):
+class Metric(SortableBase, SerializationMixin):
     """Base class for representing metrics.
 
     The `fetch_trial_data` method is the essential method to override when
@@ -50,6 +49,7 @@ class Metric(SortableBase):
         """
         self._name = name
         self.lower_is_better = lower_is_better
+        # pyre-fixme[4]: Attribute must be annotated.
         self.properties = properties or {}
 
     @classmethod
@@ -73,9 +73,10 @@ class Metric(SortableBase):
 
     @property
     def fetch_multi_group_by_metric(self) -> Type[Metric]:
-        """Metric class, with which to group this metric in `Experiment._metrics_by_class`,
-        which is used to combine metrics on experiment into groups and then fetch their
-        data via `Metric.fetch_trial_data_multi` for each group.
+        """Metric class, with which to group this metric in
+        `Experiment._metrics_by_class`, which is used to combine metrics on experiment
+        into groups and then fetch their data via `Metric.fetch_trial_data_multi` for
+        each group.
 
         NOTE: By default, this property will just return the class on which it is
         defined; however, in some cases it is useful to group metrics by their
@@ -83,25 +84,7 @@ class Metric(SortableBase):
         """
         return self.__class__
 
-    @classmethod
-    def serialize_init_args(cls, metric: Metric) -> Dict[str, Any]:
-        """Serialize the properties needed to initialize the metric.
-        Used for storage.
-        """
-        return serialize_init_args(
-            object=metric, exclude_fields=["name", "lower_is_better", "precomp_config"]
-        )
-
-    @classmethod
-    def deserialize_init_args(cls, args: Dict[str, Any]) -> Dict[str, Any]:
-        """Given a dictionary, extract the properties needed to initialize the metric.
-        Used for storage.
-        """
-        return extract_init_args(args=args, class_=cls)
-
-    def fetch_trial_data(
-        self, trial: core.base_trial.BaseTrial, **kwargs: Any
-    ) -> AbstractDataFrameData:
+    def fetch_trial_data(self, trial: core.base_trial.BaseTrial, **kwargs: Any) -> Data:
         """Fetch data for one trial."""
         raise NotImplementedError(
             f"Metric {self.name} does not implement data-fetching logic."
@@ -109,7 +92,7 @@ class Metric(SortableBase):
 
     def fetch_experiment_data(
         self, experiment: core.experiment.Experiment, **kwargs: Any
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         """Fetch this metric's data for an experiment.
 
         Default behavior is to fetch data from all trials expecting data
@@ -129,14 +112,13 @@ class Metric(SortableBase):
     @classmethod
     def fetch_trial_data_multi(
         cls, trial: core.base_trial.BaseTrial, metrics: Iterable[Metric], **kwargs: Any
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         """Fetch multiple metrics data for one trial.
 
         Default behavior calls `fetch_trial_data` for each metric.
         Subclasses should override this to trial data computation for multiple metrics.
         """
         dat = cls.data_constructor.from_multiple_data(
-            # pyre-fixme [6]: Expect `Iterable[Data]` got `List[AbstractDataFrameData]`
             [metric.fetch_trial_data(trial, **kwargs) for metric in metrics]
         )
         return dat
@@ -148,14 +130,13 @@ class Metric(SortableBase):
         metrics: Iterable[Metric],
         trials: Optional[Iterable[core.base_trial.BaseTrial]] = None,
         **kwargs: Any,
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         """Fetch multiple metrics data for an experiment.
 
         Default behavior calls `fetch_trial_data_multi` for each trial.
         Subclasses should override to batch data computation across trials + metrics.
         """
         return cls.data_constructor.from_multiple_data(
-            # pyre-fixme [6]: Expect `Iterable[Data]` got `List[AbstractDataFrameData]`
             [
                 cls.fetch_trial_data_multi(trial, metrics, **kwargs)
                 if trial.status.expecting_data
@@ -171,9 +152,11 @@ class Metric(SortableBase):
         metrics: Iterable[Metric],
         trials: Optional[Iterable[core.base_trial.BaseTrial]] = None,
         **kwargs: Any,
-    ) -> AbstractDataFrameData:
+    ) -> Tuple[Data, bool]:
         """Fetch or lookup (with fallback to fetching) data for given metrics,
-        depending on whether they are available while running.
+        depending on whether they are available while running. Return a tuple
+        containing the data, along with a boolean that will be True if new
+        data was fetched, and False if all data was looked up from cache.
 
         If metric is available while running, its data can change (and therefore
         we should always re-fetch it). If metric is available only upon trial
@@ -191,9 +174,7 @@ class Metric(SortableBase):
             fetched_data = cls.fetch_experiment_data_multi(
                 experiment=experiment, metrics=metrics, trials=trials, **kwargs
             )
-            if not fetched_data.df.empty:
-                experiment.attach_data(fetched_data)
-            return fetched_data
+            return fetched_data, True
 
         # If this metric is available only upon trial completion, look up data
         # on experiment and only fetch data that is not already cached.
@@ -205,13 +186,13 @@ class Metric(SortableBase):
             completed_trials = [t for t in trials if t.status.is_completed]
 
         if not completed_trials:
-            return cls.data_constructor()
+            return cls.data_constructor(), False
 
         trials_data = []
+        contains_new_data = False
         for trial in completed_trials:
             cached_trial_data = experiment.lookup_data_for_trial(
                 trial_index=trial.index,
-                merge_across_timestamps=kwargs.get("merge_across_timestamps", False),
             )[0]
 
             cached_metric_names = cached_trial_data.metric_names
@@ -229,29 +210,28 @@ class Metric(SortableBase):
                     trials=[trial],
                     **kwargs,
                 )
-
+                contains_new_data = True
             except NotImplementedError:
                 # Metric does not implement fetching logic and only uses lookup.
                 fetched_trial_data = cls.data_constructor()
 
             final_data = cls.data_constructor.from_multiple_data(
-                # pyre-fixme [6]: Incompatible paramtype: Expected `Data`
-                #   but got `AbstractDataFrameData`.
                 [cached_trial_data, fetched_trial_data]
             )
-            if not final_data.df.empty:
-                experiment.attach_data(final_data)
-            trials_data.append(final_data)
 
-        return cls.data_constructor.from_multiple_data(
-            trials_data, subset_metrics=[m.name for m in metrics]
+            trials_data.append(final_data)
+        return (
+            cls.data_constructor.from_multiple_data(
+                trials_data, subset_metrics=[m.name for m in metrics]
+            ),
+            contains_new_data,
         )
 
-    def clone(self) -> "Metric":
+    def clone(self) -> Metric:
         """Create a copy of this Metric."""
         cls = type(self)
         return cls(
-            **serialize_init_args(self),
+            **cls.deserialize_init_args(args=cls.serialize_init_args(obj=self)),
         )
 
     def __repr__(self) -> str:
