@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,7 +7,6 @@
 import logging
 from typing import Any, Dict, List, Optional, Set
 
-from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
 from ax.core.base_trial import BaseTrial
 from ax.core.data import Data
@@ -187,8 +186,12 @@ class MultiTypeExperiment(Experiment):
 
     @copy_doc(Experiment.fetch_data)
     def fetch_data(
-        self, metrics: Optional[List[Metric]] = None, **kwargs: Any
-    ) -> AbstractDataFrameData:
+        self,
+        metrics: Optional[List[Metric]] = None,
+        combine_with_last_data: bool = False,
+        overwrite_existing_data: bool = False,
+        **kwargs: Any,
+    ) -> Data:
         return self.default_data_constructor.from_multiple_data(
             [
                 trial.fetch_data(**kwargs, metrics=metrics)
@@ -201,7 +204,7 @@ class MultiTypeExperiment(Experiment):
     @copy_doc(Experiment._fetch_trial_data)
     def _fetch_trial_data(
         self, trial_index: int, metrics: Optional[List[Metric]] = None, **kwargs: Any
-    ) -> AbstractDataFrameData:
+    ) -> Data:
         trial = self.trials[trial_index]
         metrics = [
             metric
@@ -246,7 +249,6 @@ class MultiTypeExperiment(Experiment):
         """
         if trial.trial_type is None or not self.supports_trial_type(trial.trial_type):
             raise ValueError(f"Batch type `{trial.trial_type}` is not supported.")
-        # pyre-fixme[6]: Expected `str` for 1st param but got `Optional[str]`.
         return self._trial_type_to_runner[trial.trial_type]
 
     def supports_trial_type(self, trial_type: Optional[str]) -> bool:
