@@ -1,10 +1,11 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 from itertools import groupby
+from logging import Logger
 from typing import Dict, List, Optional
 
 from ax.core.metric import Metric
@@ -15,26 +16,19 @@ from ax.core.outcome_constraint import (
     OutcomeConstraint,
     ScalarizedOutcomeConstraint,
 )
+from ax.core.risk_measures import RiskMeasure
+from ax.exceptions.core import UserInputError
 from ax.utils.common.base import Base
 from ax.utils.common.logger import get_logger
 
-
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 
 TRefPoint = List[ObjectiveThreshold]
 
-OC_TEMPLATE: str = (
-    "OptimizationConfig(objective={objective}, outcome_constraints=[{constraints}])"
-)
-MOOC_TEMPLATE: str = (
-    "OptimizationConfig(objective={objective}, outcome_constraints=[{constraints}], "
-    "objective_thresholds=[{thresholds}])"
-)
-
 
 class OptimizationConfig(Base):
-    """An optimization configuration, which comprises an objective
-    and outcome constraints.
+    """An optimization configuration, which comprises an objective,
+    outcome constraints and an optional risk measure.
 
     There is no minimum or maximum number of outcome constraints, but an
     individual metric can have at most two constraints--which is how we
@@ -45,21 +39,27 @@ class OptimizationConfig(Base):
         self,
         objective: Objective,
         outcome_constraints: Optional[List[OutcomeConstraint]] = None,
+        risk_measure: Optional[RiskMeasure] = None,
     ) -> None:
         """Inits OptimizationConfig.
 
         Args:
             objective: Metric+direction to use for the optimization.
             outcome_constraints: Constraints on metrics.
+            risk_measure: An optional risk measure, used for robust optimization.
+                Must be used with a `RobustSearchSpace`.
         """
         constraints: List[OutcomeConstraint] = (
             [] if outcome_constraints is None else outcome_constraints
         )
         self._validate_optimization_config(
-            objective=objective, outcome_constraints=constraints
+            objective=objective,
+            outcome_constraints=constraints,
+            risk_measure=risk_measure,
         )
         self._objective: Objective = objective
         self._outcome_constraints: List[OutcomeConstraint] = constraints
+        self.risk_measure: Optional[RiskMeasure] = risk_measure
 
     def clone(self) -> "OptimizationConfig":
         """Make a copy of this optimization config."""
@@ -69,14 +69,18 @@ class OptimizationConfig(Base):
         self,
         objective: Optional[Objective] = None,
         outcome_constraints: Optional[List[OutcomeConstraint]] = None,
+        risk_measure: Optional[RiskMeasure] = None,
     ) -> "OptimizationConfig":
         """Make a copy of this optimization config."""
         objective = objective or self.objective.clone()
         outcome_constraints = outcome_constraints or [
             constraint.clone() for constraint in self.outcome_constraints
         ]
+        risk_measure = risk_measure or self.risk_measure
         return OptimizationConfig(
-            objective=objective, outcome_constraints=outcome_constraints
+            objective=objective,
+            outcome_constraints=outcome_constraints,
+            risk_measure=risk_measure,
         )
 
     @property
@@ -87,7 +91,9 @@ class OptimizationConfig(Base):
     @objective.setter
     def objective(self, objective: Objective) -> None:
         """Set objective if not present in outcome constraints."""
-        self._validate_optimization_config(objective, self.outcome_constraints)
+        self._validate_optimization_config(
+            objective, self.outcome_constraints, self.risk_measure
+        )
         self._objective = objective
 
     @property
@@ -120,11 +126,17 @@ class OptimizationConfig(Base):
             **objective_metrics,
         }
 
+    @property
+    def is_moo_problem(self) -> bool:
+        return self.objective is not None and isinstance(self.objective, MultiObjective)
+
     @outcome_constraints.setter
     def outcome_constraints(self, outcome_constraints: List[OutcomeConstraint]) -> None:
         """Set outcome constraints if valid, else raise."""
         self._validate_optimization_config(
-            objective=self.objective, outcome_constraints=outcome_constraints
+            objective=self.objective,
+            outcome_constraints=outcome_constraints,
+            risk_measure=self.risk_measure,
         )
         self._outcome_constraints = outcome_constraints
 
@@ -132,8 +144,10 @@ class OptimizationConfig(Base):
     def _validate_optimization_config(
         objective: Objective,
         outcome_constraints: Optional[List[OutcomeConstraint]] = None,
+        risk_measure: Optional[RiskMeasure] = None,
     ) -> None:
-        """Ensure outcome constraints are valid.
+        """Ensure outcome constraints are valid and the risk measure is
+        compatible with the objective.
 
         Either one or two outcome constraints can reference one metric.
         If there are two constraints, they must have different 'ops': one
@@ -142,10 +156,12 @@ class OptimizationConfig(Base):
             than the bound of the LEQ op.
 
         Args:
+            objective: Metric+direction to use for the optimization.
             outcome_constraints: Constraints to validate.
+            risk_measure: An optional risk measure to validate.
         """
         if type(objective) == MultiObjective:
-            # Raise error on exact equality; scalarizedObjective is OK
+            # Raise error on exact equality; `ScalarizedObjective` is OK
             raise ValueError(
                 (
                     "OptimizationConfig does not support MultiObjective. "
@@ -153,7 +169,7 @@ class OptimizationConfig(Base):
                 )
             )
         outcome_constraints = outcome_constraints or []
-        # only vaidate outcome_constraints
+        # Only vaidate `outcome_constraints`
         outcome_constraints = [
             constraint
             for constraint in outcome_constraints
@@ -202,18 +218,26 @@ class OptimizationConfig(Base):
                 raise ValueError(f"Duplicate outcome constraints {metric_name}")
 
     def __repr__(self) -> str:
-        return OC_TEMPLATE.format(
-            objective=repr(self.objective),
-            constraints=", ".join(
-                constraint.__repr__() for constraint in self.outcome_constraints
-            ),
+        base_repr = (
+            f"{self.__class__.__name__}("
+            "objective=" + repr(self.objective) + ", "
+            "outcome_constraints=" + repr(self.outcome_constraints)
         )
+        if self.risk_measure is None:
+            end_repr = ")"
+        else:
+            end_repr = ", risk_measure=" + repr(self.risk_measure) + ")"
+        return base_repr + end_repr
+
+    def __hash__(self) -> int:
+        """Make the class hashable to support grouping of GeneratorRuns."""
+        return hash(repr(self))
 
 
 class MultiObjectiveOptimizationConfig(OptimizationConfig):
     """An optimization configuration for multi-objective optimization,
-    which comprises multiple objective, outcome constraints, and objective
-    thresholds.
+    which comprises multiple objective, outcome constraints, objective
+    thresholds, and an optional risk measure.
 
     There is no minimum or maximum number of outcome constraints, but an
     individual metric can have at most two constraints--which is how we
@@ -229,6 +253,7 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         objective: Objective,
         outcome_constraints: Optional[List[OutcomeConstraint]] = None,
         objective_thresholds: Optional[List[ObjectiveThreshold]] = None,
+        risk_measure: Optional[RiskMeasure] = None,
     ) -> None:
         """Inits OptimizationConfig.
 
@@ -238,6 +263,8 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
             objective_thesholds: Thresholds objectives must exceed. Used for
                 multi-objective optimization and for calculating frontiers
                 and hypervolumes.
+            risk_measure: An optional risk measure, used for robust optimization.
+                Must be used with a `RobustSearchSpace`.
         """
         constraints: List[OutcomeConstraint] = (
             [] if outcome_constraints is None else outcome_constraints
@@ -247,16 +274,19 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
             objective=objective,
             outcome_constraints=constraints,
             objective_thresholds=objective_thresholds,
+            risk_measure=risk_measure,
         )
         self._objective: Objective = objective
         self._outcome_constraints: List[OutcomeConstraint] = constraints
         self._objective_thresholds: List[ObjectiveThreshold] = objective_thresholds
+        self.risk_measure: Optional[RiskMeasure] = risk_measure
 
     def clone_with_args(
         self,
         objective: Optional[Objective] = None,
         outcome_constraints: Optional[List[OutcomeConstraint]] = None,
         objective_thresholds: Optional[List[ObjectiveThreshold]] = None,
+        risk_measure: Optional[RiskMeasure] = None,
     ) -> "MultiObjectiveOptimizationConfig":
         """Make a copy of this optimization config."""
         objective = objective or self.objective.clone()
@@ -266,10 +296,12 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         objective_thresholds = objective_thresholds or [
             ot.clone() for ot in self.objective_thresholds
         ]
+        risk_measure = risk_measure or self.risk_measure
         return MultiObjectiveOptimizationConfig(
             objective=objective,
             outcome_constraints=outcome_constraints,
             objective_thresholds=objective_thresholds,
+            risk_measure=risk_measure,
         )
 
     @property
@@ -284,13 +316,13 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
             objective=objective,
             outcome_constraints=self.outcome_constraints,
             objective_thresholds=self.objective_thresholds,
+            risk_measure=self.risk_measure,
         )
         self._objective = objective
 
     @property
     def all_constraints(self) -> List[OutcomeConstraint]:
         """Get all constraints and thresholds."""
-        # pyre-ignore[58]: `+` not supported for Lists of different types.
         return self.outcome_constraints + self.objective_thresholds
 
     @property
@@ -324,7 +356,9 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
     ) -> None:
         """Set outcome constraints if valid, else raise."""
         self._validate_optimization_config(
-            objective=self.objective, objective_thresholds=objective_thresholds
+            objective=self.objective,
+            objective_thresholds=objective_thresholds,
+            risk_measure=self.risk_measure,
         )
         self._objective_thresholds = objective_thresholds
 
@@ -340,8 +374,10 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         objective: Objective,
         outcome_constraints: Optional[List[OutcomeConstraint]] = None,
         objective_thresholds: Optional[List[ObjectiveThreshold]] = None,
+        risk_measure: Optional[RiskMeasure] = None,
     ) -> None:
-        """Ensure outcome constraints are valid.
+        """Ensure outcome constraints are valid and the risk measure is
+        compatible with the objective.
 
         Either one or two outcome constraints can reference one metric.
         If there are two constraints, they must have different 'ops': one
@@ -350,7 +386,10 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
             than the bound of the LEQ op.
 
         Args:
+            objective: Metric+direction to use for the optimization.
             outcome_constraints: Constraints to validate.
+            objective_thesholds: Thresholds objectives must exceed.
+            risk_measure: An optional risk measure to validate.
         """
         if not isinstance(objective, (MultiObjective, ScalarizedObjective)):
             raise TypeError(
@@ -363,10 +402,12 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
             )
         outcome_constraints = outcome_constraints or []
         objective_thresholds = objective_thresholds or []
-
-        maybe_raise_wrong_direction_warning(
-            objective=objective, objective_thresholds=objective_thresholds
-        )
+        if isinstance(objective, MultiObjective):
+            objectives_by_name = {obj.metric.name: obj for obj in objective.objectives}
+            check_objective_thresholds_match_objectives(
+                objectives_by_name=objectives_by_name,
+                objective_thresholds=objective_thresholds,
+            )
 
         unconstrainable_metrics = objective.get_unconstrainable_metrics()
         OptimizationConfig._validate_outcome_constraints(
@@ -375,37 +416,56 @@ class MultiObjectiveOptimizationConfig(OptimizationConfig):
         )
 
     def __repr__(self) -> str:
-        return MOOC_TEMPLATE.format(
-            objective=repr(self.objective),
-            constraints=", ".join(
-                constraint.__repr__() for constraint in self.outcome_constraints
-            ),
-            thresholds=", ".join(
-                threshold.__repr__() for threshold in self.objective_thresholds
-            ),
+        base_repr = (
+            f"{self.__class__.__name__}("
+            "objective=" + repr(self.objective) + ", "
+            "outcome_constraints=" + repr(self.outcome_constraints) + ", "
+            "objective_thresholds=" + repr(self.objective_thresholds)
         )
+        if self.risk_measure is None:
+            end_repr = ")"
+        else:
+            end_repr = ", risk_measure=" + repr(self.risk_measure) + ")"
+        return base_repr + end_repr
 
 
-def maybe_raise_wrong_direction_warning(
-    objective: Objective,
+def check_objective_thresholds_match_objectives(
+    objectives_by_name: Dict[str, Objective],
     objective_thresholds: List[ObjectiveThreshold],
 ) -> None:
-    """Warn if thresholds on objective_metrics bound from the wrong direction."""
-    if not isinstance(objective, MultiObjective):
-        return
-
-    objectives_by_name = {obj.metric.name: obj for obj in objective.objectives}
+    """Error if thresholds on objective_metrics bound from the wrong direction or
+    if there is a mismatch between objective thresholds and objectives.
+    """
+    obj_thresh_metrics = set()
     for threshold in objective_thresholds:
         metric_name = threshold.metric.name
+        if metric_name not in objectives_by_name:
+            raise UserInputError(
+                f"Objective threshold {threshold} is on metric '{metric_name}', "
+                f"but that metric is not among the objectives."
+            )
+        if metric_name in obj_thresh_metrics:
+            raise UserInputError(
+                "More than one objective threshold specified for metric "
+                f"{metric_name}."
+            )
+        obj_thresh_metrics.add(metric_name)
+
         if metric_name in objectives_by_name:
             minimize = objectives_by_name[metric_name].minimize
             bounded_above = threshold.op == ComparisonOp.LEQ
             is_aligned = minimize == bounded_above
             if not is_aligned:
-                message = (
-                    f"Constraint on {metric_name} bounds from "
+                raise UserInputError(
+                    f"Objective threshold on {metric_name} bounds from "
                     f"{'above' if bounded_above else 'below'} "
                     f"but {metric_name} is being "
                     f"{'minimized' if minimize else 'maximized'}."
-                ).format(metric_name)
-                raise ValueError(message)
+                )
+
+    obj_metrics = set(objectives_by_name.keys())
+    if objective_thresholds and obj_thresh_metrics.symmetric_difference(obj_metrics):
+        raise UserInputError(
+            f"Objective thresholds: {obj_thresh_metrics} do not match objectives: "
+            f"{obj_metrics}."
+        )
