@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -75,7 +75,7 @@ class ParameterConstraint(SortableBase):
         )
         # Expected `int` for 2nd anonymous parameter to call `int.__le__` but got
         # `float`.
-        return weighted_sum <= self._bound
+        return weighted_sum <= self._bound + 1e-8  # allow for numerical imprecision
 
     def clone(self) -> ParameterConstraint:
         """Clone."""
