@@ -1,12 +1,12 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 import math
 from time import sleep
-from unittest.mock import PropertyMock, patch
+from unittest.mock import patch, PropertyMock
 
 import numpy as np
 from ax.core.arm import Arm
@@ -31,7 +31,7 @@ from ax.utils.testing.core_stubs import (
 
 
 class BatchTrialTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.experiment = get_experiment()
         self.experiment.status_quo = None
         self.batch = self.experiment.new_batch_trial()
@@ -43,7 +43,7 @@ class BatchTrialTest(TestCase):
         self.weights = weights[1:]
         self.batch.add_arms_and_weights(arms=self.arms, weights=self.weights)
 
-    def testEq(self):
+    def testEq(self) -> None:
         new_batch_trial = self.experiment.new_batch_trial()
         self.assertNotEqual(self.batch, new_batch_trial)
 
@@ -51,7 +51,7 @@ class BatchTrialTest(TestCase):
         abandoned_arm_2 = get_abandoned_arm()
         self.assertEqual(abandoned_arm, abandoned_arm_2)
 
-    def testBasicProperties(self):
+    def testBasicProperties(self) -> None:
         self.assertEqual(self.experiment, self.batch.experiment)
         self.assertEqual(self.batch.index, 0)
         self.assertEqual(self.batch.status, TrialStatus.CANDIDATE)
@@ -64,14 +64,14 @@ class BatchTrialTest(TestCase):
         # Test empty arms
         self.assertEqual(len(self.experiment.new_batch_trial().abandoned_arms), 0)
 
-    def testUndefinedSetters(self):
+    def testUndefinedSetters(self) -> None:
         with self.assertRaises(NotImplementedError):
             self.batch.arm_weights = get_arm_weights1()
 
         with self.assertRaises(NotImplementedError):
             self.batch.status = TrialStatus.RUNNING
 
-    def testBasicSetter(self):
+    def testBasicSetter(self) -> None:
         self.batch.runner = SyntheticRunner()
         self.assertIsNotNone(self.batch.runner)
 
@@ -82,7 +82,7 @@ class BatchTrialTest(TestCase):
         with self.assertRaises(ValueError):
             self.batch.trial_type = ""
 
-    def testAddArm(self):
+    def testAddArm(self) -> None:
         self.assertEqual(len(self.batch.arms), len(self.arms))
         self.assertEqual(len(self.batch.generator_run_structs), 1)
         self.assertEqual(sum(self.batch.weights), sum(self.weights))
@@ -96,7 +96,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(len(self.batch.generator_run_structs), 2)
         self.assertEqual(sum(self.batch.weights), sum(self.weights) + 3)
 
-    def testAddGeneratorRun(self):
+    def testAddGeneratorRun(self) -> None:
         self.assertEqual(len(self.batch.arms), len(self.arms))
         self.assertEqual(len(self.batch.generator_run_structs), 1)
         self.assertEqual(sum(self.batch.weights), sum(self.weights))
@@ -115,7 +115,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(len(self.batch.generator_run_structs), 2)
         self.assertEqual(sum(self.batch.weights), sum(self.weights) + 2)
 
-    def testInitWithGeneratorRun(self):
+    def testInitWithGeneratorRun(self) -> None:
         generator_run = GeneratorRun(arms=self.arms, weights=self.weights)
         batch = self.experiment.new_batch_trial(generator_run=generator_run)
         batch.add_arms_and_weights(arms=self.arms, weights=self.weights)
@@ -124,7 +124,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(len(batch.arms), len(self.arms))
         self.assertEqual(len(self.batch.generator_run_structs), 1)
 
-    def testStatusQuoOverlap(self):
+    def testStatusQuoOverlap(self) -> None:
         new_sq = Arm(parameters={"w": 0.95, "x": 1, "y": "foo", "z": True})
         # Set status quo to existing arm
         self.batch.set_status_quo_with_weight(self.arms[0], self.sq_weight)
@@ -146,7 +146,7 @@ class BatchTrialTest(TestCase):
             "status_quo_0",
         )
 
-    def testStatusQuo(self):
+    def testStatusQuo(self) -> None:
         tot_weight = sum(self.batch.weights)
         new_sq = Arm(parameters={"w": 0.95, "x": 1, "y": "foo", "z": True})
 
@@ -184,7 +184,7 @@ class BatchTrialTest(TestCase):
                 Arm(new_sq.parameters, name="new_name"), 1
             )
 
-    def testStatusQuoOptimizeForPower(self):
+    def testStatusQuoOptimizeForPower(self) -> None:
         self.experiment.status_quo = self.status_quo
         batch = self.experiment.new_batch_trial(optimize_for_power=True)
         self.assertEqual(batch._status_quo_weight_override, 1)
@@ -207,7 +207,7 @@ class BatchTrialTest(TestCase):
             )
         )
 
-    def testArmsByName(self):
+    def testArmsByName(self) -> None:
         # Initializes empty
         newbatch = self.experiment.new_batch_trial()
         self.assertEqual(newbatch.arms_by_name, {})
@@ -234,7 +234,7 @@ class BatchTrialTest(TestCase):
             {"0_0": self.batch.arms[0]},
         )
 
-    def testBatchLifecycle(self):
+    def testBatchLifecycle(self) -> None:
         # Check that state of trial statuses mapping on experiment: there should only be
         # one index, 0, among the `CANDIDATE` trials.
         trial_idcs_by_status = iter(self.experiment.trial_indices_by_status.values())
@@ -253,6 +253,8 @@ class BatchTrialTest(TestCase):
                 self.experiment.trial_indices_by_status[TrialStatus.STAGED], {0}
             )
             self.assertTrue(
+                # pyre-fixme[6]: For 1st param expected `Iterable[object]` but got
+                #  `bool`.
                 all(len(idcs) == 0)
                 for status, idcs in self.experiment.trial_indices_by_status.items()
                 if status != TrialStatus.STAGED
@@ -279,6 +281,8 @@ class BatchTrialTest(TestCase):
                 self.experiment.trial_indices_by_status[TrialStatus.RUNNING], {0}
             )
             self.assertTrue(
+                # pyre-fixme[6]: For 1st param expected `Iterable[object]` but got
+                #  `bool`.
                 all(len(idcs) == 0)
                 for status, idcs in self.experiment.trial_indices_by_status.items()
                 if status != TrialStatus.RUNNING
@@ -298,6 +302,8 @@ class BatchTrialTest(TestCase):
                 self.experiment.trial_indices_by_status[TrialStatus.COMPLETED], {0}
             )
             self.assertTrue(
+                # pyre-fixme[6]: For 1st param expected `Iterable[object]` but got
+                #  `bool`.
                 all(len(idcs) == 0)
                 for status, idcs in self.experiment.trial_indices_by_status.items()
                 if status != TrialStatus.COMPLETED
@@ -328,12 +334,14 @@ class BatchTrialTest(TestCase):
                 self.experiment.trial_indices_by_status[TrialStatus.CANDIDATE], {0}
             )
             self.assertTrue(
+                # pyre-fixme[6]: For 1st param expected `Iterable[object]` but got
+                #  `bool`.
                 all(len(idcs) == 0)
                 for status, idcs in self.experiment.trial_indices_by_status.items()
                 if status != TrialStatus.CANDIDATE
             )
 
-    def testAbandonBatchTrial(self):
+    def testAbandonBatchTrial(self) -> None:
         reason = "BatchTrial behaved poorly"
         self.batch.mark_abandoned(reason)
 
@@ -341,7 +349,7 @@ class BatchTrialTest(TestCase):
         self.assertIsNotNone(self.batch.time_completed)
         self.assertEqual(self.batch.abandoned_reason, reason)
 
-    def testFailedBatchTrial(self):
+    def testFailedBatchTrial(self) -> None:
         self.batch.runner = SyntheticRunner()
         self.batch.run()
         self.batch.mark_failed()
@@ -349,7 +357,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(self.batch.status, TrialStatus.FAILED)
         self.assertIsNotNone(self.batch.time_completed)
 
-    def testEarlyStoppedBatchTrial(self):
+    def testEarlyStoppedBatchTrial(self) -> None:
         self.batch.runner = SyntheticRunner()
         self.batch.run()
         self.batch.mark_early_stopped()
@@ -357,7 +365,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(self.batch.status, TrialStatus.EARLY_STOPPED)
         self.assertIsNotNone(self.batch.time_completed)
 
-    def testAbandonArm(self):
+    def testAbandonArm(self) -> None:
         arm = self.batch.arms[0]
         reason = "Bad arm"
         self.batch.mark_arm_abandoned(arm.name, reason)
@@ -375,14 +383,14 @@ class BatchTrialTest(TestCase):
                 Arm(parameters={"x": 3, "y": "fooz", "z": False})
             )
 
-    def testClone(self):
+    def testClone(self) -> None:
         new_batch_trial = self.batch.clone()
         self.assertEqual(len(new_batch_trial.generator_run_structs), 1)
         self.assertEqual(len(new_batch_trial.arms), 2)
         self.assertEqual(new_batch_trial.runner, self.batch.runner)
         self.assertEqual(new_batch_trial.trial_type, self.batch.trial_type)
 
-    def testRunner(self):
+    def testRunner(self) -> None:
         # Verify BatchTrial without runner will fail
         with self.assertRaises(ValueError):
             self.batch.run()
@@ -410,7 +418,7 @@ class BatchTrialTest(TestCase):
             self.assertEqual(b2.deployed_name, "test_1")
             self.assertEqual(b2.status, TrialStatus.STAGED)
 
-    def testIsFactorial(self):
+    def testIsFactorial(self) -> None:
         self.assertFalse(self.batch.is_factorial)
 
         # Insufficient factors
@@ -442,12 +450,14 @@ class BatchTrialTest(TestCase):
         )
         self.assertTrue(new_batch_trial.is_factorial)
 
-    def testNormalizedArmWeights(self):
+    def testNormalizedArmWeights(self) -> None:
         new_batch_trial = self.experiment.new_batch_trial()
         parameterizations = [
             {"w": 0.75, "x": 1, "y": "foo", "z": True},
             {"w": 0.77, "x": 2, "y": "foo", "z": True},
         ]
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Dict[str, Union[float, str]]`.
         arms = [Arm(parameters=p) for i, p in enumerate(parameterizations)]
         new_batch_trial.add_arms_and_weights(arms=arms, weights=[2, 1])
 
@@ -473,7 +483,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(batch_arm_parameters, arm_parameters)
         self.assertTrue(np.allclose(list(arm_weights.values()), [0.67, 0.33]))
 
-    def testAddGeneratorRunValidation(self):
+    def testAddGeneratorRunValidation(self) -> None:
         new_batch_trial = self.experiment.new_batch_trial()
         new_arms = [
             Arm(name="0_1", parameters={"w": 0.75, "x": 1, "y": "foo", "z": True}),
@@ -483,7 +493,7 @@ class BatchTrialTest(TestCase):
         with self.assertRaises(ValueError):
             new_batch_trial.add_generator_run(gr)
 
-    def testSetStatusQuoAndOptimizePower(self):
+    def testSetStatusQuoAndOptimizePower(self) -> None:
         batch_trial = self.experiment.new_batch_trial()
         status_quo = Arm(
             name="status_quo", parameters={"w": 0.0, "x": 1, "y": "foo", "z": True}
@@ -499,6 +509,8 @@ class BatchTrialTest(TestCase):
             {"w": 0.75, "x": 1, "y": "foo", "z": True},
             {"w": 0.77, "x": 2, "y": "foo", "z": True},
         ]
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Dict[str, Union[float, str]]`.
         arms = [Arm(parameters=p) for i, p in enumerate(parameterizations)]
         batch_trial.add_arms_and_weights(arms=arms)
         batch_trial.set_status_quo_and_optimize_power(status_quo)
@@ -524,6 +536,8 @@ class BatchTrialTest(TestCase):
             {"w": 0.77, "x": 2, "y": "foo", "z": True},
             {"w": 0.0, "x": 1, "y": "foo", "z": True},
         ]
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Dict[str, Union[float, str]]`.
         arms = [Arm(parameters=p) for i, p in enumerate(parameterizations)]
         batch_trial.add_arms_and_weights(arms=arms)
         batch_trial.set_status_quo_and_optimize_power(status_quo)
@@ -532,13 +546,13 @@ class BatchTrialTest(TestCase):
         # its weight comes from _status_quo_weight_override
         self.assertEqual(batch_trial._status_quo_weight_override, np.sqrt(2))
 
-    def testRepr(self):
+    def testRepr(self) -> None:
         self.assertEqual(
             str(self.batch),
             "BatchTrial(experiment_name='test', index=0, status=TrialStatus.CANDIDATE)",
         )
 
-    def test_TTL(self):
+    def test_TTL(self) -> None:
         # Verify that TLL is checked on execution of the `status` property.
         self.batch.ttl_seconds = 1
         self.batch.mark_running(no_runner_required=True)
@@ -567,7 +581,7 @@ class BatchTrialTest(TestCase):
         self.assertEqual(batch_trial._status, TrialStatus.FAILED)
         self.assertIn(2, self.experiment.trial_indices_by_status[TrialStatus.FAILED])
 
-    def test_get_candidate_metadata_from_all_generator_runs(self):
+    def test_get_candidate_metadata_from_all_generator_runs(self) -> None:
         gr_1 = get_generator_run()
         gr_2 = get_generator_run2()
         self.batch.add_generator_run(gr_1)
@@ -600,6 +614,8 @@ class BatchTrialTest(TestCase):
         new_cand_metadata = {
             a.signature: {"md_key": f"md_val_{a.signature}"} for a in gr_3.arms
         }
+        # pyre-fixme[8]: Attribute has type `Optional[Dict[str, Optional[Dict[str,
+        #  typing.Any]]]]`; used as `Dict[str, Dict[str, str]]`.
         gr_3._candidate_metadata_by_arm_signature = new_cand_metadata
         self.batch.add_generator_run(gr_3)
         gr_3 = self.batch._generator_run_structs[-1].generator_run
@@ -619,7 +635,7 @@ class BatchTrialTest(TestCase):
                 self.batch._get_candidate_metadata(arm.name),
             )
 
-    def testSortable(self):
+    def testSortable(self) -> None:
         new_batch_trial = self.experiment.new_batch_trial()
         self.assertTrue(self.batch < new_batch_trial)
 
