@@ -1,17 +1,18 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from typing import Type
+import logging
+from typing import Dict, Type
 from unittest.mock import patch
 
 import pandas as pd
 from ax.core.arm import Arm
 from ax.core.base_trial import TrialStatus
 from ax.core.data import Data
-from ax.core.experiment import DataType, Experiment
+from ax.core.experiment import Experiment
 from ax.core.map_data import MapData
 from ax.core.map_metric import MapMetric
 from ax.core.metric import Metric
@@ -20,30 +21,36 @@ from ax.core.search_space import SearchSpace
 from ax.exceptions.core import UnsupportedError
 from ax.metrics.branin import BraninMetric
 from ax.runners.synthetic import SyntheticRunner
-from ax.utils.common.constants import Keys
+from ax.service.ax_client import AxClient
+from ax.utils.common.constants import EXPERIMENT_IS_TEST_WARNING, Keys
 from ax.utils.common.testutils import TestCase
 from ax.utils.testing.core_stubs import (
     get_arm,
     get_branin_arms,
+    get_branin_experiment,
+    get_branin_experiment_with_multi_objective,
+    get_branin_experiment_with_timestamp_map_metric,
     get_branin_optimization_config,
     get_branin_search_space,
-    get_branin_experiment,
-    get_branin_experiment_with_timestamp_map_metric,
     get_data,
     get_experiment,
-    get_experiment_with_map_data,
     get_experiment_with_map_data_type,
-    get_map_data,
     get_optimization_config,
+    get_scalarized_outcome_constraint,
     get_search_space,
     get_sobol,
     get_status_quo,
-    get_scalarized_outcome_constraint,
 )
 
+DUMMY_RUN_METADATA_KEY = "test_run_metadata_key"
+DUMMY_RUN_METADATA_VALUE = "test_run_metadata_value"
+DUMMY_RUN_METADATA: Dict[str, str] = {DUMMY_RUN_METADATA_KEY: DUMMY_RUN_METADATA_VALUE}
+DUMMY_ABANDONED_REASON = "test abandoned reason"
+DUMMY_ARM_NAME = "test_arm_name"
+
 
 class ExperimentTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.experiment = get_experiment()
 
     def _setupBraninExperiment(self, n: int) -> Experiment:
@@ -62,7 +69,7 @@ class ExperimentTest(TestCase):
         batch_2.run()
         return exp
 
-    def testExperimentInit(self):
+    def testExperimentInit(self) -> None:
         self.assertEqual(self.experiment.name, "test")
         self.assertEqual(self.experiment.description, "test description")
         self.assertEqual(self.experiment.name, "test")
@@ -70,7 +77,7 @@ class ExperimentTest(TestCase):
         self.assertEqual(self.experiment.experiment_type, None)
         self.assertEqual(self.experiment.num_abandoned_arms, 0)
 
-    def testExperimentName(self):
+    def testExperimentName(self) -> None:
         self.assertTrue(self.experiment.has_name)
         self.experiment.name = None
         self.assertFalse(self.experiment.has_name)
@@ -78,11 +85,11 @@ class ExperimentTest(TestCase):
             self.experiment.name
         self.experiment.name = "test"
 
-    def testExperimentType(self):
+    def testExperimentType(self) -> None:
         self.experiment.experiment_type = "test"
         self.assertEqual(self.experiment.experiment_type, "test")
 
-    def testEq(self):
+    def testEq(self) -> None:
         self.assertEqual(self.experiment, self.experiment)
 
         experiment2 = Experiment(
@@ -94,13 +101,13 @@ class ExperimentTest(TestCase):
         )
         self.assertNotEqual(self.experiment, experiment2)
 
-    def testDBId(self):
+    def testDBId(self) -> None:
         self.assertIsNone(self.experiment.db_id)
         some_id = 123456789
         self.experiment.db_id = some_id
         self.assertEqual(self.experiment.db_id, some_id)
 
-    def testTrackingMetricsMerge(self):
+    def testTrackingMetricsMerge(self) -> None:
         # Tracking and optimization metrics should get merged
         # m1 is on optimization_config while m3 is not
         exp = Experiment(
@@ -109,9 +116,10 @@ class ExperimentTest(TestCase):
             optimization_config=get_optimization_config(),
             tracking_metrics=[Metric(name="m1"), Metric(name="m3")],
         )
+        # pyre-fixme[16]: Optional type has no attribute `metrics`.
         self.assertEqual(len(exp.optimization_config.metrics) + 1, len(exp.metrics))
 
-    def testBasicBatchCreation(self):
+    def testBasicBatchCreation(self) -> None:
         batch = self.experiment.new_batch_trial()
         self.assertEqual(len(self.experiment.trials), 1)
         self.assertEqual(self.experiment.trials[0], batch)
@@ -125,16 +133,90 @@ class ExperimentTest(TestCase):
             new_exp = get_experiment()
             new_exp._attach_trial(batch)
 
-    def testRepr(self):
+    def testRepr(self) -> None:
         self.assertEqual("Experiment(test)", str(self.experiment))
 
-    def testBasicProperties(self):
+    def testBasicProperties(self) -> None:
         self.assertEqual(self.experiment.status_quo, get_status_quo())
         self.assertEqual(self.experiment.search_space, get_search_space())
         self.assertEqual(self.experiment.optimization_config, get_optimization_config())
         self.assertEqual(self.experiment.is_test, True)
 
-    def testMetricSetters(self):
+    def testOnlyRangeParameterConstraints(self) -> None:
+        self.assertEqual(0, 0)
+        self.assertTrue(True)
+
+        ax_client = AxClient()
+
+        # Create an experiment with valid parameter constraints
+        ax_client.create_experiment(
+            name="experiment",
+            # pyre-fixme[6]: For 2nd param expected `List[Dict[str, Union[None,
+            #  Dict[str, List[str]], List[Union[None, bool, float, int, str]], bool,
+            #  float, int, str]]]` but got `List[Dict[str, Union[List[float], str]]]`.
+            parameters=[
+                {
+                    "name": "x1",
+                    "type": "range",
+                    "bounds": [0.0, 1.0],
+                },
+                {
+                    "name": "x2",
+                    "type": "range",
+                    "bounds": [0.0, 1.0],
+                },
+            ],
+            objective_name="objective",
+            parameter_constraints=["x1 + x2 <= 1"],
+        )
+
+        # Try (and fail) to create an experiment with constraints on choice
+        # paramaters
+        with self.assertRaises(UnsupportedError):
+            ax_client.create_experiment(
+                name="experiment",
+                # pyre-fixme[6]: For 2nd param expected `List[Dict[str, Union[None,
+                #  Dict[str, List[str]], List[Union[None, bool, float, int, str]],
+                #  bool, float, int, str]]]` but got `List[Dict[str, Union[List[float],
+                #  str]]]`.
+                parameters=[
+                    {
+                        "name": "x1",
+                        "type": "choice",
+                        "values": [0.0, 1.0],
+                    },
+                    {
+                        "name": "x2",
+                        "type": "range",
+                        "bounds": [0.0, 1.0],
+                    },
+                ],
+                objective_name="objective",
+                parameter_constraints=["x1 + x2 <= 1"],
+            )
+
+        # Try (and fail) to create an experiment with constraints on fixed
+        # parameters
+        with self.assertRaises(UnsupportedError):
+            ax_client.create_experiment(
+                name="experiment",
+                # pyre-fixme[6]: For 2nd param expected `List[Dict[str, Union[None,
+                #  Dict[str, List[str]], List[Union[None, bool, float, int, str]],
+                #  bool, float, int, str]]]` but got `List[Dict[str, Union[List[float],
+                #  float, str]]]`.
+                parameters=[
+                    {"name": "x1", "type": "fixed", "value": 0.0},
+                    {
+                        "name": "x2",
+                        "type": "range",
+                        "bounds": [0.0, 1.0],
+                    },
+                ],
+                objective_name="objective",
+                parameter_constraints=["x1 + x2 <= 1"],
+            )
+
+    def testMetricSetters(self) -> None:
         # Establish current metrics size
         self.assertEqual(
             len(get_optimization_config().metrics) + 1, len(self.experiment.metrics)
@@ -206,7 +288,7 @@ class ExperimentTest(TestCase):
         with self.assertRaises(ValueError):
             self.experiment.remove_tracking_metric(metric_name="m5")
 
-    def testSearchSpaceSetter(self):
+    def testSearchSpaceSetter(self) -> None:
         one_param_ss = SearchSpace(parameters=[get_search_space().parameters["w"]])
 
         # Verify all search space ok with no trials
@@ -233,7 +315,7 @@ class ExperimentTest(TestCase):
         with self.assertRaises(ValueError):
             self.experiment.search_space = extra_param_ss
 
-    def testStatusQuoSetter(self):
+    def testStatusQuoSetter(self) -> None:
         sq_parameters = self.experiment.status_quo.parameters
         self.experiment.status_quo = None
         self.assertIsNone(self.experiment.status_quo)
@@ -297,13 +379,14 @@ class ExperimentTest(TestCase):
         batch.add_arms_and_weights(arms=arms)
         self.assertIsNone(exp.status_quo)
         exp.status_quo = arms[0]
+        # pyre-fixme[16]: Optional type has no attribute `name`.
         self.assertEqual(exp.status_quo.name, "0_0")
 
         # Try setting sq to existing arm with different name
         with self.assertRaises(ValueError):
             exp.status_quo = Arm(arms[0].parameters, name="new_name")
 
-    def testRegisterArm(self):
+    def testRegisterArm(self) -> None:
         # Create a new arm, register on experiment
         parameters = self.experiment.status_quo.parameters
         parameters["w"] = 3.5
@@ -312,7 +395,7 @@ class ExperimentTest(TestCase):
         self.assertEqual(self.experiment.arms_by_name[arm.name], arm)
         self.assertEqual(self.experiment.arms_by_signature[arm.signature], arm)
 
-    def testFetchAndStoreData(self):
+    def testFetchAndStoreData(self) -> None:
         n = 10
         exp = self._setupBraninExperiment(n)
         batch = exp.trials[0]
@@ -372,14 +455,10 @@ class ExperimentTest(TestCase):
         # Test retrieving full exp data
         self.assertEqual(len(exp.lookup_data_for_ts(t2).df), 4 * n)
 
-        # Test merging multiple timestamps of data
-        self.assertEqual(
-            len(exp.lookup_data_for_trial(0, merge_across_timestamps=True)), 2
-        )
-
         with self.assertRaisesRegex(ValueError, ".* for metric"):
             exp.attach_data(batch_data, combine_with_last_data=True)
 
+        self.assertEqual(len(full_dict[0]), 5)  # 5 data objs for batch 0
         new_data = Data(
             df=pd.DataFrame.from_records(
                 [
@@ -394,7 +473,8 @@ class ExperimentTest(TestCase):
             )
         )
         t3 = exp.attach_data(new_data, combine_with_last_data=True)
-        self.assertEqual(len(full_dict[0]), 6)  # 6 data objs for batch 0 now
+        # still 5 data objs, since we combined last one
+        self.assertEqual(len(full_dict[0]), 5)
         self.assertIn("z", exp.lookup_data_for_ts(t3).df["metric_name"].tolist())
 
         # Verify we don't get the data if the trial is abandoned
@@ -422,7 +502,48 @@ class ExperimentTest(TestCase):
             exp.fetch_data(metrics=[Metric(name="not_on_experiment")]), Data()
         )
 
-    def testEmptyMetrics(self):
+    def testOverwriteExistingData(self) -> None:
+        n = 10
+        exp = self._setupBraninExperiment(n)
+
+        # automatically attaches data
+        data = exp.fetch_data()
+
+        # can't set both combine_with_last_data and overwrite_existing_data
+        with self.assertRaises(UnsupportedError):
+            exp.attach_data(
+                data, combine_with_last_data=True, overwrite_existing_data=True
+            )
+
+        # data exists for two trials
+        # data has been attached once for each trial
+        self.assertEqual(len(exp._data_by_trial), 2)
+        self.assertEqual(len(exp._data_by_trial[0]), 1)
+        self.assertEqual(len(exp._data_by_trial[1]), 1)
+
+        exp.attach_data(data)
+        # data has been attached twice for each trial
+        self.assertEqual(len(exp._data_by_trial), 2)
+        self.assertEqual(len(exp._data_by_trial[0]), 2)
+        self.assertEqual(len(exp._data_by_trial[1]), 2)
+
+        ts = exp.attach_data(data, overwrite_existing_data=True)
+        # previous two attachment are overwritten,
+        # now only one data (most recent one) per trial
+        self.assertEqual(len(exp._data_by_trial), 2)
+        self.assertEqual(len(exp._data_by_trial[0]), 1)
+        self.assertEqual(len(exp._data_by_trial[1]), 1)
+        self.assertTrue(ts in exp._data_by_trial[0])
+        self.assertTrue(ts in exp._data_by_trial[1])
+
+        with self.assertRaisesRegex(
+            ValueError, "new data contains only a subset of the metrics"
+        ):
+            # prevent users from overwriting metrics
+            data.df["metric_name"] = "a"
+            exp.attach_data(data, overwrite_existing_data=True)
+
+    def testEmptyMetrics(self) -> None:
         empty_experiment = Experiment(
             name="test_experiment", search_space=get_search_space()
         )
@@ -440,7 +561,7 @@ class ExperimentTest(TestCase):
         batch.mark_completed()
         self.assertFalse(empty_experiment.fetch_data().df.empty)
 
-    def testNumArmsNoDeduplication(self):
+    def testNumArmsNoDeduplication(self) -> None:
         exp = Experiment(name="test_experiment", search_space=get_search_space())
         arm = get_arm()
         exp.new_batch_trial().add_arm(arm)
@@ -450,7 +571,7 @@ class ExperimentTest(TestCase):
         trial.mark_arm_abandoned(trial.arms[0].name)
         self.assertEqual(exp.num_abandoned_arms, 1)
 
-    def testExperimentWithoutName(self):
+    def testExperimentWithoutName(self) -> None:
         exp = Experiment(
             search_space=get_branin_search_space(),
             tracking_metrics=[BraninMetric(name="b", param_names=["x1", "x2"])],
@@ -462,7 +583,7 @@ class ExperimentTest(TestCase):
         batch.run()
         self.assertEqual(batch.run_metadata, {"name": "0"})
 
-    def testExperimentRunner(self):
+    def testExperimentRunner(self) -> None:
         original_runner = SyntheticRunner()
         self.experiment.runner = original_runner
         batch = self.experiment.new_batch_trial()
@@ -485,6 +606,8 @@ class ExperimentTest(TestCase):
         self.assertEqual(tibs[TrialStatus.CANDIDATE], {1})
 
         identifier = {"new_runner": True}
+        # pyre-fixme[6]: For 1st param expected `Optional[str]` but got `Dict[str,
+        #  bool]`.
         new_runner = SyntheticRunner(dummy_metadata=identifier)
 
         self.experiment.reset_runners(new_runner)
@@ -495,7 +618,7 @@ class ExperimentTest(TestCase):
         # Update candidate trial runners.
         self.assertEqual(self.experiment.trials[1].runner, new_runner)
 
-    def testFetchTrialsData(self):
+    def testFetchTrialsData(self) -> None:
         exp = self._setupBraninExperiment(n=5)
         batch_0 = exp.trials[0]
         batch_1 = exp.trials[1]
@@ -515,6 +638,9 @@ class ExperimentTest(TestCase):
             exp.fetch_trials_data(trial_indices=[0, 1]),
             Data.from_multiple_data([batch_0_data, batch_1_data]),
         )
+
+        self.assertEqual(len(exp.data_by_trial[0]), 2)
+
         with self.assertRaisesRegex(ValueError, ".* not associated .*"):
             exp.fetch_trials_data(trial_indices=[2])
         # Try to fetch data when there are only metrics and no attached data.
@@ -531,7 +657,7 @@ class ExperimentTest(TestCase):
             set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
         )
 
-    def test_immutable_search_space_and_opt_config(self):
+    def test_immutable_search_space_and_opt_config(self) -> None:
         mutable_exp = self._setupBraninExperiment(n=5)
         self.assertFalse(mutable_exp.immutable_search_space_and_opt_config)
         immutable_exp = Experiment(
@@ -560,7 +686,7 @@ class ExperimentTest(TestCase):
         )
         self.assertTrue(immutable_exp_2.immutable_search_space_and_opt_config)
 
-    def test_fetch_as_class(self):
+    def test_fetch_as_class(self) -> None:
         class MyMetric(Metric):
             @property
             def fetch_multi_group_by_metric(self) -> Type[Metric]:
@@ -580,6 +706,8 @@ class ExperimentTest(TestCase):
         f"{BraninMetric.__module__}.BraninMetric.fetch_experiment_data_multi",
         side_effect=BraninMetric.fetch_experiment_data_multi,
     )
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def test_prefer_lookup_where_possible(self, mock_fetch_exp_data_multi):
         # By default, `BraninMetric` is available while trial is running.
         exp = self._setupBraninExperiment(n=5)
@@ -604,6 +732,7 @@ class ExperimentTest(TestCase):
 
             # 2. Newly completed trials => fetch case.
             mock_fetch_exp_data_multi.reset_mock()
+            # pyre-fixme[16]: Optional type has no attribute `mark_completed`.
             exp.trials.get(0).mark_completed()
             exp.trials.get(1).mark_completed()
             dat = exp.fetch_data()
@@ -619,137 +748,15 @@ class ExperimentTest(TestCase):
             # All fetched data should get cached, so no fetch should happen next time.
             exp.fetch_data()
             mock_fetch_exp_data_multi.assert_not_called()
+            # No new data should be attached to the experiment
+            self.assertEqual(len(exp._data_by_trial), 2)
 
-
-class ExperimentWithMapDataTest(TestCase):
-    def setUp(self):
-        self.experiment = get_experiment_with_map_data_type()
-
-    def _setupBraninExperiment(self, n: int) -> Experiment:
-        exp = get_branin_experiment_with_timestamp_map_metric()
-        batch = exp.new_batch_trial()
-        batch.add_arms_and_weights(arms=get_branin_arms(n=n, seed=0))
-        batch.run()
-
-        batch_2 = exp.new_batch_trial()
-        batch_2.add_arms_and_weights(arms=get_branin_arms(n=3 * n, seed=1))
-        batch_2.run()
-        return exp
-
-    def testEmptyMetrics(self):
-        empty_experiment = Experiment(
-            name="test_experiment",
-            search_space=get_search_space(),
-            default_data_type=DataType.MAP_DATA,
-        )
-        self.assertEqual(empty_experiment.num_trials, 0)
-        empty_experiment.add_tracking_metric(Metric(name="ax_test_metric"))
-        self.assertTrue(empty_experiment.fetch_data().df.empty)
-        empty_experiment.attach_data(get_map_data())
-
-    def testFetchDataWithMapData(self):
-        evaluations = {
-            "0_0": [
-                ({"epoch": 1}, {"no_fetch_impl_metric": (3.7, 0.5)}),
-                ({"epoch": 2}, {"no_fetch_impl_metric": (3.8, 0.5)}),
-                ({"epoch": 3}, {"no_fetch_impl_metric": (3.9, 0.5)}),
-                ({"epoch": 4}, {"no_fetch_impl_metric": (4.0, 0.5)}),
-            ],
-        }
-
-        self.experiment.add_tracking_metric(
-            metric=MapMetric(name="no_fetch_impl_metric")
-        )
-        self.experiment.new_trial()
-        self.experiment.trials[0].mark_running(no_runner_required=True)
-        first_epoch = MapData.from_map_evaluations(
-            evaluations={
-                arm_name: partial_results[0:1]
-                for arm_name, partial_results in evaluations.items()
-            },
-            trial_index=0,
-        )
-        self.experiment.attach_data(first_epoch)
-        remaining_epochs = MapData.from_map_evaluations(
-            evaluations={
-                arm_name: partial_results[1:4]
-                for arm_name, partial_results in evaluations.items()
-            },
-            trial_index=0,
-        )
-        self.experiment.attach_data(remaining_epochs)
-        self.experiment.trials[0].mark_completed()
-
-        # merge_across_timestamps = False
-        expected_data = remaining_epochs
-        actual_data = self.experiment.lookup_data()  # False by default.
-        self.assertEqual(expected_data, actual_data)
-
-        # merge_across_timestamps = True
-        expected_data = MapData.from_map_evaluations(
-            evaluations=evaluations, trial_index=0
-        )
-        actual_data = self.experiment.lookup_data(merge_across_timestamps=True)
-        self.assertEqual(expected_data, actual_data)
-
-    def testFetchTrialsData(self):
-        exp = self._setupBraninExperiment(n=5)
-        batch_0 = exp.trials[0]
-        batch_1 = exp.trials[1]
-        batch_0.mark_completed()
-        batch_1.mark_completed()
-        batch_0_data = exp.fetch_trials_data(trial_indices=[0])
-        self.assertEqual(set(batch_0_data.df["trial_index"].values), {0})
-        self.assertEqual(
-            set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
-        )
-        batch_1_data = exp.fetch_trials_data(trial_indices=[1])
-        self.assertEqual(set(batch_1_data.df["trial_index"].values), {1})
-        self.assertEqual(
-            set(batch_1_data.df["arm_name"].values), {a.name for a in batch_1.arms}
-        )
-        self.assertEqual(
-            exp.fetch_trials_data(trial_indices=[0, 1]),
-            MapData.from_multiple_data([batch_0_data, batch_1_data]),
-        )
-        with self.assertRaisesRegex(ValueError, ".* not associated .*"):
-            exp.fetch_trials_data(trial_indices=[2])
-        # Try to fetch data when there are only metrics and no attached data.
-        exp.remove_tracking_metric(metric_name="b")  # Remove implemented metric.
-        exp.add_tracking_metric(MapMetric(name="b"))  # Add unimplemented metric.
-        self.assertEqual(len(exp.fetch_trials_data(trial_indices=[0]).df), 30)
-        # Try fetching attached data.
-        exp.attach_data(batch_0_data)
-        exp.attach_data(batch_1_data)
-        self.assertEqual(exp.fetch_trials_data(trial_indices=[0]), batch_0_data)
-        self.assertEqual(exp.fetch_trials_data(trial_indices=[1]), batch_1_data)
-        self.assertEqual(set(batch_0_data.df["trial_index"].values), {0})
-        self.assertEqual(
-            set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
-        )
-
-    def testFetchTrialDataLookupOnly(self):
-        experiment = get_experiment_with_map_data()
-        self.assertTrue(experiment.optimization_config.objective.metric)
-        experiment.trials[0].mark_running(no_runner_required=True)
-        actual_data = experiment.fetch_trials_data(trial_indices=[0], lookup_only=True)
-        expected_data = get_map_data()
-        self.assertNotEqual(expected_data, actual_data)
-        with patch(
-            f"{MapMetric.__module__}.MapMetric.is_available_while_running",
-            return_value=True,
-        ):
-            experiment = get_experiment_with_map_data()
-            self.assertTrue(experiment.optimization_config.objective.metric)
-            experiment.trials[0].mark_running(no_runner_required=True)
-            actual_data = experiment.lookup_data()
-            expected_data = get_map_data()
-            self.assertEqual(expected_data, actual_data)
-
-    def testWarmStartFromOldExperiment(self):
+    def testWarmStartFromOldExperiment(self) -> None:
         # create old_experiment
-        len_old_trials = 5
-        i_failed_trial = 3
+        len_old_trials = 7
+        i_failed_trial = 1
+        i_abandoned_trial = 3
+        i_running_trial = 5
         old_experiment = get_branin_experiment()
         for i_old_trial in range(len_old_trials):
             sobol_run = get_sobol(search_space=old_experiment.search_space).gen(n=1)
@@ -757,6 +764,10 @@ class ExperimentWithMapDataTest(TestCase):
             trial.mark_running(no_runner_required=True)
             if i_old_trial == i_failed_trial:
                 trial.mark_failed()
+            elif i_old_trial == i_abandoned_trial:
+                trial.mark_abandoned(reason=DUMMY_ABANDONED_REASON)
+            elif i_old_trial == i_running_trial:
+                pass
             else:
                 trial.mark_completed()
         # make metric noiseless for exact reproducibility
@@ -778,25 +789,274 @@ class ExperimentWithMapDataTest(TestCase):
         new_experiment = get_branin_experiment()
         # make metric noiseless for exact reproducibility
         new_experiment.optimization_config.objective.metric.noise_sd = 0
-        new_experiment.warm_start_from_old_experiment(old_experiment=old_experiment)
+        for _, trial in old_experiment.trials.items():
+            trial._run_metadata = DUMMY_RUN_METADATA
+        # name one arm to test name-preserving logic.
+        old_experiment.trials[0].arm._name = DUMMY_ARM_NAME
+        new_experiment.warm_start_from_old_experiment(
+            old_experiment=old_experiment,
+            copy_run_metadata_keys=[DUMMY_RUN_METADATA_KEY],
+        )
         self.assertEqual(len(new_experiment.trials), len(old_experiment.trials) - 1)
         i_old_trial = 0
         for _, trial in new_experiment.trials.items():
             # skip failed trial
             i_old_trial += i_old_trial == i_failed_trial
             self.assertEqual(
-                trial.arm.parameters, old_experiment.trials[i_old_trial].arm.parameters
+                # pyre-fixme[16]: `BaseTrial` has no attribute `arm`.
+                trial.arm.parameters,
+                old_experiment.trials[i_old_trial].arm.parameters,
             )
             self.assertRegex(
                 trial._properties["source"], "Warm start.*Experiment.*trial"
             )
+            self.assertDictEqual(trial.run_metadata, DUMMY_RUN_METADATA)
             i_old_trial += 1
 
         # Check that the data was attached for correct trials
         old_df = old_experiment.fetch_data().df
         new_df = new_experiment.fetch_data().df
 
-        self.assertEqual(len(new_df), len_old_trials - 1)
+        self.assertEqual(len(new_df), len_old_trials - 2)
+        pd.testing.assert_frame_equal(
+            old_df.drop(["arm_name", "trial_index"], axis=1),
+            new_df.drop(["arm_name", "trial_index"], axis=1),
+        )
+
+        # Check name-preserving logic.
+        self.assertEqual(new_experiment.trials[0].arm.name, DUMMY_ARM_NAME)
+
+        # check that all non-failed/abandoned trials are copied to new_experiment
+        new_experiment = get_branin_experiment()
+        # make metric noiseless for exact reproducibility
+        new_experiment.optimization_config.objective.metric.noise_sd = 0
+        new_experiment.warm_start_from_old_experiment(
+            old_experiment=old_experiment,
+            copy_run_metadata_keys=[DUMMY_RUN_METADATA_KEY],
+            trial_statuses_to_copy=[TrialStatus.COMPLETED],
+        )
+        self.assertEqual(len(new_experiment.trials), len(old_experiment.trials) - 3)
+
+        # Warm start from an experiment with only a subset of metrics
+        map_data_experiment = get_branin_experiment_with_timestamp_map_metric()
+        map_data_experiment.warm_start_from_old_experiment(
+            old_experiment=old_experiment
+        )
+        self.assertEqual(
+            len(map_data_experiment.trials), len(old_experiment.trials) - 1
+        )
+
+    def test_is_test_warning(self) -> None:
+        experiments_module = "ax.core.experiment"
+        with self.subTest("it warns on construction for a test"):
+            with self.assertLogs(experiments_module, level=logging.INFO) as logger:
+                exp = Experiment(
+                    search_space=get_search_space(),
+                    is_test=True,
+                )
+                self.assertIn(
+                    f"INFO:{experiments_module}:{EXPERIMENT_IS_TEST_WARNING}",
+                    logger.output,
+                )
+
+        with self.subTest("it does not warn on construction for a non test"):
+            with self.assertLogs(experiments_module, level=logging.INFO) as logger:
+                logging.getLogger(experiments_module).info(
+                    "there must be at least one log or the assertLogs statement fails"
+                )
+                exp = Experiment(
+                    search_space=get_search_space(),
+                    is_test=False,
+                )
+                self.assertNotIn(
+                    f"INFO:{experiments_module}:{EXPERIMENT_IS_TEST_WARNING}",
+                    logger.output,
+                )
+
+        with self.subTest("it warns on setting is_test to True"):
+            with self.assertLogs(experiments_module, level=logging.INFO) as logger:
+                exp.is_test = True
+                self.assertIn(
+                    f"INFO:{experiments_module}:{EXPERIMENT_IS_TEST_WARNING}",
+                    logger.output,
+                )
+
+        with self.subTest("it does not warn on setting is_test to False"):
+            with self.assertLogs(experiments_module, level=logging.INFO) as logger:
+                logging.getLogger(experiments_module).info(
+                    "there must be at least one log or the assertLogs statement fails"
+                )
+                exp.is_test = False
+                self.assertNotIn(
+                    f"INFO:{experiments_module}:{EXPERIMENT_IS_TEST_WARNING}",
+                    logger.output,
+                )
+
+
+class ExperimentWithMapDataTest(TestCase):
+    def setUp(self) -> None:
+        self.experiment = get_experiment_with_map_data_type()
+
+    def _setupBraninExperiment(self, n: int, incremental: bool = False) -> Experiment:
+        exp = get_branin_experiment_with_timestamp_map_metric()
+        batch = exp.new_batch_trial()
+        batch.add_arms_and_weights(arms=get_branin_arms(n=n, seed=0))
+        batch.run()
+
+        batch_2 = exp.new_batch_trial()
+        batch_2.add_arms_and_weights(arms=get_branin_arms(n=3 * n, seed=1))
+        batch_2.run()
+        return exp
+
+    def testFetchDataWithMapData(self) -> None:
+        evaluations = {
+            "0_0": [
+                ({"epoch": 1}, {"no_fetch_impl_metric": (3.7, 0.5)}),
+                ({"epoch": 2}, {"no_fetch_impl_metric": (3.8, 0.5)}),
+                ({"epoch": 3}, {"no_fetch_impl_metric": (3.9, 0.5)}),
+                ({"epoch": 4}, {"no_fetch_impl_metric": (4.0, 0.5)}),
+            ],
+        }
+
+        self.experiment.add_tracking_metric(
+            metric=MapMetric(name="no_fetch_impl_metric")
+        )
+        self.experiment.new_trial()
+        self.experiment.trials[0].mark_running(no_runner_required=True)
+        first_epoch = MapData.from_map_evaluations(
+            # pyre-fixme[6]: For 1st param expected `Dict[str, List[Tuple[Dict[str, H...
+            evaluations={
+                arm_name: partial_results[0:1]
+                for arm_name, partial_results in evaluations.items()
+            },
+            trial_index=0,
+        )
+        self.experiment.attach_data(first_epoch)
+        remaining_epochs = MapData.from_map_evaluations(
+            # pyre-fixme[6]: For 1st param expected `Dict[str, List[Tuple[Dict[str, H...
+            evaluations={
+                arm_name: partial_results[1:4]
+                for arm_name, partial_results in evaluations.items()
+            },
+            trial_index=0,
+        )
+        self.experiment.attach_data(remaining_epochs)
+        self.experiment.trials[0].mark_completed()
+
+        expected_data = remaining_epochs
+        actual_data = self.experiment.lookup_data()
+        self.assertEqual(expected_data, actual_data)
+
+    def testFetchTrialsData(self) -> None:
+        exp = self._setupBraninExperiment(n=5)
+        batch_0 = exp.trials[0]
+        batch_1 = exp.trials[1]
+        batch_0.mark_completed()
+        batch_1.mark_completed()
+        batch_0_data = exp.fetch_trials_data(trial_indices=[0])
+        self.assertEqual(set(batch_0_data.df["trial_index"].values), {0})
+        self.assertEqual(
+            set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
+        )
+        batch_1_data = exp.fetch_trials_data(trial_indices=[1])
+        self.assertEqual(set(batch_1_data.df["trial_index"].values), {1})
+        self.assertEqual(
+            set(batch_1_data.df["arm_name"].values), {a.name for a in batch_1.arms}
+        )
+        self.assertEqual(
+            exp.fetch_trials_data(trial_indices=[0, 1]).df.shape[0],
+            len(exp.arms_by_name) * 2,
+        )
+
+        with self.assertRaisesRegex(ValueError, ".* not associated .*"):
+            exp.fetch_trials_data(trial_indices=[2])
+        # Try to fetch data when there are only metrics and no attached data.
+        exp.remove_tracking_metric(metric_name="branin")  # Remove implemented metric.
+        exp.add_tracking_metric(
+            BraninMetric(name="branin", param_names=["x1", "x2"])
+        )  # Add unimplemented metric.
+        # pyre-fixme[16]: `Data` has no attribute `map_df`.
+        self.assertEqual(len(exp.fetch_trials_data(trial_indices=[0]).map_df), 10)
+        # Try fetching attached data.
+        exp.attach_data(batch_0_data)
+        exp.attach_data(batch_1_data)
+        pd.testing.assert_frame_equal(
+            exp.fetch_trials_data(trial_indices=[0]).df, batch_0_data.df
+        )
+        pd.testing.assert_frame_equal(
+            exp.fetch_trials_data(trial_indices=[1]).df, batch_1_data.df
+        )
+        self.assertEqual(set(batch_0_data.df["trial_index"].values), {0})
+        self.assertEqual(
+            set(batch_0_data.df["arm_name"].values), {a.name for a in batch_0.arms}
+        )
+
+    def test_is_moo_problem(self) -> None:
+        exp = get_branin_experiment()
+        self.assertFalse(exp.is_moo_problem)
+        exp = get_branin_experiment_with_multi_objective()
+        self.assertTrue(exp.is_moo_problem)
+        exp._optimization_config = None
+        self.assertFalse(exp.is_moo_problem)
+
+    def testWarmStartMapData(self) -> None:
+        # create old_experiment
+        len_old_trials = 7
+        i_failed_trial = 1
+        i_abandoned_trial = 3
+        i_running_trial = 5
+        old_experiment = get_branin_experiment_with_timestamp_map_metric()
+        for i_old_trial in range(len_old_trials):
+            sobol_run = get_sobol(search_space=old_experiment.search_space).gen(n=1)
+            trial = old_experiment.new_trial(generator_run=sobol_run)
+            trial.mark_running(no_runner_required=True)
+            if i_old_trial == i_failed_trial:
+                trial.mark_failed()
+            elif i_old_trial == i_abandoned_trial:
+                trial.mark_abandoned(reason=DUMMY_ABANDONED_REASON)
+            elif i_old_trial == i_running_trial:
+                pass
+            else:
+                trial.mark_completed()
+        # make metric noiseless for exact reproducibility
+        old_experiment.optimization_config.objective.metric.noise_sd = 0
+        old_experiment.fetch_data()
+
+        # check that all non-failed trials are copied to new_experiment
+        new_experiment = get_branin_experiment_with_timestamp_map_metric()
+        # make metric noiseless for exact reproducibility
+        new_experiment.optimization_config.objective.metric.noise_sd = 0
+        for _, trial in old_experiment.trials.items():
+            trial._run_metadata = DUMMY_RUN_METADATA
+        new_experiment.warm_start_from_old_experiment(
+            old_experiment=old_experiment,
+            copy_run_metadata_keys=[DUMMY_RUN_METADATA_KEY],
+        )
+        self.assertEqual(len(new_experiment.trials), len(old_experiment.trials) - 1)
+        i_old_trial = 0
+        for _, trial in new_experiment.trials.items():
+            # skip failed trial
+            i_old_trial += i_old_trial == i_failed_trial
+            self.assertEqual(
+                # pyre-fixme[16]: `BaseTrial` has no attribute `arm`.
+                trial.arm.parameters,
+                old_experiment.trials[i_old_trial].arm.parameters,
+            )
+            self.assertRegex(
+                trial._properties["source"], "Warm start.*Experiment.*trial"
+            )
+            self.assertDictEqual(trial.run_metadata, DUMMY_RUN_METADATA)
+            i_old_trial += 1
+
+        # Check that the data was attached for correct trials
+        old_df = old_experiment.fetch_data().df
+        new_df = new_experiment.fetch_data().df
+
+        old_df = old_df.sort_values(by=["arm_name", "metric_name"], ignore_index=True)
+        new_df = new_df.sort_values(by=["arm_name", "metric_name"], ignore_index=True)
+
+        # Factor 2 comes from 2 rows per trial in this test experiment
+        self.assertEqual(len(new_df), (len_old_trials - 2) * 2)
         pd.testing.assert_frame_equal(
             old_df.drop(["arm_name", "trial_index"], axis=1),
             new_df.drop(["arm_name", "trial_index"], axis=1),
