@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,14 +12,14 @@ METRIC_STRING = "MapMetric('m1')"
 
 
 class MapMetricTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         pass
 
-    def testInit(self):
+    def testInit(self) -> None:
         metric = MapMetric(name="m1", lower_is_better=False)
         self.assertEqual(str(metric), METRIC_STRING)
 
-    def testEq(self):
+    def testEq(self) -> None:
         metric1 = MapMetric(name="m1", lower_is_better=False)
         metric2 = MapMetric(name="m1", lower_is_better=False)
         self.assertEqual(metric1, metric2)
@@ -27,11 +27,11 @@ class MapMetricTest(TestCase):
         metric3 = MapMetric(name="m1", lower_is_better=True)
         self.assertNotEqual(metric1, metric3)
 
-    def testClone(self):
+    def testClone(self) -> None:
         metric1 = MapMetric(name="m1", lower_is_better=False)
         self.assertEqual(metric1, metric1.clone())
 
-    def testSortable(self):
+    def testSortable(self) -> None:
         metric1 = MapMetric(name="m1", lower_is_better=False)
         metric2 = MapMetric(name="m2", lower_is_better=False)
         self.assertTrue(metric1 < metric2)
