@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -13,14 +13,14 @@ METRIC_STRING = "Metric('m1')"
 
 
 class MetricTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         pass
 
-    def testInit(self):
+    def testInit(self) -> None:
         metric = Metric(name="m1", lower_is_better=False)
         self.assertEqual(str(metric), METRIC_STRING)
 
-    def testEq(self):
+    def testEq(self) -> None:
         metric1 = Metric(name="m1", lower_is_better=False)
         metric2 = Metric(name="m1", lower_is_better=False)
         self.assertEqual(metric1, metric2)
@@ -28,7 +28,7 @@ class MetricTest(TestCase):
         metric3 = Metric(name="m1", lower_is_better=True)
         self.assertNotEqual(metric1, metric3)
 
-    def testClone(self):
+    def testClone(self) -> None:
         metric1 = Metric(name="m1", lower_is_better=False)
         self.assertEqual(metric1, metric1.clone())
 
@@ -38,7 +38,7 @@ class MetricTest(TestCase):
         metric3 = get_factorial_metric(name="factorial")
         self.assertEqual(metric3, metric3.clone())
 
-    def testSortable(self):
+    def testSortable(self) -> None:
         metric1 = Metric(name="m1", lower_is_better=False)
         metric2 = Metric(name="m2", lower_is_better=False)
         self.assertTrue(metric1 < metric2)
