@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -19,7 +19,7 @@ Objective(metric_name="m3", minimize=False)])
 
 
 class ObjectiveTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.metrics = {
             "m1": Metric(name="m1"),
             "m2": Metric(name="m2", lower_is_better=True),
@@ -42,7 +42,7 @@ class ObjectiveTest(TestCase):
             metrics=[self.metrics["m1"], self.metrics["m2"]]
         )
 
-    def testInit(self):
+    def testInit(self) -> None:
         with self.assertRaises(ValueError):
             ScalarizedObjective(
                 metrics=[self.metrics["m1"], self.metrics["m2"]], weights=[1.0]
@@ -65,8 +65,9 @@ class ObjectiveTest(TestCase):
             self.objective.get_unconstrainable_metrics(), [self.metrics["m1"]]
         )
 
-    def testMultiObjective(self):
+    def testMultiObjective(self) -> None:
         with self.assertRaises(NotImplementedError):
+            # pyre-fixme[7]: Expected `None` but got `Metric`.
             return self.multi_objective.metric
 
         self.assertEqual(self.multi_objective.metrics, list(self.metrics.values()))
@@ -84,9 +85,12 @@ class ObjectiveTest(TestCase):
                 'Objective(metric_name="m3", minimize=False)])'
             ),
         )
-        self.assertEqual(self.multi_objective.get_unconstrainable_metrics(), [])
+        self.assertEqual(
+            self.multi_objective.get_unconstrainable_metrics(),
+            [self.metrics["m1"], self.metrics["m2"], self.metrics["m3"]],
+        )
 
-    def testMultiObjectiveBackwardsCompatibility(self):
+    def testMultiObjectiveBackwardsCompatibility(self) -> None:
         multi_objective = MultiObjective(
             metrics=[self.metrics["m1"], self.metrics["m2"], self.metrics["m3"]]
         )
@@ -101,8 +105,9 @@ class ObjectiveTest(TestCase):
         minimizes = [obj.minimize for obj in multi_objective_min.objectives]
         self.assertEqual(minimizes, [True, False, True])
 
-    def testScalarizedObjective(self):
+    def testScalarizedObjective(self) -> None:
         with self.assertRaises(NotImplementedError):
+            # pyre-fixme[7]: Expected `None` but got `Metric`.
             return self.scalarized_objective.metric
 
         self.assertEqual(
@@ -118,4 +123,7 @@ class ObjectiveTest(TestCase):
                 "minimize=False)"
             ),
         )
-        self.assertEqual(self.scalarized_objective.get_unconstrainable_metrics(), [])
+        self.assertEqual(
+            self.scalarized_objective.get_unconstrainable_metrics(),
+            [self.metrics["m1"], self.metrics["m2"]],
+        )
