@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,12 +14,14 @@ from ax.core.base_trial import TrialStatus
 from ax.core.batch_trial import BatchTrial
 from ax.core.data import Data
 from ax.core.generator_run import GeneratorRun
-from ax.core.map_data import MapData
+from ax.core.map_data import MapData, MapKeyInfo
 from ax.core.observation import (
     Observation,
     ObservationData,
     ObservationFeatures,
     observations_from_data,
+    observations_from_map_data,
+    recombine_observations,
     separate_observations,
 )
 from ax.core.trial import Trial
@@ -27,7 +29,7 @@ from ax.utils.common.testutils import TestCase
 
 
 class ObservationsTest(TestCase):
-    def testObservationFeatures(self):
+    def testObservationFeatures(self) -> None:
         t = np.datetime64("now")
         attrs = {
             "parameters": {"x": 0, "y": "a"},
@@ -36,6 +38,14 @@ class ObservationsTest(TestCase):
             "end_time": t,
             "random_split": 1,
         }
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[Dict[str, typing.Any]]`
+        #  but got `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[int64]` but got
+        #  `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[Timestamp]` but got
+        #  `Union[Dict[str, Union[int, str]], int, datetime64]`.
         obsf = ObservationFeatures(**attrs)
         for k, v in attrs.items():
             self.assertEqual(getattr(obsf, k), v)
@@ -44,26 +54,60 @@ class ObservationsTest(TestCase):
         printstr += "start_time={t}, end_time={t}, ".format(t=t)
         printstr += "random_split=1)"
         self.assertEqual(repr(obsf), printstr)
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[Dict[str, typing.Any]]`
+        #  but got `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[int64]` but got
+        #  `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[Timestamp]` but got
+        #  `Union[Dict[str, Union[int, str]], int, datetime64]`.
         obsf2 = ObservationFeatures(**attrs)
         self.assertEqual(hash(obsf), hash(obsf2))
         a = {obsf, obsf2}
         self.assertEqual(len(a), 1)
         self.assertEqual(obsf, obsf2)
         attrs.pop("trial_index")
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[Dict[str, typing.Any]]`
+        #  but got `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[int64]` but got
+        #  `Union[Dict[str, Union[int, str]], int, datetime64]`.
+        # pyre-fixme[6]: For 1st param expected `Optional[Timestamp]` but got
+        #  `Union[Dict[str, Union[int, str]], int, datetime64]`.
         obsf3 = ObservationFeatures(**attrs)
         self.assertNotEqual(obsf, obsf3)
         self.assertFalse(obsf == 1)
 
-    def testObservationFeaturesFromArm(self):
+    def testClone(self) -> None:
+        # Test simple cloning.
         arm = Arm({"x": 0, "y": "a"})
+        # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
+        obsf = ObservationFeatures.from_arm(arm, trial_index=3)
+        self.assertIsNot(obsf, obsf.clone())
+        self.assertEqual(obsf, obsf.clone())
+
+        # Test cloning with swapping parameters.
+        clone_with_new_params = obsf.clone(replace_parameters={"x": 1, "y": "b"})
+        self.assertNotEqual(obsf, clone_with_new_params)
+        obsf.parameters = {"x": 1, "y": "b"}
+        self.assertEqual(obsf, clone_with_new_params)
+
+    def testObservationFeaturesFromArm(self) -> None:
+        arm = Arm({"x": 0, "y": "a"})
+        # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
         obsf = ObservationFeatures.from_arm(arm, trial_index=3)
         self.assertEqual(obsf.parameters, arm.parameters)
         self.assertEqual(obsf.trial_index, 3)
 
-    def testUpdateFeatures(self):
+    def testUpdateFeatures(self) -> None:
         parameters = {"x": 0, "y": "a"}
         new_parameters = {"z": "foo"}
 
+        # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Dict[str, Union[int, str]]`.
+        # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
         obsf = ObservationFeatures(parameters=parameters, trial_index=3)
 
         # Ensure None trial_index doesn't override existing value
@@ -72,10 +116,14 @@ class ObservationsTest(TestCase):
 
         # Test override
         new_obsf = ObservationFeatures(
+            # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Dict[str, str]`.
             parameters=new_parameters,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
             trial_index=4,
             start_time=pd.Timestamp("2005-02-25"),
             end_time=pd.Timestamp("2005-02-26"),
+            # pyre-fixme[6]: For 5th param expected `Optional[int64]` but got `int`.
             random_split=7,
         )
         obsf.update_features(new_obsf)
@@ -85,12 +133,16 @@ class ObservationsTest(TestCase):
         self.assertEqual(obsf.start_time, pd.Timestamp("2005-02-25"))
         self.assertEqual(obsf.end_time, pd.Timestamp("2005-02-26"))
 
-    def testObservationData(self):
+    def testObservationData(self) -> None:
         attrs = {
             "metric_names": ["a", "b"],
             "means": np.array([4.0, 5.0]),
             "covariance": np.array([[1.0, 4.0], [3.0, 6.0]]),
         }
+        # pyre-fixme[6]: For 1st param expected `List[str]` but got
+        #  `Union[List[str], ndarray]`.
+        # pyre-fixme[6]: For 1st param expected `ndarray` but got `Union[List[str],
+        #  ndarray]`.
         obsd = ObservationData(**attrs)
         self.assertEqual(obsd.metric_names, attrs["metric_names"])
         self.assertTrue(np.array_equal(obsd.means, attrs["means"]))
@@ -102,8 +154,13 @@ class ObservationsTest(TestCase):
         printstr = "ObservationData(metric_names=['a', 'b'], means=[ 4.  5.], "
         printstr += "covariance=[[ 1.  4.]\n [ 3.  6.]])"
         self.assertEqual(repr(obsd), printstr)
+        self.assertEqual(obsd.means_dict, {"a": 4.0, "b": 5.0})
+        self.assertEqual(
+            obsd.covariance_matrix,
+            {"a": {"a": 1.0, "b": 4.0}, "b": {"a": 3.0, "b": 6.0}},
+        )
 
-    def testObservationDataValidation(self):
+    def testObservationDataValidation(self) -> None:
         with self.assertRaises(ValueError):
             ObservationData(
                 metric_names=["a", "b"],
@@ -117,7 +174,7 @@ class ObservationsTest(TestCase):
                 covariance=np.array([1.0, 4.0]),
             )
 
-    def testObservationDataEq(self):
+    def testObservationDataEq(self) -> None:
         od1 = ObservationData(
             metric_names=["a", "b"],
             means=np.array([4.0, 5.0]),
@@ -137,7 +194,7 @@ class ObservationsTest(TestCase):
         self.assertNotEqual(od1, od3)
         self.assertFalse(od1 == 1)
 
-    def testObservation(self):
+    def testObservation(self) -> None:
         obs = Observation(
             features=ObservationFeatures(parameters={"x": 20}),
             data=ObservationData(
@@ -171,7 +228,7 @@ class ObservationsTest(TestCase):
         self.assertNotEqual(obs, obs3)
         self.assertNotEqual(obs, 1)
 
-    def testObservationsFromData(self):
+    def testObservationsFromData(self) -> None:
         truth = [
             {
                 "arm_name": "0_0",
@@ -199,6 +256,11 @@ class ObservationsTest(TestCase):
             },
         ]
         arms = {
+            # pyre-fixme[6]: For 1st param expected `Optional[str]` but got
+            #  `Union[Dict[str, Union[int, str]], float, str]`.
+            # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Union[Dict[str, Union[int, str]], float,
+            #  str]`.
             obs["arm_name"]: Arm(name=obs["arm_name"], parameters=obs["parameters"])
             for obs in truth
         }
@@ -241,7 +303,7 @@ class ObservationsTest(TestCase):
             )
             self.assertEqual(obs.arm_name, cname_truth[i])
 
-    def testObservationsFromDataWithFidelities(self):
+    def testObservationsFromDataWithFidelities(self) -> None:
         truth = {
             0.5: {
                 "arm_name": "0_0",
@@ -281,6 +343,12 @@ class ObservationsTest(TestCase):
             },
         }
         arms = {
+            # pyre-fixme[6]: For 1st param expected `Optional[str]` but got
+            #  `Union[Dict[str, Union[float, str]], Dict[str, Union[int, str]], float,
+            #  ndarray, str]`.
+            # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Union[Dict[str, Union[float, str]],
+            #  Dict[str, Union[int, str]], float, ndarray, str]`.
             obs["arm_name"]: Arm(name=obs["arm_name"], parameters=obs["parameters"])
             for _, obs in truth.items()
         }
@@ -303,6 +371,8 @@ class ObservationsTest(TestCase):
 
         self.assertEqual(len(observations), 3)
         for obs in observations:
+            # pyre-fixme[6]: For 1st param expected `float` but got `Union[None,
+            #  bool, float, int, str]`.
             t = truth[obs.features.parameters["z"]]
             self.assertEqual(obs.features.parameters, t["updated_parameters"])
             self.assertEqual(obs.features.trial_index, t["trial_index"])
@@ -311,7 +381,7 @@ class ObservationsTest(TestCase):
             self.assertTrue(np.array_equal(obs.data.covariance, t["covariance_t"]))
             self.assertEqual(obs.arm_name, t["arm_name"])
 
-    def testObservationsFromMapData(self):
+    def testObservationsFromMapData(self) -> None:
         truth = {
             0.5: {
                 "arm_name": "0_0",
@@ -354,6 +424,12 @@ class ObservationsTest(TestCase):
             },
         }
         arms = {
+            # pyre-fixme[6]: For 1st param expected `Optional[str]` but got
+            #  `Union[Dict[str, Union[float, str]], Dict[str, Union[int, str]], float,
+            #  ndarray, str]`.
+            # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Union[Dict[str, Union[float, str]],
+            #  Dict[str, Union[int, str]], float, ndarray, str]`.
             obs["arm_name"]: Arm(name=obs["arm_name"], parameters=obs["parameters"])
             for _, obs in truth.items()
         }
@@ -371,11 +447,20 @@ class ObservationsTest(TestCase):
         df = pd.DataFrame(list(truth.values()))[
             ["arm_name", "trial_index", "mean", "sem", "metric_name", "z", "timestamp"]
         ]
-        data = MapData(df=df, map_keys=["z", "timestamp"])
-        observations = observations_from_data(experiment, data)
+        data = MapData(
+            df=df,
+            map_key_infos=[
+                MapKeyInfo(key="z", default_value=0.0),
+                MapKeyInfo(key="timestamp", default_value=0.0),
+            ],
+        )
+        observations = observations_from_map_data(experiment, data)
 
         self.assertEqual(len(observations), 3)
+
         for obs in observations:
+            # pyre-fixme[6]: For 1st param expected `float` but got `Union[None,
+            #  bool, float, int, str]`.
             t = truth[obs.features.parameters["z"]]
             self.assertEqual(obs.features.parameters, t["updated_parameters"])
             self.assertEqual(obs.features.trial_index, t["trial_index"])
@@ -385,7 +470,7 @@ class ObservationsTest(TestCase):
             self.assertEqual(obs.arm_name, t["arm_name"])
             self.assertEqual(obs.features.metadata, {"timestamp": t["timestamp"]})
 
-    def testObservationsFromDataAbandoned(self):
+    def testObservationsFromDataAbandoned(self) -> None:
         truth = {
             0.5: {
                 "arm_name": "0_0",
@@ -441,6 +526,12 @@ class ObservationsTest(TestCase):
             },
         }
         arms = {
+            # pyre-fixme[6]: For 1st param expected `Optional[str]` but got
+            #  `Union[Dict[str, Union[float, str]], Dict[str, Union[int, str]], float,
+            #  ndarray, str]`.
+            # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Union[Dict[str, Union[float, str]],
+            #  Dict[str, Union[int, str]], float, ndarray, str]`.
             obs["arm_name"]: Arm(name=obs["arm_name"], parameters=obs["parameters"])
             for _, obs in truth.items()
         }
@@ -451,11 +542,20 @@ class ObservationsTest(TestCase):
                 Trial(experiment, GeneratorRun(arms=[arms[obs["arm_name"]]]))
             )
             for _, obs in list(truth.items())[:-1]
+            # pyre-fixme[16]: Item `Dict` of `Union[Dict[str, typing.Union[float,
+            #  str]], Dict[str, typing.Union[int, str]], float, ndarray, str]` has no
+            #  attribute `startswith`.
             if not obs["arm_name"].startswith("2")
         }
         batch = BatchTrial(experiment, GeneratorRun(arms=[arms["2_0"], arms["2_1"]]))
+        # pyre-fixme[6]: For 1st param expected
+        #  `SupportsKeysAndGetItem[Union[Dict[str, Union[float, str]], Dict[str,
+        #  Union[int, str]], float, ndarray, str], Trial]` but got `Dict[int,
+        #  BatchTrial]`.
         trials.update({2: batch})
+        # pyre-fixme[16]: Optional type has no attribute `mark_abandoned`.
         trials.get(1).mark_abandoned()
+        # pyre-fixme[16]: Optional type has no attribute `mark_arm_abandoned`.
         trials.get(2).mark_arm_abandoned(arm_name="2_1")
         type(experiment).arms_by_name = PropertyMock(return_value=arms)
         type(experiment).trials = PropertyMock(return_value=trials)
@@ -477,7 +577,7 @@ class ObservationsTest(TestCase):
         )
         self.assertEqual(len(obs_with_abandoned), 4)
 
-    def testObservationsFromDataWithSomeMissingTimes(self):
+    def testObservationsFromDataWithSomeMissingTimes(self) -> None:
         truth = [
             {
                 "arm_name": "0_0",
@@ -517,6 +617,11 @@ class ObservationsTest(TestCase):
             },
         ]
         arms = {
+            # pyre-fixme[6]: For 1st param expected `Optional[str]` but got
+            #  `Union[None, Dict[str, Union[int, str]], float, str]`.
+            # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Union[None, Dict[str, Union[int, str]],
+            #  float, str]`.
             obs["arm_name"]: Arm(name=obs["arm_name"], parameters=obs["parameters"])
             for obs in truth
         }
@@ -559,7 +664,7 @@ class ObservationsTest(TestCase):
             )
             self.assertEqual(obs.arm_name, cname_truth[i])
 
-    def testSeparateObservations(self):
+    def testSeparateObservations(self) -> None:
         obs = Observation(
             features=ObservationFeatures(parameters={"x": 20}),
             data=ObservationData(
@@ -575,6 +680,11 @@ class ObservationsTest(TestCase):
                 means=np.array([1]), covariance=np.array([[2]]), metric_names=["a"]
             ),
         )
+        with self.assertRaises(ValueError):
+            recombine_observations(observation_features=obs_feats, observation_data=[])
+        new_obs = recombine_observations(obs_feats, obs_data)[0]
+        self.assertEqual(new_obs.features, obs.features)
+        self.assertEqual(new_obs.data, obs.data)
         obs_feats, obs_data = separate_observations(observations=[obs], copy=True)
         self.assertEqual(obs.features, ObservationFeatures(parameters={"x": 20}))
         self.assertEqual(
@@ -584,7 +694,7 @@ class ObservationsTest(TestCase):
             ),
         )
 
-    def testObservationsWithCandidateMetadata(self):
+    def testObservationsWithCandidateMetadata(self) -> None:
         SOME_METADATA_KEY = "metadatum"
         truth = [
             {
@@ -605,6 +715,11 @@ class ObservationsTest(TestCase):
             },
         ]
         arms = {
+            # pyre-fixme[6]: For 1st param expected `Optional[str]` but got
+            #  `Union[Dict[str, Union[int, str]], float, str]`.
+            # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Union[Dict[str, Union[int, str]], float,
+            #  str]`.
             obs["arm_name"]: Arm(name=obs["arm_name"], parameters=obs["parameters"])
             for obs in truth
         }
@@ -634,6 +749,7 @@ class ObservationsTest(TestCase):
         observations = observations_from_data(experiment, data)
         for observation in observations:
             self.assertEqual(
+                # pyre-fixme[16]: Optional type has no attribute `get`.
                 observation.features.metadata.get(SOME_METADATA_KEY),
                 f"value_{observation.features.trial_index}",
             )
