@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -15,32 +15,39 @@ from ax.core.outcome_constraint import (
     OutcomeConstraint,
     ScalarizedOutcomeConstraint,
 )
+from ax.core.risk_measures import RiskMeasure
 from ax.core.types import ComparisonOp
+from ax.exceptions.core import UserInputError
 from ax.utils.common.testutils import TestCase
 
 
 OC_STR = (
     'OptimizationConfig(objective=Objective(metric_name="m1", minimize=False), '
-    "outcome_constraints=[OutcomeConstraint(m2 >= -0.25%), "
-    "OutcomeConstraint(m2 <= 0.25%), "
-    "ScalarizedOutcomeConstraint(metric_names=['m1', 'm2'], "
+    "outcome_constraints=[OutcomeConstraint(m3 >= -0.25%), "
+    "OutcomeConstraint(m4 <= 0.25%), "
+    "ScalarizedOutcomeConstraint(metric_names=['m3', 'm4'], "
     "weights=[0.5, 0.5], >= -0.25%)])"
 )
 
 MOOC_STR = (
-    "OptimizationConfig(objective=MultiObjective(objectives="
+    "MultiObjectiveOptimizationConfig(objective=MultiObjective(objectives="
     '[Objective(metric_name="m1", minimize=True), '
     'Objective(metric_name="m2", minimize=False)]), '
-    "outcome_constraints=[OutcomeConstraint(m2 >= -0.25%), "
-    "OutcomeConstraint(m2 <= 0.25%)], objective_thresholds=[])"
+    "outcome_constraints=[OutcomeConstraint(m3 >= -0.25%), "
+    "OutcomeConstraint(m3 <= 0.25%)], objective_thresholds=[])"
 )
 
 
 class OptimizationConfigTest(TestCase):
-    def setUp(self):
-        self.metrics = {"m1": Metric(name="m1"), "m2": Metric(name="m2")}
+    def setUp(self) -> None:
+        self.metrics = {
+            "m1": Metric(name="m1"),
+            "m2": Metric(name="m2"),
+            "m3": Metric(name="m3"),
+            "m4": Metric(name="m4"),
+        }
         self.objective = Objective(metric=self.metrics["m1"], minimize=False)
-        self.alt_objective = Objective(metric=self.metrics["m2"], minimize=False)
+        self.alt_objective = Objective(metric=self.metrics["m3"], minimize=False)
         self.multi_objective = MultiObjective(
             objectives=[self.objective, self.alt_objective],
         )
@@ -48,13 +55,13 @@ class OptimizationConfigTest(TestCase):
             metrics=[self.metrics["m1"], self.metrics["m2"]]
         )
         self.outcome_constraint = OutcomeConstraint(
-            metric=self.metrics["m2"], op=ComparisonOp.GEQ, bound=-0.25
+            metric=self.metrics["m3"], op=ComparisonOp.GEQ, bound=-0.25
         )
         self.additional_outcome_constraint = OutcomeConstraint(
-            metric=self.metrics["m2"], op=ComparisonOp.LEQ, bound=0.25
+            metric=self.metrics["m4"], op=ComparisonOp.LEQ, bound=0.25
         )
         self.scalarized_outcome_constraint = ScalarizedOutcomeConstraint(
-            metrics=[self.metrics["m1"], self.metrics["m2"]],
+            metrics=[self.metrics["m3"], self.metrics["m4"]],
             weights=[0.5, 0.5],
             op=ComparisonOp.GEQ,
             bound=-0.25,
@@ -64,8 +71,16 @@ class OptimizationConfigTest(TestCase):
             self.additional_outcome_constraint,
             self.scalarized_outcome_constraint,
         ]
+        self.single_output_risk_measure = RiskMeasure(
+            risk_measure="Expectation",
+            options={"n_w": 2},
+        )
+        self.multi_output_risk_measure = RiskMeasure(
+            risk_measure="MultiOutputExpectation",
+            options={"n_w": 2},
+        )
 
-    def testInit(self):
+    def testInit(self) -> None:
         config1 = OptimizationConfig(
             objective=self.objective, outcome_constraints=self.outcome_constraints
         )
@@ -87,12 +102,30 @@ class OptimizationConfigTest(TestCase):
         config2.outcome_constraints = self.outcome_constraints
         self.assertEqual(config2.outcome_constraints, self.outcome_constraints)
 
-    def testEq(self):
+        # Risk measure is correctly registered.
+        self.assertIsNone(config2.risk_measure)
+        config3 = OptimizationConfig(
+            objective=self.objective,
+            outcome_constraints=self.outcome_constraints,
+            risk_measure=self.single_output_risk_measure,
+        )
+        expected_str = (
+            OC_STR[:-1] + ", risk_measure=RiskMeasure(risk_measure=Expectation, "
+            "options={'n_w': 2}))"
+        )
+        self.assertEqual(str(config3), expected_str)
+        self.assertIs(config3.risk_measure, self.single_output_risk_measure)
+
+    def testEq(self) -> None:
         config1 = OptimizationConfig(
-            objective=self.objective, outcome_constraints=self.outcome_constraints
+            objective=self.objective,
+            outcome_constraints=self.outcome_constraints,
+            risk_measure=self.single_output_risk_measure,
         )
         config2 = OptimizationConfig(
-            objective=self.objective, outcome_constraints=self.outcome_constraints
+            objective=self.objective,
+            outcome_constraints=self.outcome_constraints,
+            risk_measure=self.single_output_risk_measure,
         )
         self.assertEqual(config1, config2)
 
@@ -105,7 +138,7 @@ class OptimizationConfigTest(TestCase):
         )
         self.assertNotEqual(config1, config3)
 
-    def testConstraintValidation(self):
+    def testConstraintValidation(self) -> None:
         # Can build OptimizationConfig with MultiObjective
         with self.assertRaises(ValueError):
             OptimizationConfig(objective=self.multi_objective)
@@ -118,7 +151,14 @@ class OptimizationConfigTest(TestCase):
             OptimizationConfig(
                 objective=self.objective, outcome_constraints=[objective_constraint]
             )
-
+        # Using an outcome constraint for ScalarizedObjective should also raise
+        with self.assertRaisesRegex(
+            ValueError, "Cannot constrain on objective metric."
+        ):
+            OptimizationConfig(
+                objective=self.m2_objective,
+                outcome_constraints=[objective_constraint],
+            )
         # Two outcome_constraints on the same metric with the same op
         # should raise.
         duplicate_constraint = OutcomeConstraint(
@@ -135,6 +175,7 @@ class OptimizationConfigTest(TestCase):
         # Three outcome_constraints on the same metric should raise.
         opposing_constraint = OutcomeConstraint(
             metric=self.outcome_constraint.metric,
+            # pyre-fixme[6]: For 2nd param expected `ComparisonOp` but got `bool`.
             op=not self.outcome_constraint.op,
             bound=self.outcome_constraint.bound,
         )
@@ -149,6 +190,7 @@ class OptimizationConfigTest(TestCase):
         add_bound = 1 if self.outcome_constraint.op == ComparisonOp.LEQ else -1
         opposing_constraint = OutcomeConstraint(
             metric=self.outcome_constraint.metric,
+            # pyre-fixme[6]: For 2nd param expected `ComparisonOp` but got `bool`.
             op=not self.outcome_constraint.op,
             bound=self.outcome_constraint.bound + add_bound,
         )
@@ -162,6 +204,7 @@ class OptimizationConfigTest(TestCase):
         # bounds should not raise.
         opposing_constraint = OutcomeConstraint(
             metric=self.outcome_constraint.metric,
+            # pyre-fixme[6]: For 2nd param expected `ComparisonOp` but got `bool`.
             op=not self.outcome_constraint.op,
             bound=self.outcome_constraint.bound + 1,
         )
@@ -173,15 +216,17 @@ class OptimizationConfigTest(TestCase):
             config.outcome_constraints, [self.outcome_constraint, opposing_constraint]
         )
 
-    def testClone(self):
+    def testClone(self) -> None:
         config1 = OptimizationConfig(
-            objective=self.objective, outcome_constraints=self.outcome_constraints
+            objective=self.objective,
+            outcome_constraints=self.outcome_constraints,
+            risk_measure=self.single_output_risk_measure,
         )
         self.assertEqual(config1, config1.clone())
 
 
 class MultiObjectiveOptimizationConfigTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.metrics = {
             "m1": Metric(name="m1", lower_is_better=True),
             "m2": Metric(name="m2", lower_is_better=False),
@@ -199,18 +244,31 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         self.multi_objective_just_m2 = MultiObjective(
             objectives=[self.objectives["o2"]]
         )
+        self.scalarized_objective = ScalarizedObjective(
+            metrics=list(self.metrics.values()), weights=[1.0, 1.0, 1.0]
+        )
         self.outcome_constraint = OutcomeConstraint(
-            metric=self.metrics["m2"], op=ComparisonOp.GEQ, bound=-0.25
+            metric=self.metrics["m3"], op=ComparisonOp.GEQ, bound=-0.25
         )
         self.additional_outcome_constraint = OutcomeConstraint(
-            metric=self.metrics["m2"], op=ComparisonOp.LEQ, bound=0.25
+            metric=self.metrics["m3"], op=ComparisonOp.LEQ, bound=0.25
         )
         self.outcome_constraints = [
             self.outcome_constraint,
             self.additional_outcome_constraint,
         ]
         self.objective_thresholds = [
-            ObjectiveThreshold(metric=self.metrics["m2"], bound=-1.0, relative=False)
+            ObjectiveThreshold(metric=self.metrics["m1"], bound=-1.0, relative=False),
+            ObjectiveThreshold(metric=self.metrics["m2"], bound=-1.0, relative=False),
+        ]
+        self.relative_objective_thresholds = [
+            ObjectiveThreshold(metric=self.metrics["m1"], bound=-1.0, relative=True),
+            ObjectiveThreshold(
+                metric=self.metrics["m2"],
+                op=ComparisonOp.GEQ,
+                bound=-1.0,
+                relative=True,
+            ),
         ]
         self.m1_constraint = OutcomeConstraint(
             metric=self.metrics["m1"], op=ComparisonOp.LEQ, bound=0.1, relative=True
@@ -218,8 +276,16 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         self.m3_constraint = OutcomeConstraint(
             metric=self.metrics["m3"], op=ComparisonOp.GEQ, bound=0.1, relative=True
         )
+        self.single_output_risk_measure = RiskMeasure(
+            risk_measure="Expectation",
+            options={"n_w": 2},
+        )
+        self.multi_output_risk_measure = RiskMeasure(
+            risk_measure="MultiOutputExpectation",
+            options={"n_w": 2},
+        )
 
-    def testInit(self):
+    def testInit(self) -> None:
         config1 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective, outcome_constraints=self.outcome_constraints
         )
@@ -228,7 +294,7 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
             config1.objective = self.objective  # Wrong objective type
         # updating constraints is fine.
         config1.outcome_constraints = [self.outcome_constraint]
-        self.assertEqual(len(config1.metrics), 2)
+        self.assertEqual(len(config1.metrics), 3)
 
         # objective without outcome_constraints is also supported
         config2 = MultiObjectiveOptimizationConfig(objective=self.multi_objective)
@@ -262,20 +328,43 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         # verify relative_objective_thresholds works:
         config5 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective,
-            objective_thresholds=[self.outcome_constraint],
+            objective_thresholds=self.relative_objective_thresholds,
         )
         threshold = config5.objective_thresholds[0]
         self.assertTrue(threshold.relative)
-        self.assertEqual(threshold.bound, -0.25)
+        self.assertEqual(threshold.bound, -1.0)
 
         # ValueError on wrong direction constraints
-        with self.assertRaises(ValueError):
+        with self.assertRaises(UserInputError):
             MultiObjectiveOptimizationConfig(
                 objective=self.multi_objective,
+                # pyre-fixme[6]: For 2nd param expected
+                #  `Optional[List[ObjectiveThreshold]]` but got
+                #  `List[OutcomeConstraint]`.
                 objective_thresholds=[self.additional_outcome_constraint],
             )
 
-    def testEq(self):
+        # Test with risk measures.
+        self.assertIsNone(config5.risk_measure)
+        config6 = MultiObjectiveOptimizationConfig(
+            objective=self.multi_objective,
+            outcome_constraints=self.outcome_constraints,
+            risk_measure=self.multi_output_risk_measure,
+        )
+        self.assertIs(config6.risk_measure, self.multi_output_risk_measure)
+        expected_str = (
+            MOOC_STR[:-1] + ", risk_measure=RiskMeasure(risk_measure="
+            "MultiOutputExpectation, options={'n_w': 2}))"
+        )
+        self.assertEqual(str(config6), expected_str)
+        # With scalarized objective.
+        config7 = MultiObjectiveOptimizationConfig(
+            objective=self.scalarized_objective,
+            risk_measure=self.single_output_risk_measure,
+        )
+        self.assertIs(config7.risk_measure, self.single_output_risk_measure)
+
+    def testEq(self) -> None:
         config1 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective, outcome_constraints=self.outcome_constraints
         )
@@ -285,7 +374,7 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         self.assertEqual(config1, config2)
 
         new_outcome_constraint = OutcomeConstraint(
-            metric=self.metrics["m2"], op=ComparisonOp.LEQ, bound=0.5
+            metric=self.metrics["m3"], op=ComparisonOp.LEQ, bound=0.5
         )
         config3 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective,
@@ -293,11 +382,22 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         )
         self.assertNotEqual(config1, config3)
 
-    def testConstraintValidation(self):
+    def testConstraintValidation(self) -> None:
         # Cannot build with non-MultiObjective
         with self.assertRaises(TypeError):
             MultiObjectiveOptimizationConfig(objective=self.objective)
 
+        # Using an outcome constraint for an objective should raise
+        outcome_constraint_m1 = OutcomeConstraint(
+            metric=self.metrics["m1"], op=ComparisonOp.LEQ, bound=1234, relative=False
+        )
+        with self.assertRaisesRegex(
+            ValueError, "Cannot constrain on objective metric."
+        ):
+            MultiObjectiveOptimizationConfig(
+                objective=self.multi_objective,
+                outcome_constraints=[outcome_constraint_m1],
+            )
         # Two outcome_constraints on the same metric with the same op
         # should raise.
         duplicate_constraint = OutcomeConstraint(
@@ -314,6 +414,7 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         # Three outcome_constraints on the same metric should raise.
         opposing_constraint = OutcomeConstraint(
             metric=self.outcome_constraint.metric,
+            # pyre-fixme[6]: For 2nd param expected `ComparisonOp` but got `bool`.
             op=not self.outcome_constraint.op,
             bound=self.outcome_constraint.bound,
         )
@@ -328,6 +429,7 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         add_bound = 1 if self.outcome_constraint.op == ComparisonOp.LEQ else -1
         opposing_constraint = OutcomeConstraint(
             metric=self.outcome_constraint.metric,
+            # pyre-fixme[6]: For 2nd param expected `ComparisonOp` but got `bool`.
             op=not self.outcome_constraint.op,
             bound=self.outcome_constraint.bound + add_bound,
         )
@@ -341,6 +443,7 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         # bounds should not raise.
         opposing_constraint = OutcomeConstraint(
             metric=self.outcome_constraint.metric,
+            # pyre-fixme[6]: For 2nd param expected `ComparisonOp` but got `bool`.
             op=not self.outcome_constraint.op,
             bound=self.outcome_constraint.bound + 1,
         )
@@ -352,7 +455,7 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
             config.outcome_constraints, [self.outcome_constraint, opposing_constraint]
         )
 
-    def testClone(self):
+    def testClone(self) -> None:
         config1 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective, outcome_constraints=self.outcome_constraints
         )
@@ -361,21 +464,22 @@ class MultiObjectiveOptimizationConfigTest(TestCase):
         config2 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective,
             objective_thresholds=self.objective_thresholds,
-            outcome_constraints=[self.m1_constraint],
         )
         self.assertEqual(config2, config2.clone())
 
-    def testCloneWithArgs(self):
+    def testCloneWithArgs(self) -> None:
         config1 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective_just_m2,
             outcome_constraints=[self.m1_constraint],
         )
         config2 = MultiObjectiveOptimizationConfig(
             objective=self.multi_objective_just_m2,
-            objective_thresholds=self.objective_thresholds,
+            objective_thresholds=[self.objective_thresholds[1]],
             outcome_constraints=[self.m1_constraint],
         )
         self.assertEqual(
-            config1.clone_with_args(objective_thresholds=self.objective_thresholds),
+            config1.clone_with_args(
+                objective_thresholds=[self.objective_thresholds[1]]
+            ),
             config2,
         )
