@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,25 +8,25 @@ import numpy as np
 import pandas as pd
 from ax.core.data import Data
 from ax.core.metric import Metric
-from ax.core.objective import Objective, MultiObjective
+from ax.core.objective import MultiObjective, Objective
 from ax.core.optimization_config import (
     MultiObjectiveOptimizationConfig,
     OptimizationConfig,
 )
-from ax.core.outcome_constraint import OutcomeConstraint, ObjectiveThreshold
+from ax.core.outcome_constraint import ObjectiveThreshold, OutcomeConstraint
 from ax.core.types import ComparisonOp
 from ax.core.utils import (
-    MissingMetrics,
     best_feasible_objective,
     get_missing_metrics,
     get_missing_metrics_by_name,
-    feasible_hypervolume,
+    MissingMetrics,
 )
+from ax.modelbridge.modelbridge_utils import feasible_hypervolume
 from ax.utils.common.testutils import TestCase
 
 
 class UtilsTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.df = pd.DataFrame(
             [
                 {
@@ -109,12 +109,12 @@ class UtilsTest(TestCase):
             ],
         )
 
-    def test_get_missing_metrics_by_name(self):
+    def test_get_missing_metrics_by_name(self) -> None:
         expected = {"a": {("0_1", 1)}, "b": {("0_2", 1)}}
         actual = get_missing_metrics_by_name(self.data, ["a", "b"])
         self.assertEqual(actual, expected)
 
-    def test_get_missing_metrics(self):
+    def test_get_missing_metrics(self) -> None:
         expected = MissingMetrics(
             {"a": {("0_1", 1)}},
             {"b": {("0_2", 1)}},
@@ -123,14 +123,14 @@ class UtilsTest(TestCase):
         actual = get_missing_metrics(self.data, self.optimization_config)
         self.assertEqual(actual, expected)
 
-    def test_best_feasible_objective(self):
+    def test_best_feasible_objective(self) -> None:
         bfo = best_feasible_objective(
             self.optimization_config,
             values={"a": np.array([1.0, 3.0, 2.0]), "b": np.array([0.0, -1.0, 0.0])},
         )
         self.assertEqual(list(bfo), [1.0, 1.0, 2.0])
 
-    def test_feasible_hypervolume(self):
+    def test_feasible_hypervolume(self) -> None:
         ma = Metric(name="a", lower_is_better=False)
         mb = Metric(name="b", lower_is_better=True)
         mc = Metric(name="c", lower_is_better=False)
