@@ -1,12 +1,12 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Dict, List, Optional
+from typing import Any, Dict, List, Optional, TYPE_CHECKING
 
 from ax.core.arm import Arm
 from ax.core.base_trial import BaseTrial, immutable_once_run
@@ -58,6 +58,7 @@ class Trial(BaseTrial):
             ttl_seconds=ttl_seconds,
             index=index,
         )
+        # pyre-fixme[4]: Attribute must be annotated.
         self._generator_run = None
         if generator_run is not None:
             self.add_generator_run(generator_run=generator_run)
@@ -91,7 +92,9 @@ class Trial(BaseTrial):
         return generator_run.arms[0]
 
     @immutable_once_run
-    def add_arm(self, arm: Arm) -> Trial:
+    def add_arm(
+        self, arm: Arm, candidate_metadata: Optional[Dict[str, Any]] = None
+    ) -> Trial:
         """Add arm to the trial.
 
         Returns:
@@ -99,7 +102,17 @@ class Trial(BaseTrial):
         """
 
         return self.add_generator_run(
-            generator_run=GeneratorRun(arms=[arm], type=GeneratorRunType.MANUAL.name)
+            generator_run=GeneratorRun(
+                arms=[arm],
+                type=GeneratorRunType.MANUAL.name,
+                # pyre-ignore[6]: In call `GeneratorRun.__init__`, for 3rd parameter
+                # `candidate_metadata_by_arm_signature`
+                # expected `Optional[Dict[str, Optional[Dict[str, typing.Any]]]]`
+                # but got `Optional[Dict[str, Dict[str, typing.Any]]]`
+                candidate_metadata_by_arm_signature=None
+                if candidate_metadata is None
+                else {arm.signature: candidate_metadata.copy()},
+            )
         )
 
     @immutable_once_run
@@ -141,8 +154,6 @@ class Trial(BaseTrial):
             arms: list of a single arm
                 attached to this trial if there is one, else None.
         """
-        # pyre-fixme[7]: Expected `List[Arm]` but got `Union[List[Optional[Arm]],
-        #  List[_T]]`.
         return [self.arm] if self.arm is not None else []
 
     @property
@@ -155,7 +166,6 @@ class Trial(BaseTrial):
                 arm name to arm if one is attached to this trial,
                 else None.
         """
-        # pyre-fixme[16]: `Optional` has no attribute `name`.
         return {self.arm.name: self.arm} if self.arm is not None else {}
 
     @property
