@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,50 +8,52 @@ import time
 from abc import ABC
 from collections import OrderedDict
 from copy import deepcopy
-from dataclasses import dataclass
+from dataclasses import dataclass, field
+
+from logging import Logger
 from typing import Any, Dict, List, MutableMapping, Optional, Set, Tuple, Type
 
 from ax.core.arm import Arm
 from ax.core.data import Data
 from ax.core.experiment import Experiment
-from ax.core.generator_run import GeneratorRun, extract_arm_predictions
+from ax.core.generator_run import extract_arm_predictions, GeneratorRun
 from ax.core.observation import (
     Observation,
     ObservationData,
     ObservationFeatures,
     observations_from_data,
+    recombine_observations,
     separate_observations,
 )
-from ax.core.optimization_config import (
-    OptimizationConfig,
-)
+from ax.core.optimization_config import OptimizationConfig
+from ax.core.parameter import ParameterType, RangeParameter
 from ax.core.search_space import SearchSpace
-from ax.core.types import (
-    TCandidateMetadata,
-    TConfig,
-    TGenMetadata,
-    TModelCov,
-    TModelMean,
-    TModelPredict,
-)
-from ax.modelbridge.modelbridge_utils import clamp_observation_features
+from ax.core.types import TCandidateMetadata, TModelCov, TModelMean, TModelPredict
 from ax.modelbridge.transforms.base import Transform
 from ax.modelbridge.transforms.cast import Cast
+from ax.models.types import TConfig
 from ax.utils.common.logger import get_logger
-from ax.utils.common.typeutils import not_none
+from ax.utils.common.typeutils import checked_cast, not_none
+
+logger: Logger = get_logger(__name__)
 
 
-logger = get_logger(__name__)
-
-
-@dataclass
+@dataclass(frozen=True)
 class BaseGenArgs:
     search_space: SearchSpace
-    optimization_config: OptimizationConfig
+    optimization_config: Optional[OptimizationConfig]
     pending_observations: Dict[str, List[ObservationFeatures]]
     fixed_features: ObservationFeatures
 
 
+@dataclass(frozen=True)
+class GenResults:
+    observation_features: List[ObservationFeatures]
+    weights: List[float]
+    best_observation_features: Optional[ObservationFeatures] = None
+    gen_metadata: Dict[str, Any] = field(default_factory=dict)
+
+
 class ModelBridge(ABC):
     """The main object for using models in Ax.
 
@@ -80,6 +82,7 @@ class ModelBridge(ABC):
     def __init__(
         self,
         search_space: SearchSpace,
+        # pyre-fixme[2]: Parameter annotation cannot be `Any`.
         model: Any,
         transforms: Optional[List[Type[Transform]]] = None,
         experiment: Optional[Experiment] = None,
@@ -135,28 +138,24 @@ class ModelBridge(ABC):
         self._model_kwargs: Optional[Dict[str, Any]] = None
         self._bridge_kwargs: Optional[Dict[str, Any]] = None
 
+        # pyre-fixme[4]: Attribute must be annotated.
         self._model_space = search_space.clone()
         self._raw_transforms = transforms
         self._transform_configs: Optional[Dict[str, TConfig]] = transform_configs
         self._fit_out_of_design = fit_out_of_design
         self._fit_abandoned = fit_abandoned
         imm = experiment and experiment.immutable_search_space_and_opt_config
+        # pyre-fixme[4]: Attribute must be annotated.
         self._experiment_has_immutable_search_space_and_opt_config = imm
         if experiment is not None:
             if self._optimization_config is None:
                 self._optimization_config = experiment.optimization_config
             self._arms_by_signature = experiment.arms_by_signature
 
-        observations = (
-            observations_from_data(
-                experiment=experiment,
-                data=data,
-                include_abandoned=self._fit_abandoned,
-            )
-            if experiment is not None and data is not None
-            else []
-        )
-        obs_feats_raw, obs_data_raw = self._set_training_data(
+        # Convert Data to Observations
+        observations = self._prepare_observations(experiment=experiment, data=data)
+
+        observations_raw = self._set_training_data(
             observations=observations, search_space=search_space
         )
         # Set model status quo
@@ -166,9 +165,8 @@ class ModelBridge(ABC):
             status_quo_name=status_quo_name,
             status_quo_features=status_quo_features,
         )
-        obs_feats, obs_data, search_space = self._transform_data(
-            obs_feats=obs_feats_raw,
-            obs_data=obs_data_raw,
+        observations, search_space = self._transform_data(
+            observations=observations_raw,
             search_space=search_space,
             transforms=transforms,
             transform_configs=transform_configs,
@@ -180,23 +178,32 @@ class ModelBridge(ABC):
             self._fit(
                 model=model,
                 search_space=search_space,
-                observation_features=obs_feats,
-                observation_data=obs_data,
+                observations=observations,
             )
+            # pyre-fixme[4]: Attribute must be annotated.
             self.fit_time = time.time() - t_fit_start
+            # pyre-fixme[4]: Attribute must be annotated.
             self.fit_time_since_gen = float(self.fit_time)
         except NotImplementedError:
             self.fit_time = 0.0
             self.fit_time_since_gen = 0.0
 
+    def _prepare_observations(
+        self, experiment: Optional[Experiment], data: Optional[Data]
+    ) -> List[Observation]:
+        if experiment is None or data is None:
+            return []
+        return observations_from_data(
+            experiment=experiment, data=data, include_abandoned=self._fit_abandoned
+        )
+
     def _transform_data(
         self,
-        obs_feats: List[ObservationFeatures],
-        obs_data: List[ObservationData],
+        observations: List[Observation],
         search_space: SearchSpace,
         transforms: Optional[List[Type[Transform]]],
         transform_configs: Optional[Dict[str, TConfig]],
-    ) -> Tuple[List[ObservationFeatures], List[ObservationData], SearchSpace]:
+    ) -> Tuple[List[Observation], SearchSpace]:
         """Initialize transforms and apply them to provided data."""
         # Initialize transforms
         search_space = search_space.clone()
@@ -207,52 +214,48 @@ class ModelBridge(ABC):
             for t in transforms:
                 t_instance = t(
                     search_space=search_space,
-                    observation_features=obs_feats,
-                    observation_data=obs_data,
+                    observations=observations,
+                    modelbridge=self,
                     config=transform_configs.get(t.__name__, None),
                 )
                 search_space = t_instance.transform_search_space(search_space)
-                obs_feats = t_instance.transform_observation_features(obs_feats)
-                obs_data = t_instance.transform_observation_data(obs_data, obs_feats)
+                observations = t_instance.transform_observations(observations)
                 self.transforms[t.__name__] = t_instance
 
-        return obs_feats, obs_data, search_space
+        return observations, search_space
 
     def _prepare_training_data(
         self, observations: List[Observation]
-    ) -> Tuple[List[ObservationFeatures], List[ObservationData]]:
+    ) -> List[Observation]:
         observation_features, observation_data = separate_observations(observations)
         if len(observation_features) != len(set(observation_features)):
             raise ValueError(
                 "Observation features not unique."
                 "Something went wrong constructing training data..."
             )
-        return observation_features, observation_data
+        return observations
 
     def _set_training_data(
         self, observations: List[Observation], search_space: SearchSpace
-    ) -> Tuple[List[ObservationFeatures], List[ObservationData]]:
+    ) -> List[Observation]:
         """Store training data, not-transformed.
 
         If the modelbridge specifies _fit_out_of_design, all training data is
         returned. Otherwise, only in design points are returned.
         """
-        observation_features, observation_data = self._prepare_training_data(
-            observations=observations
-        )
+        observations = self._prepare_training_data(observations=observations)
         self._training_data = deepcopy(observations)
         self._metric_names: Set[str] = set()
-        for obsd in observation_data:
-            self._metric_names.update(obsd.metric_names)
+        for obs in observations:
+            self._metric_names.update(obs.data.metric_names)
         return self._process_in_design(
             search_space=search_space,
-            observation_features=observation_features,
-            observation_data=observation_data,
+            observations=observations,
         )
 
     def _extend_training_data(
         self, observations: List[Observation]
-    ) -> Tuple[List[ObservationFeatures], List[ObservationData]]:
+    ) -> List[Observation]:
         """Extend and return training data, not-transformed.
 
         If the modelbridge specifies _fit_out_of_design, all training data is
@@ -261,15 +264,11 @@ class ModelBridge(ABC):
         Args:
             observations: New observations.
 
-        Returns:
-            observation_features: New + old observation features.
-            observation_data: New + old observation data.
+        Returns: New + old observations.
         """
-        observation_features, observation_data = self._prepare_training_data(
-            observations=observations
-        )
-        for obsd in observation_data:
-            for metric_name in obsd.metric_names:
+        observations = self._prepare_training_data(observations=observations)
+        for obs in observations:
+            for metric_name in obs.data.metric_names:
                 if metric_name not in self._metric_names:
                     raise ValueError(
                         f"Unrecognised metric {metric_name}; cannot update "
@@ -278,38 +277,43 @@ class ModelBridge(ABC):
                     )
         # Initialize with all points in design.
         self._training_data.extend(deepcopy(observations))
-        all_observation_features, all_observation_data = separate_observations(
-            self.get_training_data()
-        )
+        all_observations = self.get_training_data()
         return self._process_in_design(
             search_space=self._model_space,
-            observation_features=all_observation_features,
-            observation_data=all_observation_data,
+            observations=all_observations,
         )
 
     def _process_in_design(
         self,
         search_space: SearchSpace,
-        observation_features: List[ObservationFeatures],
-        observation_data: List[ObservationData],
-    ) -> Tuple[List[ObservationFeatures], List[ObservationData]]:
+        observations: List[Observation],
+    ) -> List[Observation]:
         """Set training_in_design, and decide whether to filter out of design points."""
         # Don't filter points.
         if self._fit_out_of_design:
             # Use all data for training
             # Set training_in_design to True for all observations so that
             # all observations are used in CV and plotting
-            self.training_in_design = [True] * len(observation_features)
-            return observation_features, observation_data
-        in_design = [
-            search_space.check_membership(obsf.parameters)
-            for obsf in observation_features
-        ]
+            self.training_in_design = [True] * len(observations)
+            return observations
+        in_design = self._compute_in_design(
+            search_space=search_space, observations=observations
+        )
         self.training_in_design = in_design
-        in_design_indices = [i for i, in_design in enumerate(in_design) if in_design]
-        in_design_features = [observation_features[i] for i in in_design_indices]
-        in_design_data = [observation_data[i] for i in in_design_indices]
-        return in_design_features, in_design_data
+        in_design_obs = [
+            observations[i] for i, is_in_design in enumerate(in_design) if is_in_design
+        ]
+        return in_design_obs
+
+    def _compute_in_design(
+        self,
+        search_space: SearchSpace,
+        observations: List[Observation],
+    ) -> List[bool]:
+        return [
+            search_space.check_membership(obs.features.parameters)
+            for obs in observations
+        ]
 
     def _set_status_quo(
         self,
@@ -337,7 +341,6 @@ class ModelBridge(ABC):
             and experiment is not None
             and experiment.status_quo is not None
         ):
-            # pyre-fixme[16]: `Optional` has no attribute `name`.
             status_quo_name = experiment.status_quo.name
 
         if status_quo_name is not None:
@@ -421,10 +424,10 @@ class ModelBridge(ABC):
 
     def _fit(
         self,
+        # pyre-fixme[2]: Parameter annotation cannot be `Any`.
         model: Any,
         search_space: SearchSpace,
-        observation_features: List[ObservationFeatures],
-        observation_data: List[ObservationData],
+        observations: List[Observation],
     ) -> None:
         """Apply terminal transform and fit model."""
         raise NotImplementedError  # pragma: no cover
@@ -445,14 +448,13 @@ class ModelBridge(ABC):
         observation_data = self._predict(observation_features)
 
         # Apply reverse transforms, in reverse order
-        for t in reversed(self.transforms.values()):  # noqa T484
-            observation_features = t.untransform_observation_features(
-                observation_features
-            )
-            observation_data = t.untransform_observation_data(
-                observation_data, observation_features
-            )
-        return observation_data
+        pred_observations = recombine_observations(
+            observation_features=observation_features, observation_data=observation_data
+        )
+
+        for t in reversed(list(self.transforms.values())):
+            pred_observations = t.untransform_observations(pred_observations)
+        return [obs.data for obs in pred_observations]
 
     def _single_predict(
         self, observation_features: List[ObservationFeatures]
@@ -541,29 +543,17 @@ class ModelBridge(ABC):
             experiment: Experiment, in which this data was obtained.
         """
         t_update_start = time.time()
-        observations = (
-            observations_from_data(
-                experiment=experiment,
-                data=new_data,
-                include_abandoned=self._fit_abandoned,
-            )
-            if experiment is not None and new_data is not None
-            else []
-        )
-        obs_feats_raw, obs_data_raw = self._extend_training_data(
-            observations=observations
-        )
-        obs_feats, obs_data, search_space = self._transform_data(
-            obs_feats=obs_feats_raw,
-            obs_data=obs_data_raw,
+        observations = self._prepare_observations(experiment=experiment, data=new_data)
+        obs_raw = self._extend_training_data(observations=observations)
+        observations, search_space = self._transform_data(
+            observations=obs_raw,
             search_space=self._model_space,
             transforms=self._raw_transforms,
             transform_configs=self._transform_configs,
         )
         self._update(
             search_space=search_space,
-            observation_features=obs_feats,
-            observation_data=obs_data,
+            observations=observations,
         )
         self.fit_time += time.time() - t_update_start
         self.fit_time_since_gen += time.time() - t_update_start
@@ -571,8 +561,7 @@ class ModelBridge(ABC):
     def _update(
         self,
         search_space: SearchSpace,
-        observation_features: List[ObservationFeatures],
-        observation_data: List[ObservationData],
+        observations: List[Observation],
     ) -> None:
         """Apply terminal transform and update model.
 
@@ -596,10 +585,8 @@ class ModelBridge(ABC):
             pending_observations = {}
         if fixed_features is None:
             fixed_features = ObservationFeatures({})
-
         if optimization_config is None:
             optimization_config = (
-                # pyre-fixme[16]: `Optional` has no attribute `clone`.
                 self._optimization_config.clone()
                 if self._optimization_config is not None
                 else None
@@ -640,6 +627,9 @@ class ModelBridge(ABC):
         model_gen_options: Optional[TConfig] = None,
     ) -> GeneratorRun:
         """
+        Generate new points from the underlying model according to
+        search_space, optimization_config and other parameters.
+
         Args:
             n: Number of points to generate
             search_space: Search space
@@ -651,8 +641,11 @@ class ModelBridge(ABC):
                 generation.
             model_gen_options: A config dictionary that is passed along to the
                 model.
+
+        Returns:
+            A GeneratorRun object that contains the generated points and other metadata.
         """
-        t_gen_start = time.time()
+        t_gen_start = time.monotonic()
         # Get modifiable versions
         if search_space is None:
             search_space = self._model_space
@@ -666,7 +659,7 @@ class ModelBridge(ABC):
         )
 
         # Apply terminal transform and gen
-        observation_features, weights, best_obsf, gen_metadata = self._gen(
+        gen_results = self._gen(
             n=n,
             search_space=base_gen_args.search_space,
             optimization_config=base_gen_args.optimization_config,
@@ -674,8 +667,11 @@ class ModelBridge(ABC):
             fixed_features=base_gen_args.fixed_features,
             model_gen_options=model_gen_options,
         )
+
+        observation_features = gen_results.observation_features
+        best_obsf = gen_results.best_observation_features
         # Apply reverse transforms
-        for t in reversed(self.transforms.values()):  # noqa T484
+        for t in reversed(list(self.transforms.values())):
             observation_features = t.untransform_observation_features(
                 observation_features
             )
@@ -723,7 +719,7 @@ class ModelBridge(ABC):
         optimization_config = None if immutable else base_gen_args.optimization_config
         gr = GeneratorRun(
             arms=arms,
-            weights=weights,
+            weights=gen_results.weights,
             optimization_config=optimization_config,
             search_space=None if immutable else base_gen_args.search_space,
             model_predictions=model_predictions,
@@ -731,11 +727,11 @@ class ModelBridge(ABC):
             if best_arm is None
             else (best_arm, best_point_predictions),
             fit_time=self.fit_time_since_gen,
-            gen_time=time.time() - t_gen_start,
+            gen_time=time.monotonic() - t_gen_start,
             model_key=self._model_key,
             model_kwargs=self._model_kwargs,
             bridge_kwargs=self._bridge_kwargs,
-            gen_metadata=gen_metadata,
+            gen_metadata=gen_results.gen_metadata,
             model_state_after_gen=self._get_serialized_model_state(),
             candidate_metadata_by_arm_signature=candidate_metadata,
         )
@@ -750,12 +746,7 @@ class ModelBridge(ABC):
         pending_observations: Dict[str, List[ObservationFeatures]],
         fixed_features: ObservationFeatures,
         model_gen_options: Optional[TConfig],
-    ) -> Tuple[
-        List[ObservationFeatures],
-        List[float],
-        Optional[ObservationFeatures],
-        TGenMetadata,
-    ]:
+    ) -> GenResults:
         """Apply terminal transform, gen, and reverse terminal transform on
         output.
         """
@@ -777,36 +768,34 @@ class ModelBridge(ABC):
         """
         # Apply transforms to cv_training_data and cv_test_points
         cv_test_points = deepcopy(cv_test_points)
-        obs_feats, obs_data = separate_observations(
-            observations=cv_training_data, copy=True
-        )
+        cv_training_data = deepcopy(cv_training_data)
         search_space = self._model_space.clone()
         for t in self.transforms.values():
-            obs_feats = t.transform_observation_features(obs_feats)
-            obs_data = t.transform_observation_data(obs_data, obs_feats)
+            cv_training_data = t.transform_observations(cv_training_data)
             cv_test_points = t.transform_observation_features(cv_test_points)
             search_space = t.transform_search_space(search_space)
 
+        obs_feats, obs_data = separate_observations(observations=cv_training_data)
         # Apply terminal transform, and get predictions.
         cv_predictions = self._cross_validate(
             search_space=search_space,
-            obs_feats=obs_feats,
-            obs_data=obs_data,
+            cv_training_data=cv_training_data,
             cv_test_points=cv_test_points,
         )
         # Apply reverse transforms, in reverse order
-        for t in reversed(self.transforms.values()):
-            cv_test_points = t.untransform_observation_features(cv_test_points)
-            cv_predictions = t.untransform_observation_data(
-                cv_predictions, cv_test_points
-            )
-        return cv_predictions
+        cv_test_observations = [
+            Observation(features=obsf, data=cv_predictions[i])
+            for i, obsf in enumerate(cv_test_points)
+        ]
+
+        for t in reversed(list(self.transforms.values())):
+            cv_test_observations = t.untransform_observations(cv_test_observations)
+        return [obs.data for obs in cv_test_observations]
 
     def _cross_validate(
         self,
         search_space: SearchSpace,
-        obs_feats: List[ObservationFeatures],
-        obs_data: List[ObservationData],
+        cv_training_data: List[Observation],
         cv_test_points: List[ObservationFeatures],
     ) -> List[ObservationData]:
         """Apply the terminal transform, make predictions on the test points,
@@ -814,30 +803,6 @@ class ModelBridge(ABC):
         """
         raise NotImplementedError  # pragma: no cover
 
-    def evaluate_acquisition_function(
-        self, observation_features: List[ObservationFeatures]
-    ) -> List[float]:
-        """Evaluate the acquisition function for given set of observation
-        features.
-
-        Args:
-            observation_features: A list of observation features, representing
-                parameterizations, for which to evaluate the acquisition function.
-
-        Returns:
-            A list of acquisition function values, in the same order as the
-            input observation features.
-        """
-        obs_feats = deepcopy(observation_features)
-        for t in self.transforms.values():
-            obs_feats = t.transform_observation_features(obs_feats)
-        return self._evaluate_acquisition_function(observation_features=obs_feats)
-
-    def _evaluate_acquisition_function(
-        self, observation_features: List[ObservationFeatures]
-    ) -> List[float]:
-        raise NotImplementedError  # pragma: no cover
-
     def _set_kwargs_to_save(
         self,
         model_key: str,
@@ -863,17 +828,18 @@ class ModelBridge(ABC):
     def _deserialize_model_state(
         self, serialized_state: Dict[str, Any]
     ) -> Dict[str, Any]:
-        model = not_none(self.model)
-        return model.deserialize_state(serialized_state=serialized_state)
+        model = not_none(self.model)  # pragma: no cover
+        return model.deserialize_state(  # pragma: no cover
+            serialized_state=serialized_state
+        )
 
     def feature_importances(self, metric_name: str) -> Dict[str, float]:
         raise NotImplementedError(
             "Feature importance not available for this model type"
         )
 
-    def transform_observation_data(
-        self, observation_data: List[ObservationData]
-    ) -> Any:
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
+    def transform_observations(self, observations: List[Observation]) -> Any:
         """Applies transforms to given observation features and returns them in the
         model space.
 
@@ -884,18 +850,18 @@ class ModelBridge(ABC):
             Transformed values. This could be e.g. a torch Tensor, depending
             on the ModelBridge subclass.
         """
-        obsd = deepcopy(observation_data)
+        observations = deepcopy(observations)
         for t in self.transforms.values():
-            obsd = t.transform_observation_data(obsd, [])
+            observations = t.transform_observations(observations)
         # Apply terminal transform and return
-        return self._transform_observation_data(obsd)
+        return self._transform_observations(observations)
 
-    def _transform_observation_data(
-        self, observation_data: List[ObservationData]
-    ) -> Any:
-        """Apply terminal transform to given observation features and return result."""
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
+    def _transform_observations(self, observations: List[Observation]) -> Any:
+        """Apply terminal transform to given observations and return result."""
         raise NotImplementedError  # pragma: no cover
 
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
     def transform_observation_features(
         self, observation_features: List[ObservationFeatures]
     ) -> Any:
@@ -915,12 +881,14 @@ class ModelBridge(ABC):
         # Apply terminal transform and return
         return self._transform_observation_features(obsf)
 
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
     def _transform_observation_features(
         self, observation_features: List[ObservationFeatures]
     ) -> Any:
         """Apply terminal transform to given observation features and return result."""
         raise NotImplementedError  # pragma: no cover
 
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
     def transform_optimization_config(
         self,
         optimization_config: OptimizationConfig,
@@ -987,3 +955,32 @@ def gen_arms(
         if of.metadata:
             candidate_metadata[arm.signature] = of.metadata
     return arms, candidate_metadata or None  # None if empty cand. metadata.
+
+
+def clamp_observation_features(
+    observation_features: List[ObservationFeatures], search_space: SearchSpace
+) -> List[ObservationFeatures]:
+    range_parameters = [
+        p for p in search_space.parameters.values() if isinstance(p, RangeParameter)
+    ]
+    for obsf in observation_features:
+        for p in range_parameters:
+            if p.name not in obsf.parameters:
+                continue
+            if p.parameter_type == ParameterType.FLOAT:
+                val = checked_cast(float, obsf.parameters[p.name])
+            else:
+                val = checked_cast(int, obsf.parameters[p.name])
+            if val < p.lower:
+                logger.info(
+                    f"Untransformed parameter {val} "
+                    f"less than lower bound {p.lower}, clamping"
+                )
+                obsf.parameters[p.name] = p.lower
+            elif val > p.upper:
+                logger.info(
+                    f"Untransformed parameter {val} "
+                    f"greater than upper bound {p.upper}, clamping"
+                )
+                obsf.parameters[p.name] = p.upper
+    return observation_features
