@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -21,11 +21,14 @@ from ax.models.torch.alebo import ALEBO
 
 
 def get_ALEBOInitializer(
-    search_space: SearchSpace, B: np.ndarray, **model_kwargs: Any
+    search_space: SearchSpace,
+    B: np.ndarray,
+    seed: Optional[int] = None,
+    **model_kwargs: Any,
 ) -> RandomModelBridge:
     return RandomModelBridge(
         search_space=search_space,
-        model=ALEBOInitializer(B=B, **model_kwargs),
+        model=ALEBOInitializer(B=B, seed=seed, **model_kwargs),
         transforms=ALEBO_X_trans,
     )
 
@@ -85,7 +88,9 @@ class ALEBOStrategy(GenerationStrategy):
         self.init_size = init_size
         self.dtype = dtype
         self.device = device
+        # pyre-fixme[4]: Attribute must be annotated.
         self.random_kwargs = random_kwargs if random_kwargs is not None else {}
+        # pyre-fixme[4]: Attribute must be annotated.
         self.gp_kwargs = gp_kwargs if gp_kwargs is not None else {}
         self.gp_gen_kwargs = gp_gen_kwargs
 
@@ -128,5 +133,6 @@ class ALEBOStrategy(GenerationStrategy):
     ) -> torch.Tensor:
         """Generate the projection matrix B as a (d x D) tensor"""
         B0 = torch.randn(d, D, dtype=dtype, device=device)
-        B = B0 / torch.sqrt((B0 ** 2).sum(dim=0))
+        # pyre-fixme[58]: `**` is not supported for operand types `Tensor` and `int`.
+        B = B0 / torch.sqrt((B0**2).sum(dim=0))
         return B
