@@ -1,9 +1,10 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import warnings
 from unittest import mock
 
 import numpy as np
@@ -20,13 +21,23 @@ from ax.core.observation import (
 from ax.core.optimization_config import OptimizationConfig
 from ax.core.parameter import FixedParameter, ParameterType, RangeParameter
 from ax.core.search_space import SearchSpace
-from ax.modelbridge.base import ModelBridge, gen_arms, unwrap_observation_data
+from ax.modelbridge.base import (
+    clamp_observation_features,
+    gen_arms,
+    GenResults,
+    ModelBridge,
+    unwrap_observation_data,
+)
+from ax.modelbridge.registry import Models
 from ax.modelbridge.transforms.log import Log
 from ax.models.base import Model
 from ax.utils.common.constants import Keys
 from ax.utils.common.testutils import TestCase
 from ax.utils.testing.core_stubs import (
+    get_branin_experiment_with_multi_objective,
+    get_experiment,
     get_experiment_with_repeated_arms,
+    get_non_monolithic_branin_moo_data,
     get_optimization_config_no_constraints,
     get_search_space_for_range_value,
     get_search_space_for_range_values,
@@ -57,6 +68,8 @@ class BaseModelBridgeTest(TestCase):
         return_value=([Arm(parameters={})], None),
     )
     @mock.patch("ax.modelbridge.base.ModelBridge._fit", autospec=True)
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def testModelBridge(self, mock_fit, mock_gen_arms, mock_observations_from_data):
         # Test that on init transforms are stored and applied in the correct order
         transforms = [transform_1, transform_2]
@@ -65,8 +78,12 @@ class BaseModelBridgeTest(TestCase):
         modelbridge = ModelBridge(
             search_space=ss,
             model=Model(),
+            # pyre-fixme[6]: For 3rd param expected
+            #  `Optional[List[Type[Transform]]]` but got `List[Type[Union[transform_1,
+            #  transform_2]]]`.
             transforms=transforms,
             experiment=exp,
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
             data=0,
         )
         self.assertFalse(
@@ -77,8 +94,7 @@ class BaseModelBridgeTest(TestCase):
         )
         fit_args = mock_fit.mock_calls[0][2]
         self.assertTrue(fit_args["search_space"] == get_search_space_for_value(8.0))
-        self.assertTrue(fit_args["observation_features"] == [])
-        self.assertTrue(fit_args["observation_data"] == [])
+        self.assertTrue(fit_args["observations"] == [])
         self.assertTrue(mock_observations_from_data.called)
 
         # Test prediction on out of design features.
@@ -118,6 +134,7 @@ class BaseModelBridgeTest(TestCase):
         )
         modelbridge.predict([get_observation2().features])
         # Observation features sent to _predict are un-transformed afterwards
+        # pyre-fixme[16]: Callable `_predict` has no attribute `assert_called_with`.
         modelbridge._predict.assert_called_with([get_observation2().features])
 
         # Check that _single_predict is equivalent here.
@@ -129,7 +146,9 @@ class BaseModelBridgeTest(TestCase):
         modelbridge._gen = mock.MagicMock(
             "ax.modelbridge.base.ModelBridge._gen",
             autospec=True,
-            return_value=([get_observation1trans().features], [2], None, {}),
+            return_value=GenResults(
+                observation_features=[get_observation1trans().features], weights=[2]
+            ),
         )
         oc = OptimizationConfig(objective=Objective(metric=Metric(name="test_metric")))
         modelbridge._set_kwargs_to_save(
@@ -143,6 +162,7 @@ class BaseModelBridgeTest(TestCase):
             fixed_features=ObservationFeatures({"x": 5}),
         )
         self.assertEqual(gr._model_key, "TestModel")
+        # pyre-fixme[16]: Callable `_gen` has no attribute `assert_called_with`.
         modelbridge._gen.assert_called_with(
             n=1,
             search_space=SearchSpace([FixedParameter("x", ParameterType.FLOAT, 8.0)]),
@@ -197,10 +217,11 @@ class BaseModelBridgeTest(TestCase):
         cv_predictions = modelbridge.cross_validate(
             cv_training_data=cv_training_data, cv_test_points=cv_test_points
         )
+        # pyre-fixme[16]: Callable `_cross_validate` has no attribute
+        #  `assert_called_with`.
         modelbridge._cross_validate.assert_called_with(
             search_space=SearchSpace([FixedParameter("x", ParameterType.FLOAT, 8.0)]),
-            obs_feats=[get_observation2trans().features],
-            obs_data=[get_observation2trans().data],
+            cv_training_data=[get_observation2trans()],
             cv_test_points=[get_observation1().features],  # untransformed after
         )
         self.assertTrue(cv_predictions == [get_observation1().data])
@@ -236,6 +257,7 @@ class BaseModelBridgeTest(TestCase):
         autospec=True,
         return_value=([get_observation1(), get_observation2()]),
     )
+    # pyre-fixme[3]: Return type must be annotated.
     def test_ood_gen(self, _):
         # Test fit_out_of_design by returning OOD candidats
         exp = get_experiment_for_value()
@@ -245,6 +267,7 @@ class BaseModelBridgeTest(TestCase):
             model=Model(),
             transforms=[],
             experiment=exp,
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
             data=0,
             fit_out_of_design=True,
         )
@@ -252,7 +275,7 @@ class BaseModelBridgeTest(TestCase):
         modelbridge._gen = mock.MagicMock(
             "ax.modelbridge.base.ModelBridge._gen",
             autospec=True,
-            return_value=([obs], [2], None, {}),
+            return_value=GenResults(observation_features=[obs], weights=[2]),
         )
         gr = modelbridge.gen(n=1)
         self.assertEqual(gr.arms[0].parameters, obs.parameters)
@@ -263,6 +286,7 @@ class BaseModelBridgeTest(TestCase):
             model=Model(),
             transforms=[],
             experiment=exp,
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
             data=0,
             fit_out_of_design=False,
         )
@@ -270,7 +294,7 @@ class BaseModelBridgeTest(TestCase):
         modelbridge._gen = mock.MagicMock(
             "ax.modelbridge.base.ModelBridge._gen",
             autospec=True,
-            return_value=([obs], [2], None, {}),
+            return_value=GenResults(observation_features=[obs], weights=[2]),
         )
         gr = modelbridge.gen(n=1)
         self.assertEqual(gr.arms[0].parameters, {"x": 1.0})
@@ -281,6 +305,8 @@ class BaseModelBridgeTest(TestCase):
         return_value=([get_observation1()]),
     )
     @mock.patch("ax.modelbridge.base.ModelBridge._fit", autospec=True)
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def testSetStatusQuo(self, mock_fit, mock_observations_from_data):
         # NOTE: If empty data object is not passed, observations are not
         # extracted, even with mock.
@@ -299,6 +325,7 @@ class BaseModelBridgeTest(TestCase):
             0,
             [],
             get_experiment_for_value(),
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
             0,
             status_quo_features=get_observation1().features,
         )
@@ -310,6 +337,7 @@ class BaseModelBridgeTest(TestCase):
         sq = Arm(name="1_1", parameters={"x": 3.0})
         exp._status_quo = sq
         # Check that we set SQ to arm 1_1
+        # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
         modelbridge = ModelBridge(get_search_space_for_value(), 0, [], exp, 0)
         self.assertEqual(modelbridge.status_quo, get_observation1())
 
@@ -320,6 +348,7 @@ class BaseModelBridgeTest(TestCase):
                 0,
                 [],
                 exp,
+                # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
                 0,
                 status_quo_features=get_observation1().features,
                 status_quo_name="1_1",
@@ -327,7 +356,13 @@ class BaseModelBridgeTest(TestCase):
 
         # Left as None if features or name don't exist
         modelbridge = ModelBridge(
-            get_search_space_for_value(), 0, [], exp, 0, status_quo_name="1_0"
+            get_search_space_for_value(),
+            0,
+            [],
+            exp,
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
+            0,
+            status_quo_name="1_0",
         )
         self.assertIsNone(modelbridge.status_quo)
         modelbridge = ModelBridge(
@@ -335,11 +370,51 @@ class BaseModelBridgeTest(TestCase):
             0,
             [],
             get_experiment_for_value(),
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
             0,
             status_quo_features=ObservationFeatures(parameters={"x": 3.0, "y": 10.0}),
         )
         self.assertIsNone(modelbridge.status_quo)
 
+    @mock.patch(
+        "ax.modelbridge.base.ModelBridge._gen",
+        autospec=True,
+    )
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
+    def test_status_quo_for_non_monolithic_data(self, mock_gen):
+        mock_gen.return_value = GenResults(
+            observation_features=[
+                ObservationFeatures(
+                    parameters={"x1": float(i), "x2": float(i)}, trial_index=np.int64(1)
+                )
+                for i in range(5)
+            ],
+            # pyre-fixme[6]: For 2nd param expected `List[float]` but got `List[int]`.
+            weights=[1] * 5,
+        )
+        exp = get_branin_experiment_with_multi_objective(with_status_quo=True)
+        sobol = Models.SOBOL(search_space=exp.search_space)
+        exp.new_batch_trial(sobol.gen(5)).set_status_quo_and_optimize_power(
+            status_quo=exp.status_quo
+        ).run()
+
+        # create data where metrics vary in start and end times
+        data = get_non_monolithic_branin_moo_data()
+        with warnings.catch_warnings(record=True) as ws:
+            bridge = ModelBridge(
+                experiment=exp,
+                data=data,
+                model=Model(),
+                search_space=exp.search_space,
+            )
+        # just testing it doesn't error
+        bridge.gen(5)
+        self.assertTrue(any("start_time" in str(w.message) for w in ws))
+        self.assertTrue(any("end_time" in str(w.message) for w in ws))
+        # pyre-fixme[16]: Optional type has no attribute `arm_name`.
+        self.assertEqual(bridge.status_quo.arm_name, "status_quo")
+
     @mock.patch(
         "ax.modelbridge.base.observations_from_data",
         autospec=True,
@@ -353,12 +428,16 @@ class BaseModelBridgeTest(TestCase):
         ),
     )
     @mock.patch("ax.modelbridge.base.ModelBridge._fit", autospec=True)
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def testSetStatusQuoMultipleObs(self, mock_fit, mock_observations_from_data):
         exp = get_experiment_with_repeated_arms(2)
 
         trial_index = 1
         status_quo_features = ObservationFeatures(
+            # pyre-fixme[16]: `BaseTrial` has no attribute `status_quo`.
             parameters=exp.trials[trial_index].status_quo.parameters,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
             trial_index=trial_index,
         )
         modelbridge = ModelBridge(
@@ -366,6 +445,7 @@ class BaseModelBridgeTest(TestCase):
             0,
             [],
             exp,
+            # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
             0,
             status_quo_features=status_quo_features,
         )
@@ -380,6 +460,8 @@ class BaseModelBridgeTest(TestCase):
         return_value=([get_observation1(), get_observation1()]),
     )
     @mock.patch("ax.modelbridge.base.ModelBridge._fit", autospec=True)
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def testSetTrainingDataDupFeatures(self, mock_fit, mock_observations_from_data):
         # Throws an error if repeated features in observations.
         with self.assertRaises(ValueError):
@@ -388,11 +470,12 @@ class BaseModelBridgeTest(TestCase):
                 0,
                 [],
                 get_experiment_for_value(),
+                # pyre-fixme[6]: For 5th param expected `Optional[Data]` but got `int`.
                 0,
                 status_quo_name="1_1",
             )
 
-    def testUnwrapObservationData(self):
+    def testUnwrapObservationData(self) -> None:
         observation_data = [get_observation1().data, get_observation2().data]
         f, cov = unwrap_observation_data(observation_data)
         self.assertEqual(f["a"], [2.0, 2.0])
@@ -408,17 +491,23 @@ class BaseModelBridgeTest(TestCase):
         with self.assertRaises(ValueError):
             unwrap_observation_data(observation_data + [od3])
 
-    def testGenArms(self):
+    def testGenArms(self) -> None:
         p1 = {"x": 0, "y": 1}
         p2 = {"x": 4, "y": 8}
         observation_features = [
+            # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Dict[str, int]`.
             ObservationFeatures(parameters=p1),
+            # pyre-fixme[6]: For 1st param expected `Dict[str, Union[None, bool,
+            #  float, int, str]]` but got `Dict[str, int]`.
             ObservationFeatures(parameters=p2),
         ]
         arms, candidate_metadata = gen_arms(observation_features=observation_features)
         self.assertEqual(arms[0].parameters, p1)
         self.assertIsNone(candidate_metadata)
 
+        # pyre-fixme[6]: For 2nd param expected `Dict[str, Union[None, bool, float,
+        #  int, str]]` but got `Dict[str, int]`.
         arm = Arm(name="1_1", parameters=p1)
         arms_by_signature = {arm.signature: arm}
         observation_features[0].metadata = {"some_key": "some_val_0"}
@@ -439,11 +528,15 @@ class BaseModelBridgeTest(TestCase):
     @mock.patch(
         "ax.modelbridge.base.ModelBridge._gen",
         autospec=True,
-        return_value=([get_observation1trans().features], [2], None, {}),
+        return_value=GenResults(
+            observation_features=[get_observation1trans().features], weights=[2]
+        ),
     )
     @mock.patch(
         "ax.modelbridge.base.ModelBridge.predict", autospec=True, return_value=None
     )
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def testGenWithDefaults(self, _, mock_gen):
         exp = get_experiment_for_value()
         exp.optimization_config = get_optimization_config_no_constraints()
@@ -468,11 +561,14 @@ class BaseModelBridgeTest(TestCase):
     @mock.patch(
         "ax.modelbridge.base.ModelBridge._gen",
         autospec=True,
-        return_value=([get_observation1trans().features], [2], None, {}),
+        return_value=GenResults(
+            observation_features=[get_observation1trans().features], weights=[2]
+        ),
     )
     @mock.patch(
         "ax.modelbridge.base.ModelBridge.predict", autospec=True, return_value=None
     )
+    # pyre-fixme[3]: Return type must be annotated.
     def test_gen_on_experiment_with_imm_ss_and_opt_conf(self, _, __):
         exp = get_experiment_for_value()
         exp._properties[Keys.IMMUTABLE_SEARCH_SPACE_AND_OPT_CONF] = True
@@ -492,16 +588,17 @@ class BaseModelBridgeTest(TestCase):
         "ax.modelbridge.base.ModelBridge._gen",
         autospec=True,
         side_effect=[
-            ([get_observation1trans().features], [2], None, {}),
-            ([get_observation2trans().features], [2], None, {}),
-            ([get_observation2().features], [2], None, {}),
+            GenResults([get_observation1trans().features], [2]),
+            GenResults([get_observation2trans().features], [2]),
+            GenResults([get_observation2().features], weights=[2]),
         ],
     )
     @mock.patch("ax.modelbridge.base.ModelBridge._update", autospec=True)
+    # pyre-fixme[3]: Return type must be annotated.
     def test_update(self, _mock_update, _mock_gen):
         exp = get_experiment_for_value()
         exp.optimization_config = get_optimization_config_no_constraints()
-        ss = get_search_space_for_range_values()
+        ss = get_search_space_for_range_values(min=0, max=1000)
         exp.search_space = ss
         modelbridge = ModelBridge(
             search_space=ss, model=Model(), transforms=[Log], experiment=exp
@@ -552,3 +649,53 @@ class BaseModelBridgeTest(TestCase):
                 ),
                 experiment=exp,
             )
+
+
+class testClampObservationFeatures(TestCase):
+    def testClampObservationFeaturesNearBounds(self) -> None:
+        cases = [
+            (
+                ObservationFeatures(
+                    parameters={"w": 1.0, "x": 2, "y": "foo", "z": True}
+                ),
+                ObservationFeatures(
+                    parameters={"w": 1.0, "x": 2, "y": "foo", "z": True}
+                ),
+            ),
+            (
+                ObservationFeatures(
+                    parameters={"w": 0.0, "x": 2, "y": "foo", "z": True}
+                ),
+                ObservationFeatures(
+                    parameters={"w": 0.5, "x": 2, "y": "foo", "z": True}
+                ),
+            ),
+            (
+                ObservationFeatures(
+                    parameters={"w": 100.0, "x": 2, "y": "foo", "z": True}
+                ),
+                ObservationFeatures(
+                    parameters={"w": 5.5, "x": 2, "y": "foo", "z": True}
+                ),
+            ),
+            (
+                ObservationFeatures(
+                    parameters={"w": 1.0, "x": 0, "y": "foo", "z": True}
+                ),
+                ObservationFeatures(
+                    parameters={"w": 1.0, "x": 1, "y": "foo", "z": True}
+                ),
+            ),
+            (
+                ObservationFeatures(
+                    parameters={"w": 1.0, "x": 11, "y": "foo", "z": True}
+                ),
+                ObservationFeatures(
+                    parameters={"w": 1.0, "x": 10, "y": "foo", "z": True}
+                ),
+            ),
+        ]
+        search_space = get_experiment().search_space
+        for obs_ft, expected_obs_ft in cases:
+            actual_obs_ft = clamp_observation_features([obs_ft], search_space)
+            self.assertEqual(actual_obs_ft[0], expected_obs_ft)
