@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -24,7 +24,7 @@ from ax.utils.common.testutils import TestCase
 
 
 class DerelativizeTransformTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         m = mock.patch.object(ModelBridge, "__abstractmethods__", frozenset())
         self.addCleanup(m.stop)
         m.start()
@@ -73,11 +73,19 @@ class DerelativizeTransformTest(TestCase):
             ]
         ),
     )
+    # pyre-fixme[3]: Return type must be annotated.
     def testDerelativizeTransform(
-        self, mock_predict, mock_fit, mock_observations_from_data
+        self,
+        # pyre-fixme[2]: Parameter must be annotated.
+        mock_predict,
+        # pyre-fixme[2]: Parameter must be annotated.
+        mock_fit,
+        # pyre-fixme[2]: Parameter must be annotated.
+        mock_observations_from_data,
     ):
         t = Derelativize(
-            search_space=None, observation_features=None, observation_data=None
+            search_space=None,
+            observations=[],
         )
 
         # ModelBridge with in-design status quo
@@ -159,7 +167,7 @@ class DerelativizeTransformTest(TestCase):
         self.assertTrue(obsf == obsf2)
 
         # Test with relative constraint, out-of-design status quo
-        mock_predict.side_effect = Exception()
+        mock_predict.side_effect = RuntimeError()
         g = ModelBridge(
             search_space=search_space,
             model=None,
@@ -227,14 +235,14 @@ class DerelativizeTransformTest(TestCase):
                 ),
             ],
         )
-        with self.assertRaises(Exception):
+        with self.assertRaises(RuntimeError):
+            #  `None`.
             oc = t.transform_optimization_config(oc, g, None)
 
         # Bypasses error if use_raw_sq
         t2 = Derelativize(
             search_space=None,
-            observation_features=None,
-            observation_data=None,
+            observations=[],
             config={"use_raw_status_quo": True},
         )
         oc2 = t2.transform_optimization_config(deepcopy(oc), g, None)
@@ -248,15 +256,18 @@ class DerelativizeTransformTest(TestCase):
             data=Data(),
         )
         with self.assertRaises(ValueError):
+            #  `None`.
             oc = t.transform_optimization_config(oc, g, None)
 
         # Raises error with relative constraint, no modelbridge
         with self.assertRaises(ValueError):
+            #  `None`.
             oc = t.transform_optimization_config(oc, None, None)
 
-    def testErrors(self):
+    def testErrors(self) -> None:
         t = Derelativize(
-            search_space=None, observation_features=None, observation_data=None
+            search_space=None,
+            observations=[],
         )
         oc = OptimizationConfig(
             objective=Objective(Metric("c")),
@@ -269,6 +280,8 @@ class DerelativizeTransformTest(TestCase):
         )
         g = ModelBridge(search_space, None, [])
         with self.assertRaises(ValueError):
+            #  `None`.
             t.transform_optimization_config(oc, None, None)
         with self.assertRaises(ValueError):
+            #  `None`.
             t.transform_optimization_config(oc, g, None)
