@@ -1,11 +1,16 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import pandas as pd
 import torch
+from ax.core.data import Data
+from ax.core.metric import Metric
+from ax.core.objective import MultiObjective, Objective
 from ax.core.observation import ObservationFeatures
+from ax.core.optimization_config import MultiObjectiveOptimizationConfig
 from ax.core.outcome_constraint import ComparisonOp, ObjectiveThreshold
 from ax.core.parameter import RangeParameter
 from ax.modelbridge.discrete import DiscreteModelBridge
@@ -18,34 +23,56 @@ from ax.modelbridge.factory import (
     get_GPMES,
     get_MOO_EHVI,
     get_MOO_NEHVI,
-    get_MTGP_NEHVI,
     get_MOO_PAREGO,
-    get_MTGP_PAREGO,
     get_MOO_RS,
     get_MTGP,
+    get_MTGP_NEHVI,
+    get_MTGP_PAREGO,
     get_sobol,
     get_thompson,
     get_uniform,
 )
-from ax.modelbridge.multi_objective_torch import MultiObjectiveTorchModelBridge
 from ax.modelbridge.random import RandomModelBridge
 from ax.modelbridge.torch import TorchModelBridge
+from ax.modelbridge.transforms.winsorize import WinsorizationConfig
 from ax.models.discrete.eb_thompson import EmpiricalBayesThompsonSampler
 from ax.models.discrete.thompson import ThompsonSampler
 from ax.utils.common.testutils import TestCase
 from ax.utils.testing.core_stubs import (
     get_branin_experiment,
     get_branin_experiment_with_multi_objective,
-    get_multi_type_experiment_with_multi_objective,
     get_branin_optimization_config,
     get_factorial_experiment,
     get_multi_type_experiment,
+    get_multi_type_experiment_with_multi_objective,
 )
+from ax.utils.testing.mock import fast_botorch_optimize
 from botorch.models.multitask import MultiTaskGP
 
 
-class ModelBridgeFactoryTest(TestCase):
-    def test_sobol_GPEI(self):
+# pyre-fixme[3]: Return type must be annotated.
+def get_multi_obj_exp_and_opt_config():
+    multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
+    # pyre-fixme[16]: Optional type has no attribute `objective`.
+    metrics = multi_obj_exp.optimization_config.objective.metrics
+    multi_objective_thresholds = [
+        ObjectiveThreshold(
+            metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
+        ),
+        ObjectiveThreshold(
+            metric=metrics[1], bound=0.0, relative=False, op=ComparisonOp.GEQ
+        ),
+    ]
+    # pyre-fixme[16]: Optional type has no attribute `clone_with_args`.
+    optimization_config = multi_obj_exp.optimization_config.clone_with_args(
+        objective_thresholds=multi_objective_thresholds
+    )
+    return multi_obj_exp, optimization_config
+
+
+class ModelBridgeFactoryTestSingleObjective(TestCase):
+    @fast_botorch_optimize
+    def test_sobol_GPEI(self) -> None:
         """Tests sobol + GPEI instantiation."""
         exp = get_branin_experiment()
         # Check that factory generates a valid sobol modelbridge.
@@ -65,7 +92,12 @@ class ModelBridgeFactoryTest(TestCase):
         botorch = get_botorch(experiment=exp, data=exp.fetch_data())
         self.assertIsInstance(botorch, TorchModelBridge)
 
-    def test_MTGP(self):
+        # Check that .gen returns without failure
+        gpei_run = gpei.gen(n=1)
+        self.assertEqual(len(gpei_run.arms), 1)
+
+    @fast_botorch_optimize
+    def test_MTGP(self) -> None:
         """Tests MTGP instantiation."""
         # Test Multi-type MTGP
         exp = get_multi_type_experiment(add_trials=True)
@@ -84,6 +116,9 @@ class ModelBridgeFactoryTest(TestCase):
             t.run().mark_completed()
         mtgp = get_MTGP(experiment=exp, data=exp.fetch_data(), trial_index=0)
         self.assertIsInstance(mtgp, TorchModelBridge)
+        # mtgp_run = mtgp.gen(
+        #     n=1
+        # )  # TODO[T110948251]: This is broken at the ChoiceEncode level
 
         with self.assertRaises(ValueError):
             get_MTGP(experiment=exp, data=exp.fetch_data(), trial_index=9)
@@ -98,7 +133,8 @@ class ModelBridgeFactoryTest(TestCase):
         with self.assertRaises(ValueError):
             get_MTGP(experiment=exp, data=exp.fetch_data(), trial_index=0)
 
-    def test_GPKG(self):
+    @fast_botorch_optimize
+    def test_GPKG(self) -> None:
         """Tests GPKG instantiation."""
         exp = get_branin_experiment(with_batch=True)
         with self.assertRaises(ValueError):
@@ -107,12 +143,27 @@ class ModelBridgeFactoryTest(TestCase):
         gpkg = get_GPKG(experiment=exp, data=exp.fetch_data())
         self.assertIsInstance(gpkg, TorchModelBridge)
 
+        # Check that .gen returns without failure
+        gr = gpkg.gen(n=1)
+        self.assertEqual(len(gr.arms), 1)
+
         # test transform_configs with winsorization
         configs = {
-            "Winsorize": {"winsorization_lower": 0.1, "winsorization_upper": 0.1}
+            "Winsorize": {
+                "winsorization_config": WinsorizationConfig(
+                    lower_quantile_margin=0.1,
+                    upper_quantile_margin=0.1,
+                )
+            }
         }
         gpkg_win = get_GPKG(
-            experiment=exp, data=exp.fetch_data(), transform_configs=configs
+            experiment=exp,
+            data=exp.fetch_data(),
+            # pyre-fixme[6]: For 3rd param expected `Optional[Dict[str, Dict[str,
+            #  Union[None, Dict[str, typing.Any], OptimizationConfig,
+            #  AcquisitionFunction, float, int, str]]]]` but got `Dict[str, Dict[str,
+            #  WinsorizationConfig]]`.
+            transform_configs=configs,
         )
         self.assertIsInstance(gpkg_win, TorchModelBridge)
         self.assertEqual(gpkg_win._transform_configs, configs)
@@ -129,7 +180,8 @@ class ModelBridgeFactoryTest(TestCase):
         gpkg_mf = get_GPKG(experiment=exp, data=exp.fetch_data())
         self.assertIsInstance(gpkg_mf, TorchModelBridge)
 
-    def test_GPMES(self):
+    @fast_botorch_optimize
+    def test_GPMES(self) -> None:
         """Tests GPMES instantiation."""
         exp = get_branin_experiment(with_batch=True)
         with self.assertRaises(ValueError):
@@ -138,12 +190,27 @@ class ModelBridgeFactoryTest(TestCase):
         gpmes = get_GPMES(experiment=exp, data=exp.fetch_data())
         self.assertIsInstance(gpmes, TorchModelBridge)
 
+        # Check that .gen returns without failure
+        gr = gpmes.gen(n=1)
+        self.assertEqual(len(gr.arms), 1)
+
         # test transform_configs with winsorization
         configs = {
-            "Winsorize": {"winsorization_lower": 0.1, "winsorization_upper": 0.1}
+            "Winsorize": {
+                "winsorization_config": WinsorizationConfig(
+                    lower_quantile_margin=0.1,
+                    upper_quantile_margin=0.1,
+                )
+            }
         }
         gpmes_win = get_GPMES(
-            experiment=exp, data=exp.fetch_data(), transform_configs=configs
+            experiment=exp,
+            data=exp.fetch_data(),
+            # pyre-fixme[6]: For 3rd param expected `Optional[Dict[str, Dict[str,
+            #  Union[None, Dict[str, typing.Any], OptimizationConfig,
+            #  AcquisitionFunction, float, int, str]]]]` but got `Dict[str, Dict[str,
+            #  WinsorizationConfig]]`.
+            transform_configs=configs,
         )
         self.assertIsInstance(gpmes_win, TorchModelBridge)
         self.assertEqual(gpmes_win._transform_configs, configs)
@@ -160,7 +227,7 @@ class ModelBridgeFactoryTest(TestCase):
         gpmes_mf = get_GPMES(experiment=exp, data=exp.fetch_data())
         self.assertIsInstance(gpmes_mf, TorchModelBridge)
 
-    def test_model_kwargs(self):
+    def test_model_kwargs(self) -> None:
         """Tests that model kwargs are passed correctly."""
         exp = get_branin_experiment()
         sobol = get_sobol(
@@ -171,9 +238,10 @@ class ModelBridgeFactoryTest(TestCase):
             sobol_run = sobol.gen(1)
             exp.new_batch_trial().add_generator_run(sobol_run).run().mark_completed()
         with self.assertRaises(TypeError):
+            # pyre-fixme[28]: Unexpected keyword argument `nonexistent`.
             get_sobol(search_space=exp.search_space, nonexistent=True)
 
-    def test_factorial(self):
+    def test_factorial(self) -> None:
         """Tests factorial instantiation."""
         exp = get_factorial_experiment()
         factorial = get_factorial(exp.search_space)
@@ -181,7 +249,7 @@ class ModelBridgeFactoryTest(TestCase):
         factorial_run = factorial.gen(n=-1)
         self.assertEqual(len(factorial_run.arms), 24)
 
-    def test_empirical_bayes_thompson(self):
+    def test_empirical_bayes_thompson(self) -> None:
         """Tests EB/TS instantiation."""
         exp = get_factorial_experiment()
         factorial = get_factorial(exp.search_space)
@@ -197,7 +265,7 @@ class ModelBridgeFactoryTest(TestCase):
         thompson_run = eb_thompson.gen(n=5)
         self.assertEqual(len(thompson_run.arms), 5)
 
-    def test_thompson(self):
+    def test_thompson(self) -> None:
         """Tests TS instantiation."""
         exp = get_factorial_experiment()
         factorial = get_factorial(exp.search_space)
@@ -208,25 +276,41 @@ class ModelBridgeFactoryTest(TestCase):
         thompson = get_thompson(experiment=exp, data=data)
         self.assertIsInstance(thompson.model, ThompsonSampler)
 
-    def test_uniform(self):
+    def test_uniform(self) -> None:
         exp = get_branin_experiment()
         uniform = get_uniform(exp.search_space)
         self.assertIsInstance(uniform, RandomModelBridge)
         uniform_run = uniform.gen(n=5)
         self.assertEqual(len(uniform_run.arms), 5)
 
-    def test_MOO_RS(self):
+
+class ModelBridgeFactoryTestMultiObjective(TestCase):
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
+    def test_single_objective_error(self, factory_fn=get_MOO_RS):
         single_obj_exp = get_branin_experiment(with_batch=True)
         with self.assertRaises(ValueError):
-            get_MOO_RS(experiment=single_obj_exp, data=single_obj_exp.fetch_data())
+            factory_fn(
+                experiment=single_obj_exp,
+                data=single_obj_exp.fetch_data(),
+            )
 
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
+    def test_data_error_and_get_multi_obj_exp(self, factory_fn=get_MOO_RS):
         multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
         with self.assertRaises(ValueError):
-            get_MOO_RS(experiment=multi_obj_exp, data=multi_obj_exp.fetch_data())
+            factory_fn(experiment=multi_obj_exp, data=multi_obj_exp.fetch_data())
 
         multi_obj_exp.trials[0].run().mark_completed()
+        return multi_obj_exp
+
+    @fast_botorch_optimize
+    def test_MOO_RS(self) -> None:
+        self.test_single_objective_error(get_MOO_RS)
+        multi_obj_exp = self.test_data_error_and_get_multi_obj_exp(get_MOO_RS)
         moo_rs = get_MOO_RS(experiment=multi_obj_exp, data=multi_obj_exp.fetch_data())
-        self.assertIsInstance(moo_rs, MultiObjectiveTorchModelBridge)
+        self.assertIsInstance(moo_rs, TorchModelBridge)
         self.assertEqual(
             {
                 "acquisition_function_kwargs": {
@@ -239,20 +323,14 @@ class ModelBridgeFactoryTest(TestCase):
         moo_rs_run = moo_rs.gen(n=2)
         self.assertEqual(len(moo_rs_run.arms), 2)
 
-    def test_MOO_PAREGO(self):
-        single_obj_exp = get_branin_experiment(with_batch=True)
-        with self.assertRaises(ValueError):
-            get_MOO_PAREGO(experiment=single_obj_exp, data=single_obj_exp.fetch_data())
-
-        multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
-        with self.assertRaises(ValueError):
-            get_MOO_PAREGO(experiment=multi_obj_exp, data=multi_obj_exp.fetch_data())
-
-        multi_obj_exp.trials[0].run().mark_completed()
+    @fast_botorch_optimize
+    def test_MOO_PAREGO(self) -> None:
+        self.test_single_objective_error(get_MOO_PAREGO)
+        multi_obj_exp = self.test_data_error_and_get_multi_obj_exp(get_MOO_PAREGO)
         moo_parego = get_MOO_PAREGO(
             experiment=multi_obj_exp, data=multi_obj_exp.fetch_data()
         )
-        self.assertIsInstance(moo_parego, MultiObjectiveTorchModelBridge)
+        self.assertIsInstance(moo_parego, TorchModelBridge)
         self.assertEqual(
             {
                 "acquisition_function_kwargs": {
@@ -265,80 +343,38 @@ class ModelBridgeFactoryTest(TestCase):
         moo_parego_run = moo_parego.gen(n=2)
         self.assertEqual(len(moo_parego_run.arms), 2)
 
-    def test_MOO_EHVI(self):
-        single_obj_exp = get_branin_experiment(with_batch=True)
-        metrics = single_obj_exp.optimization_config.objective.metrics
-        objective_thresholds = [
-            ObjectiveThreshold(
-                metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            )
-        ]
-        # ValueError: Multi-objective optimization requires multiple objectives.
-        with self.assertRaises(ValueError):
-            get_MOO_EHVI(
-                experiment=single_obj_exp,
-                data=single_obj_exp.fetch_data(),
-                objective_thresholds=objective_thresholds,
-            )
-        multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
-        metrics = multi_obj_exp.optimization_config.objective.metrics
-        multi_objective_thresholds = [
-            ObjectiveThreshold(
-                metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-            ObjectiveThreshold(
-                metric=metrics[1], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-        ]
+    @fast_botorch_optimize
+    def test_MOO_EHVI(self) -> None:
+        self.test_single_objective_error(get_MOO_EHVI)
+        multi_obj_exp, optimization_config = get_multi_obj_exp_and_opt_config()
         # ValueError: MultiObjectiveOptimization requires non-empty data.
         with self.assertRaises(ValueError):
             get_MOO_EHVI(
                 experiment=multi_obj_exp,
                 data=multi_obj_exp.fetch_data(),
-                objective_thresholds=multi_objective_thresholds,
+                optimization_config=optimization_config,
             )
 
         multi_obj_exp.trials[0].run().mark_completed()
         moo_ehvi = get_MOO_EHVI(
             experiment=multi_obj_exp,
             data=multi_obj_exp.fetch_data(),
-            objective_thresholds=multi_objective_thresholds,
+            optimization_config=optimization_config,
         )
-        self.assertIsInstance(moo_ehvi, MultiObjectiveTorchModelBridge)
+        self.assertIsInstance(moo_ehvi, TorchModelBridge)
         moo_ehvi_run = moo_ehvi.gen(n=1)
         self.assertEqual(len(moo_ehvi_run.arms), 1)
 
-    def test_MTGP_PAREGO(self):
+    @fast_botorch_optimize
+    def test_MTGP_PAREGO(self) -> None:
         """Tests MTGP ParEGO instantiation."""
-        # Test Multi-type MTGP
-        single_obj_exp = get_branin_experiment(with_batch=True)
-        metrics = single_obj_exp.optimization_config.objective.metrics
-        metrics[0].lower_is_better = True
-        objective_thresholds = [
-            ObjectiveThreshold(metric=metrics[0], bound=0.0, relative=False)
-        ]
-        with self.assertRaises(ValueError):
-            get_MTGP_PAREGO(
-                experiment=single_obj_exp,
-                data=single_obj_exp.fetch_data(),
-                objective_thresholds=objective_thresholds,
-            )
-
-        multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
-        metrics = multi_obj_exp.optimization_config.objective.metrics
-        multi_objective_thresholds = [
-            ObjectiveThreshold(
-                metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-            ObjectiveThreshold(
-                metric=metrics[1], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-        ]
+        self.test_single_objective_error(get_MTGP_PAREGO)
+        multi_obj_exp, optimization_config = get_multi_obj_exp_and_opt_config()
         with self.assertRaises(ValueError):
             get_MTGP_PAREGO(
                 experiment=multi_obj_exp,
                 data=multi_obj_exp.fetch_data(),
-                objective_thresholds=multi_objective_thresholds,
+                optimization_config=optimization_config,
             )
 
         multi_obj_exp.trials[0].run()
@@ -351,15 +387,18 @@ class ModelBridgeFactoryTest(TestCase):
         mt_ehvi = get_MTGP_PAREGO(
             experiment=multi_obj_exp,
             data=multi_obj_exp.fetch_data(),
-            objective_thresholds=multi_objective_thresholds,
             trial_index=1,
+            optimization_config=optimization_config,
         )
-        self.assertIsInstance(mt_ehvi, MultiObjectiveTorchModelBridge)
+        self.assertIsInstance(mt_ehvi, TorchModelBridge)
+        # pyre-fixme[16]: Optional type has no attribute `model`.
         self.assertIsInstance(mt_ehvi.model.model.models[0], MultiTaskGP)
         task_covar_factor = mt_ehvi.model.model.models[0].task_covar_module.covar_factor
         self.assertEqual(task_covar_factor.shape, torch.Size([2, 2]))
         mt_ehvi_run = mt_ehvi.gen(
-            n=1, fixed_features=ObservationFeatures(parameters={}, trial_index=1)
+            n=1,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
+            fixed_features=ObservationFeatures(parameters={}, trial_index=1),
         )
         self.assertEqual(len(mt_ehvi_run.arms), 1)
 
@@ -368,8 +407,8 @@ class ModelBridgeFactoryTest(TestCase):
             get_MTGP_PAREGO(
                 experiment=multi_obj_exp,
                 data=multi_obj_exp.fetch_data(),
-                objective_thresholds=multi_objective_thresholds,
                 trial_index=999,
+                optimization_config=optimization_config,
             )
 
         # Multi-type + multi-objective experiment
@@ -380,79 +419,103 @@ class ModelBridgeFactoryTest(TestCase):
         mt_ehvi = get_MTGP_PAREGO(
             experiment=multi_type_multi_obj_exp,
             data=data,
-            objective_thresholds=multi_objective_thresholds,
+            optimization_config=optimization_config,
         )
 
-    def test_MOO_NEHVI(self):
-        single_obj_exp = get_branin_experiment(with_batch=True)
-        metrics = single_obj_exp.optimization_config.objective.metrics
-        metrics[0].lower_is_better = True
-        objective_thresholds = [
-            ObjectiveThreshold(metric=metrics[0], bound=0.0, relative=False)
-        ]
-        with self.assertRaises(ValueError):
-            get_MOO_NEHVI(
-                experiment=single_obj_exp,
-                data=single_obj_exp.fetch_data(),
-                objective_thresholds=objective_thresholds,
-            )
-
-        multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
-        metrics = multi_obj_exp.optimization_config.objective.metrics
-        multi_objective_thresholds = [
-            ObjectiveThreshold(
-                metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-            ObjectiveThreshold(
-                metric=metrics[1], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-        ]
+    @fast_botorch_optimize
+    def test_MOO_NEHVI(self) -> None:
+        self.test_single_objective_error(get_MOO_NEHVI)
+        multi_obj_exp, optimization_config = get_multi_obj_exp_and_opt_config()
         with self.assertRaises(ValueError):
             get_MOO_NEHVI(
                 experiment=multi_obj_exp,
                 data=multi_obj_exp.fetch_data(),
-                objective_thresholds=multi_objective_thresholds,
+                optimization_config=optimization_config,
             )
 
         multi_obj_exp.trials[0].run()
         moo_ehvi = get_MOO_NEHVI(
             experiment=multi_obj_exp,
             data=multi_obj_exp.fetch_data(),
-            objective_thresholds=multi_objective_thresholds,
+            optimization_config=optimization_config,
         )
-        self.assertIsInstance(moo_ehvi, MultiObjectiveTorchModelBridge)
+        self.assertIsInstance(moo_ehvi, TorchModelBridge)
         moo_ehvi_run = moo_ehvi.gen(n=1)
         self.assertEqual(len(moo_ehvi_run.arms), 1)
 
-    def test_MTGP_NEHVI(self):
-        single_obj_exp = get_branin_experiment(with_batch=True)
-        metrics = single_obj_exp.optimization_config.objective.metrics
-        metrics[0].lower_is_better = True
+    @fast_botorch_optimize
+    def test_MOO_with_more_outcomes_than_thresholds(self) -> None:
+        experiment = get_branin_experiment_with_multi_objective(
+            has_optimization_config=False
+        )
+        metric_c = Metric(name="c", lower_is_better=False)
+        metric_a = Metric(name="a", lower_is_better=False)
         objective_thresholds = [
-            ObjectiveThreshold(metric=metrics[0], bound=0.0, relative=False)
+            ObjectiveThreshold(
+                metric=metric_c,
+                bound=2.0,
+                relative=False,
+            ),
+            ObjectiveThreshold(
+                metric=metric_a,
+                bound=1.0,
+                relative=False,
+            ),
         ]
-        with self.assertRaises(ValueError):
-            get_MTGP_NEHVI(
-                experiment=single_obj_exp,
-                data=single_obj_exp.fetch_data(),
-                objective_thresholds=objective_thresholds,
+        experiment.optimization_config = MultiObjectiveOptimizationConfig(
+            objective=MultiObjective(
+                objectives=[
+                    Objective(metric=metric_a),
+                    Objective(metric=metric_c),
+                ]
+            ),
+            objective_thresholds=objective_thresholds,
+        )
+        experiment.add_tracking_metric(Metric(name="b", lower_is_better=False))
+        sobol = get_sobol(
+            search_space=experiment.search_space,
+        )
+        sobol_run = sobol.gen(1)
+        experiment.new_batch_trial().add_generator_run(sobol_run).run().mark_completed()
+        data = Data(
+            pd.DataFrame(
+                data={
+                    "arm_name": ["0_0", "0_0", "0_0"],
+                    "metric_name": ["a", "b", "c"],
+                    "mean": [1.0, 2.0, 3.0],
+                    "trial_index": [0, 0, 0],
+                    "sem": [0, 0, 0],
+                }
             )
+        )
+        test_names_to_fns = {
+            "MOO_NEHVI": get_MOO_NEHVI,
+            "MOO_EHVI": get_MOO_NEHVI,
+            "MOO_PAREGO": get_MOO_PAREGO,
+            "MOO_RS": get_MOO_RS,
+        }
+        for test_name, factory_fn in test_names_to_fns.items():
+            with self.subTest(test_name):
+                moo_model = factory_fn(
+                    experiment=experiment,
+                    data=data,
+                )
+                moo_gr = moo_model.gen(n=1)
+                # pyre-fixme[16]: Optional type has no attribute `__getitem__`.
+                obj_t = moo_gr.gen_metadata["objective_thresholds"]
+                self.assertEqual(obj_t[0], objective_thresholds[1])
+                self.assertEqual(obj_t[1], objective_thresholds[0])
+                self.assertEqual(len(obj_t), 2)
 
-        multi_obj_exp = get_branin_experiment_with_multi_objective(with_batch=True)
-        metrics = multi_obj_exp.optimization_config.objective.metrics
-        multi_objective_thresholds = [
-            ObjectiveThreshold(
-                metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-            ObjectiveThreshold(
-                metric=metrics[1], bound=0.0, relative=False, op=ComparisonOp.GEQ
-            ),
-        ]
+    @fast_botorch_optimize
+    def test_MTGP_NEHVI(self) -> None:
+        self.test_single_objective_error(get_MTGP_NEHVI)
+        multi_obj_exp, optimization_config = get_multi_obj_exp_and_opt_config()
         with self.assertRaises(ValueError):
             get_MTGP_NEHVI(
                 experiment=multi_obj_exp,
                 data=multi_obj_exp.fetch_data(),
-                objective_thresholds=multi_objective_thresholds,
+                optimization_config=optimization_config,
             )
 
         multi_obj_exp.trials[0].run()
@@ -465,15 +528,18 @@ class ModelBridgeFactoryTest(TestCase):
         mt_ehvi = get_MTGP_NEHVI(
             experiment=multi_obj_exp,
             data=multi_obj_exp.fetch_data(),
-            objective_thresholds=multi_objective_thresholds,
             trial_index=1,
+            optimization_config=optimization_config,
         )
-        self.assertIsInstance(mt_ehvi, MultiObjectiveTorchModelBridge)
+        self.assertIsInstance(mt_ehvi, TorchModelBridge)
+        # pyre-fixme[16]: Optional type has no attribute `model`.
         self.assertIsInstance(mt_ehvi.model.model.models[0], MultiTaskGP)
         task_covar_factor = mt_ehvi.model.model.models[0].task_covar_module.covar_factor
         self.assertEqual(task_covar_factor.shape, torch.Size([2, 2]))
         mt_ehvi_run = mt_ehvi.gen(
-            n=1, fixed_features=ObservationFeatures(parameters={}, trial_index=1)
+            n=1,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
+            fixed_features=ObservationFeatures(parameters={}, trial_index=1),
         )
         self.assertEqual(len(mt_ehvi_run.arms), 1)
 
@@ -482,8 +548,8 @@ class ModelBridgeFactoryTest(TestCase):
             get_MTGP_NEHVI(
                 experiment=multi_obj_exp,
                 data=multi_obj_exp.fetch_data(),
-                objective_thresholds=multi_objective_thresholds,
                 trial_index=999,
+                optimization_config=optimization_config,
             )
 
         # Multi-type + multi-objective experiment
@@ -494,5 +560,5 @@ class ModelBridgeFactoryTest(TestCase):
         mt_ehvi = get_MTGP_NEHVI(
             experiment=multi_type_multi_obj_exp,
             data=data,
-            objective_thresholds=multi_objective_thresholds,
+            optimization_config=optimization_config,
         )
