@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,38 +11,44 @@ import pandas as pd
 import torch
 from ax.core.data import Data
 from ax.core.observation import ObservationFeatures
+from ax.core.outcome_constraint import ObjectiveThreshold
+from ax.core.types import ComparisonOp
 from ax.modelbridge.discrete import DiscreteModelBridge
 from ax.modelbridge.random import RandomModelBridge
 from ax.modelbridge.registry import (
-    MODEL_KEY_TO_MODEL_SETUP,
     Cont_X_trans,
+    get_model_from_generator_run,
+    MODEL_KEY_TO_MODEL_SETUP,
     Models,
     Y_trans,
-    get_model_from_generator_run,
 )
 from ax.modelbridge.torch import TorchModelBridge
 from ax.models.base import Model
 from ax.models.discrete.eb_thompson import EmpiricalBayesThompsonSampler
 from ax.models.discrete.thompson import ThompsonSampler
+from ax.models.random.alebo_initializer import ALEBOInitializer
 from ax.models.torch.alebo import ALEBO
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 from ax.models.torch.botorch_modular.model import BoTorchModel
 from ax.models.torch.botorch_modular.surrogate import Surrogate
+from ax.models.torch.botorch_moo import MultiObjectiveBotorchModel
 from ax.utils.common.kwargs import get_function_argument_names
 from ax.utils.common.testutils import TestCase
 from ax.utils.testing.core_stubs import (
     get_branin_data,
     get_branin_experiment,
+    get_branin_experiment_with_multi_objective,
     get_branin_optimization_config,
     get_factorial_experiment,
 )
+from ax.utils.testing.mock import fast_botorch_optimize
 from botorch.acquisition.monte_carlo import qExpectedImprovement
 from botorch.models.gp_regression import FixedNoiseGP
-from torch import float64 as torch_float64
 
 
 class ModelRegistryTest(TestCase):
-    def test_botorch_modular(self):
+    @fast_botorch_optimize
+    def test_botorch_modular(self) -> None:
         exp = get_branin_experiment(with_batch=True)
         exp.trials[0].run()
         gpei = Models.BOTORCH_MODULAR(
@@ -63,7 +69,11 @@ class ModelRegistryTest(TestCase):
         # FixedNoiseGP should be picked since experiment data has fixed noise.
         self.assertIsInstance(gpei.model.surrogate.model, FixedNoiseGP)
 
-    def test_enum_sobol_GPEI(self):
+        gr = gpei.gen(n=1)
+        self.assertIsNotNone(gr.best_arm_predictions)
+
+    @fast_botorch_optimize
+    def test_enum_sobol_GPEI(self) -> None:
         """Tests Sobol and GPEI instantiation through the Models enum."""
         exp = get_branin_experiment()
         # Check that factory generates a valid sobol modelbridge.
@@ -114,13 +124,14 @@ class ModelRegistryTest(TestCase):
             gpei._bridge_kwargs,
             {
                 "transform_configs": None,
-                "torch_dtype": torch_float64,
+                "torch_dtype": torch.float64,
                 "torch_device": None,
                 "status_quo_name": None,
                 "status_quo_features": None,
                 "optimization_config": None,
                 "transforms": Cont_X_trans + Y_trans,
                 "fit_out_of_design": False,
+                "fit_abandoned": False,
                 "default_model_gen_options": None,
             },
         )
@@ -129,7 +140,7 @@ class ModelRegistryTest(TestCase):
         )
         self.assertIsInstance(gpei, TorchModelBridge)
 
-    def test_enum_model_kwargs(self):
+    def test_enum_model_kwargs(self) -> None:
         """Tests that kwargs are passed correctly when instantiating through the
         Models enum."""
         exp = get_branin_experiment()
@@ -141,7 +152,7 @@ class ModelRegistryTest(TestCase):
             sobol_run = sobol.gen(1)
             exp.new_batch_trial().add_generator_run(sobol_run).run()
 
-    def test_enum_factorial(self):
+    def test_enum_factorial(self) -> None:
         """Tests factorial instantiation through the Models enum."""
         exp = get_factorial_experiment()
         factorial = Models.FACTORIAL(exp.search_space)
@@ -149,7 +160,7 @@ class ModelRegistryTest(TestCase):
         factorial_run = factorial.gen(n=-1)
         self.assertEqual(len(factorial_run.arms), 24)
 
-    def test_enum_empirical_bayes_thompson(self):
+    def test_enum_empirical_bayes_thompson(self) -> None:
         """Tests EB/TS instantiation through the Models enum."""
         exp = get_factorial_experiment()
         factorial = Models.FACTORIAL(exp.search_space)
@@ -165,7 +176,7 @@ class ModelRegistryTest(TestCase):
         thompson_run = eb_thompson.gen(n=5)
         self.assertEqual(len(thompson_run.arms), 5)
 
-    def test_enum_thompson(self):
+    def test_enum_thompson(self) -> None:
         """Tests TS instantiation through the Models enum."""
         exp = get_factorial_experiment()
         factorial = Models.FACTORIAL(exp.search_space)
@@ -176,7 +187,7 @@ class ModelRegistryTest(TestCase):
         thompson = Models.THOMPSON(experiment=exp, data=data)
         self.assertIsInstance(thompson.model, ThompsonSampler)
 
-    def test_enum_uniform(self):
+    def test_enum_uniform(self) -> None:
         """Tests uniform random instantiation through the Models enum."""
         exp = get_branin_experiment()
         uniform = Models.UNIFORM(exp.search_space)
@@ -184,7 +195,7 @@ class ModelRegistryTest(TestCase):
         uniform_run = uniform.gen(n=5)
         self.assertEqual(len(uniform_run.arms), 5)
 
-    def test_view_defaults(self):
+    def test_view_defaults(self) -> None:
         """Checks that kwargs are correctly constructed from default kwargs +
         standard kwargs."""
         self.assertEqual(
@@ -196,6 +207,7 @@ class ModelRegistryTest(TestCase):
                     "init_position": 0,
                     "scramble": True,
                     "generated_points": None,
+                    "fallback_to_sample_polytope": False,
                 },
                 {
                     "optimization_config": None,
@@ -230,7 +242,8 @@ class ModelRegistryTest(TestCase):
             ),
         )
 
-    def test_get_model_from_generator_run(self):
+    @fast_botorch_optimize
+    def test_get_model_from_generator_run(self) -> None:
         """Tests that it is possible to restore a model from a generator run it
         produced, if `Models` registry was used.
         """
@@ -239,13 +252,20 @@ class ModelRegistryTest(TestCase):
         gr = initial_sobol.gen(n=1)
         # Restore the model as it was before generation.
         sobol = get_model_from_generator_run(
-            generator_run=gr, experiment=exp, data=exp.fetch_data(), after_gen=False
+            generator_run=gr,
+            experiment=exp,
+            data=exp.fetch_data(),
+            models_enum=Models,
+            after_gen=False,
         )
         self.assertEqual(sobol.model.init_position, 0)
         self.assertEqual(sobol.model.seed, 239)
         # Restore the model as it was after generation (to resume generation).
         sobol_after_gen = get_model_from_generator_run(
-            generator_run=gr, experiment=exp, data=exp.fetch_data()
+            generator_run=gr,
+            experiment=exp,
+            data=exp.fetch_data(),
+            models_enum=Models,
         )
         self.assertEqual(sobol_after_gen.model.init_position, 1)
         self.assertEqual(sobol_after_gen.model.seed, 239)
@@ -260,7 +280,7 @@ class ModelRegistryTest(TestCase):
         gr._bridge_kwargs = gpei._bridge_kwargs
         gr._model_state_after_gen = {}
         gpei_restored = get_model_from_generator_run(
-            gr, experiment=exp, data=get_branin_data()
+            gr, experiment=exp, data=get_branin_data(), models_enum=Models
         )
         for key in gpei.__dict__:
             self.assertIn(key, gpei_restored.__dict__)
@@ -286,7 +306,7 @@ class ModelRegistryTest(TestCase):
                 continue
             self.assertEqual(original, restored)
 
-    def test_ModelSetups_do_not_share_kwargs(self):
+    def test_ModelSetups_do_not_share_kwargs(self) -> None:
         """Tests that none of the preset model and bridge combinations share a
         kwarg.
         """
@@ -298,7 +318,8 @@ class ModelRegistryTest(TestCase):
             # Intersection of two sets should be empty
             self.assertEqual(model_args & bridge_args, set())
 
-    def test_ST_MTGP(self):
+    @fast_botorch_optimize
+    def test_ST_MTGP(self) -> None:
         """Tests single type MTGP instantiation."""
         # Test Single-type MTGP
         exp = get_branin_experiment()
@@ -310,7 +331,9 @@ class ModelRegistryTest(TestCase):
             t.set_status_quo_with_weight(status_quo=t.arms[0], weight=0.5)
             t.run().mark_completed()
         status_quo_features = ObservationFeatures(
+            # pyre-fixme[16]: `BaseTrial` has no attribute `status_quo`.
             parameters=exp.trials[0].status_quo.parameters,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
             trial_index=0,
         )
         mtgp = Models.ST_MTGP(
@@ -331,6 +354,7 @@ class ModelRegistryTest(TestCase):
         with self.assertRaises(ValueError):
             status_quo_features = ObservationFeatures(
                 parameters=exp.trials[0].status_quo.parameters,
+                # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
                 trial_index=0,
             )
             Models.ST_MTGP(
@@ -339,7 +363,8 @@ class ModelRegistryTest(TestCase):
                 status_quo_features=status_quo_features,
             )
 
-    def test_ALEBO(self):
+    @fast_botorch_optimize
+    def test_ALEBO(self) -> None:
         """Tests Alebo fitting and generations"""
         experiment = get_branin_experiment(with_batch=True)
         B = np.array([[1.0, 2.0]])
@@ -357,8 +382,79 @@ class ModelRegistryTest(TestCase):
             experiment=experiment,
             search_space=None,
             data=data,
-            B=torch.from_numpy(B).float(),
+            B=torch.from_numpy(B).double(),
         )
         self.assertIsInstance(m, TorchModelBridge)
         self.assertIsInstance(m.model, ALEBO)
         self.assertTrue(np.array_equal(m.model.B.numpy(), B))
+
+    def test_ALEBO_Initializer(self) -> None:
+        """Tests Alebo Initializer generations"""
+        experiment = get_branin_experiment(with_batch=True)
+        B = np.array([[1.0, 2.0]])
+        m = Models.ALEBO_INITIALIZER(
+            experiment=experiment,
+            search_space=None,
+            B=B,
+        )
+        self.assertIsInstance(m, RandomModelBridge)
+        self.assertIsInstance(m.model, ALEBOInitializer)
+
+        gr = m.gen(n=2)
+        self.assertEqual(len(gr.arms), 2)
+
+    @fast_botorch_optimize
+    def test_ST_MTGP_NEHVI(self) -> None:
+        """Tests single type MTGP NEHVI instantiation."""
+        multi_obj_exp = get_branin_experiment_with_multi_objective(
+            with_batch=True,
+            with_status_quo=True,
+        )
+        # pyre-fixme[16]: Optional type has no attribute `objective`.
+        metrics = multi_obj_exp.optimization_config.objective.metrics
+        multi_objective_thresholds = [
+            ObjectiveThreshold(
+                metric=metrics[0], bound=0.0, relative=False, op=ComparisonOp.GEQ
+            ),
+            ObjectiveThreshold(
+                metric=metrics[1], bound=0.0, relative=False, op=ComparisonOp.GEQ
+            ),
+        ]
+        sobol = Models.SOBOL(search_space=multi_obj_exp.search_space)
+        self.assertIsInstance(sobol, RandomModelBridge)
+        for _ in range(2):
+            sobol_run = sobol.gen(n=1)
+            t = multi_obj_exp.new_batch_trial().add_generator_run(sobol_run)
+            t.set_status_quo_with_weight(status_quo=t.arms[0], weight=0.5)
+            t.run().mark_completed()
+        status_quo_features = ObservationFeatures(
+            # pyre-fixme[16]: `BaseTrial` has no attribute `status_quo`.
+            parameters=multi_obj_exp.trials[0].status_quo.parameters,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
+            trial_index=0,
+        )
+        # pyre-fixme[16]: Optional type has no attribute `clone_with_args`.
+        optimization_config = multi_obj_exp.optimization_config.clone_with_args(
+            objective_thresholds=multi_objective_thresholds
+        )
+        mtgp = Models.ST_MTGP_NEHVI(
+            experiment=multi_obj_exp,
+            data=multi_obj_exp.fetch_data(),
+            status_quo_features=status_quo_features,
+            optimization_config=optimization_config,
+        )
+        self.assertIsInstance(mtgp, TorchModelBridge)
+        self.assertIsInstance(mtgp.model, MultiObjectiveBotorchModel)
+
+        # test it can generate
+        mtgp_run = mtgp.gen(
+            n=1,
+            # pyre-fixme[6]: For 2nd param expected `Optional[int64]` but got `int`.
+            fixed_features=ObservationFeatures(parameters={}, trial_index=1),
+        )
+        self.assertEqual(len(mtgp_run.arms), 1)
+
+        # test a generated trial can be completed
+        t = multi_obj_exp.new_batch_trial().add_generator_run(mtgp_run)
+        t.set_status_quo_with_weight(status_quo=t.arms[0], weight=0.5)
+        t.run().mark_completed()
