@@ -1,13 +1,14 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 from copy import deepcopy
 
+import numpy as np
 from ax.core.arm import Arm
-from ax.core.observation import ObservationFeatures
+from ax.core.observation import Observation, ObservationData, ObservationFeatures
 from ax.core.parameter import (
     ChoiceParameter,
     FixedParameter,
@@ -15,12 +16,14 @@ from ax.core.parameter import (
     RangeParameter,
 )
 from ax.core.search_space import SearchSpace
+from ax.exceptions.core import UnsupportedError
 from ax.modelbridge.transforms.search_space_to_choice import SearchSpaceToChoice
 from ax.utils.common.testutils import TestCase
+from ax.utils.testing.core_stubs import get_robust_search_space
 
 
 class SearchSpaceToChoiceTest(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.search_space = SearchSpace(
             parameters=[
                 RangeParameter(
@@ -51,24 +54,27 @@ class SearchSpaceToChoiceTest(TestCase):
                 parameters={"arms": Arm(parameters={"a": 3, "b": "c"}).signature}
             ),
         ]
+        self.observations = [
+            Observation(
+                data=ObservationData([], np.array([]), np.empty((0, 0))), features=obsf
+            )
+            for obsf in self.observation_features
+        ]
         self.t = SearchSpaceToChoice(
             search_space=self.search_space,
-            observation_features=self.observation_features,
-            observation_data=None,
+            observations=self.observations,
         )
         self.t2 = SearchSpaceToChoice(
             search_space=self.search_space,
-            observation_features=[self.observation_features[0]],
-            observation_data=None,
+            observations=self.observations[:1],
         )
         self.t3 = SearchSpaceToChoice(
             search_space=self.search_space,
-            observation_features=self.observation_features,
-            observation_data=None,
+            observations=self.observations,
             config={"use_ordered": True},
         )
 
-    def testTransformSearchSpace(self):
+    def testTransformSearchSpace(self) -> None:
         ss2 = self.search_space.clone()
         ss2 = self.t.transform_search_space(ss2)
         self.assertEqual(len(ss2.parameters), 1)
@@ -107,11 +113,10 @@ class SearchSpaceToChoiceTest(TestCase):
         with self.assertRaises(ValueError):
             SearchSpaceToChoice(
                 search_space=ss3,
-                observation_features=self.observation_features,
-                observation_data=None,
+                observations=[],
             )
 
-    def testTransformSearchSpaceWithFixedParam(self):
+    def testTransformSearchSpaceWithFixedParam(self) -> None:
         ss2 = self.search_space.clone()
         ss2 = self.t2.transform_search_space(ss2)
         self.assertEqual(len(ss2.parameters), 1)
@@ -122,9 +127,18 @@ class SearchSpaceToChoiceTest(TestCase):
         )
         self.assertEqual(ss2.parameters.get("arms"), expected_parameter)
 
-    def testTransformObservationFeatures(self):
+    def testTransformObservationFeatures(self) -> None:
         obs_ft2 = deepcopy(self.observation_features)
         obs_ft2 = self.t.transform_observation_features(obs_ft2)
         self.assertEqual(obs_ft2, self.transformed_features)
         obs_ft2 = self.t.untransform_observation_features(obs_ft2)
         self.assertEqual(obs_ft2, self.observation_features)
+
+    def test_w_robust_search_space(self) -> None:
+        rss = get_robust_search_space()
+        # Raises an error in __init__.
+        with self.assertRaisesRegex(UnsupportedError, "transform is not supported"):
+            SearchSpaceToChoice(
+                search_space=rss,
+                observations=[],
+            )
