@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,7 +9,7 @@ from ax.utils.common.testutils import TestCase
 
 
 class TransformUtilsTest(TestCase):
-    def test_closest_lookup_dict(self):
+    def test_closest_lookup_dict(self) -> None:
         # test empty lookup
         d = ClosestLookupDict()
         with self.assertRaises(RuntimeError):
@@ -24,4 +24,5 @@ class TransformUtilsTest(TestCase):
         self.assertEqual(d[0], "a")
         self.assertEqual(d[6], "c")
         with self.assertRaises(ValueError):
+            # pyre-fixme[6]: For 1st param expected `Number` but got `str`.
             d["str_key"] = 3
