@@ -1,23 +1,30 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 import math
-from typing import List, Optional
+from logging import Logger
+from typing import List, Optional, TYPE_CHECKING
 
-from ax.core.observation import ObservationData, ObservationFeatures
+from ax.core.observation import Observation, ObservationData
 from ax.core.search_space import SearchSpace
-from ax.core.types import TConfig
+from ax.exceptions.core import DataRequiredError
 from ax.modelbridge.transforms.base import Transform
 from ax.modelbridge.transforms.utils import get_data
+from ax.models.types import TConfig
 from ax.utils.common.logger import get_logger
 from ax.utils.common.typeutils import checked_cast
 from scipy import stats
 
 
-logger = get_logger(__name__)
+if TYPE_CHECKING:
+    # import as module to make sphinx-autodoc-typehints happy
+    from ax import modelbridge as modelbridge_module  # noqa F401  # pragma: no cover
+
+
+logger: Logger = get_logger(__name__)
 
 
 # TODO(jej): Add OptimizationConfig validation - can't transform outcome constraints.
@@ -26,28 +33,29 @@ class PercentileY(Transform):
 
     def __init__(
         self,
-        search_space: SearchSpace,
-        observation_features: List[ObservationFeatures],
-        observation_data: List[ObservationData],
+        search_space: Optional[SearchSpace] = None,
+        observations: Optional[List[Observation]] = None,
+        modelbridge: Optional["modelbridge_module.base.ModelBridge"] = None,
         config: Optional[TConfig] = None,
     ) -> None:
-        if len(observation_data) == 0:
-            raise ValueError(
-                "Percentile transform requires non-empty observation data."
-            )
+        assert observations is not None, "PercentileY requires observations"
+        if len(observations) == 0:
+            raise DataRequiredError("Percentile transform requires non-empty data.")
+        observation_data = [obs.data for obs in observations]
         metric_values = get_data(observation_data=observation_data)
+        # pyre-fixme[4]: Attribute must be annotated.
         self.percentiles = {
             metric_name: vals for metric_name, vals in metric_values.items()
         }
         if config is not None and "winsorize" in config:
+            # pyre-fixme[4]: Attribute must be annotated.
             self.winsorize = checked_cast(bool, (config.get("winsorize") or False))
         else:
             self.winsorize = False
 
-    def transform_observation_data(
+    def _transform_observation_data(
         self,
         observation_data: List[ObservationData],
-        observation_features: List[ObservationFeatures],
     ) -> List[ObservationData]:
         """Map observation data to empirical CDF quantiles in place."""
         # TODO (jej): Transform covariances.
@@ -86,9 +94,5 @@ class PercentileY(Transform):
 
     def _map(self, val: float, metric_name: str) -> float:
         vals = self.percentiles[metric_name]
-        mapped_val = (
-            # pyre-fixme[16]: `scipy.stats` has no attr `percentileofscore`.
-            stats.percentileofscore(vals, val, kind="weak")
-            / 100.0
-        )
+        mapped_val = stats.percentileofscore(vals, val, kind="weak") / 100.0
         return mapped_val
