@@ -1,19 +1,24 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from typing import List, Optional
+from typing import List, Optional, TYPE_CHECKING
 
 from ax.core.arm import Arm
-from ax.core.observation import ObservationData, ObservationFeatures
+from ax.core.observation import Observation, ObservationFeatures
 from ax.core.parameter import ChoiceParameter, FixedParameter, ParameterType
-from ax.core.search_space import SearchSpace
-from ax.core.types import TConfig
+from ax.core.search_space import RobustSearchSpace, SearchSpace
+from ax.exceptions.core import UnsupportedError
 from ax.modelbridge.transforms.base import Transform
+from ax.models.types import TConfig
 from ax.utils.common.typeutils import checked_cast
 
+if TYPE_CHECKING:
+    # import as module to make sphinx-autodoc-typehints happy
+    from ax import modelbridge as modelbridge_module  # noqa F401  # pragma: no cover
+
 
 class SearchSpaceToChoice(Transform):
     """Replaces the search space with a single choice parameter, whose values
@@ -29,15 +34,16 @@ class SearchSpaceToChoice(Transform):
 
     def __init__(
         self,
-        search_space: SearchSpace,
-        observation_features: List[ObservationFeatures],
-        observation_data: List[ObservationData],
+        search_space: Optional[SearchSpace] = None,
+        observations: Optional[List[Observation]] = None,
+        modelbridge: Optional["modelbridge_module.base.ModelBridge"] = None,
         config: Optional[TConfig] = None,
     ) -> None:
+        assert search_space is not None, "SearchSpaceToChoice requires search space"
+        assert observations is not None, "SeachSpaceToChoice requires observations"
         super().__init__(
             search_space=search_space,
-            observation_features=observation_features,
-            observation_data=observation_data,
+            observations=observations,
             config=config,
         )
         if any(p.is_fidelity for p in search_space.parameters.values()):
@@ -45,13 +51,18 @@ class SearchSpaceToChoice(Transform):
                 "Cannot perform SearchSpaceToChoice conversion if fidelity "
                 "parameters are present"
             )
+        if isinstance(search_space, RobustSearchSpace):
+            raise UnsupportedError(
+                "SearchSpaceToChoice transform is not supported for RobustSearchSpace."
+            )
         self.parameter_name = "arms"
+        # pyre-fixme[4]: Attribute must be annotated.
         self.signature_to_parameterization = {
-            Arm(parameters=obsf.parameters).signature: obsf.parameters
-            for obsf in observation_features
+            Arm(parameters=obs.features.parameters).signature: obs.features.parameters
+            for obs in observations
         }
 
-    def transform_search_space(self, search_space: SearchSpace) -> SearchSpace:
+    def _transform_search_space(self, search_space: SearchSpace) -> SearchSpace:
         values = list(self.signature_to_parameterization.keys())
         if len(values) > 1:
             parameter = ChoiceParameter(
@@ -59,6 +70,7 @@ class SearchSpaceToChoice(Transform):
                 parameter_type=ParameterType.STRING,
                 values=values,
                 is_ordered=checked_cast(bool, self.config.get("use_ordered", False)),
+                sort_values=False,
             )
         else:
             parameter = FixedParameter(
