@@ -1,14 +1,16 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import dataclasses
 from itertools import chain, product
 from unittest import mock
 
 import torch
 from ax.core.search_space import SearchSpaceDigest
+from ax.exceptions.core import DataRequiredError
 from ax.models.torch.botorch import BotorchModel, get_rounding_func
 from ax.models.torch.botorch_defaults import (
     get_and_fit_model,
@@ -16,19 +18,22 @@ from ax.models.torch.botorch_defaults import (
     recommend_best_out_of_sample_point,
 )
 from ax.models.torch.utils import sample_simplex
+from ax.models.torch_base import TorchOptConfig
 from ax.utils.common.testutils import TestCase
+from ax.utils.testing.mock import fast_botorch_optimize
 from ax.utils.testing.torch_stubs import get_torch_test_data
 from botorch.acquisition.utils import get_infeasible_cost
 from botorch.models import FixedNoiseGP, ModelListGP
 from botorch.models.transforms.input import Warp
-from botorch.utils import get_objective_weights_transform
+from botorch.utils.datasets import FixedNoiseDataset
+from botorch.utils.objective import get_objective_weights_transform
 from gpytorch.likelihoods import _GaussianLikelihoodBase
 from gpytorch.mlls import ExactMarginalLogLikelihood, LeaveOneOutPseudoLikelihood
 from gpytorch.priors import GammaPrior
 from gpytorch.priors.lkj_prior import LKJCovariancePrior
 
 
-FIT_MODEL_MO_PATH = f"{get_and_fit_model.__module__}.fit_gpytorch_model"
+FIT_MODEL_MO_PATH = f"{get_and_fit_model.__module__}.fit_gpytorch_mll"
 SAMPLE_SIMPLEX_UTIL_PATH = f"{sample_simplex.__module__}.sample_simplex"
 SAMPLE_HYPERSPHERE_UTIL_PATH = f"{sample_simplex.__module__}.sample_hypersphere"
 CHEBYSHEV_SCALARIZATION_PATH = (
@@ -41,36 +46,43 @@ def dummy_func(X: torch.Tensor) -> torch.Tensor:
 
 
 class BotorchModelTest(TestCase):
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def test_fixed_rank_BotorchModel(self, dtype=torch.float, cuda=False):
-        Xs1, Ys1, Yvars1, bounds, _, fns, __package__ = get_torch_test_data(
+        Xs1, Ys1, Yvars1, bounds, _, fns, _ = get_torch_test_data(
             dtype=dtype, cuda=cuda, constant_noise=True
         )
         Xs2, Ys2, Yvars2, _, _, _, _ = get_torch_test_data(
             dtype=dtype, cuda=cuda, constant_noise=True
         )
         model = BotorchModel(multitask_gp_ranks={"y": 2, "w": 1})
+        datasets = [
+            FixedNoiseDataset(X=Xs1[0], Y=Ys1[0], Yvar=Yvars1[0]),
+            FixedNoiseDataset(X=Xs2[0], Y=Ys2[0], Yvar=Yvars2[0]),
+        ]
 
         with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
             model.fit(
-                Xs=Xs1 + Xs2,
-                Ys=Ys1 + Ys2,
-                Yvars=Yvars1 + Yvars2,
+                datasets=datasets,
+                metric_names=["y", "w"],
                 search_space_digest=SearchSpaceDigest(
                     feature_names=fns,
                     bounds=bounds,
                     task_features=[0],
                 ),
-                metric_names=["y", "w"],
             )
             _mock_fit_model.assert_called_once()
 
         # Check ranks
+        # pyre-fixme[16]: Optional type has no attribute `models`.
         model_list = model.model.models
         self.assertEqual(model_list[0]._rank, 2)
         self.assertEqual(model_list[1]._rank, 1)
 
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def test_fixed_prior_BotorchModel(self, dtype=torch.float, cuda=False):
-        Xs1, Ys1, Yvars1, bounds, _, fns, __package__ = get_torch_test_data(
+        Xs1, Ys1, Yvars1, bounds, _, fns, _ = get_torch_test_data(
             dtype=dtype, cuda=cuda, constant_noise=True
         )
         Xs2, Ys2, Yvars2, _, _, _, _ = get_torch_test_data(
@@ -83,23 +95,41 @@ class BotorchModelTest(TestCase):
                 "eta": 0.6,
             }
         }
+        # pyre-fixme[6]: For 1st param expected `(TorchModel, List[Tuple[float, float...
+        # pyre-fixme[6]: For 1st param expected `(AcquisitionFunction, Tensor, int, O...
+        # pyre-fixme[6]: For 1st param expected `(Model, Tensor) -> Tuple[Tensor,
+        #  Tensor]` but got `Dict[str, Union[Type[LKJCovariancePrior], float,
+        #  GammaPrior]]`.
+        # pyre-fixme[6]: For 1st param expected `(Model, Tensor,
+        #  Optional[Tuple[Tensor, Tensor]], Optional[Tensor], Optional[Tensor], Any) ->
+        #  AcquisitionFunction` but got `Dict[str, Union[Type[LKJCovariancePrior],
+        #  float, GammaPrior]]`.
+        # pyre-fixme[6]: For 1st param expected `(List[Tensor], List[Tensor],
+        #  List[Tensor], List[int], List[int], List[str], Optional[Dict[str, Tensor]],
+        #  Any) -> Model` but got `Dict[str, Union[Type[LKJCovariancePrior], float,
+        #  GammaPrior]]`.
+        # pyre-fixme[6]: For 1st param expected `bool` but got `Dict[str,
+        #  Union[Type[LKJCovariancePrior], float, GammaPrior]]`.
         model = BotorchModel(**kwargs)
+        datasets = [
+            FixedNoiseDataset(X=Xs1[0], Y=Ys1[0], Yvar=Yvars1[0]),
+            FixedNoiseDataset(X=Xs2[0], Y=Ys2[0], Yvar=Yvars2[0]),
+        ]
 
         with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
             model.fit(
-                Xs=Xs1 + Xs2,
-                Ys=Ys1 + Ys2,
-                Yvars=Yvars1 + Yvars2,
+                datasets=datasets,
+                metric_names=["y", "w"],
                 search_space_digest=SearchSpaceDigest(
                     feature_names=fns,
                     bounds=bounds,
                     task_features=[0],
                 ),
-                metric_names=["y", "w"],
             )
             _mock_fit_model.assert_called_once()
 
         # Check ranks
+        # pyre-fixme[16]: Optional type has no attribute `models`.
         model_list = model.model.models
         for i in range(1):
             self.assertIsInstance(
@@ -117,6 +147,9 @@ class BotorchModelTest(TestCase):
                 0.6,
             )
 
+    @fast_botorch_optimize
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def test_BotorchModel(self, dtype=torch.float, cuda=False):
         Xs1, Ys1, Yvars1, bounds, tfs, fns, mns = get_torch_test_data(
             dtype=dtype, cuda=cuda, constant_noise=True
@@ -130,20 +163,24 @@ class BotorchModelTest(TestCase):
                     use_input_warping=use_input_warping,
                     use_loocv_pseudo_likelihood=use_loocv_pseudo_likelihood,
                 )
+
                 # Test ModelListGP
+
                 # make training data different for each output
                 Xs2_diff = [Xs2[0] + 0.1]
+                datasets = [
+                    FixedNoiseDataset(X=Xs1[0], Y=Ys1[0], Yvar=Yvars1[0]),
+                    FixedNoiseDataset(X=Xs2_diff[0], Y=Ys2[0], Yvar=Yvars2[0]),
+                ]
                 with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
                     model.fit(
-                        Xs=Xs1 + Xs2_diff,
-                        Ys=Ys1 + Ys2,
-                        Yvars=Yvars1 + Yvars2,
+                        datasets=datasets,
+                        metric_names=mns,
                         search_space_digest=SearchSpaceDigest(
                             feature_names=fns,
                             bounds=bounds,
                             task_features=tfs,
                         ),
-                        metric_names=mns,
                     )
                     _mock_fit_model.assert_called_once()
                     if use_loocv_pseudo_likelihood:
@@ -162,6 +199,7 @@ class BotorchModelTest(TestCase):
                 self.assertIsInstance(model.model, ModelListGP)
 
                 # Check fitting
+                # pyre-fixme[16]: Optional type has no attribute `models`.
                 model_list = model.model.models
                 self.assertTrue(torch.equal(model_list[0].train_inputs[0], Xs1[0]))
                 self.assertTrue(torch.equal(model_list[1].train_inputs[0], Xs2_diff[0]))
@@ -183,17 +221,19 @@ class BotorchModelTest(TestCase):
                         self.assertFalse(hasattr(m, "input_transform"))
 
             # Test batched multi-output FixedNoiseGP
+            datasets_block = [
+                FixedNoiseDataset(X=Xs1[0], Y=Ys1[0], Yvar=Yvars1[0]),
+                FixedNoiseDataset(X=Xs2[0], Y=Ys2[0], Yvar=Yvars2[0]),
+            ]
             with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
                 model.fit(
-                    Xs=Xs1 + Xs2,
-                    Ys=Ys1 + Ys2,
-                    Yvars=Yvars1 + Yvars2,
+                    datasets=datasets_block,
+                    metric_names=["y1", "y2"],
                     search_space_digest=SearchSpaceDigest(
                         feature_names=fns,
                         bounds=bounds,
                         task_features=tfs,
                     ),
-                    metric_names=mns,
                 )
                 _mock_fit_model.assert_called_once()
 
@@ -214,7 +254,8 @@ class BotorchModelTest(TestCase):
 
                 if not use_input_warping:
                     expected_train_inputs = expected_train_inputs.unsqueeze(0).expand(
-                        torch.Size([2]) + Xs1[0].shape
+                        torch.Size([2])
+                        + Xs1[0].shape  # pyre-fixme[58]: Unsupported operand
                     )
                     expected_train_targets = torch.cat(Ys1 + Ys2, dim=-1).permute(1, 0)
                 else:
@@ -237,8 +278,11 @@ class BotorchModelTest(TestCase):
                 self.assertIsInstance(m.likelihood, _GaussianLikelihoodBase)
 
             # Check infeasible cost can be computed on the model
-            device = torch.device("cuda") if cuda else torch.device("cpu")
-            objective_weights = torch.tensor([1.0, 0.0], dtype=dtype, device=device)
+            tkwargs = {
+                "device": torch.device("cuda" if cuda else "cpu"),
+                "dtype": dtype,
+            }
+            objective_weights = torch.tensor([1.0, 0.0], **tkwargs)
             objective_transform = get_objective_weights_transform(objective_weights)
             infeasible_cost = torch.tensor(
                 get_infeasible_cost(
@@ -246,27 +290,28 @@ class BotorchModelTest(TestCase):
                 )
             )
             expected_infeasible_cost = -1 * torch.min(
+                # pyre-fixme[20]: Argument `1` expected.
                 objective_transform(
                     model.model.posterior(Xs1[0]).mean
                     - 6 * model.model.posterior(Xs1[0]).variance.sqrt()
                 ).min(),
-                torch.tensor(0.0, dtype=dtype, device=device),
+                torch.tensor(0.0, **tkwargs),
             )
             self.assertTrue(
                 torch.abs(infeasible_cost - expected_infeasible_cost) < 1e-5
             )
 
             # Check prediction
-            X = torch.tensor([[6.0, 7.0, 8.0]], dtype=dtype, device=device)
+            X = torch.tensor([[6.0, 7.0, 8.0]], **tkwargs)
             f_mean, f_cov = model.predict(X)
             self.assertTrue(f_mean.shape == torch.Size([1, 2]))
             self.assertTrue(f_cov.shape == torch.Size([1, 2, 2]))
 
             # Check generation
-            objective_weights = torch.tensor([1.0, 0.0], dtype=dtype, device=device)
+            objective_weights = torch.tensor([1.0, 0.0], **tkwargs)
             outcome_constraints = (
-                torch.tensor([[0.0, 1.0]], dtype=dtype, device=device),
-                torch.tensor([[5.0]], dtype=dtype, device=device),
+                torch.tensor([[0.0, 1.0]], **tkwargs),
+                torch.tensor([[5.0]], **tkwargs),
             )
             linear_constraints = (
                 torch.tensor([[0.0, 1.0, 1.0]]),
@@ -274,103 +319,121 @@ class BotorchModelTest(TestCase):
             )
             fixed_features = None
             pending_observations = [
-                torch.tensor([[1.0, 3.0, 4.0]], dtype=dtype, device=device),
-                torch.tensor([[2.0, 6.0, 8.0]], dtype=dtype, device=device),
+                torch.tensor([[1.0, 3.0, 4.0]], **tkwargs),
+                torch.tensor([[2.0, 6.0, 8.0]], **tkwargs),
             ]
             n = 3
 
-            X_dummy = torch.tensor([[[1.0, 2.0, 3.0]]], dtype=dtype, device=device)
-            acqfv_dummy = torch.tensor([[[1.0, 2.0, 3.0]]], dtype=dtype, device=device)
-            model_gen_options = {}
+            X_dummy = torch.tensor([[[1.0, 2.0, 3.0]]], **tkwargs)
+            acqfv_dummy = torch.tensor([[[1.0, 2.0, 3.0]]], **tkwargs)
+            model_gen_options = {"subset_model": False}
             # test sequential optimize
+            search_space_digest = SearchSpaceDigest(
+                feature_names=[],
+                bounds=bounds,
+            )
             with mock.patch(
                 "ax.models.torch.botorch_defaults.optimize_acqf",
                 return_value=(X_dummy, acqfv_dummy),
             ) as mock_optimize_acqf:
-                Xgen, wgen, gen_metadata, cand_metadata = model.gen(
+                gen_results = model.gen(
                     n=n,
-                    bounds=bounds,
-                    objective_weights=objective_weights,
-                    outcome_constraints=outcome_constraints,
-                    linear_constraints=linear_constraints,
-                    fixed_features=fixed_features,
-                    pending_observations=pending_observations,
-                    model_gen_options=model_gen_options,
-                    rounding_func=dummy_func,
+                    search_space_digest=search_space_digest,
+                    torch_opt_config=TorchOptConfig(
+                        objective_weights=objective_weights,
+                        outcome_constraints=outcome_constraints,
+                        linear_constraints=linear_constraints,
+                        fixed_features=fixed_features,
+                        pending_observations=pending_observations,
+                        # pyre-fixme[6]: For 6th param expected `Dict[str,
+                        #  Union[None, Dict[str, typing.Any], OptimizationConfig,
+                        #  AcquisitionFunction, float, int, str]]` but got `Dict[str,
+                        #  bool]`.
+                        model_gen_options=model_gen_options,
+                        rounding_func=dummy_func,
+                    ),
                 )
                 # note: gen() always returns CPU tensors
-                self.assertTrue(torch.equal(Xgen, X_dummy.cpu()))
-                self.assertTrue(torch.equal(wgen, torch.ones(n, dtype=dtype)))
+                self.assertTrue(torch.equal(gen_results.points, X_dummy.cpu()))
+                self.assertTrue(
+                    torch.equal(gen_results.weights, torch.ones(n, dtype=dtype))
+                )
 
+            torch_opt_config = TorchOptConfig(
+                objective_weights=objective_weights,
+                fixed_features=fixed_features,
+                pending_observations=pending_observations,
+                model_gen_options={"optimizer_kwargs": {"joint_optimization": True}},
+            )
             # test joint optimize
             with mock.patch(
                 "ax.models.torch.botorch_defaults.optimize_acqf",
                 return_value=(X_dummy, acqfv_dummy),
             ) as mock_optimize_acqf:
-                Xgen, wgen, gen_metadata, cand_metadata = model.gen(
+                gen_results = model.gen(
                     n=n,
-                    bounds=bounds,
-                    objective_weights=objective_weights,
-                    outcome_constraints=None,
-                    linear_constraints=None,
-                    fixed_features=fixed_features,
-                    pending_observations=pending_observations,
-                    model_gen_options={
-                        "optimizer_kwargs": {"joint_optimization": True}
-                    },
+                    search_space_digest=search_space_digest,
+                    torch_opt_config=torch_opt_config,
                 )
                 # note: gen() always returns CPU tensors
-                self.assertTrue(torch.equal(Xgen, X_dummy.cpu()))
-                self.assertTrue(torch.equal(wgen, torch.ones(n, dtype=dtype)))
+                self.assertTrue(torch.equal(gen_results.points, X_dummy.cpu()))
+                self.assertTrue(
+                    torch.equal(gen_results.weights, torch.ones(n, dtype=dtype))
+                )
                 mock_optimize_acqf.assert_called_once()
 
             # test that fidelity features are unsupported
             with self.assertRaises(NotImplementedError):
-                Xgen, wgen = model.gen(
+                gen_results = model.gen(
                     n=n,
-                    bounds=bounds,
-                    objective_weights=objective_weights,
-                    outcome_constraints=None,
-                    linear_constraints=None,
-                    fixed_features=fixed_features,
-                    pending_observations=pending_observations,
-                    model_gen_options={
-                        "optimizer_kwargs": {"joint_optimization": True}
-                    },
-                    target_fidelities={0: 3.0},
+                    search_space_digest=dataclasses.replace(
+                        search_space_digest,
+                        target_fidelities={0: 3.0},
+                    ),
+                    torch_opt_config=torch_opt_config,
                 )
 
             # test get_rounding_func
             dummy_rounding = get_rounding_func(rounding_func=dummy_func)
             X_temp = torch.rand(1, 2, 3, 4)
+            # pyre-fixme[29]: `Optional[typing.Callable[[torch._tensor.Tensor],
+            #  torch._tensor.Tensor]]` is not a function.
             self.assertTrue(torch.equal(X_temp, dummy_rounding(X_temp)))
 
             # Check best point selection
-            xbest = model.best_point(bounds=bounds, objective_weights=objective_weights)
             xbest = model.best_point(
-                bounds=bounds,
-                objective_weights=objective_weights,
+                search_space_digest=search_space_digest,
+                torch_opt_config=torch_opt_config,
+            )
+            torch_opt_config = dataclasses.replace(
+                torch_opt_config,
                 fixed_features={0: 100.0},
             )
+            xbest = model.best_point(
+                search_space_digest=search_space_digest,
+                torch_opt_config=torch_opt_config,
+            )
             self.assertIsNone(xbest)
 
             # test that fidelity features are unsupported
             with self.assertRaises(NotImplementedError):
                 xbest = model.best_point(
-                    bounds=bounds,
-                    objective_weights=objective_weights,
-                    fixed_features={0: 100.0},
-                    target_fidelities={0: 3.0},
+                    search_space_digest=dataclasses.replace(
+                        search_space_digest,
+                        target_fidelities={0: 3.0},
+                    ),
+                    torch_opt_config=torch_opt_config,
                 )
 
             # Test cross-validation
+            combined_datasets = [
+                FixedNoiseDataset(Xs1[0], Y=Ys1[0], Yvar=Yvars1[0]),
+                FixedNoiseDataset(Xs2[0], Y=Ys2[0], Yvar=Yvars2[0]),
+            ]
             mean, variance = model.cross_validate(
-                Xs_train=Xs1 + Xs2,
-                Ys_train=Ys1 + Ys2,
-                Yvars_train=Yvars1 + Yvars2,
-                X_test=torch.tensor(
-                    [[1.2, 3.2, 4.2], [2.4, 5.2, 3.2]], dtype=dtype, device=device
-                ),
+                datasets=combined_datasets,
+                metric_names=["y1", "y2"],
+                X_test=torch.tensor([[1.2, 3.2, 4.2], [2.4, 5.2, 3.2]], **tkwargs),
             )
             self.assertTrue(mean.shape == torch.Size([2, 2]))
             self.assertTrue(variance.shape == torch.Size([2, 2, 2]))
@@ -378,19 +441,19 @@ class BotorchModelTest(TestCase):
             # Test cross-validation with refit_on_cv
             model.refit_on_cv = True
             mean, variance = model.cross_validate(
-                Xs_train=Xs1 + Xs2,
-                Ys_train=Ys1 + Ys2,
-                Yvars_train=Yvars1 + Yvars2,
-                X_test=torch.tensor(
-                    [[1.2, 3.2, 4.2], [2.4, 5.2, 3.2]], dtype=dtype, device=device
-                ),
+                datasets=combined_datasets,
+                metric_names=["y1", "y2"],
+                X_test=torch.tensor([[1.2, 3.2, 4.2], [2.4, 5.2, 3.2]], **tkwargs),
             )
             self.assertTrue(mean.shape == torch.Size([2, 2]))
             self.assertTrue(variance.shape == torch.Size([2, 2, 2]))
 
             # Test update
             model.refit_on_update = False
-            model.update(Xs=Xs2 + Xs2, Ys=Ys2 + Ys2, Yvars=Yvars2 + Yvars2)
+            model.update(
+                datasets=[FixedNoiseDataset(Xs2[0], Y=Ys2[0], Yvar=Yvars2[0])] * 2,
+                metric_names=["y1", "y2"],
+            )
 
             # Test feature_importances
             importances = model.feature_importances()
@@ -404,26 +467,29 @@ class BotorchModelTest(TestCase):
 
             model.refit_on_update = True
             with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
-                model.update(Xs=Xs2 + Xs2, Ys=Ys2 + Ys2, Yvars=Yvars2 + Yvars2)
+                model.update(
+                    datasets=[FixedNoiseDataset(Xs2[0], Y=Ys2[0], Yvar=Yvars2[0])] * 2,
+                    metric_names=["y1", "y2"],
+                )
 
             # test unfit model CV, update, and feature_importances
             unfit_model = BotorchModel()
             with self.assertRaises(RuntimeError):
                 unfit_model.cross_validate(
-                    Xs_train=Xs1 + Xs2,
-                    Ys_train=Ys1 + Ys2,
-                    Yvars_train=Yvars1 + Yvars2,
+                    datasets=combined_datasets,
+                    metric_names=["y1", "y2"],
                     X_test=Xs1[0],
                 )
             with self.assertRaises(RuntimeError):
-                unfit_model.update(Xs=Xs1 + Xs2, Ys=Ys1 + Ys2, Yvars=Yvars1 + Yvars2)
+                unfit_model.update(
+                    datasets=combined_datasets, metric_names=["y1", "y2"]
+                )
             with self.assertRaises(RuntimeError):
                 unfit_model.feature_importances()
 
             # Test loading state dict
-            tkwargs = {"device": device, "dtype": dtype}
             true_state_dict = {
-                "mean_module.constant": [3.5004],
+                "mean_module.raw_constant": 3.5004,
                 "covar_module.raw_outputscale": 2.2438,
                 "covar_module.base_kernel.raw_lengthscale": [
                     [-0.9274, -0.9274, -0.9274]
@@ -447,7 +513,7 @@ class BotorchModelTest(TestCase):
                 Yvars=Yvars1,
                 task_features=[],
                 fidelity_features=[],
-                metric_names=mns[0],
+                metric_names=[mns[0]],
                 state_dict=true_state_dict,
                 refit_model=False,
             )
@@ -455,7 +521,7 @@ class BotorchModelTest(TestCase):
                 self.assertTrue(torch.equal(true_state_dict[k], v))
 
             # Test for some change in model parameters & buffer for refit_model=True
-            true_state_dict["mean_module.constant"] += 0.1
+            true_state_dict["mean_module.raw_constant"] += 0.1
             true_state_dict["covar_module.raw_outputscale"] += 0.1
             true_state_dict["covar_module.base_kernel.raw_lengthscale"] += 0.1
             model = get_and_fit_model(
@@ -464,7 +530,7 @@ class BotorchModelTest(TestCase):
                 Yvars=Yvars1,
                 task_features=[],
                 fidelity_features=[],
-                metric_names=mns[0],
+                metric_names=[mns[0]],
                 state_dict=true_state_dict,
                 refit_model=True,
             )
@@ -479,31 +545,37 @@ class BotorchModelTest(TestCase):
         model = BotorchModel(best_point_recommender=recommend_best_out_of_sample_point)
         with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
             model.fit(
-                Xs=Xs1 + Xs2_diff,
-                Ys=Ys1 + Ys2,
-                Yvars=Yvars1 + Yvars2,
+                # pyre-fixme[61]: `datasets` is undefined, or not always defined.
+                datasets=datasets,
+                metric_names=mns,
                 search_space_digest=SearchSpaceDigest(
                     feature_names=fns,
                     bounds=bounds,
                     task_features=tfs,
                 ),
-                metric_names=mns,
             )
         with self.assertRaises(RuntimeError):
-            xbest = model.best_point(bounds=bounds, objective_weights=objective_weights)
+            xbest = model.best_point(
+                # pyre-fixme[61]: `search_space_digest` is undefined, or not always
+                #  defined.
+                search_space_digest=search_space_digest,
+                # pyre-fixme[61]: `torch_opt_config` is undefined, or not always
+                #  defined.
+                torch_opt_config=torch_opt_config,
+            )
 
-    def test_BotorchModel_cuda(self):
+    def test_BotorchModel_cuda(self) -> None:
         if torch.cuda.is_available():
             self.test_BotorchModel(cuda=True)
 
-    def test_BotorchModel_double(self):
+    def test_BotorchModel_double(self) -> None:
         self.test_BotorchModel(dtype=torch.double)
 
-    def test_BotorchModel_double_cuda(self):
+    def test_BotorchModel_double_cuda(self) -> None:
         if torch.cuda.is_available():
             self.test_BotorchModel(dtype=torch.double, cuda=True)
 
-    def test_BotorchModelOneOutcome(self):
+    def test_BotorchModelOneOutcome(self) -> None:
         Xs1, Ys1, Yvars1, bounds, tfs, fns, mns = get_torch_test_data(
             dtype=torch.float, cuda=False, constant_noise=True
         )
@@ -516,15 +588,13 @@ class BotorchModelTest(TestCase):
             )
             with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
                 model.fit(
-                    Xs=Xs1,
-                    Ys=Ys1,
-                    Yvars=Yvars1,
+                    datasets=[FixedNoiseDataset(X=Xs1[0], Y=Ys1[0], Yvar=Yvars1[0])],
+                    metric_names=mns[:1],
                     search_space_digest=SearchSpaceDigest(
                         feature_names=fns,
                         bounds=bounds,
                         task_features=tfs,
                     ),
-                    metric_names=mns[0],
                 )
                 _mock_fit_model.assert_called_once()
                 if use_loocv_pseudo_likelihood:
@@ -541,11 +611,12 @@ class BotorchModelTest(TestCase):
             self.assertTrue(f_cov.shape == torch.Size([2, 1, 1]))
             if use_input_warping:
                 self.assertTrue(hasattr(model.model, "input_transform"))
+                # pyre-fixme[16]: Optional type has no attribute `input_transform`.
                 self.assertIsInstance(model.model.input_transform, Warp)
             else:
                 self.assertFalse(hasattr(model.model, "input_transform"))
 
-    def test_BotorchModelConstraints(self):
+    def test_BotorchModelConstraints(self) -> None:
         Xs1, Ys1, Yvars1, bounds, tfs, fns, mns = get_torch_test_data(
             dtype=torch.float, cuda=False, constant_noise=True
         )
@@ -559,20 +630,43 @@ class BotorchModelTest(TestCase):
         )
         n = 3
         model = BotorchModel()
+        search_space_digest = SearchSpaceDigest(
+            feature_names=fns,
+            bounds=bounds,
+            task_features=tfs,
+        )
         with mock.patch(FIT_MODEL_MO_PATH) as _mock_fit_model:
             model.fit(
-                Xs=Xs1 + Xs2,
-                Ys=Ys1 + Ys2,
-                Yvars=Yvars1 + Yvars2,
-                search_space_digest=SearchSpaceDigest(
-                    feature_names=fns,
-                    bounds=bounds,
-                    task_features=tfs,
-                ),
+                datasets=[
+                    FixedNoiseDataset(X=Xs1[0], Y=Ys1[0], Yvar=Yvars1[0]),
+                    FixedNoiseDataset(X=Xs2[0], Y=Ys2[0], Yvar=Yvars2[0]),
+                ],
                 metric_names=mns,
+                search_space_digest=search_space_digest,
             )
             _mock_fit_model.assert_called_once()
 
         # because there are no feasible points:
         with self.assertRaises(ValueError):
-            model.gen(n, bounds, objective_weights)
+            model.gen(
+                n,
+                search_space_digest=search_space_digest,
+                torch_opt_config=TorchOptConfig(objective_weights),
+            )
+
+    def test_botorchmodel_raises_when_no_data(self) -> None:
+        _, _, _, bounds, tfs, fns, mns = get_torch_test_data(
+            dtype=torch.float, cuda=False, constant_noise=True
+        )
+        search_space_digest = SearchSpaceDigest(
+            feature_names=fns,
+            bounds=bounds,
+            task_features=tfs,
+        )
+        model = BotorchModel()
+        with self.assertRaisesRegex(
+            DataRequiredError, "BotorchModel.fit requires non-empty data sets."
+        ):
+            model.fit(
+                datasets=[], metric_names=mns, search_space_digest=search_space_digest
+            )
