@@ -1,27 +1,29 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from unittest import mock
 
 import torch
 from ax.core.search_space import SearchSpaceDigest
 from ax.models.torch.cbo_lcea import LCEABO
 from ax.utils.common.testutils import TestCase
+from ax.utils.testing.mock import fast_botorch_optimize
 from botorch.models.contextual import LCEAGP
 from botorch.models.model_list_gp_regression import ModelListGP
+from botorch.utils.datasets import FixedNoiseDataset
 
 
 class LCEABOTest(TestCase):
-    @mock.patch("ax.models.torch.botorch_defaults.fit_gpytorch_model")
-    def testLCEABO(self, mock_model_fit):
+    @fast_botorch_optimize
+    def testLCEABO(self) -> None:
         train_X = torch.tensor(
             [[0.0, 0.0, 0.0, 0.0], [1.0, 1.0, 1.0, 1.0], [2.0, 2.0, 2.0, 2.0]]
         )
         train_Y = torch.tensor([[1.0], [2.0], [3.0]])
         train_Yvar = 0.1 * torch.ones(3, 1, dtype=torch.double)
+        training_data = [FixedNoiseDataset(X=train_X, Y=train_Y, Yvar=train_Yvar)]
 
         # Test setting attributes
         decomposition = {"1": ["0", "1"], "2": ["2", "3"]}
@@ -34,14 +36,12 @@ class LCEABOTest(TestCase):
 
         # Test fit
         m1.fit(
-            Xs=[train_X],
-            Ys=[train_Y],
-            Yvars=[train_Yvar],
+            datasets=training_data,
+            metric_names=["y"],
             search_space_digest=SearchSpaceDigest(
                 feature_names=["0", "1", "2", "3"],
                 bounds=[(0.0, 1.0) for _ in range(4)],
             ),
-            metric_names=["y"],
         )
         self.assertIsInstance(m1.model, LCEAGP)
 
@@ -74,40 +74,35 @@ class LCEABOTest(TestCase):
         # Test input decomposition indicates parameter name
         m2 = LCEABO(decomposition={"1": ["x1", "x3"], "2": ["x2", "x4"]})
         m2.fit(
-            Xs=[train_X],
-            Ys=[train_Y],
-            Yvars=[train_Yvar],
+            datasets=training_data,
+            metric_names=["y"],
             search_space_digest=SearchSpaceDigest(
                 feature_names=["x1", "x2", "x3", "x4"],
                 bounds=[(0.0, 1.0) for _ in range(4)],
             ),
-            metric_names=["y"],
         )
+        # pyre-fixme[16]: Optional type has no attribute `decomposition`.
         self.assertDictEqual(m2.model.decomposition, {"1": [0, 2], "2": [1, 3]})
 
         # Test decomposition validation in get_and_fit_model
         # does not pass feature names when decomposition uses feature names
         with self.assertRaises(ValueError):
             m2.fit(
-                Xs=[train_X],
-                Ys=[train_Y],
-                Yvars=[train_Yvar],
+                datasets=training_data,
+                metric_names=["y"],
                 search_space_digest=SearchSpaceDigest(
                     feature_names=[],
                     bounds=[(0.0, 1.0) for _ in range(4)],
                 ),
-                metric_names=[],
             )
 
         # pass wrong feature names
         with self.assertRaises(AssertionError):
             m2.fit(
-                Xs=[train_X],
-                Ys=[train_Y],
-                Yvars=[train_Yvar],
+                datasets=training_data,
+                metric_names=["y"],
                 search_space_digest=SearchSpaceDigest(
                     feature_names=["x0", "x1", "x2", "x3"],
                     bounds=[(0.0, 1.0) for _ in range(4)],
                 ),
-                metric_names=["y"],
             )
