@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -10,9 +10,11 @@ from ax.utils.common.testutils import TestCase
 
 
 class REMBOInitializerTest(TestCase):
-    def testREMBOInitializerModel(self):
+    def testREMBOInitializerModel(self) -> None:
         A = np.vstack((np.eye(2, 2), -(np.eye(2, 2))))
         # Test setting attributes
+        # pyre-fixme[6]: For 2nd param expected `List[Tuple[float, float]]` but got
+        #  `List[Tuple[int, int]]`.
         m = REMBOInitializer(A=A, bounds_d=[(-2, 2)] * 2)
         self.assertTrue(np.allclose(A, m.A))
         self.assertEqual(m.bounds_d, [(-2, 2), (-2, 2)])
