@@ -1,26 +1,32 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from __future__ import annotations
+
+import dataclasses
 import re
 from collections import OrderedDict
+from logging import Logger
 from typing import Any, Callable, Dict, List, MutableMapping, Optional, Tuple, Union
 
 import gpytorch
 import numpy as np
 import torch
 from ax.core.search_space import SearchSpaceDigest
-from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
+from ax.core.types import TCandidateMetadata
 from ax.models.random.alebo_initializer import ALEBOInitializer
 from ax.models.torch.botorch import BotorchModel
 from ax.models.torch.botorch_defaults import get_NEI
-from ax.models.torch_base import TorchModel
+from ax.models.torch.utils import _datasets_to_legacy_inputs
+from ax.models.torch_base import TorchGenResults, TorchModel, TorchOptConfig
 from ax.utils.common.docutils import copy_doc
 from ax.utils.common.logger import get_logger
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.analytic import ExpectedImprovement
+from botorch.acquisition.objective import PosteriorTransform
 from botorch.models.gp_regression import FixedNoiseGP
 from botorch.models.gpytorch import GPyTorchModel
 from botorch.models.model_list_gp_regression import ModelListGP
@@ -30,6 +36,7 @@ from botorch.optim.numpy_converter import module_to_array
 from botorch.optim.optimize import optimize_acqf
 from botorch.optim.utils import _scipy_objective_and_grad
 from botorch.posteriors.gpytorch import GPyTorchPosterior
+from botorch.utils.datasets import SupervisedDataset
 from gpytorch.distributions.multivariate_normal import MultivariateNormal
 from gpytorch.kernels.kernel import Kernel
 from gpytorch.kernels.rbf_kernel import postprocess_rbf
@@ -39,7 +46,7 @@ from scipy.optimize import approx_fprime
 from torch import Tensor
 
 
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 
 
 class ALEBOKernel(Kernel):
@@ -63,17 +70,19 @@ class ALEBOKernel(Kernel):
         super().__init__(
             has_lengthscale=False, ard_num_dims=None, eps=0.0, batch_shape=batch_shape
         )
+        # pyre-fixme[4]: Attribute must be annotated.
         self.d, D = B.shape
         assert self.d < D
         self.B = B
         # Initialize U
         Arnd = torch.randn(D, D, dtype=B.dtype, device=B.device)
-        Arnd = torch.qr(Arnd)[0]
+        Arnd = torch.linalg.qr(Arnd)[0]
         ABinv = Arnd[: self.d, :] @ torch.pinverse(B)
         # U is the upper Cholesky decomposition of Gamma, the Mahalanobis
         # matrix. Uvec is the upper triangular portion of U squeezed out into
         # a vector.
-        U = torch.cholesky(torch.mm(ABinv.t(), ABinv), upper=True)
+        U = torch.linalg.cholesky(torch.mm(ABinv.t(), ABinv)).t()
+        # pyre-fixme[4]: Attribute must be annotated.
         self.triu_indx = torch.triu_indices(self.d, self.d, device=B.device)
         Uvec = U[self.triu_indx.tolist()].repeat(*batch_shape, 1)
         self.register_parameter(name="Uvec", parameter=torch.nn.Parameter(Uvec))
@@ -94,6 +103,7 @@ class ALEBOKernel(Kernel):
         # Compute kernel distance
         z1 = torch.matmul(x1, U_t)
         z2 = torch.matmul(x2, U_t)
+
         return self.covar_dist(
             z1,
             z2,
@@ -159,6 +169,8 @@ class ALEBOGP(FixedNoiseGP):
         x = x.unsqueeze(-3).expand(
             x.shape[:-2]
             + torch.Size([self._aug_batch_shape[0]])  # pyre-ignore
+            # pyre-fixme[58]: `+` is not supported for operand types `Tuple[int,
+            #  ...]` and `Size`.
             + x.shape[-2:]
         )
         mvn_b = super().__call__(x)
@@ -177,12 +189,16 @@ class ALEBOGP(FixedNoiseGP):
         X: Tensor,
         output_indices: Optional[List[int]] = None,
         observation_noise: Union[bool, Tensor] = False,
+        posterior_transform: Optional[PosteriorTransform] = None,
         **kwargs: Any,
     ) -> GPyTorchPosterior:
         assert output_indices is None
         assert not observation_noise
         mvn = self(X)
-        return GPyTorchPosterior(mvn=mvn)
+        posterior = GPyTorchPosterior(mvn=mvn)
+        if posterior_transform is not None:
+            return posterior_transform(posterior)
+        return posterior
 
 
 def get_fitted_model(
@@ -264,8 +280,6 @@ def get_map_model(
     for _ in range(restarts):
         m = ALEBOGP(B=B, train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
         if init_state_dict is not None:
-            # pyre-fixme[6]: Expected `OrderedDict[typing.Any, typing.Any]` for 1st
-            #  param but got `Dict[str, Tensor]`.
             m.load_state_dict(init_state_dict)
         mll = ExactMarginalLogLikelihood(m.likelihood, m)
         mll.train()
@@ -312,6 +326,11 @@ def laplace_sample_U(
     epsilon = 1e-4 + 1e-3 * np.abs(x0)
     for i, _ in enumerate(x0):
         # Compute gradient of df/dx_i wrt x_i
+        # pyre-fixme[53]: Captured variable `property_dict` is not annotated.
+        # pyre-fixme[53]: Captured variable `x0` is not annotated.
+        # pyre-fixme[53]: Captured variable `i` is not annotated.
+        # pyre-fixme[3]: Return type must be annotated.
+        # pyre-fixme[2]: Parameter must be annotated.
         def f(x):
             x_all = x0.copy()
             x_all[i] = x[0]
@@ -321,7 +340,7 @@ def laplace_sample_U(
 
     # Sample only Uvec; leave mean and output scale fixed.
     assert list(property_dict.keys()) == [
-        "model.mean_module.constant",
+        "model.mean_module.raw_constant",
         "model.covar_module.raw_outputscale",
         "model.covar_module.base_kernel.Uvec",
     ]
@@ -337,7 +356,7 @@ def laplace_sample_U(
         nsamp, *attrs.shape
     )
     # Get the other properties into batch mode
-    mean_constant_batch = mll.model.mean_module.constant.repeat(nsamp, 1)
+    mean_constant_batch = mll.model.mean_module.constant.repeat(nsamp)
     output_scale_batch = mll.model.covar_module.raw_outputscale.repeat(nsamp)
     return Uvec_batch, mean_constant_batch, output_scale_batch
 
@@ -373,9 +392,10 @@ def get_batch_model(
     )
     m_b.train()
     # Set mean constant
-    m_b.mean_module.constant.requires_grad_(False)
-    m_b.mean_module.constant.copy_(mean_constant_batch)
-    m_b.mean_module.constant.requires_grad_(True)
+    # pyre-fixme[16]: `Optional` has no attribute `raw_constant`.
+    m_b.mean_module.raw_constant.requires_grad_(False)
+    m_b.mean_module.raw_constant.copy_(mean_constant_batch)
+    m_b.mean_module.raw_constant.requires_grad_(True)
     # Set output scale
     m_b.covar_module.raw_outputscale.requires_grad_(False)
     m_b.covar_module.raw_outputscale.copy_(output_scale_batch)
@@ -460,6 +480,7 @@ def ei_or_nei(
             best_f = model.train_targets.max()
         else:
             best_f = model.train_targets.min()
+        # pyre-fixme[6]: For 3rd param expected `bool` but got `Tensor`.
         return ExpectedImprovement(model=model, best_f=best_f, maximize=maximize)
     else:
         with gpytorch.settings.max_cholesky_size(2000):
@@ -507,16 +528,21 @@ def alebo_acqf_optimizer(
 
         Xrnd = torch.tensor(Xrnd_npy, dtype=B.dtype, device=B.device).unsqueeze(1)
         Yrnd = torch.matmul(Xrnd, B.t())  # Project down to the embedding
+
         with gpytorch.settings.max_cholesky_size(2000):
             with torch.no_grad():
                 alpha = acq_function(Yrnd)
 
             Yinit = initialize_q_batch_nonneg(X=Yrnd, Y=alpha, n=num_restarts)
-
+            inf_bounds = (  # all constraints are encoded via inequality_constraints
+                torch.tensor([[-float("inf")], [float("inf")]])
+                .expand(2, Yrnd.shape[-1])
+                .to(Yrnd)
+            )
             # Optimize the acquisition function, separately for each random restart.
             candidate, acq_value = optimize_acqf(
                 acq_function=acq_function,
-                bounds=[None, None],  # pyre-ignore
+                bounds=inf_bounds,
                 q=1,
                 num_restarts=num_restarts,
                 raw_samples=0,
@@ -566,6 +592,7 @@ class ALEBO(BotorchModel):
         self, B: Tensor, laplace_nsamp: int = 25, fit_restarts: int = 10
     ) -> None:
         self.B = B
+        # pyre-fixme[4]: Attribute must be annotated.
         self.Binv = torch.pinverse(B)
         self.laplace_nsamp = laplace_nsamp
         self.fit_restarts = fit_restarts
@@ -574,19 +601,19 @@ class ALEBO(BotorchModel):
             refit_on_cv=False,
             warm_start_refitting=False,
             acqf_constructor=ei_or_nei,  # pyre-ignore
+            # pyre-fixme[6]: Expected `(AcquisitionFunction, Tensor, int, Optional[Li...
             acqf_optimizer=alebo_acqf_optimizer,
         )
 
     @copy_doc(TorchModel.fit)
     def fit(
         self,
-        Xs: List[Tensor],
-        Ys: List[Tensor],
-        Yvars: List[Tensor],
-        search_space_digest: SearchSpaceDigest,
+        datasets: List[SupervisedDataset],
         metric_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
+        Xs, Ys, Yvars = _datasets_to_legacy_inputs(datasets=datasets)
         assert len(search_space_digest.task_features) == 0
         assert len(search_space_digest.fidelity_features) == 0
         for b in search_space_digest.bounds:
@@ -608,29 +635,17 @@ class ALEBO(BotorchModel):
     @copy_doc(TorchModel.best_point)
     def best_point(
         self,
-        bounds: List[Tuple[float, float]],
-        objective_weights: Tensor,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        model_gen_options: Optional[TConfig] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
+        search_space_digest: SearchSpaceDigest,
+        torch_opt_config: TorchOptConfig,
     ) -> Optional[Tensor]:
         raise NotImplementedError
 
     def gen(
         self,
         n: int,
-        bounds: List[Tuple[float, float]],
-        objective_weights: Tensor,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        pending_observations: Optional[List[Tensor]] = None,
-        model_gen_options: Optional[TConfig] = None,
-        rounding_func: Optional[Callable[[Tensor], Tensor]] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
-    ) -> Tuple[Tensor, Tensor, TGenMetadata, List[TCandidateMetadata]]:
+        search_space_digest: SearchSpaceDigest,
+        torch_opt_config: TorchOptConfig,
+    ) -> TorchGenResults:
         """Generate candidates.
 
         Candidates are generated in the linear embedding with the polytope
@@ -640,53 +655,55 @@ class ALEBO(BotorchModel):
         initializing the acquisition function optimization) and 'num_restarts'
         (number of restarts for acquisition function optimization).
         """
-        for b in bounds:
+        for b in search_space_digest.bounds:
             assert b == (-1, 1)
         # The following can be easily handled in the future when needed
-        assert linear_constraints is None
-        assert fixed_features is None
-        assert pending_observations is None
+        assert torch_opt_config.linear_constraints is None
+        assert torch_opt_config.fixed_features is None
+        assert torch_opt_config.pending_observations is None
         # Setup constraints
         A = torch.cat((self.Binv, -self.Binv))
         b = torch.ones(2 * self.Binv.shape[0], 1, dtype=self.dtype, device=self.device)
         linear_constraints = (A, b)
         noiseless = max(Yvar.min().item() for Yvar in self.Yvars) < 1e-5
-        if model_gen_options is None:
-            model_gen_options = {}
         model_gen_options = {
             "acquisition_function_kwargs": {"q": n, "noiseless": noiseless},
             "optimizer_kwargs": {
-                "raw_samples": model_gen_options.get("raw_samples", 1000),
-                "num_restarts": model_gen_options.get("num_restarts", 10),
+                "raw_samples": torch_opt_config.model_gen_options.get(
+                    "raw_samples", 1000
+                ),
+                "num_restarts": torch_opt_config.model_gen_options.get(
+                    "num_restarts", 10
+                ),
                 "B": self.B,
             },
         }
-        Xd_opt, w, gen_metadata, candidate_metadata = super().gen(
+        gen_results = super().gen(
             n=n,
-            bounds=[(-1e8, 1e8)] * self.B.shape[0],
-            objective_weights=objective_weights,
-            outcome_constraints=outcome_constraints,
-            linear_constraints=linear_constraints,
-            model_gen_options=model_gen_options,
+            search_space_digest=dataclasses.replace(
+                search_space_digest,
+                bounds=[(-1e8, 1e8)] * self.B.shape[0],
+            ),
+            torch_opt_config=dataclasses.replace(
+                torch_opt_config,
+                linear_constraints=linear_constraints,
+                model_gen_options=model_gen_options,
+            ),
         )
         # Project up
-        Xopt = (self.Binv @ Xd_opt.t()).t()
+        Xopt = (self.Binv @ gen_results.points.t()).t()
         # Sometimes numerical tolerance can have Xopt epsilon outside [-1, 1],
         # so clip it back.
         if Xopt.min() < -1 or Xopt.max() > 1:
             logger.debug(f"Clipping from [{Xopt.min()}, {Xopt.max()}]")
             Xopt = torch.clamp(Xopt, min=-1.0, max=1.0)
-        # pyre-fixme[7]: Expected `Tuple[Tensor, Tensor, Dict[str, typing.Any],
-        #  List[Optional[Dict[str, typing.Any]]]]` but got `Tuple[typing.Any, Tensor,
-        #  Dict[str, typing.Any], None]`.
-        return Xopt, w, gen_metadata, candidate_metadata
+
+        return dataclasses.replace(gen_results, points=Xopt)
 
     @copy_doc(TorchModel.update)
     def update(
         self,
-        Xs: List[Tensor],
-        Ys: List[Tensor],
-        Yvars: List[Tensor],
+        datasets: List[SupervisedDataset],
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
         **kwargs: Any,
     ) -> None:
@@ -694,6 +711,7 @@ class ALEBO(BotorchModel):
             raise RuntimeError(
                 "Cannot update model that has not been fit"
             )  # pragma: no cover
+        Xs, Ys, Yvars = _datasets_to_legacy_inputs(datasets=datasets)
         self.Xs = [(self.B @ X.t()).t() for X in Xs]  # Project down.
         self.Ys = Ys
         self.Yvars = Yvars
@@ -710,9 +728,7 @@ class ALEBO(BotorchModel):
     @copy_doc(TorchModel.cross_validate)
     def cross_validate(
         self,
-        Xs_train: List[Tensor],
-        Ys_train: List[Tensor],
-        Yvars_train: List[Tensor],
+        datasets: List[SupervisedDataset],
         X_test: Tensor,
         **kwargs: Any,
     ) -> Tuple[Tensor, Tensor]:
@@ -726,10 +742,11 @@ class ALEBO(BotorchModel):
             state_dicts = extract_map_statedict(
                 m_b=self.model, num_outputs=len(self.Xs)  # pyre-ignore
             )
-        Xs_train = [X @ self.B.t() for X in Xs_train]  # Project down.
+        Xs, Ys, Yvars = _datasets_to_legacy_inputs(datasets=datasets)
+        Xs = [X @ self.B.t() for X in Xs]  # Project down.
         X_test = X_test @ self.B.t()
         model = self.get_and_fit_model(
-            Xs=Xs_train, Ys=Ys_train, Yvars=Yvars_train, state_dicts=state_dicts
+            Xs=Xs, Ys=Ys, Yvars=Yvars, state_dicts=state_dicts
         )
         return self.model_predictor(model=model, X=X_test)  # pyre-ignore: [28]
 
