@@ -1,26 +1,24 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from typing import Any, Callable, Dict, List, Optional, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type
 
 import torch
-from ax.core.types import TConfig
 from ax.models.model_utils import best_observed_point, get_observed
-from ax.models.torch.utils import (  # noqa F401
-    _to_inequality_constraints,
-    predict_from_model,
-)
+from ax.models.torch.utils import _to_inequality_constraints
 from ax.models.torch_base import TorchModel
+from ax.models.types import TConfig
 from ax.utils.common.constants import Keys
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.fixed_feature import FixedFeatureAcquisitionFunction
 from botorch.acquisition.objective import ConstrainedMCObjective, GenericMCObjective
+from botorch.acquisition.penalized import PenalizedMCObjective
 from botorch.acquisition.utils import get_acquisition_function, get_infeasible_cost
 from botorch.exceptions.errors import UnsupportedError
-from botorch.fit import fit_gpytorch_model
+from botorch.fit import fit_gpytorch_mll
 from botorch.models.gp_regression import FixedNoiseGP, SingleTaskGP
 from botorch.models.gp_regression_fidelity import SingleTaskMultiFidelityGP
 from botorch.models.gpytorch import GPyTorchModel
@@ -42,7 +40,7 @@ from gpytorch.priors.torch_priors import GammaPrior, LogNormalPrior
 from torch import Tensor
 
 
-MIN_OBSERVED_NOISE_LEVEL = 1e-7
+MIN_OBSERVED_NOISE_LEVEL = 1e-6
 
 
 def get_and_fit_model(
@@ -164,8 +162,6 @@ def get_and_fit_model(
         model = ModelListGP(*models)
     model.to(Xs[0])
     if state_dict is not None:
-        # pyre-fixme[6]: Expected `OrderedDict[typing.Any, typing.Any]` for 1st
-        #  param but got `Dict[str, Tensor]`.
         model.load_state_dict(state_dict)
     if state_dict is None or refit_model:
         # TODO: Add bounds for optimization stability - requires revamp upstream
@@ -178,7 +174,7 @@ def get_and_fit_model(
             mll = SumMarginalLogLikelihood(model.likelihood, model, mll_cls=mll_cls)
         else:
             mll = mll_cls(model.likelihood, model)
-        mll = fit_gpytorch_model(mll, bounds=bounds)
+        mll = fit_gpytorch_mll(mll, optimizer_kwargs={"bounds": bounds})
     return model
 
 
@@ -191,7 +187,7 @@ def get_NEI(
     **kwargs: Any,
 ) -> AcquisitionFunction:
     r"""Instantiates a qNoisyExpectedImprovement acquisition function."""
-    return _get_acqusition_func(
+    return _get_acquisition_func(
         model=model,
         acquisition_function_name="qNEI",
         objective_weights=objective_weights,
@@ -202,13 +198,20 @@ def get_NEI(
     )
 
 
-def _get_acqusition_func(
+def _get_acquisition_func(
     model: Model,
     acquisition_function_name: str,
     objective_weights: Tensor,
     outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
     X_observed: Optional[Tensor] = None,
     X_pending: Optional[Tensor] = None,
+    mc_objective: Type[GenericMCObjective] = GenericMCObjective,
+    constrained_mc_objective: Optional[
+        Type[ConstrainedMCObjective]
+    ] = ConstrainedMCObjective,
+    # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+    #  `typing.Dict` to avoid runtime subscripting errors.
+    mc_objective_kwargs: Optional[Dict] = None,
     **kwargs: Any,
 ) -> AcquisitionFunction:
     r"""Instantiates a acquisition function.
@@ -229,6 +232,14 @@ def _get_acqusition_func(
             that have been submitted for evaluation) present for all objective
             outcomes and outcomes that appear in the outcome constraints (if
             there are any).
+        mc_objective: GenericMCObjective class, used for constructing a
+            MC-objective. If constructing a penalized MC-objective, pass in
+            PenalizedMCObjective together with mc_objective_kwargs .
+        constrained_mc_objective: ConstrainedMCObjective class, used when
+            applying constraints on the outcomes.
+        mc_objective_kwargs: kwargs for constructing MC-objective.
+            For GenericMCObjective, leave it as None. For PenalizedMCObjective,
+            it needs to be specified in the format of kwargs.
         mc_samples: The number of MC samples to use (default: 512).
         qmc: If True, use qMC instead of MC (default: True).
         prune_baseline: If True, prune the baseline points for NEI (default: True).
@@ -247,15 +258,26 @@ def _get_acqusition_func(
     else:
         obj_tf = get_objective_weights_transform(objective_weights)
 
+    # pyre-fixme[53]: Captured variable `obj_tf` is not annotated.
     def objective(samples: Tensor, X: Optional[Tensor] = None) -> Tensor:
         return obj_tf(samples)
 
     if outcome_constraints is None:
-        objective = GenericMCObjective(objective=objective)
+        mc_objective_kwargs = {} if mc_objective_kwargs is None else mc_objective_kwargs
+        objective = mc_objective(objective=objective, **mc_objective_kwargs)
     else:
+        if constrained_mc_objective is None:
+            raise ValueError(
+                "constrained_mc_objective cannot be set to None "
+                "when applying outcome constraints."
+            )
+        if issubclass(mc_objective, PenalizedMCObjective):
+            raise RuntimeError(
+                "Outcome constraints are not supported for PenalizedMCObjective."
+            )
         con_tfs = get_outcome_constraint_transforms(outcome_constraints)
         inf_cost = get_infeasible_cost(X=X_observed, model=model, objective=objective)
-        objective = ConstrainedMCObjective(
+        objective = constrained_mc_objective(
             objective=objective, constraints=con_tfs or [], infeasible_cost=inf_cost
         )
     return get_acquisition_function(
@@ -512,7 +534,7 @@ def _get_model(
     Returns:
         A GPyTorchModel (unfitted).
     """
-    Yvar = Yvar.clamp_min(MIN_OBSERVED_NOISE_LEVEL)  # pyre-ignore[16]
+    Yvar = Yvar.clamp_min(MIN_OBSERVED_NOISE_LEVEL)
     is_nan = torch.isnan(Yvar)
     any_nan_Yvar = torch.any(is_nan)
     all_nan_Yvar = torch.all(is_nan)
@@ -529,7 +551,7 @@ def _get_model(
         warp_tf = get_warping_transform(
             d=X.shape[-1],
             task_feature=task_feature,
-            batch_shape=X.shape[:-2],  # pyre-ignore [6]
+            batch_shape=X.shape[:-2],
         )
     else:
         warp_tf = None
@@ -624,8 +646,8 @@ def get_warping_transform(
     tf = Warp(
         indices=indices,
         # prior with a median of 1
-        concentration1_prior=LogNormalPrior(0.0, 0.75 ** 0.5),
-        concentration0_prior=LogNormalPrior(0.0, 0.75 ** 0.5),
+        concentration1_prior=LogNormalPrior(0.0, 0.75**0.5),
+        concentration0_prior=LogNormalPrior(0.0, 0.75**0.5),
         batch_shape=batch_shape,
     )
     return tf
