@@ -1,16 +1,17 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 import dataclasses
 from copy import deepcopy
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type
+from typing import Any, Dict, List, Optional, Tuple, Type
 
 import torch
 from ax.core.search_space import SearchSpaceDigest
-from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
+from ax.core.types import TCandidateMetadata, TGenMetadata
+from ax.exceptions.core import UnsupportedError
 from ax.models.torch.botorch import get_rounding_func
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 from ax.models.torch.botorch_modular.list_surrogate import ListSurrogate
@@ -19,17 +20,18 @@ from ax.models.torch.botorch_modular.utils import (
     choose_botorch_acqf_class,
     choose_model_class,
     construct_acquisition_and_optimizer_options,
+    convert_to_block_design,
     use_model_list,
-    validate_data_format,
 )
 from ax.models.torch.utils import _to_inequality_constraints
-from ax.models.torch_base import TorchModel
+from ax.models.torch_base import TorchGenResults, TorchModel, TorchOptConfig
 from ax.utils.common.base import Base
 from ax.utils.common.constants import Keys
 from ax.utils.common.docutils import copy_doc
 from ax.utils.common.typeutils import checked_cast, not_none
 from botorch.acquisition.acquisition import AcquisitionFunction
-from botorch.utils.containers import TrainingData
+from botorch.models.deterministic import FixedSingleSampleModel
+from botorch.utils.datasets import SupervisedDataset
 from torch import Tensor
 
 
@@ -84,6 +86,7 @@ class BoTorchModel(TorchModel, Base):
     _surrogate: Optional[Surrogate]
     _botorch_acqf_class: Optional[Type[AcquisitionFunction]]
     _search_space_digest: Optional[SearchSpaceDigest] = None
+    _supports_robust_optimization: bool = True
 
     def __init__(
         self,
@@ -113,14 +116,30 @@ class BoTorchModel(TorchModel, Base):
         self.refit_on_cv = refit_on_cv
         self.warm_start_refit = warm_start_refit
 
+    @property
+    def Xs(self) -> List[Tensor]:
+        """A list of tensors, each of shape ``batch_shape x n_i x d``,
+        where `n_i` is the number of training inputs for the i-th model.
+
+        NOTE: This is an accessor for ``self.surrogate.Xs``
+        and returns it unchanged.
+        """
+        return self.surrogate.Xs
+
     @property
     def surrogate(self) -> Surrogate:
+        """Ax ``Surrogate`` object (wrapper for BoTorch ``Model``), associated with
+        this model. Raises an error if one is not yet set.
+        """
         if not self._surrogate:
             raise ValueError("Surrogate has not yet been set.")
         return not_none(self._surrogate)
 
     @property
     def botorch_acqf_class(self) -> Type[AcquisitionFunction]:
+        """BoTorch ``AcquisitionFunction`` class, associated with this model.
+        Raises an error if one is not yet set.
+        """
         if not self._botorch_acqf_class:
             raise ValueError("BoTorch `AcquisitionFunction` has not yet been set.")
         return not_none(self._botorch_acqf_class)
@@ -128,18 +147,19 @@ class BoTorchModel(TorchModel, Base):
     @copy_doc(TorchModel.fit)
     def fit(
         self,
-        Xs: List[Tensor],
-        Ys: List[Tensor],
-        Yvars: List[Tensor],
-        search_space_digest: SearchSpaceDigest,
+        datasets: List[SupervisedDataset],
         metric_names: List[str],
-        target_fidelities: Optional[Dict[int, float]] = None,
+        search_space_digest: SearchSpaceDigest,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
         state_dict: Optional[Dict[str, Tensor]] = None,
         refit: bool = True,
     ) -> None:
-        # Ensure that parts of data all have equal lengths.
-        validate_data_format(Xs=Xs, Ys=Ys, Yvars=Yvars, metric_names=metric_names)
+        if not len(datasets) == len(metric_names):
+            raise ValueError(
+                "Length of datasets and metric_names must match, but your inputs "
+                f"are of lengths {len(datasets)} and {len(metric_names)}, "
+                "respectively."
+            )
 
         # store search space info for later use (e.g. during generation)
         self._search_space_digest = search_space_digest
@@ -147,17 +167,25 @@ class BoTorchModel(TorchModel, Base):
         # Choose `Surrogate` and undelying `Model` based on properties of data.
         if not self._surrogate:
             self._autoset_surrogate(
-                Xs=Xs,
-                Ys=Ys,
-                Yvars=Yvars,
-                search_space_digest=search_space_digest,
+                datasets=datasets,
                 metric_names=metric_names,
+                search_space_digest=search_space_digest,
+            )
+
+        if len(datasets) > 1 and not isinstance(self.surrogate, ListSurrogate):
+            # Note: If the datasets do not confirm to a block design then this
+            # will filter the data and drop observations to make sure that it does.
+            # This can happen e.g. if only some metrics are observed at some points.
+            datasets, metric_names = convert_to_block_design(
+                datasets=datasets,
+                metric_names=metric_names,
+                force=True,
             )
 
         self.surrogate.fit(
-            training_data=TrainingData(Xs=Xs, Ys=Ys, Yvars=Yvars),
-            search_space_digest=search_space_digest,
+            datasets=datasets,
             metric_names=metric_names,
+            search_space_digest=search_space_digest,
             candidate_metadata=candidate_metadata,
             state_dict=state_dict,
             refit=refit,
@@ -166,15 +194,13 @@ class BoTorchModel(TorchModel, Base):
     @copy_doc(TorchModel.update)
     def update(
         self,
-        Xs: List[Tensor],
-        Ys: List[Tensor],
-        Yvars: List[Tensor],
-        search_space_digest: SearchSpaceDigest,
+        datasets: List[Optional[SupervisedDataset]],
         metric_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
         if not self._surrogate:
-            raise ValueError("Cannot update model that has not been fitted.")
+            raise UnsupportedError("Cannot update model that has not been fitted.")
 
         # store search space info  for later use (e.g. during generation)
         self._search_space_digest = search_space_digest
@@ -188,11 +214,14 @@ class BoTorchModel(TorchModel, Base):
             if self.refit_on_update and not self.warm_start_refit
             else self.surrogate.model.state_dict()
         )
-
+        if any(dataset is None for dataset in datasets):
+            raise UnsupportedError(
+                f"{self.__class__.__name__}.update requires data for all outcomes."
+            )
         self.surrogate.update(
-            training_data=TrainingData(Xs=Xs, Ys=Ys, Yvars=Yvars),
-            search_space_digest=search_space_digest,
+            datasets=[not_none(dataset) for dataset in datasets],
             metric_names=metric_names,
+            search_space_digest=search_space_digest,
             candidate_metadata=candidate_metadata,
             state_dict=state_dict,
             refit=self.refit_on_update,
@@ -206,105 +235,107 @@ class BoTorchModel(TorchModel, Base):
     def gen(
         self,
         n: int,
-        bounds: List[Tuple[float, float]],
-        objective_weights: Tensor,
-        objective_thresholds: Optional[Tensor] = None,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        pending_observations: Optional[List[Tensor]] = None,
-        model_gen_options: Optional[TConfig] = None,
-        rounding_func: Optional[Callable[[Tensor], Tensor]] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
-    ) -> Tuple[Tensor, Tensor, TGenMetadata, Optional[List[TCandidateMetadata]]]:
+        search_space_digest: SearchSpaceDigest,
+        torch_opt_config: TorchOptConfig,
+    ) -> TorchGenResults:
         if self._search_space_digest is None:
             raise RuntimeError("Must `fit` the model before calling `gen`.")
         acq_options, opt_options = construct_acquisition_and_optimizer_options(
-            acqf_options=self.acquisition_options, model_gen_options=model_gen_options
+            acqf_options=self.acquisition_options,
+            model_gen_options=torch_opt_config.model_gen_options,
         )
         # update bounds / target fidelities
-        new_ssd_args = {
-            **dataclasses.asdict(self._search_space_digest),
-            "bounds": bounds,
-            "target_fidelities": target_fidelities or {},
-        }
-        search_space_digest = SearchSpaceDigest(**new_ssd_args)
-
+        search_space_digest = not_none(
+            dataclasses.replace(
+                self._search_space_digest,
+                bounds=search_space_digest.bounds,
+                target_fidelities=search_space_digest.target_fidelities or {},
+            )
+        )
         acqf = self._instantiate_acquisition(
             search_space_digest=search_space_digest,
-            objective_weights=objective_weights,
-            objective_thresholds=objective_thresholds,
-            outcome_constraints=outcome_constraints,
-            linear_constraints=linear_constraints,
-            fixed_features=fixed_features,
-            pending_observations=pending_observations,
+            torch_opt_config=torch_opt_config,
             acq_options=acq_options,
         )
-
-        botorch_rounding_func = get_rounding_func(rounding_func)
+        botorch_rounding_func = get_rounding_func(torch_opt_config.rounding_func)
         candidates, expected_acquisition_value = acqf.optimize(
             n=n,
             search_space_digest=search_space_digest,
             inequality_constraints=_to_inequality_constraints(
-                linear_constraints=linear_constraints
+                linear_constraints=torch_opt_config.linear_constraints
             ),
-            fixed_features=fixed_features,
+            fixed_features=torch_opt_config.fixed_features,
             rounding_func=botorch_rounding_func,
             optimizer_options=checked_cast(dict, opt_options),
         )
-        return (
-            candidates.detach().cpu(),
-            torch.ones(n, dtype=self.surrogate.dtype),
-            {Keys.EXPECTED_ACQF_VAL: expected_acquisition_value.tolist()},
-            None,
+        gen_metadata = self._get_gen_metadata_from_acqf(
+            acqf=acqf,
+            torch_opt_config=torch_opt_config,
+            expected_acquisition_value=expected_acquisition_value,
         )
+        return TorchGenResults(
+            points=candidates.detach().cpu(),
+            weights=torch.ones(n, dtype=self.surrogate.dtype),
+            gen_metadata=gen_metadata,
+        )
+
+    def _get_gen_metadata_from_acqf(
+        self,
+        acqf: Acquisition,
+        torch_opt_config: TorchOptConfig,
+        expected_acquisition_value: Tensor,
+    ) -> TGenMetadata:
+        gen_metadata: TGenMetadata = {
+            Keys.EXPECTED_ACQF_VAL: expected_acquisition_value.tolist()
+        }
+        if torch_opt_config.objective_weights.nonzero().numel() > 1:
+            gen_metadata["objective_thresholds"] = acqf.objective_thresholds
+            gen_metadata["objective_weights"] = acqf.objective_weights
+
+        if hasattr(acqf.acqf, "outcome_model"):
+            outcome_model = acqf.acqf.outcome_model
+            if isinstance(
+                outcome_model,
+                FixedSingleSampleModel,
+            ):
+                gen_metadata["outcome_model_fixed_draw_weights"] = outcome_model.w
+        return gen_metadata
 
     @copy_doc(TorchModel.best_point)
     def best_point(
         self,
-        bounds: List[Tuple[float, float]],
-        objective_weights: Tensor,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        model_gen_options: Optional[TConfig] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
+        search_space_digest: SearchSpaceDigest,
+        torch_opt_config: TorchOptConfig,
     ) -> Optional[Tensor]:
-        raise NotImplementedError("Coming soon.")
+        try:
+            return self.surrogate.best_in_sample_point(
+                search_space_digest=search_space_digest,
+                torch_opt_config=torch_opt_config,
+            )[0]
+        except ValueError:
+            return None
 
     @copy_doc(TorchModel.evaluate_acquisition_function)
     def evaluate_acquisition_function(
         self,
         X: Tensor,
         search_space_digest: SearchSpaceDigest,
-        objective_weights: Tensor,
-        objective_thresholds: Optional[Tensor] = None,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        pending_observations: Optional[List[Tensor]] = None,
+        torch_opt_config: TorchOptConfig,
         acq_options: Optional[Dict[str, Any]] = None,
     ) -> Tensor:
         acqf = self._instantiate_acquisition(
             search_space_digest=search_space_digest,
-            objective_weights=objective_weights,
-            objective_thresholds=objective_thresholds,
-            outcome_constraints=outcome_constraints,
-            linear_constraints=linear_constraints,
-            fixed_features=fixed_features,
-            pending_observations=pending_observations,
+            torch_opt_config=torch_opt_config,
             acq_options=acq_options,
         )
         return acqf.evaluate(X=X)
 
     def cross_validate(
         self,
-        Xs_train: List[Tensor],
-        Ys_train: List[Tensor],
-        Yvars_train: List[Tensor],
+        datasets: List[SupervisedDataset],
+        metric_names: List[str],
         X_test: Tensor,
         search_space_digest: SearchSpaceDigest,
-        metric_names: List[str],
     ) -> Tuple[Tensor, Tensor]:
         current_surrogate = self.surrogate
         # If we should be refitting but not warm-starting the refit, set
@@ -320,14 +351,17 @@ class BoTorchModel(TorchModel, Base):
         # use it to predict the test point.
         surrogate_clone = self.surrogate.clone_reset()
         self._surrogate = surrogate_clone
+        # Remove the robust_digest since we do not want to use perturbations here.
+        search_space_digest = dataclasses.replace(
+            search_space_digest,
+            robust_digest=None,
+        )
 
         try:
             self.fit(
-                Xs=Xs_train,
-                Ys=Ys_train,
-                Yvars=Yvars_train,
-                search_space_digest=search_space_digest,
+                datasets=datasets,
                 metric_names=metric_names,
+                search_space_digest=search_space_digest,
                 state_dict=state_dict,
                 refit=self.refit_on_cv,
             )
@@ -341,11 +375,9 @@ class BoTorchModel(TorchModel, Base):
 
     def _autoset_surrogate(
         self,
-        Xs: List[Tensor],
-        Ys: List[Tensor],
-        Yvars: List[Tensor],
-        search_space_digest: SearchSpaceDigest,
+        datasets: List[SupervisedDataset],
         metric_names: List[str],
+        search_space_digest: SearchSpaceDigest,
     ) -> None:
         """Sets a default surrogate on this model if one was not explicitly
         provided.
@@ -354,18 +386,18 @@ class BoTorchModel(TorchModel, Base):
         # the batched multi-output case, so we first see which model would
         # be chosen given the Yvars and the properties of data.
         botorch_model_class = choose_model_class(
-            Yvars=Yvars,
+            datasets=datasets,
             search_space_digest=search_space_digest,
         )
-        if use_model_list(Xs=Xs, botorch_model_class=botorch_model_class):
+        if use_model_list(datasets=datasets, botorch_model_class=botorch_model_class):
             # If using `ListSurrogate` / `ModelListGP`, pick submodels for each
             # outcome.
             botorch_submodel_class_per_outcome = {
                 metric_name: choose_model_class(
-                    Yvars=[Yvar],
+                    datasets=[dataset],
                     search_space_digest=search_space_digest,
                 )
-                for Yvar, metric_name in zip(Yvars, metric_names)
+                for dataset, metric_name in zip(datasets, metric_names)
             }
             self._surrogate = ListSurrogate(
                 botorch_submodel_class_per_outcome=botorch_submodel_class_per_outcome,
@@ -381,38 +413,33 @@ class BoTorchModel(TorchModel, Base):
     def _instantiate_acquisition(
         self,
         search_space_digest: SearchSpaceDigest,
-        objective_weights: Tensor,
-        objective_thresholds: Optional[Tensor] = None,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        pending_observations: Optional[List[Tensor]] = None,
+        torch_opt_config: TorchOptConfig,
         acq_options: Optional[Dict[str, Any]] = None,
     ) -> Acquisition:
-        """Set an BoTorch acquisition function class for this model if needed and
+        """Set a BoTorch acquisition function class for this model if needed and
         instantiate it.
 
         Returns:
-            BoTorch ``AcquisitionFunction`` instance.
+            A BoTorch ``AcquisitionFunction`` instance.
         """
         if not self._botorch_acqf_class:
+            if torch_opt_config.risk_measure is not None:  # pragma: no cover
+                # TODO[T131759261]: Implement selection of acqf for robust opt.
+                # This will depend on the properties of the robust search space and
+                # the risk measure being used.
+                raise NotImplementedError
             self._botorch_acqf_class = choose_botorch_acqf_class(
-                objective_thresholds=objective_thresholds,
-                outcome_constraints=outcome_constraints,
-                linear_constraints=linear_constraints,
-                fixed_features=fixed_features,
-                pending_observations=pending_observations,
+                pending_observations=torch_opt_config.pending_observations,
+                outcome_constraints=torch_opt_config.outcome_constraints,
+                linear_constraints=torch_opt_config.linear_constraints,
+                fixed_features=torch_opt_config.fixed_features,
+                objective_thresholds=torch_opt_config.objective_thresholds,
+                objective_weights=torch_opt_config.objective_weights,
             )
-
         return self.acquisition_class(
             surrogate=self.surrogate,
             botorch_acqf_class=self.botorch_acqf_class,
             search_space_digest=search_space_digest,
-            objective_weights=objective_weights,
-            objective_thresholds=objective_thresholds,
-            outcome_constraints=outcome_constraints,
-            linear_constraints=linear_constraints,
-            fixed_features=fixed_features,
-            pending_observations=pending_observations,
+            torch_opt_config=torch_opt_config,
             options=acq_options,
         )
