@@ -1,31 +1,32 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from logging import Logger
 from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import torch
-from ax.core.types import TCandidateMetadata, TConfig, TGenMetadata
+from ax.core.search_space import SearchSpaceDigest
 from ax.exceptions.core import AxError
 from ax.models.torch.botorch import (
     BotorchModel,
+    get_rounding_func,
     TAcqfConstructor,
     TBestPointRecommender,
     TModelConstructor,
     TModelPredictor,
     TOptimizer,
-    get_rounding_func,
 )
 from ax.models.torch.botorch_defaults import (
     get_and_fit_model,
-    predict_from_model,
     recommend_best_observed_point,
     scipy_optimizer,
 )
 from ax.models.torch.botorch_moo_defaults import (
     get_NEHVI,
+    infer_objective_thresholds,
     pareto_frontier_evaluator,
     scipy_optimizer_list,
 )
@@ -33,24 +34,23 @@ from ax.models.torch.frontier_utils import TFrontierEvaluator
 from ax.models.torch.utils import (
     _get_X_pending_and_observed,
     _to_inequality_constraints,
+    predict_from_model,
     randomize_objective_weights,
     subset_model,
 )
-from ax.models.torch.utils import get_outcome_constraint_transforms
-from ax.models.torch_base import TorchModel
+from ax.models.torch_base import TorchGenResults, TorchModel, TorchOptConfig
 from ax.utils.common.constants import Keys
 from ax.utils.common.docutils import copy_doc
 from ax.utils.common.logger import get_logger
 from ax.utils.common.typeutils import checked_cast, not_none
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.models.model import Model
-from botorch.utils.multi_objective.hypervolume import infer_reference_point
-from botorch.utils.multi_objective.pareto import is_non_dominated
 from torch import Tensor
 
 
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 
+# pyre-fixme[33]: Aliased annotation cannot contain `Any`.
 TOptimizerList = Callable[
     [
         List[AcquisitionFunction],
@@ -216,6 +216,7 @@ class MultiObjectiveBotorchModel(BotorchModel):
         self.acqf_optimizer = acqf_optimizer
         self.best_point_recommender = best_point_recommender
         self.frontier_evaluator = frontier_evaluator
+        # pyre-fixme[4]: Attribute must be annotated.
         self._kwargs = kwargs
         self.refit_on_cv = refit_on_cv
         self.refit_on_update = refit_on_update
@@ -232,106 +233,25 @@ class MultiObjectiveBotorchModel(BotorchModel):
         self.fidelity_features: List[int] = []
         self.metric_names: List[str] = []
 
-    def infer_objective_thresholds(
-        self,
-        objective_weights: Tensor,  # objective_directions
-        bounds: Optional[List[Tuple[float, float]]] = None,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        X_observed: Optional[Tensor] = None,
-        model: Optional[Model] = None,
-        subset_idcs: Optional[Tensor] = None,
-    ) -> Tensor:
-        """Infer objective thresholds.
-
-        Returns:
-            A `m`-dim tensor of objective thresholds, where the objective
-                threshold is `nan` if the outcome is not an objective.
-        """
-        if X_observed is None:
-            if bounds is None:
-                raise ValueError("bounds is required if X_observed is None.")
-            _, X_observed = _get_X_pending_and_observed(
-                Xs=self.Xs,
-                objective_weights=objective_weights,
-                outcome_constraints=outcome_constraints,
-                bounds=bounds,
-                linear_constraints=linear_constraints,
-                fixed_features=fixed_features,
-            )
-        if model is not None:
-            if subset_idcs is None:
-                raise ValueError(
-                    "subset_idcs must be provided if the model is provided."
-                )
-        else:
-            # subset the model
-            subset_model_results = subset_model(
-                model=self.model,  # pyre-ignore [6]
-                objective_weights=objective_weights,
-                outcome_constraints=outcome_constraints,
-            )
-            model = subset_model_results.model
-            objective_weights = subset_model_results.objective_weights
-            outcome_constraints = subset_model_results.outcome_constraints
-            subset_idcs = subset_model_results.indices
-        with torch.no_grad():
-            pred = not_none(model).posterior(not_none(X_observed)).mean
-        if outcome_constraints is not None:
-            cons_tfs = get_outcome_constraint_transforms(outcome_constraints)
-            # pyre-ignore [16]
-            feas = torch.stack([c(pred) <= 0 for c in cons_tfs], dim=-1).all(dim=-1)
-            pred = pred[feas]
-        if pred.shape[0] == 0:
-            raise AxError("There are no feasible observed points.")
-        # pyre-ignore [16]
-        obj_mask = objective_weights.nonzero().view(-1)
-        obj_weights_subset = objective_weights[obj_mask]
-        obj = pred[..., obj_mask] * obj_weights_subset
-        pareto_obj = obj[is_non_dominated(obj)]
-        objective_thresholds = infer_reference_point(
-            pareto_Y=pareto_obj,
-            scale=0.1,
-        )
-        # multiply by objective weights to return objective thresholds in the
-        # unweighted space
-        objective_thresholds = objective_thresholds * obj_weights_subset
-        full_objective_thresholds = torch.full(
-            (len(self.metric_names),),
-            float("nan"),
-            dtype=objective_weights.dtype,
-            device=objective_weights.device,
-        )
-        full_objective_thresholds[subset_idcs] = objective_thresholds.clone()
-        return full_objective_thresholds
-
     @copy_doc(TorchModel.gen)
     def gen(
         self,
         n: int,
-        bounds: List[Tuple[float, float]],
-        objective_weights: Tensor,  # objective_directions
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        objective_thresholds: Optional[Tensor] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        pending_observations: Optional[List[Tensor]] = None,
-        model_gen_options: Optional[TConfig] = None,
-        rounding_func: Optional[Callable[[Tensor], Tensor]] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
-    ) -> Tuple[Tensor, Tensor, TGenMetadata, Optional[List[TCandidateMetadata]]]:
-        options = model_gen_options or {}
+        search_space_digest: SearchSpaceDigest,
+        torch_opt_config: TorchOptConfig,
+    ) -> TorchGenResults:
+        options = torch_opt_config.model_gen_options or {}
         acf_options = options.get("acquisition_function_kwargs", {})
         optimizer_options = options.get("optimizer_kwargs", {})
 
-        if target_fidelities:
+        if search_space_digest.target_fidelities:  # untested
             raise NotImplementedError(
                 "target_fidelities not implemented for base BotorchModel"
             )
         if (
-            objective_thresholds is not None
-            and objective_weights.shape[0] != objective_thresholds.shape[0]
+            torch_opt_config.objective_thresholds is not None
+            and torch_opt_config.objective_weights.shape[0]
+            != not_none(torch_opt_config.objective_thresholds).shape[0]
         ):
             raise AxError(
                 "Objective weights and thresholds most both contain an element for"
@@ -340,23 +260,25 @@ class MultiObjectiveBotorchModel(BotorchModel):
 
         X_pending, X_observed = _get_X_pending_and_observed(
             Xs=self.Xs,
-            pending_observations=pending_observations,
-            objective_weights=objective_weights,
-            outcome_constraints=outcome_constraints,
-            bounds=bounds,
-            linear_constraints=linear_constraints,
-            fixed_features=fixed_features,
+            objective_weights=torch_opt_config.objective_weights,
+            bounds=search_space_digest.bounds,
+            pending_observations=torch_opt_config.pending_observations,
+            outcome_constraints=torch_opt_config.outcome_constraints,
+            linear_constraints=torch_opt_config.linear_constraints,
+            fixed_features=torch_opt_config.fixed_features,
         )
 
         model = not_none(self.model)
-        full_objective_thresholds = objective_thresholds
+        full_objective_thresholds = torch_opt_config.objective_thresholds
+        full_objective_weights = torch_opt_config.objective_weights
+        full_outcome_constraints = torch_opt_config.outcome_constraints
         # subset model only to the outcomes we need for the optimization
         if options.get(Keys.SUBSET_MODEL, True):
             subset_model_results = subset_model(
                 model=model,
-                objective_weights=objective_weights,
-                outcome_constraints=outcome_constraints,
-                objective_thresholds=objective_thresholds,
+                objective_weights=torch_opt_config.objective_weights,
+                outcome_constraints=torch_opt_config.outcome_constraints,
+                objective_thresholds=torch_opt_config.objective_thresholds,
             )
             model = subset_model_results.model
             objective_weights = subset_model_results.objective_weights
@@ -364,28 +286,30 @@ class MultiObjectiveBotorchModel(BotorchModel):
             objective_thresholds = subset_model_results.objective_thresholds
             idcs = subset_model_results.indices
         else:
-            idcs = torch.arange(
-                objective_weights.shape[0],
-                dtype=objective_weights.dtype,
-                device=objective_weights.device,
-            )
+            objective_weights = torch_opt_config.objective_weights
+            outcome_constraints = torch_opt_config.outcome_constraints
+            objective_thresholds = torch_opt_config.objective_thresholds
+            idcs = None
         if objective_thresholds is None:
-            full_objective_thresholds = self.infer_objective_thresholds(
-                X_observed=not_none(X_observed),
-                objective_weights=objective_weights,
-                outcome_constraints=outcome_constraints,
+            full_objective_thresholds = infer_objective_thresholds(
                 model=model,
+                X_observed=not_none(X_observed),
+                objective_weights=full_objective_weights,
+                outcome_constraints=full_outcome_constraints,
                 subset_idcs=idcs,
             )
-
             # subset the objective thresholds
-            objective_thresholds = full_objective_thresholds[idcs].clone()
-        else:
-            full_objective_thresholds = objective_thresholds
+            objective_thresholds = (
+                full_objective_thresholds
+                if idcs is None
+                else full_objective_thresholds[idcs].clone()
+            )
 
-        bounds_ = torch.tensor(bounds, dtype=self.dtype, device=self.device)
+        bounds_ = torch.tensor(
+            search_space_digest.bounds, dtype=self.dtype, device=self.device
+        )
         bounds_ = bounds_.transpose(0, 1)
-        botorch_rounding_func = get_rounding_func(rounding_func)
+        botorch_rounding_func = get_rounding_func(torch_opt_config.rounding_func)
         if acf_options.get("random_scalarization", False) or acf_options.get(
             "chebyshev_scalarization", False
         ):
@@ -417,9 +341,9 @@ class MultiObjectiveBotorchModel(BotorchModel):
                 acq_function_list=acquisition_function_list,
                 bounds=bounds_,
                 inequality_constraints=_to_inequality_constraints(
-                    linear_constraints=linear_constraints
+                    linear_constraints=torch_opt_config.linear_constraints
                 ),
-                fixed_features=fixed_features,
+                fixed_features=torch_opt_config.fixed_features,
                 rounding_func=botorch_rounding_func,
                 **optimizer_options,
             )
@@ -442,19 +366,19 @@ class MultiObjectiveBotorchModel(BotorchModel):
                 bounds=bounds_,
                 n=n,
                 inequality_constraints=_to_inequality_constraints(
-                    linear_constraints=linear_constraints
+                    linear_constraints=torch_opt_config.linear_constraints
                 ),
-                fixed_features=fixed_features,
+                fixed_features=torch_opt_config.fixed_features,
                 rounding_func=botorch_rounding_func,
                 **optimizer_options,
             )
         gen_metadata = {
             "expected_acquisition_value": expected_acquisition_value.tolist(),
-            "objective_thresholds": full_objective_thresholds.cpu(),
+            "objective_thresholds": not_none(full_objective_thresholds).cpu(),
+            "objective_weights": full_objective_weights.cpu(),
         }
-        return (
-            candidates.detach().cpu(),
-            torch.ones(n, dtype=self.dtype),
-            gen_metadata,
-            None,
+        return TorchGenResults(
+            points=candidates.detach().cpu(),
+            weights=torch.ones(n, dtype=self.dtype),
+            gen_metadata=gen_metadata,
         )
