@@ -1,29 +1,31 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from logging import Logger
 from typing import Any, Dict, List, Optional, Tuple
 
 from ax.core.search_space import SearchSpaceDigest
-from ax.core.types import TCandidateMetadata, TConfig
+from ax.core.types import TCandidateMetadata
 from ax.models.torch.alebo import ei_or_nei
 from ax.models.torch.botorch import BotorchModel
 from ax.models.torch.cbo_sac import generate_model_space_decomposition
-from ax.models.torch_base import TorchModel
+from ax.models.torch_base import TorchModel, TorchOptConfig
 from ax.utils.common.docutils import copy_doc
 from ax.utils.common.logger import get_logger
-from botorch.fit import fit_gpytorch_model
+from botorch.fit import fit_gpytorch_mll
 from botorch.models.contextual import LCEAGP
 from botorch.models.gpytorch import GPyTorchModel
 from botorch.models.model_list_gp_regression import ModelListGP
+from botorch.utils.datasets import SupervisedDataset
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
 from torch import Tensor
 
 
 MIN_OBSERVED_NOISE_LEVEL = 1e-7
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 
 
 def get_map_model(
@@ -32,9 +34,15 @@ def get_map_model(
     train_Yvar: Tensor,
     decomposition: Dict[str, List[int]],
     train_embedding: bool = True,
+    # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+    #  `typing.Dict` to avoid runtime subscripting errors.
     cat_feature_dict: Optional[Dict] = None,
+    # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+    #  `typing.Dict` to avoid runtime subscripting errors.
     embs_feature_dict: Optional[Dict] = None,
     embs_dim_list: Optional[List[int]] = None,
+    # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+    #  `typing.Dict` to avoid runtime subscripting errors.
     context_weight_dict: Optional[Dict] = None,
 ) -> Tuple[LCEAGP, ExactMarginalLogLikelihood]:
     """Obtain MAP fitting of Latent Context Embedding Additive (LCE-A) GP."""
@@ -52,7 +60,7 @@ def get_map_model(
         context_weight_dict=context_weight_dict,
     )
     mll = ExactMarginalLogLikelihood(model.likelihood, model)
-    fit_gpytorch_model(mll)
+    fit_gpytorch_mll(mll)
     return model, mll
 
 
@@ -73,8 +81,14 @@ class LCEABO(BotorchModel):
     def __init__(
         self,
         decomposition: Dict[str, List[str]],
+        # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+        #  `typing.Dict` to avoid runtime subscripting errors.
         cat_feature_dict: Optional[Dict] = None,
+        # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+        #  `typing.Dict` to avoid runtime subscripting errors.
         embs_feature_dict: Optional[Dict] = None,
+        # pyre-fixme[24]: Generic type `dict` expects 2 type parameters, use
+        #  `typing.Dict` to avoid runtime subscripting errors.
         context_weight_dict: Optional[Dict] = None,
         embs_dim_list: Optional[List[int]] = None,
         gp_model_args: Optional[Dict[str, Any]] = None,
@@ -89,8 +103,10 @@ class LCEABO(BotorchModel):
         self.embs_feature_dict = embs_feature_dict
         self.context_weight_dict = context_weight_dict
         self.embs_dim_list = embs_dim_list
+        # pyre-fixme[4]: Attribute must be annotated.
         self.gp_model_args = gp_model_args if gp_model_args is not None else {}
         self.feature_names: List[str] = []
+        # pyre-fixme[4]: Attribute must be annotated.
         self.train_embedding = self.gp_model_args.get("train_embedding", True)
         super().__init__(
             model_constructor=self.get_and_fit_model,
@@ -100,34 +116,25 @@ class LCEABO(BotorchModel):
     @copy_doc(TorchModel.fit)
     def fit(
         self,
-        Xs: List[Tensor],
-        Ys: List[Tensor],
-        Yvars: List[Tensor],
-        search_space_digest: SearchSpaceDigest,
+        datasets: List[SupervisedDataset],
         metric_names: List[str],
+        search_space_digest: SearchSpaceDigest,
         candidate_metadata: Optional[List[List[TCandidateMetadata]]] = None,
     ) -> None:
         if len(search_space_digest.feature_names) == 0:
             raise ValueError("feature names are required for LCEABO")
         self.feature_names = search_space_digest.feature_names
         super().fit(
-            Xs=Xs,
-            Ys=Ys,
-            Yvars=Yvars,
-            search_space_digest=search_space_digest,
+            datasets=datasets,
             metric_names=metric_names,
+            search_space_digest=search_space_digest,
         )
 
     @copy_doc(TorchModel.best_point)
     def best_point(
         self,
-        bounds: List[Tuple[float, float]],
-        objective_weights: Tensor,
-        outcome_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        linear_constraints: Optional[Tuple[Tensor, Tensor]] = None,
-        fixed_features: Optional[Dict[int, float]] = None,
-        model_gen_options: Optional[TConfig] = None,
-        target_fidelities: Optional[Dict[int, float]] = None,
+        search_space_digest: SearchSpaceDigest,
+        torch_opt_config: TorchOptConfig,
     ) -> Optional[Tensor]:
         raise NotImplementedError
 
