@@ -1,9 +1,10 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from copy import deepcopy
 from typing import Any, Dict, List, Optional, Tuple
 
 import numpy as np
@@ -23,10 +24,9 @@ from ax.plot.base import (
     PlotMetric,
     Z,
 )
+from ax.plot.helper import compose_annotation
 from ax.plot.scatter import _error_scatter_data, _error_scatter_trace
 from ax.utils.common.typeutils import not_none
-
-# pyre-fixme[21]: Could not find name `subplots` in `plotly`.
 from plotly import subplots
 
 
@@ -86,7 +86,7 @@ def _obs_vs_pred_dropdown_plot(
     show_context: bool = False,
     xlabel: str = "Actual Outcome",
     ylabel: str = "Predicted Outcome",
-) -> Dict[str, Any]:
+) -> go.Figure:
     """Plot a dropdown plot of observed vs. predicted values from a model.
 
     Args:
@@ -113,25 +113,20 @@ def _obs_vs_pred_dropdown_plot(
 
     for i, metric in enumerate(data.metrics):
         y_raw, se_raw, y_hat, se_hat = _error_scatter_data(
-            # Expected `List[typing.Union[PlotInSampleArm,
-            # ax.plot.base.PlotOutOfSampleArm]]` for 1st anonymous
-            # parameter to call `ax.plot.scatter._error_scatter_data` but got
-            # `List[PlotInSampleArm]`.
-            # pyre-fixme[6]:
             list(data.in_sample.values()),
             y_axis_var=PlotMetric(metric, pred=True, rel=rel),
             x_axis_var=PlotMetric(metric, pred=False, rel=rel),
             status_quo_arm=status_quo_arm,
         )
-        min_, max_ = _get_min_max_with_errors(y_raw, y_hat, se_raw or [], se_hat)
+        se_raw = (
+            [0.0 if np.isnan(se) else se for se in se_raw]
+            if se_raw is not None
+            else [0.0] * len(y_raw)
+        )
+        min_, max_ = _get_min_max_with_errors(y_raw, y_hat, se_raw, se_hat)
         traces.append(_diagonal_trace(min_, max_, visible=(i == 0)))
         traces.append(
             _error_scatter_trace(
-                # Expected `List[typing.Union[PlotInSampleArm,
-                # ax.plot.base.PlotOutOfSampleArm]]` for 1st parameter
-                # `arms` to call `ax.plot.scatter._error_scatter_trace`
-                # but got `List[PlotInSampleArm]`.
-                # pyre-fixme[6]:
                 arms=list(data.in_sample.values()),
                 hoverinfo="text",
                 show_arm_details_on_hover=True,
@@ -289,9 +284,18 @@ def _get_batch_comparison_plot_data(
         for i, mname in enumerate(x_observation.data.metric_names):
             # pyre-fixme[16]: Optional type has no attribute `__setitem__`.
             arm_data["y"][mname] = x_observation.data.means[i]
+            # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+            #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]], str]`
+            #  has no attribute `__setitem__`.
             arm_data["se"][mname] = np.sqrt(x_observation.data.covariance[i][i])
         for i, mname in enumerate(y_observation.data.metric_names):
+            # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+            #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]], str]`
+            #  has no attribute `__setitem__`.
             arm_data["y_hat"][mname] = y_observation.data.means[i]
+            # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+            #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]], str]`
+            #  has no attribute `__setitem__`.
             arm_data["se_hat"][mname] = np.sqrt(y_observation.data.covariance[i][i])
         # Expected `str` for 2nd anonymous parameter to call `dict.__setitem__` but got
         # `Optional[str]`.
@@ -306,17 +310,33 @@ def _get_batch_comparison_plot_data(
     )
 
 
-def _get_cv_plot_data(cv_results: List[CVResult]) -> PlotData:
+def _get_cv_plot_data(
+    cv_results: List[CVResult], label_dict: Optional[Dict[str, str]]
+) -> PlotData:
     if len(cv_results) == 0:
         return PlotData(
             metrics=[], in_sample={}, out_of_sample=None, status_quo_name=None
         )
 
+    if label_dict is None:
+        label_dict = {}
+    # Apply label_dict to cv_results
+    cv_results = deepcopy(cv_results)  # Copy and edit in-place
+    for cv_i in cv_results:
+        cv_i.observed.data.metric_names = [
+            label_dict.get(m, m) for m in cv_i.observed.data.metric_names
+        ]
+        cv_i.predicted.metric_names = [
+            label_dict.get(m, m) for m in cv_i.predicted.metric_names
+        ]
+
     # arm_name -> Arm data
     insample_data: Dict[str, PlotInSampleArm] = {}
 
-    # Assume input is well formed and this is consistent
-    metric_names = cv_results[0].predicted.metric_names
+    # Get the union of all metric_names seen in predictions
+    metric_names = list(
+        set().union(*(cv_result.predicted.metric_names for cv_result in cv_results))
+    )
 
     for rid, cv_result in enumerate(cv_results):
         arm_name = cv_result.observed.arm_name
@@ -332,9 +352,18 @@ def _get_cv_plot_data(cv_results: List[CVResult]) -> PlotData:
         for i, mname in enumerate(cv_result.observed.data.metric_names):
             # pyre-fixme[16]: Optional type has no attribute `__setitem__`.
             arm_data["y"][mname] = cv_result.observed.data.means[i]
+            # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+            #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]], str]`
+            #  has no attribute `__setitem__`.
             arm_data["se"][mname] = np.sqrt(cv_result.observed.data.covariance[i][i])
         for i, mname in enumerate(cv_result.predicted.metric_names):
+            # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+            #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]], str]`
+            #  has no attribute `__setitem__`.
             arm_data["y_hat"][mname] = cv_result.predicted.means[i]
+            # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+            #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]], str]`
+            #  has no attribute `__setitem__`.
             arm_data["se_hat"][mname] = np.sqrt(cv_result.predicted.covariance[i][i])
 
         # Expected `str` for 2nd anonymous parameter to call `dict.__setitem__` but got
@@ -383,9 +412,15 @@ def interact_empirical_model_validation(batch: BatchTrial, data: Data) -> AxPlot
                 metric_name = row["metric_name"]
                 # pyre-fixme[16]: Optional type has no attribute `__setitem__`.
                 arm_data["y"][metric_name] = row["mean"]
+                # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+                #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]],
+                #  str]` has no attribute `__setitem__`.
                 arm_data["se"][metric_name] = row["sem"]
                 # pyre-fixme[16]: `Optional` has no attribute `__getitem__`.
                 arm_data["y_hat"][metric_name] = predictions[0][metric_name][i]
+                # pyre-fixme[16]: Item `None` of `Union[None, Dict[typing.Any,
+                #  typing.Any], Dict[str, typing.Union[None, bool, float, int, str]],
+                #  str]` has no attribute `__setitem__`.
                 arm_data["se_hat"][metric_name] = predictions[1][metric_name][
                     metric_name
                 ][i]
@@ -405,8 +440,39 @@ def interact_empirical_model_validation(batch: BatchTrial, data: Data) -> AxPlot
     return AxPlotConfig(data=fig, plot_type=AxPlotTypes.GENERIC)
 
 
+def interact_cross_validation_plotly(
+    cv_results: List[CVResult],
+    show_context: bool = True,
+    caption: str = "",
+    label_dict: Optional[Dict[str, str]] = None,
+) -> go.Figure:
+    """Interactive cross-validation (CV) plotting; select metric via dropdown.
+
+    Note: uses the Plotly version of dropdown (which means that all data is
+    stored within the notebook).
+
+    Args:
+        cv_results: cross-validation results.
+        show_context: if True, show context on hover.
+        label_dict: optional map from real metric names to shortened names
+
+    Returns a plotly.graph_objects.Figure
+    """
+    data = _get_cv_plot_data(cv_results, label_dict=label_dict)
+    fig = _obs_vs_pred_dropdown_plot(data=data, rel=False, show_context=show_context)
+    current_bmargin = fig["layout"]["margin"].b or 90
+    caption_height = 100 * (len(caption) > 0)
+    fig["layout"]["margin"].b = current_bmargin + caption_height
+    fig["layout"]["height"] += caption_height
+    fig["layout"]["annotations"] += tuple(compose_annotation(caption))
+    fig["layout"]["title"] = "Cross-validation"
+    return fig
+
+
 def interact_cross_validation(
-    cv_results: List[CVResult], show_context: bool = True
+    cv_results: List[CVResult],
+    show_context: bool = True,
+    label_dict: Optional[Dict[str, str]] = None,
 ) -> AxPlotConfig:
     """Interactive cross-validation (CV) plotting; select metric via dropdown.
 
@@ -416,18 +482,25 @@ def interact_cross_validation(
     Args:
         cv_results: cross-validation results.
         show_context: if True, show context on hover.
+        label_dict: optional map from real metric names to shortened names
 
+    Returns an AxPlotConfig
     """
-    data = _get_cv_plot_data(cv_results)
-    fig = _obs_vs_pred_dropdown_plot(data=data, rel=False, show_context=show_context)
-    fig["layout"]["title"] = "Cross-validation"
-    return AxPlotConfig(data=fig, plot_type=AxPlotTypes.GENERIC)
+    return AxPlotConfig(
+        data=interact_cross_validation_plotly(
+            cv_results=cv_results,
+            show_context=show_context,
+            label_dict=label_dict,
+        ),
+        plot_type=AxPlotTypes.GENERIC,
+    )
 
 
 def tile_cross_validation(
     cv_results: List[CVResult],
     show_arm_details_on_hover: bool = True,
     show_context: bool = True,
+    label_dict: Optional[Dict[str, str]] = None,
 ) -> AxPlotConfig:
     """Tile version of CV plots; sorted by 'best fitting' outcomes.
 
@@ -442,15 +515,16 @@ def tile_cross_validation(
             parameterizations of arms on hover. Default is True.
         show_context: if True (default), display context on
             hover.
+        label_dict: optional map from real metric names to shortened names
 
+    Returns a plotly.graph_objects.Figure
     """
-    data = _get_cv_plot_data(cv_results)
+    data = _get_cv_plot_data(cv_results, label_dict=label_dict)
     metrics = data.metrics
 
     # make subplots (2 plots per row)
     nrows = int(np.ceil(len(metrics) / 2))
     ncols = min(len(metrics), 2)
-    # pyre-fixme[16]: Module `plotly` has no attribute `subplots`.
     fig = subplots.make_subplots(
         rows=nrows,
         cols=ncols,
@@ -476,11 +550,6 @@ def tile_cross_validation(
         )
         fig.append_trace(
             _error_scatter_trace(
-                # Expected `List[typing.Union[PlotInSampleArm,
-                # ax.plot.base.PlotOutOfSampleArm]]` for 1st anonymous
-                # parameter to call `ax.plot.scatter._error_scatter_trace` but
-                # got `List[PlotInSampleArm]`.
-                # pyre-fixme[6]:
                 list(data.in_sample.values()),
                 y_axis_var=PlotMetric(metric, pred=True, rel=False),
                 x_axis_var=PlotMetric(metric, pred=False, rel=False),
