@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -9,7 +9,7 @@ from typing import Any, List
 import pandas as pd
 import plotly.graph_objs as go
 from ax.modelbridge.base import ModelBridge
-from ax.plot.base import DECIMALS, AxPlotConfig, AxPlotTypes
+from ax.plot.base import AxPlotConfig, AxPlotTypes, DECIMALS
 from ax.plot.helper import get_plot_data
 from ax.utils.stats.statstools import marginal_effects
 from plotly import subplots
@@ -39,6 +39,7 @@ def plot_marginal_effects(model: ModelBridge, metric: str) -> AxPlotConfig:
     effect_table = marginal_effects(pd.concat(arm_dfs, 0))
 
     varnames = effect_table["Name"].unique()
+    # pyre-fixme[33]: Given annotation cannot contain `Any`.
     data: List[Any] = []
     for varname in varnames:
         var_df = effect_table[effect_table["Name"] == varname]
@@ -63,7 +64,7 @@ def plot_marginal_effects(model: ModelBridge, metric: str) -> AxPlotConfig:
     # fig.layout.margin = go.layout.Margin(l=2, r=2)
     fig.layout.title = "Marginal Effects by Factor"
     fig.layout.yaxis = {
-        "title": "% better than experiment average",
+        "title": "% higher than experiment average",
         "hoverformat": ".{}f".format(DECIMALS),
     }
     return AxPlotConfig(data=fig, plot_type=AxPlotTypes.GENERIC)
