@@ -1,11 +1,14 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import copy
+from copy import deepcopy
 from itertools import combinations
-from typing import Dict, List, NamedTuple, Optional, Tuple
+from logging import Logger
+from typing import Dict, List, NamedTuple, Optional, Tuple, Union
 
 import numpy as np
 import torch
@@ -16,10 +19,15 @@ from ax.core.metric import Metric
 from ax.core.objective import ScalarizedObjective
 from ax.core.observation import ObservationFeatures
 from ax.core.optimization_config import MultiObjectiveOptimizationConfig
-from ax.core.outcome_constraint import OutcomeConstraint
+from ax.core.outcome_constraint import ObjectiveThreshold, OutcomeConstraint
+from ax.core.search_space import RobustSearchSpace, SearchSpace
 from ax.core.types import TParameterization
 from ax.exceptions.core import AxError, UnsupportedError
-from ax.modelbridge.modelbridge_utils import observed_pareto_frontier
+from ax.modelbridge.modelbridge_utils import (
+    _get_modelbridge_training_data,
+    get_pareto_frontier_and_configs,
+    observed_pareto_frontier,
+)
 from ax.modelbridge.registry import Models
 from ax.modelbridge.torch import TorchModelBridge
 from ax.modelbridge.transforms.choice_encode import OrderedChoiceEncode
@@ -31,37 +39,47 @@ from ax.models.torch_base import TorchModel
 from ax.utils.common.logger import get_logger
 from ax.utils.stats.statstools import relativize
 from botorch.utils.multi_objective import is_non_dominated
-
+from botorch.utils.multi_objective.hypervolume import infer_reference_point
 
 # type aliases
 Mu = Dict[str, List[float]]
 Cov = Dict[str, Dict[str, List[float]]]
 
 
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 
 
 def _extract_observed_pareto_2d(
-    Y: torch.Tensor, reference_point: Tuple[float, float], minimize: bool = True
-) -> torch.Tensor:
+    Y: np.ndarray,
+    reference_point: Optional[Tuple[float, float]],
+    minimize: Union[bool, Tuple[bool, bool]] = True,
+) -> np.ndarray:
     if Y.shape[1] != 2:
         raise NotImplementedError("Currently only the 2-dim case is handled.")
-    ref_point = torch.tensor(reference_point, dtype=Y.dtype)
-    Y_pareto = Y[is_non_dominated(-1 * Y if minimize else Y)]
-    Y_pareto = (
-        Y_pareto[torch.all(Y_pareto < ref_point, dim=1)]
-        if minimize
-        else Y_pareto[torch.all(Y_pareto > ref_point, dim=1)]
-    )
-    Y_pareto = Y_pareto[torch.argsort(Y_pareto[:, 0])]
-    if Y_pareto.shape[0] == 0:
-        better = "below" if minimize else "above"
-        raise ValueError(
-            f"No Pareto-optimal points in `Y` were {better} the reference point."
-        )
+    # If `minimize` is a bool, apply to both dimensions
+    if isinstance(minimize, bool):
+        minimize = (minimize, minimize)
+    Y_copy = deepcopy(torch.from_numpy(Y).to())
+    if reference_point:
+        ref_point = torch.tensor(reference_point, dtype=Y_copy.dtype)
+        for i in range(2):
+            # Filter based on reference point
+            Y_copy = (
+                Y_copy[Y_copy[:, i] < ref_point[i]]
+                if minimize[i]
+                else Y_copy[Y_copy[:, i] > ref_point[i]]
+            )
+    for i in range(2):
+        # Flip sign in each dimension based on minimize
+        Y_copy[:, i] *= (-1) ** minimize[i]
+    Y_pareto = Y_copy[is_non_dominated(Y_copy)]
+    Y_pareto = Y_pareto[torch.argsort(input=Y_pareto[:, 0], descending=True)]
+    for i in range(2):
+        # Flip sign back
+        Y_pareto[:, i] *= (-1) ** minimize[i]
 
     assert Y_pareto.shape[1] == 2  # Y_pareto should have two outcomes.
-    return Y_pareto
+    return Y_pareto.detach().cpu().numpy()
 
 
 class ParetoFrontierResults(NamedTuple):
@@ -125,41 +143,44 @@ def get_observed_pareto_frontiers(
             data from experiment.
         rel: Relativize, if status quo on experiment.
         arm_names: If provided, computes Pareto frontier only from among the provided
-            list of arm names.
+            list of arm names, plus status quo if set on experiment.
 
     Returns: ParetoFrontierResults that can be used with interact_pareto_frontier.
     """
     if data is None:
         data = experiment.fetch_data()
-    if not isinstance(data, Data):
-        raise TypeError(
-            "Data fetched from experiment not an instance of PTS-supporting `Data`"
-        )
     if experiment.optimization_config is None:
         raise ValueError("Experiment must have an optimization config")
     if arm_names is not None:
+        if (
+            experiment.status_quo is not None
+            and experiment.status_quo.name not in arm_names
+        ):
+            # Make sure status quo is always included, for derelativization
+            arm_names.append(experiment.status_quo.name)
         data = Data(data.df[data.df["arm_name"].isin(arm_names)])
     mb = get_tensor_converter_model(experiment=experiment, data=data)
     pareto_observations = observed_pareto_frontier(modelbridge=mb)
     # Convert to ParetoFrontierResults
-    metric_names = [
+    objective_metric_names = {
         metric.name
         for metric in experiment.optimization_config.objective.metrics  # pyre-ignore
-    ]
-    pfr_means = {name: [] for name in metric_names}
-    pfr_sems = {name: [] for name in metric_names}
+    }
+    obj_metr_list = sorted(objective_metric_names)
+    pfr_means = {name: [] for name in obj_metr_list}
+    pfr_sems = {name: [] for name in obj_metr_list}
 
     for obs in pareto_observations:
         for i, name in enumerate(obs.data.metric_names):
-            pfr_means[name].append(obs.data.means[i])
-            pfr_sems[name].append(np.sqrt(obs.data.covariance[i, i]))
+            if name in objective_metric_names:
+                pfr_means[name].append(obs.data.means[i])
+                pfr_sems[name].append(np.sqrt(obs.data.covariance[i, i]))
 
     # Relativize as needed
     if rel and experiment.status_quo is not None:
         # Get status quo values
-        sq_df = data.df[
-            data.df["arm_name"] == experiment.status_quo.name  # pyre-ignore
-        ]
+        # pyre-fixme[16]: `Optional` has no attribute `name`.
+        sq_df = data.df[data.df["arm_name"] == experiment.status_quo.name]
         sq_df = sq_df.to_dict(orient="list")
         sq_means = {}
         sq_sems = {}
@@ -170,7 +191,7 @@ def get_observed_pareto_frontiers(
             # pyre-fixme[6]: Expected `_SupportsIndex` for 1st param but got `str`.
             sq_sems[metric] = sq_df["sem"][i]
         # Relativize
-        for name in metric_names:
+        for name in pfr_means:
             if np.isnan(sq_sems[name]) or np.isnan(pfr_sems[name]).any():
                 # Just relativize means
                 pfr_means[name] = [
@@ -187,7 +208,7 @@ def get_observed_pareto_frontiers(
                 )
         absolute_metrics = []
     else:
-        absolute_metrics = metric_names
+        absolute_metrics = obj_metr_list
 
     objective_thresholds = {}
     if experiment.optimization_config.objective_thresholds is not None:  # pyre-ignore
@@ -205,7 +226,7 @@ def get_observed_pareto_frontiers(
     param_dicts = [obs.features.parameters for obs in pareto_observations]
     pfr_arm_names = [obs.arm_name for obs in pareto_observations]
 
-    for metric_a, metric_b in combinations(metric_names, 2):
+    for metric_a, metric_b in combinations(obj_metr_list, 2):
         pfr_list.append(
             ParetoFrontierResults(
                 param_dicts=param_dicts,
@@ -221,6 +242,22 @@ def get_observed_pareto_frontiers(
     return pfr_list
 
 
+def to_nonrobust_search_space(search_space: SearchSpace) -> SearchSpace:
+    """Reduces a RobustSearchSpace to a SearchSpace.
+
+    This is a no-op for all other search spaces.
+    """
+    if isinstance(search_space, RobustSearchSpace):
+        return SearchSpace(
+            parameters=[p.clone() for p in search_space._parameters.values()],
+            parameter_constraints=[
+                pc.clone() for pc in search_space._parameter_constraints
+            ],
+        )
+    else:
+        return search_space
+
+
 def get_tensor_converter_model(experiment: Experiment, data: Data) -> TorchModelBridge:
     """
     Constructs a minimal model for converting things to tensors.
@@ -241,7 +278,7 @@ def get_tensor_converter_model(experiment: Experiment, data: Data) -> TorchModel
     # space to tensors.
     return TorchModelBridge(
         experiment=experiment,
-        search_space=experiment.search_space,
+        search_space=to_nonrobust_search_space(experiment.search_space),
         data=data,
         model=TorchModel(),
         transforms=[Derelativize, SearchSpaceToChoice, OrderedChoiceEncode, IntToFloat],
@@ -289,7 +326,6 @@ def compute_posterior_pareto_frontier(
     Returns:
         ParetoFrontierResults: A NamedTuple with fields listed in its definition.
     """
-    # TODO(jej): Implement using MultiObjectiveTorchModelBridge's _pareto_frontier
     model_gen_options = {
         "acquisition_function_kwargs": {"chebyshev_scalarization": chebyshev}
     }
@@ -320,18 +356,11 @@ def compute_posterior_pareto_frontier(
     # build posterior mean model
     if not data:
         try:
-            abstract_data = (
+            data = (
                 experiment.trials[trial_index].fetch_data()
                 if trial_index
                 else experiment.fetch_data()
             )
-            # TODO(jej)[T87591836] Support non-`Data` data types.
-            if not isinstance(abstract_data, Data):
-                raise TypeError(
-                    "Data passed as arg or fetched from experiment is not "
-                    "an instance of PTS-supporting `Data`"
-                )
-            data = abstract_data
         except Exception as e:
             logger.info(f"Could not fetch data from experiment or trial: {e}")
 
@@ -471,8 +500,15 @@ def _validate_outcome_constraints(
 
 
 def _build_new_optimization_config(
-    weights, primary_objective, secondary_objective, outcome_constraints=None
-):
+    # pyre-fixme[2]: Parameter must be annotated.
+    weights,
+    # pyre-fixme[2]: Parameter must be annotated.
+    primary_objective,
+    # pyre-fixme[2]: Parameter must be annotated.
+    secondary_objective,
+    # pyre-fixme[2]: Parameter must be annotated.
+    outcome_constraints=None,
+) -> MultiObjectiveOptimizationConfig:
     obj = ScalarizedObjective(
         metrics=[primary_objective, secondary_objective],
         weights=weights,
@@ -482,3 +518,73 @@ def _build_new_optimization_config(
         objective=obj, outcome_constraints=outcome_constraints
     )
     return optimization_config
+
+
+def infer_reference_point_from_experiment(
+    experiment: Experiment,
+) -> List[ObjectiveThreshold]:
+    """This functions is a wrapper around ``infer_reference_point`` to find the nadir
+    point from the pareto front of an experiment. Aside from converting experiment
+    to tensors, this wrapper transforms back and forth the objectives of the experiment
+    so that they are appropriately used by ``infer_reference_point``.
+
+    Args:
+        experiment: The experiment for which we want to infer the reference point.
+
+    Returns:
+        List of obejective thresholds representing the reference point.
+    """
+    if not experiment.is_moo_problem:
+        raise ValueError(
+            "This function works for MOO experiments only."
+            f" Experiment {experiment.name} is single objective."
+        )
+
+    # Reading experiment data.
+    mb_reference = get_tensor_converter_model(
+        experiment=experiment, data=experiment.fetch_data()
+    )
+    obs_feats, obs_data, _ = _get_modelbridge_training_data(modelbridge=mb_reference)
+
+    # Turning all the objectives to be maximized.
+    multiplier = []
+    for objective in experiment.optimization_config.objective.objectives:  # pyre-ignore
+        multiplier.append(-1 if objective.minimize else +1)
+
+    # A dummy reference point so that all observed points are considered when
+    # calculating the Pareto front.
+    dummy_rp = copy.deepcopy(
+        experiment.optimization_config.objective_thresholds  # pyre-ignore
+    )
+    for i in range(len(dummy_rp)):
+        dummy_rp[i].bound = -1 * multiplier[i] * np.inf
+
+    # Finding the pareto frontier
+    frontier_observations, f, obj_w, obj_t = get_pareto_frontier_and_configs(
+        modelbridge=mb_reference,
+        observation_features=obs_feats,
+        observation_data=obs_data,
+        objective_thresholds=dummy_rp,
+        use_model_predictions=False,
+    )
+
+    # Transforming all the objectives to be maximized.
+    f_transformed = (
+        torch.tensor(multiplier, dtype=f.dtype, device=f.device)
+        * f[:, obj_w.nonzero().view(-1)]
+    )
+
+    # Finding nadir point.
+    rp_raw = infer_reference_point(f_transformed)
+
+    # Un-transforming the reference point.
+    rp = torch.tensor(multiplier, dtype=f.dtype, device=f.device) * rp_raw
+
+    # Constructing the objective thresholds.
+    nadir_objective_thresholds = copy.deepcopy(
+        experiment.optimization_config.objective_thresholds
+    )
+    for i, obj_threshold in enumerate(nadir_objective_thresholds):
+        obj_threshold.bound = rp[i].item()
+
+    return nadir_objective_thresholds
