@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -7,7 +7,9 @@
 import numbers
 import warnings
 from collections import OrderedDict
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+
+from logging import Logger
+from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import plotly.graph_objs as go
@@ -17,36 +19,40 @@ from ax.core.observation import Observation, ObservationFeatures
 from ax.modelbridge.base import ModelBridge
 from ax.modelbridge.registry import Models
 from ax.plot.base import (
-    CI_OPACITY,
-    DECIMALS,
     AxPlotConfig,
     AxPlotTypes,
+    CI_OPACITY,
+    DECIMALS,
     PlotInSampleArm,
     PlotMetric,
     PlotOutOfSampleArm,
     Z,
 )
-from ax.plot.color import COLORS, DISCRETE_COLOR_SCALE, rgba
+from ax.plot.color import BLUE_SCALE, COLORS, DISCRETE_COLOR_SCALE, rgba
 from ax.plot.helper import (
-    TNullableGeneratorRunsDict,
     _format_CI,
     _format_dict,
     _wrap_metric,
+    arm_name_to_sort_key,
     arm_name_to_tuple,
     get_plot_data,
     infer_is_relative,
     resize_subtitles,
+    TNullableGeneratorRunsDict,
 )
+from ax.utils.common.logger import get_logger
 from ax.utils.common.typeutils import checked_cast_optional
 from ax.utils.stats.statstools import relativize
 from plotly import subplots
 
+logger: Logger = get_logger(__name__)
+
 # type aliases
 Traces = List[Dict[str, Any]]
 
 
 def _error_scatter_data(
-    arms: List[Union[PlotInSampleArm, PlotOutOfSampleArm]],
+    arms: Iterable[Union[PlotInSampleArm, PlotOutOfSampleArm]],
     y_axis_var: PlotMetric,
     x_axis_var: Optional[PlotMetric] = None,
     status_quo_arm: Optional[PlotInSampleArm] = None,
@@ -97,7 +103,7 @@ def _error_scatter_data(
 
 
 def _error_scatter_trace(
-    arms: List[Union[PlotInSampleArm, PlotOutOfSampleArm]],
+    arms: Sequence[Union[PlotInSampleArm, PlotOutOfSampleArm]],
     y_axis_var: PlotMetric,
     x_axis_var: Optional[PlotMetric] = None,
     y_axis_label: Optional[str] = None,
@@ -113,6 +119,8 @@ def _error_scatter_trace(
     show_arm_details_on_hover: bool = True,
     show_context: bool = False,
     arm_noun: str = "arm",
+    color_parameter: Optional[str] = None,
+    color_metric: Optional[str] = None,
 ) -> Dict[str, Any]:
     """Plot scatterplot with error bars.
 
@@ -147,7 +155,24 @@ def _error_scatter_trace(
         show_context: if True and show_arm_details_on_hover,
             context will be included in the hover.
         arm_noun: noun to use instead of "arm" (e.g. group)
+        color_parameter: color points according to the specified parameter,
+            cannot be used together with color_metric.
+        color_metric: color points according to the specified metric,
+            cannot be used together with color_parameter.
     """
+    if color_metric and color_parameter:
+        raise RuntimeError(
+            "color_metric and color_parameter cannot be used at the same time!"
+        )
+
+    if (color_metric or color_parameter) and not all(
+        isinstance(arm, PlotInSampleArm) for arm in arms
+    ):
+        raise RuntimeError("Color coding currently only works with in-sample arms!")
+
+    # Opportunistically sort if arm names are in {trial}_{arm} format
+    arms = sorted(arms, key=lambda a: arm_name_to_sort_key(a.name), reverse=True)
+
     x, x_se, y, y_se = _error_scatter_data(
         arms=arms,
         y_axis_var=y_axis_var,
@@ -155,6 +180,7 @@ def _error_scatter_trace(
         status_quo_arm=status_quo_arm,
     )
     labels = []
+    colors = []
 
     arm_names = [a.name for a in arms]
 
@@ -191,6 +217,13 @@ def _error_scatter_trace(
             else ""
         )
 
+        if color_parameter:
+            colors.append(arms[i].parameters[color_parameter])
+        elif color_metric:
+            # Must be PlotInSampleArm here if no error raised previously
+            # pyre-ignore[16]: `PlotOutOfSampleArm` has no attribute `y`
+            colors.append(arms[i].y[color_metric])
+
         context = (
             # Expected `Dict[str, Optional[Union[bool, float, str]]]` for 1st anonymous
             # parameter to call `ax.plot.helper._format_dict` but got
@@ -213,10 +246,22 @@ def _error_scatter_trace(
             )
         )
         i += 1
+
+    if color_metric or color_parameter:
+        rgba_blue_scale = [rgba(c) for c in BLUE_SCALE]
+        marker = {
+            "color": colors,
+            "colorscale": rgba_blue_scale,
+            "colorbar": {"title": color_metric or color_parameter},
+            "showscale": True,
+        }
+    else:
+        marker = {"color": rgba(color)}
+
     trace = go.Scatter(
         x=x,
         y=y,
-        marker={"color": rgba(color)},
+        marker=marker,
         mode="markers",
         name=name,
         text=labels,
@@ -258,6 +303,8 @@ def _multiple_metric_traces(
     rel_y: bool,
     fixed_features: Optional[ObservationFeatures] = None,
     data_selector: Optional[Callable[[Observation], bool]] = None,
+    color_parameter: Optional[str] = None,
+    color_metric: Optional[str] = None,
 ) -> Traces:
     """Plot traces for multiple metrics given a model and metrics.
 
@@ -271,11 +318,19 @@ def _multiple_metric_traces(
         rel_y: if True, use relative effects on metric_y.
         fixed_features: Fixed features to use when making model predictions.
         data_selector: Function for selecting observations for plotting.
+        color_parameter: color points according to the specified parameter,
+            cannot be used together with color_metric.
+        color_metric: color points according to the specified metric,
+            cannot be used together with color_parameter.
     """
+    metric_names = {metric_x, metric_y}
+    if color_metric is not None:
+        metric_names.add(color_metric)
+
     plot_data, _, _ = get_plot_data(
         model,
         generator_runs_dict if generator_runs_dict is not None else {},
-        {metric_x, metric_y},
+        metric_names,
         fixed_features=fixed_features,
         data_selector=data_selector,
     )
@@ -288,31 +343,26 @@ def _multiple_metric_traces(
 
     traces = [
         _error_scatter_trace(
-            # Expected `List[Union[PlotInSampleArm, PlotOutOfSampleArm]]`
-            # for 1st anonymous parameter to call
-            # `ax.plot.scatter._error_scatter_trace` but got
-            # `List[PlotInSampleArm]`.
-            # pyre-fixme[6]:
             list(plot_data.in_sample.values()),
             x_axis_var=PlotMetric(metric_x, pred=False, rel=rel_x),
             y_axis_var=PlotMetric(metric_y, pred=False, rel=rel_y),
             status_quo_arm=status_quo_arm,
             visible=False,
+            color_parameter=color_parameter,
+            color_metric=color_metric,
         ),
         _error_scatter_trace(
-            # Expected `List[Union[PlotInSampleArm, PlotOutOfSampleArm]]`
-            # for 1st anonymous parameter to call
-            # `ax.plot.scatter._error_scatter_trace` but got
-            # `List[PlotInSampleArm]`.
-            # pyre-fixme[6]:
             list(plot_data.in_sample.values()),
             x_axis_var=PlotMetric(metric_x, pred=True, rel=rel_x),
             y_axis_var=PlotMetric(metric_y, pred=True, rel=rel_y),
             status_quo_arm=status_quo_arm,
             visible=True,
+            color_parameter=color_parameter,
+            color_metric=color_metric,
         ),
     ]
 
+    # TODO: Figure out if there's a better way to color code out-of-sample points
     for i, (generator_run_name, cand_arms) in enumerate(
         (plot_data.out_of_sample or {}).items(), start=1
     ):
@@ -338,6 +388,8 @@ def plot_multiple_metrics(
     rel_y: bool = True,
     fixed_features: Optional[ObservationFeatures] = None,
     data_selector: Optional[Callable[[Observation], bool]] = None,
+    color_parameter: Optional[str] = None,
+    color_metric: Optional[str] = None,
     **kwargs: Any,
 ) -> AxPlotConfig:
     """Plot raw values or predictions of two metrics for arms.
@@ -354,7 +406,15 @@ def plot_multiple_metrics(
         rel_x: if True, use relative effects on metric_x.
         rel_y: if True, use relative effects on metric_y.
         data_selector: Function for selecting observations for plotting.
+        color_parameter: color points according to the specified parameter,
+            cannot be used together with color_metric.
+        color_metric: color points according to the specified metric,
+            cannot be used together with color_parameter.
     """
+    if color_parameter or color_metric:
+        layout_offset_x = 0.15
+    else:
+        layout_offset_x = 0
     rel = checked_cast_optional(bool, kwargs.get("rel"))
     if rel is not None:
         warnings.warn("Use `rel_x` and `rel_y` instead of `rel`.", DeprecationWarning)
@@ -369,6 +429,8 @@ def plot_multiple_metrics(
         rel_y=rel_y,
         fixed_features=fixed_features,
         data_selector=data_selector,
+        color_parameter=color_parameter,
+        color_metric=color_metric,
     )
     num_cand_traces = len(generator_runs_dict) if generator_runs_dict is not None else 0
     layout = go.Layout(
@@ -376,7 +438,7 @@ def plot_multiple_metrics(
         hovermode="closest",
         updatemenus=[
             {
-                "x": 1.25,
+                "x": 1.25 + layout_offset_x,
                 "y": 0.67,
                 "buttons": [
                     {
@@ -408,7 +470,7 @@ def plot_multiple_metrics(
                 "xanchor": "left",
             },
             {
-                "x": 1.25,
+                "x": 1.25 + layout_offset_x,
                 "y": 0.57,
                 "buttons": [
                     {
@@ -432,7 +494,7 @@ def plot_multiple_metrics(
         ],
         annotations=[
             {
-                "x": 1.18,
+                "x": 1.18 + layout_offset_x,
                 "y": 0.7,
                 "xref": "paper",
                 "yref": "paper",
@@ -441,7 +503,7 @@ def plot_multiple_metrics(
                 "yanchor": "middle",
             },
             {
-                "x": 1.18,
+                "x": 1.18 + layout_offset_x,
                 "y": 0.6,
                 "xref": "paper",
                 "yref": "paper",
@@ -463,6 +525,7 @@ def plot_multiple_metrics(
         width=800,
         height=600,
         font={"size": 10},
+        legend={"x": 1 + layout_offset_x},
     )
     fig = go.Figure(data=traces, layout=layout)
     return AxPlotConfig(data=fig, plot_type=AxPlotTypes.GENERIC)
@@ -477,8 +540,11 @@ def plot_objective_vs_constraints(
     infer_relative_constraints: Optional[bool] = False,
     fixed_features: Optional[ObservationFeatures] = None,
     data_selector: Optional[Callable[[Observation], bool]] = None,
+    color_parameter: Optional[str] = None,
+    color_metric: Optional[str] = None,
+    label_dict: Optional[Dict[str, str]] = None,
 ) -> AxPlotConfig:
-    """Plot the tradeoff between an objetive and all other metrics in a model.
+    """Plot the tradeoff between an objective and all other metrics in a model.
 
     All arms used in the model are included in the plot. Additional
     arms can be passed through via the `generator_runs_dict` argument.
@@ -501,11 +567,23 @@ def plot_objective_vs_constraints(
             Metrics that are not constraints will be relativized.
         fixed_features: Fixed features to use when making model predictions.
         data_selector: Function for selecting observations for plotting.
+        color_parameter: color points according to the specified parameter,
+            cannot be used together with color_metric.
+        color_metric: color points according to the specified metric,
+            cannot be used together with color_parameter.
+        label_dict: A dictionary that maps the label to
+            an alias to be used in the plot.
     """
+    if color_parameter or color_metric:
+        layout_offset_x = 0.15
+    else:
+        layout_offset_x = 0
     if subset_metrics is not None:
         metrics = subset_metrics
     else:
         metrics = [m for m in model.metric_names if m != objective]
+    if not label_dict:
+        _check_label_lengths(metrics + [objective])
 
     metric_dropdown = []
 
@@ -533,6 +611,8 @@ def plot_objective_vs_constraints(
         rel_y=rels[metrics[0]],
         fixed_features=fixed_features,
         data_selector=data_selector,
+        color_parameter=color_parameter,
+        color_metric=color_metric,
     )
 
     for metric in metrics:
@@ -545,6 +625,8 @@ def plot_objective_vs_constraints(
             rel_y=rels[metric],
             fixed_features=fixed_features,
             data_selector=data_selector,
+            color_parameter=color_parameter,
+            color_metric=color_metric,
         )
 
         # Current version of Plotly does not allow updating the yaxis label
@@ -555,11 +637,14 @@ def plot_objective_vs_constraints(
                     {
                         "y": [t["y"] for t in otraces],
                         "error_y.array": [t["error_y"]["array"] for t in otraces],
-                        "text": [t["text"] for t in otraces],
+                        "text": [_replace_str(t["text"], label_dict) for t in otraces],
+                    },
+                    {
+                        "yaxis.title": _replace_str(metric, label_dict)
+                        + (" (%)" if rels[metric] else ""),
                     },
-                    {"yaxis.title": metric + (" (%)" if rels[metric] else "")},
                 ],
-                "label": metric,
+                "label": _replace_str(metric, label_dict),
                 "method": "update",
             }
         )
@@ -571,7 +656,7 @@ def plot_objective_vs_constraints(
         hovermode="closest",
         updatemenus=[
             {
-                "x": 1.25,
+                "x": 1.25 + layout_offset_x,
                 "y": 0.62,
                 "buttons": [
                     {
@@ -603,7 +688,7 @@ def plot_objective_vs_constraints(
                 "xanchor": "left",
             },
             {
-                "x": 1.25,
+                "x": 1.25 + layout_offset_x,
                 "y": 0.52,
                 "buttons": [
                     {
@@ -625,7 +710,7 @@ def plot_objective_vs_constraints(
                 "xanchor": "left",
             },
             {
-                "x": 1.25,
+                "x": 1.25 + layout_offset_x,
                 "y": 0.72,
                 "yanchor": "middle",
                 "xanchor": "left",
@@ -634,7 +719,7 @@ def plot_objective_vs_constraints(
         ],
         annotations=[
             {
-                "x": 1.18,
+                "x": 1.18 + layout_offset_x,
                 "y": 0.72,
                 "xref": "paper",
                 "yref": "paper",
@@ -643,7 +728,7 @@ def plot_objective_vs_constraints(
                 "yanchor": "middle",
             },
             {
-                "x": 1.18,
+                "x": 1.18 + layout_offset_x,
                 "y": 0.62,
                 "xref": "paper",
                 "yref": "paper",
@@ -652,7 +737,7 @@ def plot_objective_vs_constraints(
                 "yanchor": "middle",
             },
             {
-                "x": 1.18,
+                "x": 1.18 + layout_offset_x,
                 "y": 0.52,
                 "xref": "paper",
                 "yref": "paper",
@@ -662,24 +747,55 @@ def plot_objective_vs_constraints(
             },
         ],
         xaxis={
-            "title": objective + (" (%)" if rels[objective] else ""),
+            "title": _replace_str(objective, label_dict)
+            + (" (%)" if rels[objective] else ""),
             "zeroline": True,
             "zerolinecolor": "red",
         },
         yaxis={
-            "title": metrics[0] + (" (%)" if rels[metrics[0]] else ""),
+            "title": _replace_str(metrics[0], label_dict)
+            + (" (%)" if rels[metrics[0]] else ""),
             "zeroline": True,
             "zerolinecolor": "red",
         },
         width=900,
         height=600,
         font={"size": 10},
+        legend={"x": 1 + layout_offset_x},
     )
 
     fig = go.Figure(data=plot_data, layout=layout)
     return AxPlotConfig(data=fig, plot_type=AxPlotTypes.GENERIC)
 
 
+def _replace_str(input_str: str, str_dict: Optional[Dict[str, str]] = None) -> str:
+    """Utility function to replace a string based on a mapping dictionary.
+
+    Args:
+        input_str: Input string to map.
+        str_dict: Mapping dictionary.
+    """
+    return str_dict[input_str] if (str_dict and input_str in str_dict) else input_str
+
+
+def _check_label_lengths(labels: List[str]) -> None:
+    """Utility function to check label length and provide a warning for long
+       labels pointing to a mapping that can be used to override them.
+
+    Args:
+        labels: List of labels to check.
+    """
+    max_len = 30
+    long_labels = [label for label in labels if len(label) > max_len]
+    if long_labels:
+        logger.info(
+            "This plot may be malformed due to long labels. You"
+            " can override long labels by passing a label_dict dictionary"
+            " to plotting functions that support it.\nHere's a list of labels"
+            f" longer than {max_len} characters:\n" + "\n".join(long_labels)
+        )
+
+
 def lattice_multiple_metrics(
     model: ModelBridge,
     generator_runs_dict: TNullableGeneratorRunsDict = None,
@@ -725,11 +841,6 @@ def lattice_multiple_metrics(
             if o1 != o2:
                 # in-sample observed and predicted
                 obs_insample_trace = _error_scatter_trace(
-                    # Expected `List[Union[PlotInSampleArm,
-                    # PlotOutOfSampleArm]]` for 1st anonymous parameter to call
-                    # `ax.plot.scatter._error_scatter_trace` but got
-                    # `List[PlotInSampleArm]`.
-                    # pyre-fixme[6]:
                     list(plot_data.in_sample.values()),
                     x_axis_var=PlotMetric(o1, pred=False, rel=rel),
                     y_axis_var=PlotMetric(o2, pred=False, rel=rel),
@@ -740,11 +851,6 @@ def lattice_multiple_metrics(
                     show_arm_details_on_hover=show_arm_details_on_hover,
                 )
                 predicted_insample_trace = _error_scatter_trace(
-                    # Expected `List[Union[PlotInSampleArm,
-                    # PlotOutOfSampleArm]]` for 1st anonymous parameter to call
-                    # `ax.plot.scatter._error_scatter_trace` but got
-                    # `List[PlotInSampleArm]`.
-                    # pyre-fixme[6]:
                     list(plot_data.in_sample.values()),
                     x_axis_var=PlotMetric(o1, pred=True, rel=rel),
                     y_axis_var=PlotMetric(o2, pred=True, rel=rel),
@@ -1005,11 +1111,6 @@ def _single_metric_traces(
 
     traces = [
         _error_scatter_trace(
-            # Expected `List[Union[PlotInSampleArm, PlotOutOfSampleArm]]`
-            # for 1st anonymous parameter to call
-            # `ax.plot.scatter._error_scatter_trace` but got
-            # `List[PlotInSampleArm]`.
-            # pyre-fixme[6]:
             list(plot_data.in_sample.values()),
             x_axis_var=None,
             y_axis_var=PlotMetric(metric, pred=True, rel=rel),
@@ -1336,7 +1437,7 @@ def tile_fitted(
     return AxPlotConfig(data=fig, plot_type=AxPlotTypes.GENERIC)
 
 
-def interact_fitted(
+def interact_fitted_plotly(
     model: ModelBridge,
     generator_runs_dict: TNullableGeneratorRunsDict = None,
     rel: bool = True,
@@ -1346,7 +1447,8 @@ def interact_fitted(
     metrics: Optional[List[str]] = None,
     fixed_features: Optional[ObservationFeatures] = None,
     data_selector: Optional[Callable[[Observation], bool]] = None,
-) -> AxPlotConfig:
+    label_dict: Optional[Dict[str, str]] = None,
+) -> go.Figure:
     """Interactive fitted outcome plots for each arm used in fitting the model.
 
     Choose the outcome to plot using a dropdown.
@@ -1363,11 +1465,15 @@ def interact_fitted(
         metrics: List of metric names to restrict to when plotting.
         fixed_features: Fixed features to use when making model predictions.
         data_selector: Function for selecting observations for plotting.
+        label_dict: A dictionary that maps the label to
+            an alias to be used in the plot.
     """
     traces_per_metric = (
         1 if generator_runs_dict is None else len(generator_runs_dict) + 1
     )
     metrics = sorted(metrics or model.metric_names)
+    if not label_dict:
+        _check_label_lengths(metrics)
 
     traces = []
     dropdown = []
@@ -1398,7 +1504,11 @@ def interact_fitted(
 
         # on dropdown change, restyle
         dropdown.append(
-            {"args": ["visible", is_visible], "label": metric, "method": "restyle"}
+            {
+                "args": ["visible", is_visible],
+                "label": _replace_str(metric, label_dict),
+                "method": "restyle",
+            }
         )
 
     layout = go.Layout(
@@ -1457,8 +1567,55 @@ def interact_fitted(
             },
         )
 
+    return go.Figure(data=traces, layout=layout)
+
+
+def interact_fitted(
+    model: ModelBridge,
+    generator_runs_dict: TNullableGeneratorRunsDict = None,
+    rel: bool = True,
+    show_arm_details_on_hover: bool = True,
+    show_CI: bool = True,
+    arm_noun: str = "arm",
+    metrics: Optional[List[str]] = None,
+    fixed_features: Optional[ObservationFeatures] = None,
+    data_selector: Optional[Callable[[Observation], bool]] = None,
+    label_dict: Optional[Dict[str, str]] = None,
+) -> AxPlotConfig:
+    """Interactive fitted outcome plots for each arm used in fitting the model.
+
+    Choose the outcome to plot using a dropdown.
+
+    Args:
+        model: model to use for predictions.
+        generator_runs_dict: a mapping from
+            generator run name to generator run.
+        rel: if True, use relative effects. Default is True.
+        show_arm_details_on_hover: if True, display
+            parameterizations of arms on hover. Default is True.
+        show_CI: if True, render confidence intervals.
+        arm_noun: noun to use instead of "arm" (e.g. group)
+        metrics: List of metric names to restrict to when plotting.
+        fixed_features: Fixed features to use when making model predictions.
+        data_selector: Function for selecting observations for plotting.
+        label_dict: A dictionary that maps the label to
+            an alias to be used in the plot.
+    """
+
     return AxPlotConfig(
-        data=go.Figure(data=traces, layout=layout), plot_type=AxPlotTypes.GENERIC
+        data=interact_fitted_plotly(
+            model=model,
+            generator_runs_dict=generator_runs_dict,
+            rel=rel,
+            show_arm_details_on_hover=show_arm_details_on_hover,
+            show_CI=show_CI,
+            arm_noun=arm_noun,
+            metrics=metrics,
+            fixed_features=fixed_features,
+            data_selector=data_selector,
+            label_dict=label_dict,
+        ),
+        plot_type=AxPlotTypes.GENERIC,
     )
 
 
@@ -1488,8 +1645,6 @@ def tile_observations(
     """
     if data is None:
         data = experiment.fetch_data()
-    if not isinstance(data, Data):
-        raise TypeError("data must be Data type")
     if arm_names is not None:
         data = Data(data.df[data.df["arm_name"].isin(arm_names)])
     m_ts = Models.THOMPSON(
