@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -18,6 +18,7 @@ from ax.storage.sqa_store.load import (
     _load_experiment,
     _load_generation_strategy_by_experiment_name,
 )
+from ax.storage.sqa_store.reduced_state import GR_LARGE_MODEL_ATTRS
 from ax.storage.sqa_store.save import (
     _save_experiment,
     _save_generation_strategy,
@@ -25,18 +26,14 @@ from ax.storage.sqa_store.save import (
 )
 from ax.storage.sqa_store.structs import DBSettings
 from ax.utils.common.testutils import TestCase
-from ax.utils.testing.core_stubs import (
-    get_experiment,
-    get_generator_run,
-    get_simple_experiment,
-)
+from ax.utils.testing.core_stubs import get_experiment, get_generator_run
 from ax.utils.testing.modeling_stubs import get_generation_strategy
 
 
 class TestWithDBSettingsBase(TestCase):
     """Tests saving/loading functionality of WithDBSettingsBase class."""
 
-    def setUp(self):
+    def setUp(self) -> None:
         self.generation_strategy = get_generation_strategy(with_experiment=True)
         self.experiment = self.generation_strategy.experiment
 
@@ -66,7 +63,7 @@ class TestWithDBSettingsBase(TestCase):
     def get_random_generation_strategy(self) -> GenerationStrategy:
         """Get an GenerationStrategy instance with random name."""
 
-        generation_strategy = get_generation_strategy()
+        generation_strategy = get_generation_strategy(with_callable_model_kwarg=False)
         gs_name = "".join(random.choice(string.ascii_letters) for i in range(8))
         generation_strategy._name = gs_name
         return generation_strategy
@@ -94,7 +91,7 @@ class TestWithDBSettingsBase(TestCase):
             )
         return experiment, generation_strategy
 
-    def test_get_experiment_and_generation_strategy_db_id(self):
+    def test_get_experiment_and_generation_strategy_db_id(self) -> None:
 
         (
             exp_id,
@@ -105,7 +102,7 @@ class TestWithDBSettingsBase(TestCase):
         self.assertIsNotNone(exp_id)
         self.assertIsNotNone(gen_id)
 
-    def test_save_experiment(self):
+    def test_save_experiment(self) -> None:
         experiment = self.get_random_experiment()
         saved = self.with_db_settings._save_experiment_to_db_if_possible(experiment)
         self.assertTrue(saved)
@@ -115,7 +112,7 @@ class TestWithDBSettingsBase(TestCase):
         self.assertIsNotNone(loaded_experiment)
         self.assertEqual(experiment, loaded_experiment)
 
-    def test_save_generation_strategy(self):
+    def test_save_generation_strategy(self) -> None:
         experiment, generation_strategy = self.init_experiment_and_generation_strategy(
             save_generation_strategy=False
         )
@@ -130,7 +127,7 @@ class TestWithDBSettingsBase(TestCase):
         self.assertIsNotNone(loaded_gs)
         self.assertEqual(loaded_gs.name, generation_strategy.name)
 
-    def test_save_load_experiment_and_generation_strategy(self):
+    def test_save_load_experiment_and_generation_strategy(self) -> None:
         experiment, generation_strategy = self.init_experiment_and_generation_strategy(
             save_generation_strategy=False
         )
@@ -160,18 +157,7 @@ class TestWithDBSettingsBase(TestCase):
         self.assertIsNotNone(db_gs)
         self.assertEqual(db_gs.name, generation_strategy.name)
 
-        simple_experiment = get_simple_experiment()
-        _save_experiment(
-            simple_experiment,
-            encoder=self.with_db_settings.db_settings.encoder,
-            decoder=self.with_db_settings.db_settings.decoder,
-        )
-        with self.assertRaisesRegex(ValueError, "Service API only"):
-            self.with_db_settings._load_experiment_and_generation_strategy(
-                simple_experiment.name
-            )
-
-    def test_update_generation_strategy(self):
+    def test_update_generation_strategy(self) -> None:
         _, generation_strategy = self.init_experiment_and_generation_strategy()
 
         generator_run = get_generator_run()
@@ -181,9 +167,10 @@ class TestWithDBSettingsBase(TestCase):
         )
         self.assertTrue(updated)
         self.assertIsNotNone(generator_run.db_id)
+        self.assertIsNotNone(generator_run.arms[0].db_id)
 
     @patch(f"{WithDBSettingsBase.__module__}.STORAGE_MINI_BATCH_SIZE", 2)
-    def test_update_generation_strategy_mini_batches(self):
+    def test_update_generation_strategy_mini_batches(self) -> None:
         _, generation_strategy = self.init_experiment_and_generation_strategy()
 
         # Check with 1 GR.
@@ -206,7 +193,7 @@ class TestWithDBSettingsBase(TestCase):
         for gr in grs:
             self.assertIsNotNone(gr.db_id)
 
-    def test_save_new_trial(self):
+    def test_save_new_trial(self) -> None:
         experiment, _ = self.init_experiment_and_generation_strategy(
             save_generation_strategy=False
         )
@@ -225,7 +212,7 @@ class TestWithDBSettingsBase(TestCase):
         self.assertEqual(len(exp.trials), 1)
         self.assertEqual(exp.trials[0].status, TrialStatus.CANDIDATE)
 
-    def test_save_updated_trial(self):
+    def test_save_updated_trial(self) -> None:
         experiment, _ = self.init_experiment_and_generation_strategy(
             save_generation_strategy=False
         )
@@ -254,7 +241,7 @@ class TestWithDBSettingsBase(TestCase):
         self.assertEqual(exp.trials[0].status, TrialStatus.RUNNING)
 
     @patch(f"{WithDBSettingsBase.__module__}.STORAGE_MINI_BATCH_SIZE", 2)
-    def test_updated_trials_mini_batch(self):
+    def test_updated_trials_mini_batch(self) -> None:
         experiment, _ = self.init_experiment_and_generation_strategy(
             save_generation_strategy=False
         )
@@ -270,7 +257,9 @@ class TestWithDBSettingsBase(TestCase):
             experiment.name, decoder=self.with_db_settings.db_settings.decoder
         )
         self.assertEqual(
-            loaded_experiment.trials.get(trial.index).status, TrialStatus.CANDIDATE
+            # pyre-fixme[16]: Optional type has no attribute `status`.
+            loaded_experiment.trials.get(trial.index).status,
+            TrialStatus.CANDIDATE,
         )
         self.assertIsNotNone(trial.db_id)
 
@@ -284,6 +273,8 @@ class TestWithDBSettingsBase(TestCase):
 
         self.with_db_settings._save_or_update_trials_in_db_if_possible(
             experiment=experiment,
+            # pyre-fixme[6]: For 2nd param expected `List[BaseTrial]` but got
+            #  `List[Trial]`.
             trials=trials,
         )
         loaded_experiment = _load_experiment(
@@ -297,7 +288,54 @@ class TestWithDBSettingsBase(TestCase):
             else:
                 self.assertEqual(t.status, TrialStatus.RUNNING)
 
-    def test_update_experiment_properties_in_db(self):
+    def test_update_reduced_state_generator_runs(self) -> None:
+        experiment, generation_strategy = self.init_experiment_and_generation_strategy(
+            save_generation_strategy=True
+        )
+
+        trials = [experiment.new_trial() for _ in range(5)]
+        grs = []
+        for t in trials:
+            gr = generation_strategy.gen(experiment)
+            grs.append(gr)
+            t.add_generator_run(gr)
+
+        self.with_db_settings._save_or_update_trials_and_generation_strategy_if_possible(  # noqa E501
+            experiment=experiment,
+            # pyre-fixme[6]: For 2nd param expected `List[BaseTrial]` but got
+            #  `List[Trial]`.
+            trials=trials,
+            generation_strategy=generation_strategy,
+            new_generator_runs=grs,
+            reduce_state_generator_runs=True,
+        )
+
+        loaded_experiment = _load_experiment(
+            experiment.name, decoder=self.with_db_settings.db_settings.decoder
+        )
+
+        # Only the last trial's generator run should have large model attributes
+        for idx, trial in loaded_experiment.trials.items():
+            for key in [f"_{attr.key}" for attr in GR_LARGE_MODEL_ATTRS]:
+                if idx < len(loaded_experiment.trials) - 1:
+                    # pyre-fixme[16]: `BaseTrial` has no attribute `generator_run`.
+                    self.assertIsNone(getattr(trial.generator_run, key))
+                else:
+                    self.assertIsNotNone(getattr(trial.generator_run, key))
+
+        loaded_generation_strategy = _load_generation_strategy_by_experiment_name(
+            experiment.name, decoder=self.with_db_settings.db_settings.decoder
+        )
+
+        # Only the last generator run should have large model attributes
+        for idx, gr in enumerate(loaded_generation_strategy._generator_runs):
+            for key in [f"_{attr.key}" for attr in GR_LARGE_MODEL_ATTRS]:
+                if idx < len(loaded_generation_strategy._generator_runs) - 1:
+                    self.assertIsNone(getattr(gr, key))
+                else:
+                    self.assertIsNotNone(getattr(gr, key))
+
+    def test_update_experiment_properties_in_db(self) -> None:
         experiment, _ = self.init_experiment_and_generation_strategy(
             save_generation_strategy=False
         )
