@@ -1,11 +1,13 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 from typing import Any, Dict, Type
 
+import torch
+
 # Ax `Acquisition` imports
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 
@@ -24,6 +26,11 @@ from botorch.acquisition.monte_carlo import (
     qExpectedImprovement,
     qNoisyExpectedImprovement,
 )
+from botorch.acquisition.multi_objective.monte_carlo import (
+    qExpectedHypervolumeImprovement,
+    qNoisyExpectedHypervolumeImprovement,
+)
+from botorch.models import SaasFullyBayesianSingleTaskGP
 
 # BoTorch `Model` imports
 from botorch.models.gp_regression import FixedNoiseGP, SingleTaskGP
@@ -36,11 +43,17 @@ from botorch.models.model import Model
 from botorch.models.model_list_gp_regression import ModelListGP
 from botorch.models.multitask import FixedNoiseMultiTaskGP, MultiTaskGP
 
+# Miscellaneous BoTorch imports
+from gpytorch.constraints import Interval
+from gpytorch.likelihoods.gaussian_likelihood import GaussianLikelihood
+from gpytorch.likelihoods.likelihood import Likelihood
+
 # BoTorch `MarginalLogLikelihood` imports
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
+from gpytorch.mlls.leave_one_out_pseudo_likelihood import LeaveOneOutPseudoLikelihood
 from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
 from gpytorch.mlls.sum_marginal_log_likelihood import SumMarginalLogLikelihood
-
+from gpytorch.priors.torch_priors import GammaPrior
 
 # NOTE: When adding a new registry for a class, make sure to make changes
 # to `CLASS_TO_REGISTRY` and `CLASS_TO_REVERSE_REGISTRY` in this file.
@@ -65,6 +78,7 @@ MODEL_REGISTRY: Dict[Type[Model], str] = {
     MultiTaskGP: "MultiTaskGP",
     SingleTaskGP: "SingleTaskGP",
     SingleTaskMultiFidelityGP: "SingleTaskMultiFidelityGP",
+    SaasFullyBayesianSingleTaskGP: "SaasFullyBayesianSingleTaskGP",
 }
 
 
@@ -73,6 +87,8 @@ Mapping of Botorch `AcquisitionFunction` classes to class name strings.
 """
 ACQUISITION_FUNCTION_REGISTRY: Dict[Type[AcquisitionFunction], str] = {
     ExpectedImprovement: "ExpectedImprovement",
+    qExpectedHypervolumeImprovement: "qExpectedHypervolumeImprovement",
+    qNoisyExpectedHypervolumeImprovement: "qNoisyExpectedHypervolumeImprovement",
     qExpectedImprovement: "qExpectedImprovement",
     qKnowledgeGradient: "qKnowledgeGradient",
     qMaxValueEntropy: "qMaxValueEntropy",
@@ -87,18 +103,31 @@ Mapping of BoTorch `MarginalLogLikelihood` classes to class name strings.
 """
 MLL_REGISTRY: Dict[Type[MarginalLogLikelihood], str] = {
     ExactMarginalLogLikelihood: "ExactMarginalLogLikelihood",
+    LeaveOneOutPseudoLikelihood: "LeaveOneOutPseudoLikelihood",
     SumMarginalLogLikelihood: "SumMarginalLogLikelihood",
 }
 
+LIKELIHOOD_REGISTRY: Dict[Type[GaussianLikelihood], str] = {
+    GaussianLikelihood: "GaussianLikelihood"
+}
+
+GPYTORCH_COMPONENT_REGISTRY: Dict[Type[torch.nn.Module], str] = {
+    Interval: "Interval",
+    GammaPrior: "GammaPrior",
+}
 
 """
 Overarching mapping from encoded classes to registry map.
 """
+# pyre-fixme[5]: Global annotation cannot contain `Any`.
 CLASS_TO_REGISTRY: Dict[Any, Dict[Type[Any], str]] = {
     Acquisition: ACQUISITION_REGISTRY,
     AcquisitionFunction: ACQUISITION_FUNCTION_REGISTRY,
+    Likelihood: LIKELIHOOD_REGISTRY,
     MarginalLogLikelihood: MLL_REGISTRY,
     Model: MODEL_REGISTRY,
+    Interval: GPYTORCH_COMPONENT_REGISTRY,
+    GammaPrior: GPYTORCH_COMPONENT_REGISTRY,
 }
 
 
@@ -124,15 +153,26 @@ REVERSE_MLL_REGISTRY: Dict[str, Type[MarginalLogLikelihood]] = {
     v: k for k, v in MLL_REGISTRY.items()
 }
 
+REVERSE_LIKELIHOOD_REGISTRY: Dict[str, Type[Likelihood]] = {
+    v: k for k, v in LIKELIHOOD_REGISTRY.items()
+}
+
+REVERSE_GPYTORCH_COMPONENT_REGISTRY: Dict[str, Type[torch.nn.Module]] = {
+    v: k for k, v in GPYTORCH_COMPONENT_REGISTRY.items()
+}
 
 """
 Overarching mapping from encoded classes to reverse registry map.
 """
+# pyre-fixme[5]: Global annotation cannot contain `Any`.
 CLASS_TO_REVERSE_REGISTRY: Dict[Any, Dict[str, Type[Any]]] = {
     Acquisition: REVERSE_ACQUISITION_REGISTRY,
     AcquisitionFunction: REVERSE_ACQUISITION_FUNCTION_REGISTRY,
+    Likelihood: REVERSE_LIKELIHOOD_REGISTRY,
     MarginalLogLikelihood: REVERSE_MLL_REGISTRY,
     Model: REVERSE_MODEL_REGISTRY,
+    Interval: REVERSE_GPYTORCH_COMPONENT_REGISTRY,
+    GammaPrior: REVERSE_GPYTORCH_COMPONENT_REGISTRY,
 }
 
 
@@ -141,3 +181,10 @@ def register_acquisition(acq_class: Type[Acquisition]) -> None:
     class_name = acq_class.__name__
     CLASS_TO_REGISTRY[Acquisition].update({acq_class: class_name})
     CLASS_TO_REVERSE_REGISTRY[Acquisition].update({class_name: acq_class})
+
+
+def register_acquisition_function(acqf_class: Type[AcquisitionFunction]) -> None:
+    """Add a custom acquisition class to the SQA and JSON registries."""
+    class_name = acqf_class.__name__
+    CLASS_TO_REGISTRY[AcquisitionFunction].update({acqf_class: class_name})
+    CLASS_TO_REVERSE_REGISTRY[AcquisitionFunction].update({class_name: acqf_class})
