@@ -1,20 +1,20 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-import pickle
+import re
 from typing import Any, Dict, Type
 
-from ax.benchmark.benchmark_problem import BenchmarkProblem, SimpleBenchmarkProblem
+from ax.benchmark.problems.hpo.torchvision import PyTorchCNNTorchvisionBenchmarkProblem
 from ax.core import ObservationFeatures
 from ax.core.arm import Arm
 from ax.core.batch_trial import BatchTrial
 from ax.core.data import Data
 from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun
-from ax.core.map_data import MapData
+from ax.core.map_data import MapData, MapKeyInfo
 from ax.core.metric import Metric
 from ax.core.multi_type_experiment import MultiTypeExperiment
 from ax.core.objective import MultiObjective, Objective, ScalarizedObjective
@@ -29,18 +29,26 @@ from ax.core.parameter_constraint import (
     ParameterConstraint,
     SumConstraint,
 )
-from ax.core.search_space import SearchSpace
-from ax.core.simple_experiment import SimpleExperiment
+from ax.core.parameter_distribution import ParameterDistribution
+from ax.core.risk_measures import RiskMeasure
+from ax.core.runner import Runner
+from ax.core.search_space import RobustSearchSpace, SearchSpace
 from ax.core.trial import Trial
+from ax.early_stopping.strategies import (
+    LogicalEarlyStoppingStrategy,
+    PercentileEarlyStoppingStrategy,
+    ThresholdEarlyStoppingStrategy,
+)
 from ax.modelbridge.generation_strategy import GenerationStep, GenerationStrategy
 from ax.modelbridge.registry import _encode_callables_as_references
 from ax.modelbridge.transforms.base import Transform
+from ax.modelbridge.transforms.winsorize import WinsorizationConfig
 from ax.models.torch.botorch_modular.model import BoTorchModel
 from ax.models.torch.botorch_modular.surrogate import Surrogate
-from ax.runners.synthetic import SyntheticRunner
 from ax.storage.botorch_modular_registry import CLASS_TO_REGISTRY
 from ax.storage.transform_registry import TRANSFORM_REGISTRY
 from ax.utils.common.serialization import serialize_init_args
+from ax.utils.common.typeutils import not_none
 
 
 def experiment_to_dict(experiment: Experiment) -> Dict[str, Any]:
@@ -64,11 +72,6 @@ def experiment_to_dict(experiment: Experiment) -> Dict[str, Any]:
     }
 
 
-def simple_experiment_to_dict(experiment: SimpleExperiment) -> Dict[str, Any]:
-    """Convert AE simple experiment to a dictionary."""
-    return experiment_to_dict(experiment)
-
-
 def multi_type_experiment_to_dict(experiment: MultiTypeExperiment) -> Dict[str, Any]:
     """Convert AE multitype experiment to a dictionary."""
     multi_type_dict = {
@@ -97,6 +100,7 @@ def batch_to_dict(batch: BatchTrial) -> Dict[str, Any]:
         "time_run_started": batch.time_run_started,
         "abandoned_reason": batch.abandoned_reason,
         "run_metadata": batch.run_metadata,
+        "stop_metadata": batch.stop_metadata,
         "generator_run_structs": batch.generator_run_structs,
         "runner": batch.runner,
         "abandoned_arms_metadata": batch._abandoned_arms_metadata,
@@ -121,6 +125,7 @@ def trial_to_dict(trial: Trial) -> Dict[str, Any]:
         "time_run_started": trial.time_run_started,
         "abandoned_reason": trial.abandoned_reason,
         "run_metadata": trial.run_metadata,
+        "stop_metadata": trial.stop_metadata,
         "generator_run": trial.generator_run,
         "runner": trial.runner,
         "num_arms_created": trial._num_arms_created,
@@ -138,6 +143,7 @@ def range_parameter_to_dict(parameter: RangeParameter) -> Dict[str, Any]:
         "lower": parameter.lower,
         "upper": parameter.upper,
         "log_scale": parameter.log_scale,
+        "logit_scale": parameter.logit_scale,
         "digits": parameter.digits,
         "is_fidelity": parameter.is_fidelity,
         "target_value": parameter.target_value,
@@ -155,6 +161,7 @@ def choice_parameter_to_dict(parameter: ChoiceParameter) -> Dict[str, Any]:
         "values": parameter.values,
         "is_fidelity": parameter.is_fidelity,
         "target_value": parameter.target_value,
+        "dependents": parameter.dependents if parameter.is_hierarchical else None,
     }
 
 
@@ -167,6 +174,7 @@ def fixed_parameter_to_dict(parameter: FixedParameter) -> Dict[str, Any]:
         "value": parameter.value,
         "is_fidelity": parameter.is_fidelity,
         "target_value": parameter.target_value,
+        "dependents": parameter.dependents if parameter.is_hierarchical else None,
     }
 
 
@@ -224,9 +232,32 @@ def search_space_to_dict(search_space: SearchSpace) -> Dict[str, Any]:
     }
 
 
+def robust_search_space_to_dict(rss: RobustSearchSpace) -> Dict[str, Any]:
+    """Convert robust search space to a dictionary."""
+    return {
+        "__type": rss.__class__.__name__,
+        "parameters": list(rss._parameters.values()),
+        "parameter_distributions": rss.parameter_distributions,
+        "num_samples": rss.num_samples,
+        "environmental_variables": list(rss._environmental_variables.values()),
+        "parameter_constraints": rss.parameter_constraints,
+    }
+
+
+def parameter_distribution_to_dict(dist: ParameterDistribution) -> Dict[str, Any]:
+    """Convert a parameter distribution to a dictionary."""
+    return {
+        "__type": dist.__class__.__name__,
+        "parameters": dist.parameters,
+        "distribution_class": dist.distribution_class,
+        "distribution_parameters": dist.distribution_parameters,
+        "multiplicative": dist.multiplicative,
+    }
+
+
 def metric_to_dict(metric: Metric) -> Dict[str, Any]:
     """Convert Ax metric to a dictionary."""
-    properties = serialize_init_args(object=metric)
+    properties = metric.serialize_init_args(obj=metric)
     properties["__type"] = metric.__class__.__name__
     return properties
 
@@ -278,6 +309,7 @@ def optimization_config_to_dict(
         "__type": optimization_config.__class__.__name__,
         "objective": optimization_config.objective,
         "outcome_constraints": optimization_config.outcome_constraints,
+        "risk_measure": optimization_config.risk_measure,
     }
 
 
@@ -290,6 +322,7 @@ def multi_objective_optimization_config_to_dict(
         "objective": multi_objective_optimization_config.objective,
         "outcome_constraints": multi_objective_optimization_config.outcome_constraints,
         "objective_thresholds": multi_objective_optimization_config.objective_thresholds,  # noqa E501
+        "risk_measure": multi_objective_optimization_config.risk_measure,
     }
 
 
@@ -320,30 +353,33 @@ def generator_run_to_dict(generator_run: GeneratorRun) -> Dict[str, Any]:
     }
 
 
-def runner_to_dict(runner: SyntheticRunner) -> Dict[str, Any]:
-    """Convert Ax synthetic runner to a dictionary."""
-    properties = serialize_init_args(object=runner)
+def runner_to_dict(runner: Runner) -> Dict[str, Any]:
+    """Convert Ax runner to a dictionary."""
+    properties = runner.serialize_init_args(obj=runner)
     properties["__type"] = runner.__class__.__name__
     return properties
 
 
 def data_to_dict(data: Data) -> Dict[str, Any]:
     """Convert Ax data to a dictionary."""
-    return {
-        "__type": data.__class__.__name__,
-        "df": data.df,
-        "description": data.description,
-    }
+    properties = data.serialize_init_args(obj=data)
+    properties["__type"] = data.__class__.__name__
+    return properties
 
 
 def map_data_to_dict(map_data: MapData) -> Dict[str, Any]:
-    """Convert Ax data to a dictionary."""
-    return {
-        "__type": map_data.__class__.__name__,
-        "df": map_data.df,
-        "map_keys": map_data.map_keys,
-        "description": map_data.description,
-    }
+    """Convert Ax map data to a dictionary."""
+    properties = map_data.serialize_init_args(obj=map_data)
+    properties["__type"] = map_data.__class__.__name__
+    return properties
+
+
+# pyre-fixme[24]: Generic type `MapKeyInfo` expects 1 type parameter.
+def map_key_info_to_dict(mki: MapKeyInfo) -> Dict[str, Any]:
+    """Convert Ax map data metadata to a dictionary."""
+    properties = serialize_init_args(object=mki)
+    properties["__type"] = mki.__class__.__name__
+    return properties
 
 
 def transform_type_to_dict(transform_type: Type[Transform]) -> Dict[str, Any]:
@@ -372,6 +408,7 @@ def generation_step_to_dict(generation_step: GenerationStep) -> Dict[str, Any]:
             generation_step.model_gen_kwargs or {}
         ),
         "index": generation_step.index,
+        "should_deduplicate": generation_step.should_deduplicate,
     }
 
 
@@ -409,36 +446,6 @@ def observation_features_to_dict(obs_features: ObservationFeatures) -> Dict[str,
     }
 
 
-def benchmark_problem_to_dict(benchmark_problem: BenchmarkProblem) -> Dict[str, Any]:
-    """Converts an Ax benchmark problem to a serializable dictionary."""
-    if isinstance(benchmark_problem, SimpleBenchmarkProblem):
-        if benchmark_problem.uses_synthetic_function:
-            function_name = benchmark_problem.f.name  # pyre-ignore[16]
-            f = None
-        else:
-            function_name = benchmark_problem.f.__name__  # pyre-ignore[16]
-            f = pickle.dumps(benchmark_problem.f, 0).decode()
-        return {
-            "__type": benchmark_problem.__class__.__name__,
-            "uses_synthetic_function": benchmark_problem.uses_synthetic_function,
-            "function_name": function_name,
-            # If the benchamrk problem uses a custom callable, pickle it.
-            "f": f,
-            "name": benchmark_problem.name,
-            "domain": benchmark_problem.domain,
-            "minimize": benchmark_problem.minimize,
-            "noise_sd": benchmark_problem.noise_sd,
-            "evaluate_suggested": benchmark_problem.evaluate_suggested,
-            "optimal_value": benchmark_problem.optimal_value,
-        }
-    elif isinstance(benchmark_problem, BenchmarkProblem):
-        properties = serialize_init_args(object=benchmark_problem)
-        properties["__type"] = benchmark_problem.__class__.__name__
-        return properties
-    else:  # pragma: no cover
-        raise ValueError(f"Expected benchmark problem, got: {benchmark_problem}.")
-
-
 def botorch_model_to_dict(model: BoTorchModel) -> Dict[str, Any]:
     """Convert Ax model to a dictionary."""
     return {
@@ -461,6 +468,7 @@ def surrogate_to_dict(surrogate: Surrogate) -> Dict[str, Any]:
     return dict_representation
 
 
+# pyre-fixme[2]: Parameter annotation cannot contain `Any`.
 def botorch_modular_to_dict(class_type: Type[Any]) -> Dict[str, Any]:
     """Convert any class to a dictionary."""
     for _class in CLASS_TO_REGISTRY:
@@ -481,3 +489,92 @@ def botorch_modular_to_dict(class_type: Type[Any]) -> Dict[str, Any]:
         f"{class_type} does not have a corresponding parent class in "
         "CLASS_TO_REGISTRY."
     )
+
+
+# pyre-fixme[2]: Parameter annotation cannot contain `Any`.
+def botorch_component_to_dict(input_obj: Type[Any]) -> Dict[str, Any]:
+    class_type = input_obj.__class__
+    state_dict = input_obj.state_dict()
+    # Cast dict values to float to avoid errors with Tensors.
+    state_dict = {k: float(v) for k, v in state_dict.items()}
+    return {
+        "__type": f"{class_type.__name__}",
+        "index": class_type,
+        "class": f"{class_type}",
+        "state_dict": state_dict,
+    }
+
+
+def percentile_early_stopping_strategy_to_dict(
+    strategy: PercentileEarlyStoppingStrategy,
+) -> Dict[str, Any]:
+    """Convert Ax percentile early stopping strategy to a dictionary."""
+    return {
+        "__type": strategy.__class__.__name__,
+        "metric_names": strategy.metric_names,
+        "percentile_threshold": strategy.percentile_threshold,
+        "min_progression": strategy.min_progression,
+        "min_curves": strategy.min_curves,
+        "trial_indices_to_ignore": strategy.trial_indices_to_ignore,
+        "true_objective_metric_name": strategy.true_objective_metric_name,
+        "seconds_between_polls": strategy.seconds_between_polls,
+        "normalize_progressions": strategy.normalize_progressions,
+    }
+
+
+def threshold_early_stopping_strategy_to_dict(
+    strategy: ThresholdEarlyStoppingStrategy,
+) -> Dict[str, Any]:
+    """Convert Ax metric-threshold early stopping strategy to a dictionary."""
+    return {
+        "__type": strategy.__class__.__name__,
+        "metric_names": strategy.metric_names,
+        "metric_threshold": strategy.metric_threshold,
+        "min_progression": strategy.min_progression,
+        "trial_indices_to_ignore": strategy.trial_indices_to_ignore,
+        "true_objective_metric_name": strategy.true_objective_metric_name,
+        "normalize_progressions": strategy.normalize_progressions,
+    }
+
+
+def logical_early_stopping_strategy_to_dict(
+    strategy: LogicalEarlyStoppingStrategy,
+) -> Dict[str, Any]:
+    return {
+        "__type": strategy.__class__.__name__,
+        "left": strategy.left,
+        "right": strategy.right,
+    }
+
+
+def winsorization_config_to_dict(config: WinsorizationConfig) -> Dict[str, Any]:
+    """Convert Ax winsorization config to a dictionary."""
+    return {
+        "__type": config.__class__.__name__,
+        "lower_quantile_margin": config.lower_quantile_margin,
+        "upper_quantile_margin": config.upper_quantile_margin,
+        "lower_boundary": config.lower_boundary,
+        "upper_boundary": config.upper_boundary,
+    }
+
+
+def pytorch_cnn_torchvision_benchmark_problem_to_dict(
+    problem: PyTorchCNNTorchvisionBenchmarkProblem,
+) -> Dict[str, Any]:
+    # unit tests for this in benchmark suite
+    return {  # pragma: no cover
+        "__type": problem.__class__.__name__,
+        "name": not_none(re.compile("(?<=::).*").search(problem.name)).group(),
+        "num_trials": problem.num_trials,
+    }
+
+
+def risk_measure_to_dict(
+    risk_measure: RiskMeasure,
+) -> Dict[str, Any]:
+    """Convert a RiskMeasure to a dictionary."""
+    return {
+        "__type": risk_measure.__class__.__name__,
+        "risk_measure": risk_measure.risk_measure,
+        "options": risk_measure.options,
+    }
