@@ -1,15 +1,15 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from collections import OrderedDict, defaultdict
+import json
+from collections import defaultdict, OrderedDict
 from enum import Enum
-from typing import List, Optional, Tuple, Type, Union
+from typing import Any, cast, Dict, List, Optional, Tuple, Type, Union
 
 import pandas as pd
-from ax.core.abstract_data import AbstractDataFrameData
 from ax.core.arm import Arm
 from ax.core.base_trial import BaseTrial, TrialStatus
 from ax.core.batch_trial import AbandonedArm, BatchTrial, GeneratorRunStruct
@@ -34,16 +34,16 @@ from ax.core.parameter_constraint import (
     ParameterConstraint,
     SumConstraint,
 )
+from ax.core.parameter_distribution import ParameterDistribution
+from ax.core.risk_measures import RiskMeasure
 from ax.core.runner import Runner
-from ax.core.search_space import SearchSpace
-from ax.core.simple_experiment import SimpleExperiment
+from ax.core.search_space import RobustSearchSpace, SearchSpace
 from ax.core.trial import Trial
 from ax.exceptions.storage import SQADecodeError
 from ax.modelbridge.generation_strategy import GenerationStrategy
-from ax.modelbridge.registry import Models, ModelRegistryBase
+from ax.modelbridge.registry import ModelRegistryBase, Models
 from ax.storage.json_store.decoder import object_from_json
-from ax.storage.metric_registry import REVERSE_METRIC_REGISTRY
-from ax.storage.runner_registry import REVERSE_RUNNER_REGISTRY
+from ax.storage.sqa_store.db import session_scope
 from ax.storage.sqa_store.sqa_classes import (
     SQAAbandonedArm,
     SQAArm,
@@ -61,6 +61,7 @@ from ax.storage.sqa_store.sqa_config import SQAConfig
 from ax.storage.utils import DomainType, MetricIntent, ParameterConstraintType
 from ax.utils.common.constants import Keys
 from ax.utils.common.typeutils import not_none
+from sqlalchemy.orm.exc import DetachedInstanceError
 
 
 class Decoder:
@@ -76,6 +77,14 @@ class Decoder:
     def __init__(self, config: SQAConfig) -> None:
         self.config = config
 
+        # TODO[T113829027] Remove this in a couple months
+        self.EXTRA_REGISTRY_ERROR_NOTE = (
+            "ATTENTION: There have been some recent "
+            "changes to Metric/Runner registration in Ax. Please see "
+            "https://ax.dev/tutorials/gpei_hartmann_developer.html#9.-Save-to-JSON-or-SQL "  # noqa
+            "for the most up-to-date information on saving custom metrics."
+        )
+
     def get_enum_name(
         self, value: Optional[int], enum: Optional[Enum]
     ) -> Optional[str]:
@@ -91,7 +100,11 @@ class Decoder:
         except ValueError:
             raise SQADecodeError(f"Value {value} is invalid for enum {enum}.")
 
-    def _init_experiment_from_sqa(self, experiment_sqa: SQAExperiment) -> Experiment:
+    def _init_experiment_from_sqa(
+        self,
+        experiment_sqa: SQAExperiment,
+        ax_object_field_overrides: Optional[Dict[str, Any]] = None,
+    ) -> Experiment:
         """First step of conversion within experiment_from_sqa."""
         opt_config, tracking_metrics = self.opt_config_and_tracking_metrics_from_sqa(
             metrics_sqa=experiment_sqa.metrics
@@ -106,9 +119,6 @@ class Decoder:
             )
         status_quo = (
             Arm(
-                # pyre-fixme[6]: Expected `Dict[str, Optional[Union[bool, float,
-                #  int, str]]]` for 1st param but got `Optional[Dict[str,
-                #  Optional[Union[bool, float, int, str]]]]`.
                 parameters=experiment_sqa.status_quo_parameters,
                 name=experiment_sqa.status_quo_name,
             )
@@ -118,7 +128,14 @@ class Decoder:
         if len(experiment_sqa.runners) == 0:
             runner = None
         elif len(experiment_sqa.runners) == 1:
-            runner = self.runner_from_sqa(experiment_sqa.runners[0])
+            runner_kwargs = (
+                ax_object_field_overrides.get("runner")
+                if ax_object_field_overrides is not None
+                else None
+            )
+            runner = self.runner_from_sqa(
+                runner_sqa=experiment_sqa.runners[0], runner_kwargs=runner_kwargs
+            )
         else:
             raise ValueError(  # pragma: no cover
                 "Multiple runners on experiment "
@@ -128,41 +145,19 @@ class Decoder:
         # `experiment_sqa.properties` is `sqlalchemy.ext.mutable.MutableDict`
         # so need to convert it to regular dict.
         properties = dict(experiment_sqa.properties or {})
-        # Remove 'subclass' from experiment's properties, since its only
-        # used for decoding to the correct experiment subclass in storage.
-        subclass = properties.pop(Keys.SUBCLASS, None)
         default_data_type = experiment_sqa.default_data_type
-        if subclass == "SimpleExperiment":
-            if opt_config is None:
-                raise SQADecodeError(  # pragma: no cover
-                    "SimpleExperiment must have an optimization config."
-                )
-            experiment = SimpleExperiment(
-                name=experiment_sqa.name,
-                search_space=search_space,
-                objective_name=opt_config.objective.metric.name,
-                minimize=opt_config.objective.minimize,
-                outcome_constraints=opt_config.outcome_constraints,
-                status_quo=status_quo,
-                properties=properties,
-                default_data_type=default_data_type,
-            )
-            experiment.description = experiment_sqa.description
-            experiment.is_test = experiment_sqa.is_test
-        else:
-            experiment = Experiment(
-                name=experiment_sqa.name,
-                description=experiment_sqa.description,
-                search_space=search_space,
-                optimization_config=opt_config,
-                tracking_metrics=tracking_metrics,
-                runner=runner,
-                status_quo=status_quo,
-                is_test=experiment_sqa.is_test,
-                properties=properties,
-                default_data_type=default_data_type,
-            )
-        return experiment
+        return Experiment(
+            name=experiment_sqa.name,
+            description=experiment_sqa.description,
+            search_space=search_space,
+            optimization_config=opt_config,
+            tracking_metrics=tracking_metrics,
+            runner=runner,
+            status_quo=status_quo,
+            is_test=experiment_sqa.is_test,
+            properties=properties,
+            default_data_type=default_data_type,
+        )
 
     def _init_mt_experiment_from_sqa(
         self, experiment_sqa: SQAExperiment
@@ -181,9 +176,6 @@ class Decoder:
             )
         status_quo = (
             Arm(
-                # pyre-fixme[6]: Expected `Dict[str, Optional[Union[bool, float,
-                #  int, str]]]` for 1st param but got `Optional[Dict[str,
-                #  Optional[Union[bool, float, int, str]]]]`.
                 parameters=experiment_sqa.status_quo_parameters,
                 name=experiment_sqa.status_quo_name,
             )
@@ -224,7 +216,10 @@ class Decoder:
         return experiment
 
     def experiment_from_sqa(
-        self, experiment_sqa: SQAExperiment, reduced_state: bool = False
+        self,
+        experiment_sqa: SQAExperiment,
+        reduced_state: bool = False,
+        ax_object_field_overrides: Optional[Dict[str, Any]] = None,
     ) -> Experiment:
         """Convert SQLAlchemy Experiment to Ax Experiment.
 
@@ -233,15 +228,24 @@ class Decoder:
             reduced_state: Whether to load experiment with a slightly reduced state
                 (without abandoned arms on experiment and without model state,
                 search space, and optimization config on generator runs).
+            ax_object_field_overrides: Mapping of object types to mapping of fields
+                to override values loaded objects will all be instantiated with fields
+                set to override value
+                current valid object types are: "runner"
         """
         subclass = (experiment_sqa.properties or {}).get(Keys.SUBCLASS)
         if subclass == "MultiTypeExperiment":
             experiment = self._init_mt_experiment_from_sqa(experiment_sqa)
         else:
-            experiment = self._init_experiment_from_sqa(experiment_sqa)
+            experiment = self._init_experiment_from_sqa(
+                experiment_sqa, ax_object_field_overrides=ax_object_field_overrides
+            )
         trials = [
             self.trial_from_sqa(
-                trial_sqa=trial, experiment=experiment, reduced_state=reduced_state
+                trial_sqa=trial,
+                experiment=experiment,
+                reduced_state=reduced_state,
+                ax_object_field_overrides=ax_object_field_overrides,
             )
             for trial in experiment_sqa.trials
         ]
@@ -288,8 +292,6 @@ class Decoder:
             parameter = RangeParameter(
                 name=parameter_sqa.name,
                 parameter_type=parameter_sqa.parameter_type,
-                # pyre-fixme[6]: Expected `float` for 3rd param but got
-                #  `Optional[float]`.
                 lower=parameter_sqa.lower,
                 upper=parameter_sqa.upper,
                 log_scale=parameter_sqa.log_scale or False,
@@ -305,13 +307,11 @@ class Decoder:
             parameter = ChoiceParameter(
                 name=parameter_sqa.name,
                 parameter_type=parameter_sqa.parameter_type,
-                # pyre-fixme[6]: Expected `List[Optional[Union[bool, float, int,
-                #  str]]]` for 3rd param but got `Optional[List[Optional[Union[bool,
-                #  float, int, str]]]]`.
                 values=parameter_sqa.choice_values,
                 is_fidelity=parameter_sqa.is_fidelity or False,
                 target_value=parameter_sqa.target_value,
                 is_ordered=parameter_sqa.is_ordered,
+                # pyre-fixme[6]: Expected `bool` for 7th param but got `Optional[bool]`.
                 is_task=parameter_sqa.is_task,
             )
         elif parameter_sqa.domain_type == DomainType.FIXED:
@@ -353,9 +353,7 @@ class Decoder:
                     "Cannot decode SQAParameterConstraint because `lower_name` or "
                     "`upper_name` was not found."
                 )
-            # pyre-fixme[6]: Expected `str` for 1st param but got `None`.
             lower_parameter = parameter_map[lower_name]
-            # pyre-fixme[6]: Expected `str` for 1st param but got `None`.
             upper_parameter = parameter_map[upper_name]
             constraint = OrderConstraint(
                 lower_parameter=lower_parameter, upper_parameter=upper_parameter
@@ -395,6 +393,56 @@ class Decoder:
         constraint.db_id = parameter_constraint_sqa.id
         return constraint
 
+    def parameter_distributions_from_sqa(
+        self,
+        parameter_constraint_sqa_list: List[SQAParameterConstraint],
+    ) -> Tuple[List[ParameterDistribution], Optional[int]]:
+        """Convert SQLAlchemy ParameterConstraints to Ax ParameterDistributions."""
+        parameter_distributions: List[ParameterDistribution] = []
+        num_samples = None
+        for parameter_constraint_sqa in parameter_constraint_sqa_list:
+            if parameter_constraint_sqa.type != ParameterConstraintType.DISTRIBUTION:
+                raise SQADecodeError(  # pragma: no cover
+                    "Parameter distribution must have type `DISTRIBUTION`. "
+                    "Received type "
+                    f"{ParameterConstraintType(parameter_constraint_sqa.type).name}."
+                )
+            num_samples = int(parameter_constraint_sqa.bound)
+            distribution = object_from_json(
+                parameter_constraint_sqa.constraint_dict,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
+            )
+            distribution.db_id = parameter_constraint_sqa.id
+            parameter_distributions.append(distribution)
+        return parameter_distributions, num_samples
+
+    def environmental_variable_from_sqa(self, parameter_sqa: SQAParameter) -> Parameter:
+        """Convert SQLAlchemy Parameter to Ax environmental variable."""
+        if parameter_sqa.domain_type == DomainType.ENVIRONMENTAL_RANGE:
+            if parameter_sqa.lower is None or parameter_sqa.upper is None:
+                raise SQADecodeError(  # pragma: no cover
+                    "`lower` and `upper` must be set for RangeParameter."
+                )
+            parameter = RangeParameter(
+                name=parameter_sqa.name,
+                parameter_type=parameter_sqa.parameter_type,
+                lower=parameter_sqa.lower,
+                upper=parameter_sqa.upper,
+                log_scale=parameter_sqa.log_scale or False,
+                digits=parameter_sqa.digits,
+                is_fidelity=parameter_sqa.is_fidelity or False,
+                target_value=parameter_sqa.target_value,
+            )
+        else:  # pragma: no cover
+            raise SQADecodeError(
+                f"Cannot decode SQAParameter because {parameter_sqa.domain_type} "
+                "is an invalid domain type."
+            )
+
+        parameter.db_id = parameter_sqa.id
+        return parameter
+
     def search_space_from_sqa(
         self,
         parameters_sqa: List[SQAParameter],
@@ -403,43 +451,76 @@ class Decoder:
         """Convert a list of SQLAlchemy Parameters and ParameterConstraints to an
         Ax SearchSpace.
         """
-        parameters = [
-            self.parameter_from_sqa(parameter_sqa=parameter_sqa)
-            for parameter_sqa in parameters_sqa
-        ]
+        parameters, environmental_variables = [], []
+        for parameter_sqa in parameters_sqa:
+            if parameter_sqa.domain_type == DomainType.ENVIRONMENTAL_RANGE:
+                environmental_variables.append(
+                    self.environmental_variable_from_sqa(parameter_sqa=parameter_sqa)
+                )
+            else:
+                parameters.append(self.parameter_from_sqa(parameter_sqa=parameter_sqa))
         parameter_constraints = [
             self.parameter_constraint_from_sqa(
                 parameter_constraint_sqa=parameter_constraint_sqa, parameters=parameters
             )
             for parameter_constraint_sqa in parameter_constraints_sqa
+            if parameter_constraint_sqa.type != ParameterConstraintType.DISTRIBUTION
         ]
+        parameter_distributions, num_samples = self.parameter_distributions_from_sqa(
+            [
+                parameter_constraint_sqa
+                for parameter_constraint_sqa in parameter_constraints_sqa
+                if parameter_constraint_sqa.type == ParameterConstraintType.DISTRIBUTION
+            ]
+        )
 
         if len(parameters) == 0:
             return None
 
-        return SearchSpace(
-            parameters=parameters, parameter_constraints=parameter_constraints
-        )
+        if num_samples is not None:
+            return RobustSearchSpace(
+                parameters=parameters,
+                parameter_distributions=parameter_distributions,
+                num_samples=num_samples,
+                environmental_variables=environmental_variables,
+                parameter_constraints=parameter_constraints,
+            )
+        else:
+            return SearchSpace(
+                parameters=parameters, parameter_constraints=parameter_constraints
+            )
 
     def metric_from_sqa_util(self, metric_sqa: SQAMetric) -> Metric:
         """Convert SQLAlchemy Metric to Ax Metric"""
-        metric_class = REVERSE_METRIC_REGISTRY.get(metric_sqa.metric_type)
-        if metric_class is None:
+        if metric_sqa.metric_type not in self.config.reverse_metric_registry:
             raise SQADecodeError(
                 f"Cannot decode SQAMetric because {metric_sqa.metric_type} "
-                f"is an invalid type."
+                f"is an invalid type. "
+                f"{self.EXTRA_REGISTRY_ERROR_NOTE}"
             )
-        args = dict(metric_sqa.properties or {})
+        metric_class = self.config.reverse_metric_registry[metric_sqa.metric_type]
+
+        args = dict(
+            object_from_json(
+                metric_sqa.properties,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
+            )
+            or {}
+        )
         args["name"] = metric_sqa.name
         args["lower_is_better"] = metric_sqa.lower_is_better
-        args = metric_class.deserialize_init_args(args=args)
-        metric = metric_class(**args)
-        metric.db_id = metric_sqa.id
-        return metric
+        try:
+            args = metric_class.deserialize_init_args(args=args)
+            metric = metric_class(**args)
+            metric.db_id = metric_sqa.id
+            return metric
+        except ValueError as err:
+            raise ValueError(f"{err} {self.EXTRA_REGISTRY_ERROR_NOTE}")
 
     def metric_from_sqa(
         self, metric_sqa: SQAMetric
-    ) -> Union[Metric, Objective, OutcomeConstraint]:
+    ) -> Union[Metric, Objective, OutcomeConstraint, RiskMeasure]:
         """Convert SQLAlchemy Metric to Ax Metric, Objective, or OutcomeConstraint."""
 
         metric = self.metric_from_sqa_util(metric_sqa)
@@ -461,7 +542,13 @@ class Decoder:
             metric_sqa.intent == MetricIntent.MULTI_OBJECTIVE
         ):  # metric_sqa is a parent whose children are individual
             # metrics in MultiObjective
-            metrics_sqa_children = metric_sqa.scalarized_objective_children_metrics
+            try:
+                metrics_sqa_children = metric_sqa.scalarized_objective_children_metrics
+            except DetachedInstanceError:
+                metrics_sqa_children = _get_scalarized_objective_children_metrics(
+                    metric_id=metric_sqa.id, decoder=self
+                )
+
             if metrics_sqa_children is None:
                 raise SQADecodeError(  # pragma: no cover
                     "Cannot decode SQAMetric to MultiObjective \
@@ -489,7 +576,14 @@ class Decoder:
                     "Cannot decode SQAMetric to Scalarized Objective \
                     because minimize is None."
                 )
-            metrics_sqa_children = metric_sqa.scalarized_objective_children_metrics
+
+            try:
+                metrics_sqa_children = metric_sqa.scalarized_objective_children_metrics
+            except DetachedInstanceError:
+                metrics_sqa_children = _get_scalarized_objective_children_metrics(
+                    metric_id=metric_sqa.id, decoder=self
+                )
+
             if metrics_sqa_children is None:
                 raise SQADecodeError(  # pragma: no cover
                     "Cannot decode SQAMetric to Scalarized Objective \
@@ -509,8 +603,7 @@ class Decoder:
             scalarized_objective = ScalarizedObjective(
                 metrics=list(metrics),
                 weights=list(weights),
-                # pyre-fixme[6]: Expected `bool` for 3nd param but got `Optional[bool]`.
-                minimize=metric_sqa.minimize,
+                minimize=not_none(metric_sqa.minimize),
             )
             scalarized_objective.db_id = metric_sqa.id
             return scalarized_objective
@@ -526,8 +619,6 @@ class Decoder:
                 )
             return OutcomeConstraint(
                 metric=metric,
-                # pyre-fixme[6]: Expected `float` for 2nd param but got
-                #  `Optional[float]`.
                 bound=metric_sqa.bound,
                 op=metric_sqa.op,
                 relative=metric_sqa.relative,
@@ -542,9 +633,18 @@ class Decoder:
                     "Cannot decode SQAMetric to Scalarized OutcomeConstraint because "
                     "bound, op, or relative is None."
                 )
-            metrics_sqa_children = (
-                metric_sqa.scalarized_outcome_constraint_children_metrics
-            )
+
+            try:
+                metrics_sqa_children = (
+                    metric_sqa.scalarized_outcome_constraint_children_metrics
+                )
+            except DetachedInstanceError:
+                metrics_sqa_children = (
+                    _get_scalarized_outcome_constraint_children_metrics(
+                        metric_id=metric_sqa.id, decoder=self
+                    )
+                )
+
             if metrics_sqa_children is None:
                 raise SQADecodeError(  # pragma: no cover
                     "Cannot decode SQAMetric to Scalarized OutcomeConstraint \
@@ -564,11 +664,9 @@ class Decoder:
             scalarized_outcome_constraint = ScalarizedOutcomeConstraint(
                 metrics=list(metrics),
                 weights=list(weights),
-                # pyre-fixme[6]: Expected `float` for 2nd param but got
-                #  `Optional[float]`.
-                bound=metric_sqa.bound,
-                op=metric_sqa.op,
-                relative=metric_sqa.relative,
+                bound=not_none(metric_sqa.bound),
+                op=not_none(metric_sqa.op),
+                relative=not_none(metric_sqa.relative),
             )
             scalarized_outcome_constraint.db_id = metric_sqa.id
             return scalarized_outcome_constraint
@@ -578,14 +676,26 @@ class Decoder:
                     "Cannot decode SQAMetric to ObjectiveThreshold because "
                     "bound, op, or relative is None."
                 )
-            return ObjectiveThreshold(
+            ot = ObjectiveThreshold(
                 metric=metric,
-                # pyre-fixme[6]: Expected `float` for 2nd param but got
-                #  `Optional[float]`.
                 bound=metric_sqa.bound,
                 relative=metric_sqa.relative,
                 op=metric_sqa.op,
             )
+            # ObjectiveThreshold constructor clones the passed-in metric, which means
+            # the db id gets lost and so we need to reset it
+            ot.metric._db_id = metric.db_id
+            return ot
+        elif metric_sqa.intent == MetricIntent.RISK_MEASURE:
+            rm = RiskMeasure(
+                **object_from_json(
+                    metric_sqa.properties,
+                    decoder_registry=self.config.json_decoder_registry,
+                    class_decoder_registry=self.config.json_class_decoder_registry,
+                )
+            )
+            rm._db_id = metric.db_id
+            return rm
         else:
             raise SQADecodeError(
                 f"Cannot decode SQAMetric because {metric_sqa.intent} "
@@ -602,6 +712,7 @@ class Decoder:
         objective_thresholds = []
         outcome_constraints = []
         tracking_metrics = []
+        risk_measure = None
         for metric_sqa in metrics_sqa:
             metric = self.metric_from_sqa(metric_sqa=metric_sqa)
             if isinstance(metric, Objective):
@@ -610,6 +721,8 @@ class Decoder:
                 objective_thresholds.append(metric)
             elif isinstance(metric, OutcomeConstraint):
                 outcome_constraints.append(metric)
+            elif isinstance(metric, RiskMeasure):
+                risk_measure = metric
             else:
                 tracking_metrics.append(metric)
 
@@ -621,10 +734,13 @@ class Decoder:
                 objective=objective,
                 outcome_constraints=outcome_constraints,
                 objective_thresholds=objective_thresholds,
+                risk_measure=risk_measure,
             )
         else:
             optimization_config = OptimizationConfig(
-                objective=objective, outcome_constraints=outcome_constraints
+                objective=objective,
+                outcome_constraints=outcome_constraints,
+                risk_measure=risk_measure,
             )
         return (optimization_config, tracking_metrics)
 
@@ -716,23 +832,45 @@ class Decoder:
             fit_time=generator_run_sqa.fit_time,
             gen_time=generator_run_sqa.gen_time,
             best_arm_predictions=best_arm_predictions,  # pyre-ignore[6]
+            # pyre-fixme[6]: Expected `Optional[Tuple[typing.Dict[str, List[float]],
+            #  typing.Dict[str, typing.Dict[str, List[float]]]]]` for 8th param but got
+            #  `Optional[typing.Tuple[Union[typing.Dict[str, List[float]],
+            #  typing.Dict[str, typing.Dict[str, List[float]]]], ...]]`.
             model_predictions=model_predictions,
             model_key=generator_run_sqa.model_key,
             model_kwargs=None
             if reduced_state
-            else object_from_json(generator_run_sqa.model_kwargs),
+            else object_from_json(
+                generator_run_sqa.model_kwargs,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
+            ),
             bridge_kwargs=None
             if reduced_state
-            else object_from_json(generator_run_sqa.bridge_kwargs),
+            else object_from_json(
+                generator_run_sqa.bridge_kwargs,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
+            ),
             gen_metadata=None
             if reduced_state
-            else object_from_json(generator_run_sqa.gen_metadata),
+            else object_from_json(
+                generator_run_sqa.gen_metadata,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
+            ),
             model_state_after_gen=None
             if reduced_state
-            else object_from_json(generator_run_sqa.model_state_after_gen),
+            else object_from_json(
+                generator_run_sqa.model_state_after_gen,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
+            ),
             generation_step_index=generator_run_sqa.generation_step_index,
             candidate_metadata_by_arm_signature=object_from_json(
-                generator_run_sqa.candidate_metadata_by_arm_signature
+                generator_run_sqa.candidate_metadata_by_arm_signature,
+                decoder_registry=self.config.json_decoder_registry,
+                class_decoder_registry=self.config.json_class_decoder_registry,
             ),
         )
         generator_run._time_created = generator_run_sqa.time_created
@@ -751,7 +889,11 @@ class Decoder:
         reduced_state: bool = False,
     ) -> GenerationStrategy:
         """Convert SQALchemy generation strategy to Ax `GenerationStrategy`."""
-        steps = object_from_json(gs_sqa.steps)
+        steps = object_from_json(
+            gs_sqa.steps,
+            decoder_registry=self.config.json_decoder_registry,
+            class_decoder_registry=self.config.json_class_decoder_registry,
+        )
         gs = GenerationStrategy(name=gs_sqa.name, steps=steps)
         gs._curr = gs._steps[gs_sqa.curr_index]
         immutable_ss_and_oc = (
@@ -786,6 +928,7 @@ class Decoder:
                 )
                 for gr in gs_sqa.generator_runs
             ]
+        gs._experiment = experiment
         if len(gs._generator_runs) > 0:
             # Generation strategy had an initialized model.
             if experiment is None:
@@ -793,7 +936,6 @@ class Decoder:
                     "Cannot decode a generation strategy with a non-zero number of "
                     "generator runs without an experiment."
                 )
-            gs._experiment = experiment
             # If model in the current step was not directly from the `Models` enum,
             # pass its type to `restore_model_from_generator_run`, which will then
             # attempt to use this type to recreate the model.
@@ -807,24 +949,36 @@ class Decoder:
         gs.db_id = gs_sqa.id
         return gs
 
-    def runner_from_sqa(self, runner_sqa: SQARunner) -> Runner:
+    def runner_from_sqa(
+        self, runner_sqa: SQARunner, runner_kwargs: Optional[Dict[str, Any]] = None
+    ) -> Runner:
         """Convert SQLAlchemy Runner to Ax Runner."""
-        runner_class = REVERSE_RUNNER_REGISTRY.get(runner_sqa.runner_type)
-        if runner_class is None:
+        if runner_sqa.runner_type not in self.config.reverse_runner_registry:
             raise SQADecodeError(
                 f"Cannot decode SQARunner because {runner_sqa.runner_type} "
-                f"is an invalid type."
+                f"is an invalid type. "
+                f"{self.EXTRA_REGISTRY_ERROR_NOTE}"
             )
-        args = runner_class.deserialize_init_args(
-            args=dict(runner_sqa.properties or {})
-        )
-        # pyre-ignore[45]: Cannot instantiate abstract class `Runner`.
-        runner = runner_class(**args)
-        runner.db_id = runner_sqa.id
-        return runner
+        runner_class = self.config.reverse_runner_registry[runner_sqa.runner_type]
+
+        try:
+            args = runner_class.deserialize_init_args(
+                args=dict(runner_sqa.properties or {})
+            )
+            args.update(runner_kwargs or {})
+            # pyre-ignore[45]: Cannot instantiate abstract class `Runner`.
+            runner = runner_class(**args)
+            runner.db_id = runner_sqa.id
+            return runner
+        except ValueError as err:
+            raise ValueError(f"{err} {self.EXTRA_REGISTRY_ERROR_NOTE}")
 
     def trial_from_sqa(
-        self, trial_sqa: SQATrial, experiment: Experiment, reduced_state: bool = False
+        self,
+        trial_sqa: SQATrial,
+        experiment: Experiment,
+        reduced_state: bool = False,
+        ax_object_field_overrides: Optional[Dict[str, Any]] = None,
     ) -> BaseTrial:
         """Convert SQLAlchemy Trial to Ax Trial.
 
@@ -841,6 +995,7 @@ class Decoder:
                 optimize_for_power=trial_sqa.optimize_for_power,
                 ttl_seconds=trial_sqa.ttl_seconds,
                 index=trial_sqa.index,
+                lifecycle_stage=trial_sqa.lifecycle_stage,
             )
             generator_run_structs = [
                 GeneratorRunStruct(
@@ -914,15 +1069,29 @@ class Decoder:
         # pyre-fixme[8]: Attribute has type `Dict[str, typing.Any]`; used as
         #  `Optional[typing.Dict[Variable[_KT], Variable[_VT]]]`.
         trial._run_metadata = (
-            # pyre-fixme[6]: Expected `Mapping[Variable[_KT], Variable[_VT]]` for
-            #  1st param but got `Optional[Dict[str, typing.Any]]`.
-            dict(trial_sqa.run_metadata)
-            if trial_sqa.run_metadata is not None
+            dict(trial_sqa.run_metadata) if trial_sqa.run_metadata is not None else None
+        )
+        # pyre-fixme[9]: _run_metadata has type `Dict[str, Any]`; used as
+        #  `Optional[Dict[str, Any]]`.
+        # pyre-fixme[8]: Attribute has type `Dict[str, typing.Any]`; used as
+        #  `Optional[typing.Dict[Variable[_KT], Variable[_VT]]]`.
+        trial._stop_metadata = (
+            dict(trial_sqa.stop_metadata)
+            if trial_sqa.stop_metadata is not None
             else None
         )
         trial._num_arms_created = trial_sqa.num_arms_created
         trial._runner = (
-            self.runner_from_sqa(trial_sqa.runner) if trial_sqa.runner else None
+            self.runner_from_sqa(
+                trial_sqa.runner,
+                runner_kwargs=(
+                    ax_object_field_overrides.get("runner")
+                    if ax_object_field_overrides is not None
+                    else None
+                ),
+            )
+            if trial_sqa.runner
+            else None
         )
         trial._generation_step_index = trial_sqa.generation_step_index
         trial._properties = dict(trial_sqa.properties or {})
@@ -932,17 +1101,57 @@ class Decoder:
     def data_from_sqa(
         self,
         data_sqa: SQAData,
-        data_constructor: Optional[Type[AbstractDataFrameData]] = None,
-    ) -> AbstractDataFrameData:
+        data_constructor: Type[Data] = Data,
+    ) -> Data:
         """Convert SQLAlchemy Data to AE Data."""
         # TODO: extract data type from SQAData after DataRegistry added.
-        data_constructor = data_constructor or Data
-        # pyre-ignore[45]: Cannot instantiate abstract class. But this is concrete.
-        dat = data_constructor(
-            description=data_sqa.description,
-            # NOTE: Need dtype=False, otherwise infers arm_names like
-            # "4_1" should be int 41.
-            df=pd.read_json(data_sqa.data_json, dtype=False),
+        kwargs = data_constructor.deserialize_init_args(
+            args=dict(
+                json.loads(data_sqa.structure_metadata_json)
+                if data_sqa.structure_metadata_json
+                else {}
+            )
         )
+
+        # Override df from deserialize_init_args with `data_json`.
+        # NOTE: Need dtype=False, otherwise infers arm_names like
+        # "4_1" should be int 41.
+        kwargs["df"] = pd.read_json(data_sqa.data_json, dtype=False)
+
+        dat = data_constructor(**kwargs)
+
         dat.db_id = data_sqa.id
         return dat
+
+
+def _get_scalarized_objective_children_metrics(
+    metric_id: int, decoder: Decoder
+) -> List[SQAMetric]:
+    """Given a metric db id, fetch its scalarized objective children metrics."""
+    metric_sqa_class = cast(
+        Type[SQAMetric],
+        decoder.config.class_to_sqa_class[Metric],
+    )
+    with session_scope() as session:
+        query = session.query(metric_sqa_class).filter_by(
+            scalarized_objective_id=metric_id
+        )
+        metrics_sqa = query.all()
+    return metrics_sqa
+
+
+def _get_scalarized_outcome_constraint_children_metrics(
+    metric_id: int, decoder: Decoder
+) -> List[SQAMetric]:
+    """Given a metric db id, fetch its scalarized outcome constraint
+    children metrics."""
+    metric_sqa_class = cast(
+        Type[SQAMetric],
+        decoder.config.class_to_sqa_class[Metric],
+    )
+    with session_scope() as session:
+        query = session.query(metric_sqa_class).filter_by(
+            scalarized_outcome_constraint_id=metric_id
+        )
+        metrics_sqa = query.all()
+    return metrics_sqa
