@@ -1,16 +1,18 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 from enum import Enum
-from typing import Any, Dict, List, Optional, Tuple, Type, cast
 
-from ax.core.abstract_data import AbstractDataFrameData
+from logging import Logger
+from typing import Any, cast, Dict, List, Optional, Tuple, Type
+
 from ax.core.arm import Arm
 from ax.core.base_trial import BaseTrial
 from ax.core.batch_trial import AbandonedArm, BatchTrial
+from ax.core.data import Data
 from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun, GeneratorRunType
 from ax.core.metric import Metric
@@ -31,15 +33,14 @@ from ax.core.parameter_constraint import (
     ParameterConstraint,
     SumConstraint,
 )
+from ax.core.parameter_distribution import ParameterDistribution
+from ax.core.risk_measures import RiskMeasure
 from ax.core.runner import Runner
-from ax.core.search_space import SearchSpace
-from ax.core.simple_experiment import SimpleExperiment
+from ax.core.search_space import RobustSearchSpace, SearchSpace
 from ax.core.trial import Trial
 from ax.exceptions.storage import SQAEncodeError
 from ax.modelbridge.generation_strategy import GenerationStrategy
 from ax.storage.json_store.encoder import object_to_json
-from ax.storage.metric_registry import METRIC_REGISTRY
-from ax.storage.runner_registry import RUNNER_REGISTRY
 from ax.storage.sqa_store.sqa_classes import (
     SQAAbandonedArm,
     SQAArm,
@@ -58,10 +59,10 @@ from ax.storage.utils import DomainType, MetricIntent, ParameterConstraintType
 from ax.utils.common.base import Base
 from ax.utils.common.constants import Keys
 from ax.utils.common.logger import get_logger
+from ax.utils.common.serialization import serialize_init_args
 from ax.utils.common.typeutils import checked_cast, not_none
 
-
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 
 
 class Encoder:
@@ -77,6 +78,14 @@ class Encoder:
     def __init__(self, config: SQAConfig) -> None:
         self.config = config
 
+        # TODO[T113829027] Remove this in a couple months
+        self.EXTRA_REGISTRY_ERROR_NOTE = (
+            "ATTENTION: There have been some recent "
+            "changes to Metric/Runner registration in Ax. Please see "
+            "https://ax.dev/tutorials/gpei_hartmann_developer.html#9.-Save-to-JSON-or-SQL "  # noqa
+            "for the most up-to-date information on saving custom metrics."
+        )
+
     @classmethod
     def validate_experiment_metadata(
         cls,
@@ -86,13 +95,13 @@ class Encoder:
         """Validates required experiment metadata."""
         if experiment.db_id is not None:
             if existing_sqa_experiment_id is None:
-                raise Exception(
+                raise ValueError(
                     f"Experiment with ID {experiment.db_id} was already saved to the "
                     "database with a different name. Changing the name of an "
                     "experiment is not allowed."
                 )
             elif experiment.db_id != existing_sqa_experiment_id:
-                raise Exception(
+                raise ValueError(
                     f"experiment.db_id = {experiment.db_id} but the experiment in the "
                     f"database with the name {experiment.name} has the id "
                     f"{existing_sqa_experiment_id}."
@@ -100,7 +109,7 @@ class Encoder:
         else:
             # experiment.db_id is None
             if existing_sqa_experiment_id is not None:
-                raise Exception(
+                raise ValueError(
                     f"An experiment already exists with the name {experiment.name}. "
                     "If you need to override this existing experiment, first delete it "
                     "via `delete_experiment` in ax/ax/storage/sqa_store/delete.py, "
@@ -123,10 +132,7 @@ class Encoder:
             raise SQAEncodeError(f"Value {value} is invalid for enum {enum}.")
 
     def experiment_to_sqa(self, experiment: Experiment) -> SQAExperiment:
-        """Convert Ax Experiment to SQLAlchemy and compile a list of (object,
-        sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
+        """Convert Ax Experiment to SQLAlchemy.
 
         In addition to creating and storing a new Experiment object, we need to
         create and store copies of the Trials, Metrics, Parameters,
@@ -184,9 +190,6 @@ class Encoder:
         elif experiment.runner:
             runners.append(self.runner_to_sqa(not_none(experiment.runner)))
 
-        if isinstance(experiment, SimpleExperiment):
-            properties[Keys.SUBCLASS] = "SimpleExperiment"
-
         # pyre-ignore[9]: Expected `Base` for 1st...yping.Type[Experiment]`.
         experiment_class: Type[SQAExperiment] = self.config.class_to_sqa_class[
             Experiment
@@ -214,7 +217,11 @@ class Encoder:
 
     def parameter_to_sqa(self, parameter: Parameter) -> SQAParameter:
         """Convert Ax Parameter to SQLAlchemy."""
-        # pyre-fixme: Expected `Base` for 1st...typing.Type[Parameter]`.
+        if parameter.is_hierarchical:  # pragma: no cover
+            raise NotImplementedError(
+                "SQL storage is not yet implemented for hierarchical search spaces."
+            )
+        # pyre-fixme[9]: Expected `Base` for 1st...typing.Type[Parameter]`.
         parameter_class: SQAParameter = self.config.class_to_sqa_class[Parameter]
         if isinstance(parameter, RangeParameter):
             # pyre-fixme[29]: `SQAParameter` is not a function.
@@ -297,11 +304,10 @@ class Encoder:
         self, search_space: Optional[SearchSpace]
     ) -> Tuple[List[SQAParameter], List[SQAParameterConstraint]]:
         """Convert Ax SearchSpace to a list of SQLAlchemy Parameters and
-        ParameterConstraints and compile a list of (object,
-        sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
+        ParameterConstraints.
         """
+        if isinstance(search_space, RobustSearchSpace):
+            return self.robust_search_space_to_sqa(rss=search_space)
         parameters, parameter_constraints = [], []
         if search_space is not None:
             for parameter in search_space.parameters.values():
@@ -316,6 +322,84 @@ class Encoder:
 
         return parameters, parameter_constraints
 
+    def parameter_distribution_to_sqa(
+        self, distribution: ParameterDistribution, num_samples: int
+    ) -> SQAParameterConstraint:
+        """Convert Ax ParameterDistribution to SQLAlchemy.
+
+        NOTE: This saves the distributions as json blobs in `constraint_dict`
+        to avoid creating a new table in the short term. If robust optimization
+        sees more usage in the long term, the proper solution would be to
+        make a new table for these.
+        """
+        # pyre-fixme[9]: parameter_constraint_cl... used as type `SQABase`.
+        param_constraint_cls: SQAParameterConstraint = self.config.class_to_sqa_class[
+            ParameterConstraint
+        ]
+        # pyre-fixme[29]: `SQAParameterConstraint` is not a function.
+        return param_constraint_cls(
+            id=distribution.db_id,
+            type=ParameterConstraintType.DISTRIBUTION,
+            constraint_dict=object_to_json(
+                distribution,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
+            ),
+            bound=num_samples,
+        )
+
+    def environmental_variable_to_sqa(self, parameter: Parameter) -> SQAParameter:
+        """Convert Ax environmental variables to SQLAlchemy.
+
+        Since these are effectively just range parameters with an associated
+        distribution, which is stored separately, we will store these as new
+        parameter types.
+        """
+        # pyre-fixme: Expected `Base` for 1st...typing.Type[Parameter]`.
+        parameter_class: SQAParameter = self.config.class_to_sqa_class[Parameter]
+        if isinstance(parameter, RangeParameter):
+            # pyre-fixme[29]: `SQAParameter` is not a function.
+            return parameter_class(
+                id=parameter.db_id,
+                name=parameter.name,
+                domain_type=DomainType.ENVIRONMENTAL_RANGE,
+                parameter_type=parameter.parameter_type,
+                lower=float(parameter.lower),
+                upper=float(parameter.upper),
+                log_scale=parameter.log_scale,
+                digits=parameter.digits,
+                is_fidelity=parameter.is_fidelity,
+                target_value=parameter.target_value,
+            )
+        else:  # pragma: no cover
+            raise SQAEncodeError(
+                "Cannot encode environmental variable to SQLAlchemy because "
+                f"the corresponding parameter type ({type(parameter)}) is invalid."
+            )
+
+    def robust_search_space_to_sqa(
+        self, rss: RobustSearchSpace
+    ) -> Tuple[List[SQAParameter], List[SQAParameterConstraint]]:
+        parameters, parameter_constraints = [], []
+        for parameter in rss._parameters.values():
+            parameters.append(self.parameter_to_sqa(parameter=parameter))
+        for parameter in rss._environmental_variables.values():
+            parameters.append(self.environmental_variable_to_sqa(parameter=parameter))
+        for parameter_constraint in rss.parameter_constraints:
+            parameter_constraints.append(
+                self.parameter_constraint_to_sqa(
+                    parameter_constraint=parameter_constraint
+                )
+            )
+        for distribution in rss.parameter_distributions:
+            parameter_constraints.append(
+                self.parameter_distribution_to_sqa(
+                    distribution=distribution,
+                    num_samples=rss.num_samples,
+                )
+            )
+        return parameters, parameter_constraints
+
     def get_metric_type_and_properties(
         self, metric: Metric
     ) -> Tuple[int, Dict[str, Any]]:
@@ -324,17 +408,22 @@ class Encoder:
         json blob.
         """
         metric_class = type(metric)
-        metric_type = METRIC_REGISTRY.get(metric_class)
+        metric_type = self.config.metric_registry.get(metric_class)
         if metric_type is None:
             raise SQAEncodeError(
                 "Cannot encode metric to SQLAlchemy because metric's "
                 f"subclass ({metric_class}) is missing from the registry. "
                 "The metric registry currently contains the following: "
-                f"{','.join(map(str, METRIC_REGISTRY.keys()))}"
+                f"{','.join(map(str, self.config.metric_registry.keys()))} "
+                f"{self.EXTRA_REGISTRY_ERROR_NOTE}"
             )  # pragma: no cover
 
-        properties = metric_class.serialize_init_args(metric=metric)
-        return metric_type, properties
+        properties = metric_class.serialize_init_args(obj=metric)
+        return metric_type, object_to_json(
+            properties,
+            encoder_registry=self.config.json_encoder_registry,
+            class_encoder_registry=self.config.json_class_encoder_registry,
+        )
 
     def metric_to_sqa(self, metric: Metric) -> SQAMetric:
         """Convert Ax Metric to SQLAlchemy."""
@@ -366,11 +455,7 @@ class Encoder:
         }
 
     def objective_to_sqa(self, objective: Objective) -> SQAMetric:
-        """Convert Ax Objective to SQLAlchemy and compile a list of (object,
-        sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
-        """
+        """Convert Ax Objective to SQLAlchemy."""
         if isinstance(objective, ScalarizedObjective):
             objective_sqa = self.scalarized_objective_to_sqa(objective)
 
@@ -397,10 +482,7 @@ class Encoder:
         return checked_cast(SQAMetric, objective_sqa)
 
     def multi_objective_to_sqa(self, multi_objective: MultiObjective) -> SQAMetric:
-        """Convert Ax Multi Objective to SQLAlchemy and compile a list of (object,
-        sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
+        """Convert Ax Multi Objective to SQLAlchemy.
 
         Returns: A parent `SQAMetric`, whose children are the `SQAMetric`-s
             corresponding to `metrics` attribute of `MultiObjective`.
@@ -416,7 +498,7 @@ class Encoder:
             )
             children_objectives.append(
                 objective_cls(  # pyre-ignore[29]: `SQAMetric` is not a func.
-                    id=objective.db_id,
+                    id=objective.metric.db_id,
                     name=objective.metric.name,
                     metric_type=type_and_properties[0],
                     intent=MetricIntent.OBJECTIVE,
@@ -433,7 +515,7 @@ class Encoder:
             parent_metric_cls(  # pyre-ignore[29]: `SQAMetric` is not a func.
                 id=multi_objective.db_id,
                 name="multi_objective",
-                metric_type=METRIC_REGISTRY[Metric],
+                metric_type=self.config.metric_registry[Metric],
                 intent=MetricIntent.MULTI_OBJECTIVE,
                 scalarized_objective_children_metrics=children_objectives,
             )
@@ -441,10 +523,7 @@ class Encoder:
         return parent_metric
 
     def scalarized_objective_to_sqa(self, objective: ScalarizedObjective) -> SQAMetric:
-        """Convert Ax Scalarized Objective to SQLAlchemy and compile a list of
-        (object, sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
+        """Convert Ax Scalarized Objective to SQLAlchemy.
 
         Returns: A parent `SQAMetric`, whose children are the `SQAMetric`-s
             corresponding to `metrics` attribute of `ScalarizedObjective`.
@@ -483,7 +562,7 @@ class Encoder:
         parent_metric = parent_metric_cls(  # pyre-ignore[29]: `SQAMetric` not a func.
             id=objective.db_id,
             name="scalarized_objective",
-            metric_type=METRIC_REGISTRY[Metric],
+            metric_type=self.config.metric_registry[Metric],
             intent=MetricIntent.SCALARIZED_OBJECTIVE,
             minimize=objective.minimize,
             lower_is_better=objective.minimize,
@@ -557,7 +636,7 @@ class Encoder:
         parent_metric = parent_metric_cls(  # pyre-ignore[29]: `SQAMetric` not a func.
             id=outcome_constraint.db_id,
             name="scalarized_outcome_constraint",
-            metric_type=METRIC_REGISTRY[Metric],
+            metric_type=self.config.metric_registry[Metric],
             intent=MetricIntent.SCALARIZED_OUTCOME_CONSTRAINT,
             bound=outcome_constraint.bound,
             op=outcome_constraint.op,
@@ -588,15 +667,23 @@ class Encoder:
             lower_is_better=metric.lower_is_better,
         )
 
+    def risk_measure_to_sqa(self, risk_measure: RiskMeasure) -> SQAMetric:
+        """Convert Ax RiskMeasure to SQLAlchemy."""
+        # pyre-fixme: Expected `Base` for 1st...t `typing.Type[Metric]`.
+        metric_class: SQAMetric = self.config.class_to_sqa_class[Metric]
+        # pyre-fixme[29]: `SQAMetric` is not a function.
+        return metric_class(
+            id=risk_measure.db_id,
+            name="risk measure",
+            metric_type=self.config.metric_registry[Metric],
+            intent=MetricIntent.RISK_MEASURE,
+            properties=serialize_init_args(risk_measure),
+        )
+
     def optimization_config_to_sqa(
         self, optimization_config: Optional[OptimizationConfig]
     ) -> List[SQAMetric]:
-        """Convert Ax OptimizationConfig to a list of SQLAlchemy Metrics
-        and compile a list of (object, sqa_counterpart) tuples to set `db_id`
-        on user-facing classes after the conversion is complete and the SQL
-        session is flushed (SQLAlchemy classes receive their `id` attributes
-        during `session.flush()`).
-        """
+        """Convert Ax OptimizationConfig to a list of SQLAlchemy Metrics."""
         if optimization_config is None:
             return []
 
@@ -614,6 +701,11 @@ class Encoder:
                     objective_threshold=threshold
                 )
                 metrics_sqa.append(threshold_sqa)
+        if optimization_config.risk_measure is not None:
+            risk_measure_sqa = self.risk_measure_to_sqa(
+                risk_measure=optimization_config.risk_measure
+            )
+            metrics_sqa.append(risk_measure_sqa)
         return metrics_sqa
 
     def arm_to_sqa(self, arm: Arm, weight: Optional[float] = 1.0) -> SQAArm:
@@ -640,17 +732,18 @@ class Encoder:
         )
 
     def generator_run_to_sqa(
-        self, generator_run: GeneratorRun, weight: Optional[float] = None
+        self,
+        generator_run: GeneratorRun,
+        weight: Optional[float] = None,
+        reduced_state: bool = False,
     ) -> SQAGeneratorRun:
-        """Convert Ax GeneratorRun to SQLAlchemy and compile a list of (object,
-        sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
+        """Convert Ax GeneratorRun to SQLAlchemy.
 
         In addition to creating and storing a new GeneratorRun object, we need to
         create and store copies of the Arms, Metrics, Parameters, and
         ParameterConstraints owned by this GeneratorRun.
         """
+
         arms = []
         for arm, arm_weight in generator_run.arm_weights.items():
             arms.append(self.arm_to_sqa(arm=arm, weight=arm_weight))
@@ -664,15 +757,12 @@ class Encoder:
         best_arm_parameters = None
         best_arm_predictions = None
         if generator_run.best_arm_predictions is not None:
-            # pyre-fixme[16]: `Optional` has no attribute `__getitem__`.
             best_arm = generator_run.best_arm_predictions[0]
+            # pyre-fixme[16]: `Optional` has no attribute `__getitem__`.
             best_arm_predictions = list(generator_run.best_arm_predictions[1])
             best_arm_name = best_arm._name
             best_arm_parameters = best_arm.parameters
         model_predictions = (
-            # pyre-fixme[6]: Expected `Iterable[Variable[_T]]` for 1st param but got
-            #  `Optional[typing.Tuple[Dict[str, List[float]], Dict[str, Dict[str,
-            #  List[float]]]]]`.
             list(generator_run.model_predictions)
             if generator_run.model_predictions is not None
             else None
@@ -704,19 +794,48 @@ class Encoder:
             best_arm_predictions=best_arm_predictions,
             model_predictions=model_predictions,
             model_key=generator_run._model_key,
-            model_kwargs=object_to_json(generator_run._model_kwargs),
-            bridge_kwargs=object_to_json(generator_run._bridge_kwargs),
-            gen_metadata=object_to_json(generator_run._gen_metadata),
-            model_state_after_gen=object_to_json(generator_run._model_state_after_gen),
+            model_kwargs=object_to_json(
+                generator_run._model_kwargs,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
+            )
+            if not reduced_state
+            else None,
+            bridge_kwargs=object_to_json(
+                generator_run._bridge_kwargs,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
+            )
+            if not reduced_state
+            else None,
+            gen_metadata=object_to_json(
+                generator_run._gen_metadata,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
+            )
+            if not reduced_state
+            else None,
+            model_state_after_gen=object_to_json(
+                generator_run._model_state_after_gen,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
+            )
+            if not reduced_state
+            else None,
             generation_step_index=generator_run._generation_step_index,
             candidate_metadata_by_arm_signature=object_to_json(
-                generator_run._candidate_metadata_by_arm_signature
+                generator_run._candidate_metadata_by_arm_signature,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
             ),
         )
         return gr_sqa
 
     def generation_strategy_to_sqa(
-        self, generation_strategy: GenerationStrategy, experiment_id: Optional[int]
+        self,
+        generation_strategy: GenerationStrategy,
+        experiment_id: Optional[int],
+        generator_run_reduced_state: bool = False,
     ) -> SQAGenerationStrategy:
         """Convert an Ax `GenerationStrategy` to SQLAlchemy, preserving its state,
         so that the restored generation strategy can be resumed from the point
@@ -727,15 +846,24 @@ class Encoder:
             cast(Type[Base], GenerationStrategy)
         ]
         generator_runs_sqa = []
-        for gr in generation_strategy._generator_runs:
-            gr_sqa = self.generator_run_to_sqa(gr)
+        for idx, gr in enumerate(generation_strategy._generator_runs):
+            # Never reduce the state of the last generator run because that
+            # generator run is needed to recreate the model when reloading the
+            # generation strategy.
+            is_last_gr = idx == len(generation_strategy._generator_runs) - 1
+            reduced_state = generator_run_reduced_state and not is_last_gr
+            gr_sqa = self.generator_run_to_sqa(gr, reduced_state=reduced_state)
             generator_runs_sqa.append(gr_sqa)
 
         # pyre-fixme[29]: `SQAGenerationStrategy` is not a function.
         gs_sqa = gs_class(
             id=generation_strategy.db_id,
             name=generation_strategy.name,
-            steps=object_to_json(generation_strategy._steps),
+            steps=object_to_json(
+                generation_strategy._steps,
+                encoder_registry=self.config.json_encoder_registry,
+                class_encoder_registry=self.config.json_class_encoder_registry,
+            ),
             curr_index=generation_strategy._curr.index,
             generator_runs=generator_runs_sqa,
             experiment_id=experiment_id,
@@ -747,15 +875,16 @@ class Encoder:
     ) -> SQARunner:
         """Convert Ax Runner to SQLAlchemy."""
         runner_class = type(runner)
-        runner_type = RUNNER_REGISTRY.get(runner_class)
+        runner_type = self.config.runner_registry.get(runner_class)
         if runner_type is None:
             raise SQAEncodeError(
                 "Cannot encode runner to SQLAlchemy because runner's "
                 f"subclass ({runner_class}) is missing from the registry. "
                 "The runner registry currently contains the following: "
-                f"{','.join(map(str, RUNNER_REGISTRY.keys()))}"
+                f"{','.join(map(str, self.config.runner_registry.keys()))} "
+                f"{self.EXTRA_REGISTRY_ERROR_NOTE}"
             )  # pragma: no cover
-        properties = runner_class.serialize_init_args(runner=runner)
+        properties = runner_class.serialize_init_args(obj=runner)
 
         # pyre-fixme: Expected `Base` for 1st...t `typing.Type[Runner]`.
         runner_class: SQARunner = self.config.class_to_sqa_class[Runner]
@@ -767,15 +896,15 @@ class Encoder:
             trial_type=trial_type,
         )
 
-    def trial_to_sqa(self, trial: BaseTrial) -> SQATrial:
-        """Convert Ax Trial to SQLAlchemy and compile a list of (object,
-        sqa_counterpart) tuples to set `db_id` on user-facing classes after
-        the conversion is complete and the SQL session is flushed (SQLAlchemy
-        classes receive their `id` attributes during `session.flush()`).
+    def trial_to_sqa(
+        self, trial: BaseTrial, generator_run_reduced_state: bool = False
+    ) -> SQATrial:
+        """Convert Ax Trial to SQLAlchemy.
 
         In addition to creating and storing a new Trial object, we need to
         create and store the GeneratorRuns and Runner that it owns.
         """
+
         runner = None
         if trial.runner:
             runner = self.runner_to_sqa(runner=not_none(trial.runner))
@@ -784,10 +913,12 @@ class Encoder:
         generator_runs = []
         status_quo_name = None
         optimize_for_power = None
+        lifecycle_stage = None
 
         if isinstance(trial, Trial) and trial.generator_run:
             gr_sqa = self.generator_run_to_sqa(
-                generator_run=not_none(trial.generator_run)
+                generator_run=not_none(trial.generator_run),
+                reduced_state=generator_run_reduced_state,
             )
             generator_runs.append(gr_sqa)
 
@@ -804,6 +935,8 @@ class Encoder:
 
             trial_status_quo = trial.status_quo
             trial_status_quo_weight_override = trial._status_quo_weight_override
+            lifecycle_stage = trial.lifecycle_stage
+
             if (
                 trial_status_quo is not None
                 and trial_status_quo_weight_override is not None
@@ -848,6 +981,7 @@ class Encoder:
             optimize_for_power=optimize_for_power,
             ttl_seconds=trial.ttl_seconds,
             run_metadata=trial.run_metadata,
+            stop_metadata=trial.stop_metadata,
             status=trial.status,
             status_quo_name=status_quo_name,
             time_completed=trial.time_completed,
@@ -860,25 +994,30 @@ class Encoder:
             runner=runner,
             generation_step_index=trial._generation_step_index,
             properties=trial._properties,
+            lifecycle_stage=lifecycle_stage,
         )
         return trial_sqa
 
     def data_to_sqa(
-        self, data: AbstractDataFrameData, trial_index: Optional[int], timestamp: int
+        self, data: Data, trial_index: Optional[int], timestamp: int
     ) -> SQAData:
         """Convert Ax data to SQLAlchemy."""
         # pyre-fixme: Expected `Base` for 1st...ot `typing.Type[Data]`.
-        data_class: SQAData = self.config.class_to_sqa_class[type(data)]
+        data_class: SQAData = self.config.class_to_sqa_class[Data]
         import json
 
         # pyre-fixme[29]: `SQAData` is not a function.
         return data_class(
             id=data.db_id,
-            data_json=data.df.to_json(),
+            data_json=data.true_df.to_json(),
             description=data.description,
             time_created=timestamp,
             trial_index=trial_index,
             structure_metadata_json=json.dumps(
-                object_to_json(data.serialize_init_args(data))
+                object_to_json(
+                    data.serialize_init_args(data),
+                    encoder_registry=self.config.json_encoder_registry,
+                    class_encoder_registry=self.config.json_class_encoder_registry,
+                )
             ),
         )
