@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -8,6 +8,7 @@ from datetime import datetime
 from typing import Any, Dict, List, Optional
 
 from ax.core.base_trial import TrialStatus
+from ax.core.batch_trial import LifecycleStage
 from ax.core.parameter import ParameterType
 from ax.core.types import (
     ComparisonOp,
@@ -17,14 +18,15 @@ from ax.core.types import (
     TParamValue,
 )
 from ax.storage.sqa_store.db import (
+    Base,
     LONG_STRING_FIELD_LENGTH,
     LONGTEXT_BYTES,
     NAME_OR_TYPE_FIELD_LENGTH,
-    Base,
 )
 from ax.storage.sqa_store.json import (
     JSONEncodedDict,
     JSONEncodedList,
+    JSONEncodedLongTextDict,
     JSONEncodedObject,
     JSONEncodedTextDict,
 )
@@ -157,17 +159,18 @@ class SQAMetric(Base):
     # pyre-fixme[8]: Attribute has type `Optional[str]`; used as `Column[str]`.
     canonical_name: Optional[str] = Column(String(NAME_OR_TYPE_FIELD_LENGTH))
 
+    # pyre-fixme[4]: Attribute must be annotated.
     scalarized_objective_id = Column(Integer, ForeignKey("metric_v2.id"))
 
     # Relationship containing SQAMetric(s) only defined for the parent metric
     # of Multi/Scalarized Objective contains all children of the parent metric
     # join_depth argument: used for loading self-referential relationships
     # https://docs.sqlalchemy.org/en/13/orm/self_referential.html#configuring-self-referential-eager-loading
+    # pyre-fixme[4]: Attribute must be annotated.
     scalarized_objective_children_metrics = relationship(
         "SQAMetric",
         cascade="all, delete-orphan",
-        lazy="selectin",
-        join_depth=5,
+        lazy=True,
         foreign_keys=[scalarized_objective_id],
     )
 
@@ -176,12 +179,13 @@ class SQAMetric(Base):
     #  `Column[decimal.Decimal]`.
     scalarized_objective_weight: Optional[float] = Column(Float)
 
+    # pyre-fixme[4]: Attribute must be annotated.
     scalarized_outcome_constraint_id = Column(Integer, ForeignKey("metric_v2.id"))
+    # pyre-fixme[4]: Attribute must be annotated.
     scalarized_outcome_constraint_children_metrics = relationship(
         "SQAMetric",
         cascade="all, delete-orphan",
-        lazy="selectin",
-        join_depth=5,
+        lazy=True,
         foreign_keys=[scalarized_outcome_constraint_id],
     )
     # pyre-fixme[8]: Attribute has type `Optional[float]`; used as
@@ -311,7 +315,7 @@ class SQARunner(Base):
     experiment_id: Optional[int] = Column(Integer, ForeignKey("experiment_v2.id"))
     # pyre-fixme[8]: Attribute has type `Optional[Dict[str, typing.Any]]`; used as
     #  `Column[typing.Any]`.
-    properties: Optional[Dict[str, Any]] = Column(JSONEncodedTextDict, default={})
+    properties: Optional[Dict[str, Any]] = Column(JSONEncodedLongTextDict, default={})
     # pyre-fixme[8]: Attribute has type `int`; used as `Column[int]`.
     runner_type: int = Column(Integer, nullable=False)
     # pyre-fixme[8]: Attribute has type `Optional[int]`; used as `Column[int]`.
@@ -383,6 +387,8 @@ class SQATrial(Base):
     index: int = Column(Integer, index=True, nullable=False)
     # pyre-fixme[8]: Attribute has type `bool`; used as `Column[bool]`.
     is_batch: bool = Column("is_batched", Boolean, nullable=False, default=True)
+    # pyre-fixme[8]: Attribute has type `LifecycleStage`; used as `Column[DataType]`.
+    lifecycle_stage: LifecycleStage = Column(IntEnum(LifecycleStage), nullable=True)
     # pyre-fixme[8]: Attribute has type `int`; used as `Column[int]`.
     num_arms_created: int = Column(Integer, nullable=False, default=0)
     # pyre-fixme[8]: Attribute has type `Optional[bool]`; used as `Column[bool]`.
@@ -391,7 +397,10 @@ class SQATrial(Base):
     ttl_seconds: Optional[int] = Column(Integer)
     # pyre-fixme[8]: Attribute has type `Optional[Dict[str, typing.Any]]`; used as
     #  `Column[typing.Any]`.
-    run_metadata: Optional[Dict[str, Any]] = Column(JSONEncodedTextDict)
+    run_metadata: Optional[Dict[str, Any]] = Column(JSONEncodedLongTextDict)
+    # pyre-fixme[8]: Attribute has type `Optional[Dict[str, typing.Any]]`; used as
+    #  `Column[typing.Any]`.
+    stop_metadata: Optional[Dict[str, Any]] = Column(JSONEncodedTextDict)
     # pyre-fixme[8]: Attribute has type `TrialStatus`; used as `Column[typing.Any]`.
     status: TrialStatus = Column(
         IntEnum(TrialStatus), nullable=False, default=TrialStatus.CANDIDATE
