@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -12,11 +12,18 @@ from sqlalchemy import types
 
 
 class BaseNullableEnum(types.TypeDecorator):
+    cache_ok = True
+
+    # pyre-fixme[2]: Parameter annotation cannot be `Any`.
     def __init__(self, enum: Any, *arg: List[Any], **kw: Dict[Any, Any]) -> None:
         types.TypeDecorator.__init__(self, *arg, **kw)
+        # pyre-fixme[4]: Attribute must be annotated.
         self._member_map = enum._member_map_
+        # pyre-fixme[4]: Attribute must be annotated.
         self._value2member_map = enum._value2member_map_
 
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
+    # pyre-fixme[2]: Parameter annotation cannot be `Any`.
     def process_bind_param(self, value: Any, dialect: Any) -> Any:
         if value is None:
             return value  # pragma: no cover
@@ -31,6 +38,8 @@ class BaseNullableEnum(types.TypeDecorator):
             )
         return val._value_
 
+    # pyre-fixme[3]: Return annotation cannot be `Any`.
+    # pyre-fixme[2]: Parameter annotation cannot be `Any`.
     def process_result_value(self, value: Any, dialect: Any) -> Any:
         if value is None:
             return value  # pragma: no cover
