@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,11 +14,10 @@ from ax.utils.testing.core_stubs import (
     get_branin_objective,
     get_branin_outcome_constraint,
     get_choice_parameter,
-    get_ordered_choice_parameter,
-    get_task_choice_parameter,
     get_experiment_with_batch_and_single_trial,
     get_experiment_with_batch_trial,
     get_experiment_with_data,
+    get_experiment_with_map_data,
     get_experiment_with_multi_objective,
     get_experiment_with_scalarized_objective_and_outcome_constraint,
     get_factorial_metric,
@@ -29,19 +28,24 @@ from ax.utils.testing.core_stubs import (
     get_metric,
     get_objective,
     get_order_constraint,
+    get_ordered_choice_parameter,
     get_outcome_constraint,
     get_parameter_constraint,
     get_range_parameter,
+    get_risk_measure,
+    get_robust_search_space,
+    get_robust_search_space_environmental,
     get_scalarized_objective,
     get_scalarized_outcome_constraint,
-    get_simple_experiment_with_batch_trial,
     get_sum_constraint1,
     get_sum_constraint2,
     get_synthetic_runner,
+    get_task_choice_parameter,
     get_trial,
 )
 
 
+# pyre-fixme[5]: Global expression must be annotated.
 TEST_CASES = [
     (
         "AbandonedArm",
@@ -112,6 +116,12 @@ TEST_CASES = [
         Encoder.experiment_to_sqa,
         Decoder.experiment_from_sqa,
     ),
+    (
+        "Experiment",
+        get_experiment_with_map_data,
+        Encoder.experiment_to_sqa,
+        Decoder.experiment_from_sqa,
+    ),
     (
         "FixedParameter",
         get_fixed_parameter,
@@ -142,26 +152,14 @@ TEST_CASES = [
         Encoder.metric_to_sqa,
         Decoder.metric_from_sqa,
     ),
+    ("Metric", get_metric, Encoder.metric_to_sqa, Decoder.metric_from_sqa),
+    ("Objective", get_objective, Encoder.objective_to_sqa, Decoder.metric_from_sqa),
     (
         "OrderConstraint",
         get_order_constraint,
         Encoder.parameter_constraint_to_sqa,
         Decoder.parameter_constraint_from_sqa,
     ),
-    (
-        "ParameterConstraint",
-        get_parameter_constraint,
-        Encoder.parameter_constraint_to_sqa,
-        Decoder.parameter_constraint_from_sqa,
-    ),
-    ("Metric", get_metric, Encoder.metric_to_sqa, Decoder.metric_from_sqa),
-    ("Objective", get_objective, Encoder.objective_to_sqa, Decoder.metric_from_sqa),
-    (
-        "ScalarizedObjective",
-        get_scalarized_objective,
-        Encoder.objective_to_sqa,
-        Decoder.metric_from_sqa,
-    ),
     (
         "OutcomeConstraint",
         get_outcome_constraint,
@@ -169,10 +167,10 @@ TEST_CASES = [
         Decoder.metric_from_sqa,
     ),
     (
-        "ScalarizedOutcomeConstraint",
-        get_scalarized_outcome_constraint,
-        Encoder.outcome_constraint_to_sqa,
-        Decoder.metric_from_sqa,
+        "ParameterConstraint",
+        get_parameter_constraint,
+        Encoder.parameter_constraint_to_sqa,
+        Decoder.parameter_constraint_from_sqa,
     ),
     (
         "RangeParameter",
@@ -181,10 +179,36 @@ TEST_CASES = [
         Decoder.parameter_from_sqa,
     ),
     (
-        "SimpleExperiment",
-        get_simple_experiment_with_batch_trial,
-        Encoder.experiment_to_sqa,
-        Decoder.experiment_from_sqa,
+        "RiskMeasure",
+        get_risk_measure,
+        Encoder.risk_measure_to_sqa,
+        Decoder.metric_from_sqa,
+    ),
+    (
+        "RobustSearchSpace",
+        get_robust_search_space,
+        # Using generic search space methods for full-stack test coverage.
+        Encoder.search_space_to_sqa,
+        Decoder.search_space_from_sqa,
+    ),
+    (
+        "RobustSearchSpace",
+        get_robust_search_space_environmental,
+        # Using generic search space methods for full-stack test coverage.
+        Encoder.search_space_to_sqa,
+        Decoder.search_space_from_sqa,
+    ),
+    (
+        "ScalarizedObjective",
+        get_scalarized_objective,
+        Encoder.objective_to_sqa,
+        Decoder.metric_from_sqa,
+    ),
+    (
+        "ScalarizedOutcomeConstraint",
+        get_scalarized_outcome_constraint,
+        Encoder.outcome_constraint_to_sqa,
+        Decoder.metric_from_sqa,
     ),
     (
         "SyntheticRunner",
