@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -14,12 +14,15 @@ import pandas as pd
 from ax.utils.common.typeutils import numpy_type_to_python_type
 
 
+# pyre-fixme[24]: Generic type `Callable` expects 2 type parameters.
 def equality_typechecker(eq_func: Callable) -> Callable:
     """A decorator to wrap all __eq__ methods to ensure that the inputs
     are of the right type.
     """
 
     # no type annotation for now; breaks sphinx-autodoc-typehints
+    # pyre-fixme[3]: Return type must be annotated.
+    # pyre-fixme[2]: Parameter must be annotated.
     def _type_safe_equals(self, other):
         if not isinstance(other, self.__class__):
             return False
@@ -28,6 +31,7 @@ def equality_typechecker(eq_func: Callable) -> Callable:
     return _type_safe_equals
 
 
+# pyre-fixme[2]: Parameter annotation cannot contain `Any`.
 def same_elements(list1: List[Any], list2: List[Any]) -> bool:
     """Compare equality of two lists of core Ax objects.
 
@@ -103,6 +107,7 @@ def object_attribute_dicts_equal(
     return not bool(unequal_type or unequal_value)
 
 
+# pyre-fixme[3]: Return annotation cannot contain `Any`.
 def object_attribute_dicts_find_unequal_fields(
     one_dict: Dict[str, Any],
     other_dict: Dict[str, Any],
@@ -155,9 +160,11 @@ def object_attribute_dicts_find_unequal_fields(
                 # bridge is the same.
                 equal = isinstance(one_val.model, type(other_val.model))
         elif isinstance(one_val, list):
-            equal = same_elements(one_val, other_val)
+            equal = isinstance(other_val, list) and same_elements(one_val, other_val)
         elif isinstance(one_val, dict):
-            equal = sorted(one_val.keys()) == sorted(other_val.keys())
+            equal = isinstance(other_val, dict) and sorted(one_val.keys()) == sorted(
+                other_val.keys()
+            )
             equal = equal and same_elements(
                 list(one_val.values()), list(other_val.values())
             )
