@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -11,7 +11,7 @@ from ax.utils.common.testutils import TestCase
 
 
 class TestSerializationUtils(TestCase):
-    def test_named_tuple_to_dict(self):
+    def test_named_tuple_to_dict(self) -> None:
         class Foo(NamedTuple):
             x: int
             y: str
