@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -21,7 +21,7 @@ from ax.utils.report.render import (
 
 
 class RenderTest(TestCase):
-    def testRenderReportElements(self):
+    def testRenderReportElements(self) -> None:
         elements = [
             p_html("foobar"),
             h2_html("foobar"),
