@@ -1,9 +1,10 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from logging import Logger
 from typing import List, Tuple, Union
 
 import numpy as np
@@ -11,8 +12,7 @@ import pandas as pd
 from ax.core.data import Data
 from ax.utils.common.logger import get_logger
 
-
-logger = get_logger(__name__)
+logger: Logger = get_logger(__name__)
 num_mixed = Union[np.ndarray, List[float]]
 
 
@@ -104,7 +104,8 @@ def positive_part_james_stein(
     contribution from the uncertainty in the sum of squared deviations of `y_i`
     from the mean of all `y_i`.
 
-    For more information, see https://fburl.com/empirical_bayes.
+    For more information, see
+    https://ax.dev/docs/models.html#empirical-bayes-and-thompson-sampling.
 
     Args:
         means: Means of each arm
@@ -133,7 +134,7 @@ def positive_part_james_stein(
     sigma_hat_i = np.sqrt(
         (1 - phi_i) * sigma2_i
         + phi_i * sigma2_i / K
-        + 2 * phi_i ** 2 * (y_i - ybar) ** 2 / (K - 3)
+        + 2 * phi_i**2 * (y_i - ybar) ** 2 / (K - 3)
     )
     return mu_hat_i, sigma_hat_i
 
@@ -215,11 +216,11 @@ def relativize(
     c = m_t / mean_c
     r_hat = (m_t - mean_c) / np.abs(mean_c)
     if bias_correction:
-        r_hat = r_hat - m_t * sem_c ** 2 / np.abs(mean_c) ** 3
+        r_hat = r_hat - m_t * sem_c**2 / np.abs(mean_c) ** 3
     # If everything's the same, then set r_hat to zero
     same = (m_t == mean_c) & (s_t == sem_c)
     r_hat = ~same * r_hat
-    var = ((s_t ** 2) - 2 * c * cov_t + (c ** 2) * (sem_c ** 2)) / (mean_c ** 2)
+    var = ((s_t**2) - 2 * c * cov_t + (c**2) * (sem_c**2)) / (mean_c**2)
     if as_percent:
         return (r_hat * 100, np.sqrt(var) * 100)
     else:
