@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
@@ -16,7 +16,7 @@ from ax.utils.stats.statstools import (
 
 
 class InverseVarianceWeightingTest(TestCase):
-    def test_bad_arg_ivw(self):
+    def test_bad_arg_ivw(self) -> None:
         with self.assertRaises(ValueError):
             inverse_variance_weight(
                 np.array([0]), np.array([1]), conflicting_noiseless="foo"
@@ -24,28 +24,28 @@ class InverseVarianceWeightingTest(TestCase):
         with self.assertRaises(ValueError):
             inverse_variance_weight(np.array([1, 2]), np.array([1]))
 
-    def test_very_simple_ivw(self):
+    def test_very_simple_ivw(self) -> None:
         means = np.array([1, 1, 1])
         variances = np.array([1, 1, 1])
         new_mean, new_var = inverse_variance_weight(means, variances)
         self.assertEqual(new_mean, 1.0)
         self.assertEqual(new_var, 1 / 3)
 
-    def test_simple_ivw(self):
+    def test_simple_ivw(self) -> None:
         means = np.array([1, 2, 3])
         variances = np.array([1, 1, 1])
         new_mean, new_var = inverse_variance_weight(means, variances)
         self.assertEqual(new_mean, 2.0)
         self.assertEqual(new_var, 1 / 3)
 
-    def test_another_simple_ivw(self):
+    def test_another_simple_ivw(self) -> None:
         means = np.array([1, 3])
         variances = np.array([1, 3])
         new_mean, new_var = inverse_variance_weight(means, variances)
         self.assertEqual(new_mean, 1.5)
         self.assertEqual(new_var, 0.75)
 
-    def test_conflicting_noiseless_ivw(self):
+    def test_conflicting_noiseless_ivw(self) -> None:
         means = np.array([1, 2, 1])
         variances = np.array([0, 0, 1])
 
@@ -58,7 +58,7 @@ class InverseVarianceWeightingTest(TestCase):
 
 
 class MarginalEffectsTest(TestCase):
-    def test_marginal_effects(self):
+    def test_marginal_effects(self) -> None:
         df = pd.DataFrame(
             {
                 "mean": [1, 2, 3, 4],
@@ -73,7 +73,7 @@ class MarginalEffectsTest(TestCase):
 
 
 class RelativizeDataTest(TestCase):
-    def test_relativize_data(self):
+    def test_relativize_data(self) -> None:
         data = Data(
             df=pd.DataFrame(
                 [
