@@ -1,24 +1,37 @@
 #!/usr/bin/env python3
-# Copyright (c) Facebook, Inc. and its affiliates.
+# Copyright (c) Meta Platforms, Inc. and affiliates.
 #
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
 
 from collections import OrderedDict
-from datetime import datetime
-from typing import Any, Dict, Iterable, List, MutableMapping, Optional, Type, cast
+from datetime import datetime, timedelta
+
+from logging import Logger
+from typing import (
+    Any,
+    cast,
+    Dict,
+    Iterable,
+    List,
+    MutableMapping,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+)
 
 import numpy as np
 import pandas as pd
 import torch
 from ax.core.arm import Arm
-from ax.core.base_trial import BaseTrial
+from ax.core.base_trial import BaseTrial, TrialStatus
 from ax.core.batch_trial import AbandonedArm, BatchTrial
 from ax.core.data import Data
 from ax.core.experiment import DataType, Experiment
 from ax.core.generator_run import GeneratorRun
-from ax.core.map_data import MapData
+from ax.core.map_data import MapData, MapKeyInfo
 from ax.core.map_metric import MapMetric
 from ax.core.metric import Metric
 from ax.core.multi_type_experiment import MultiTypeExperiment
@@ -44,8 +57,9 @@ from ax.core.parameter_constraint import (
     ParameterConstraint,
     SumConstraint,
 )
-from ax.core.search_space import SearchSpace
-from ax.core.simple_experiment import SimpleExperiment
+from ax.core.parameter_distribution import ParameterDistribution
+from ax.core.risk_measures import RiskMeasure
+from ax.core.search_space import HierarchicalSearchSpace, RobustSearchSpace, SearchSpace
 from ax.core.trial import Trial
 from ax.core.types import (
     ComparisonOp,
@@ -55,16 +69,29 @@ from ax.core.types import (
     TModelPredictArm,
     TParameterization,
 )
+from ax.early_stopping.strategies import (
+    BaseEarlyStoppingStrategy,
+    PercentileEarlyStoppingStrategy,
+    ThresholdEarlyStoppingStrategy,
+)
+from ax.early_stopping.strategies.logical import (
+    AndEarlyStoppingStrategy,
+    OrEarlyStoppingStrategy,
+)
+from ax.exceptions.core import UserInputError
+from ax.global_stopping.strategies.base import BaseGlobalStoppingStrategy
 from ax.metrics.branin import AugmentedBraninMetric, BraninMetric
 from ax.metrics.branin_map import BraninTimestampMapMetric
 from ax.metrics.factorial import FactorialMetric
 from ax.metrics.hartmann6 import AugmentedHartmann6Metric, Hartmann6Metric
 from ax.modelbridge.factory import Cont_X_trans, get_factorial, get_sobol
+from ax.modelbridge.transforms.winsorize import WinsorizationConfig
 from ax.models.torch.botorch_modular.acquisition import Acquisition
 from ax.models.torch.botorch_modular.list_surrogate import ListSurrogate
 from ax.models.torch.botorch_modular.model import BoTorchModel
 from ax.models.torch.botorch_modular.surrogate import Surrogate
 from ax.runners.synthetic import SyntheticRunner
+from ax.service.utils.scheduler_options import SchedulerOptions, TrialType
 from ax.utils.common.logger import get_logger
 from ax.utils.common.typeutils import checked_cast, not_none
 from ax.utils.measurement.synthetic_functions import branin
@@ -72,12 +99,14 @@ from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.monte_carlo import qExpectedImprovement
 from botorch.models.gp_regression import SingleTaskGP
 from botorch.models.model import Model
+from gpytorch.constraints import Interval
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
 from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
+from gpytorch.priors.torch_priors import GammaPrior
 
+logger: Logger = get_logger(__name__)
 
-logger = get_logger(__name__)
-
+TEST_SOBOL_SEED = 1234
 
 ##############################
 # Experiments
@@ -96,7 +125,7 @@ def get_experiment(with_status_quo: bool = True) -> Experiment:
     )
 
 
-def get_experiment_with_map_data_type():
+def get_experiment_with_map_data_type() -> Experiment:
     return Experiment(
         name="test_map_data",
         search_space=get_search_space(),
@@ -116,13 +145,16 @@ def get_branin_experiment(
     with_status_quo: bool = False,
     with_fidelity_parameter: bool = False,
     with_choice_parameter: bool = False,
+    with_str_choice_param: bool = False,
     search_space: Optional[SearchSpace] = None,
     minimize: bool = False,
     named: bool = True,
+    with_completed_trial: bool = False,
 ) -> Experiment:
     search_space = search_space or get_branin_search_space(
         with_fidelity_parameter=with_fidelity_parameter,
         with_choice_parameter=with_choice_parameter,
+        with_str_choice_param=with_str_choice_param,
     )
     exp = Experiment(
         name="branin_test_experiment" if named else None,
@@ -144,34 +176,99 @@ def get_branin_experiment(
             sobol_run
         )
 
-    if with_trial:
+    if with_trial or with_completed_trial:
         sobol_generator = get_sobol(search_space=exp.search_space)
         sobol_run = sobol_generator.gen(n=1)
-        exp.new_trial(generator_run=sobol_run)
+        trial = exp.new_trial(generator_run=sobol_run)
+
+        if with_completed_trial:
+            trial.mark_running(no_runner_required=True)
+            exp.attach_data(get_branin_data(trials=[trial]))  # Add data for one trial
+            trial.mark_completed()
 
     return exp
 
 
-def get_branin_experiment_with_timestamp_map_metric(rate: Optional[float] = None):
-    return Experiment(
+def get_robust_branin_experiment(
+    risk_measure: Optional[RiskMeasure] = None,
+    optimization_config: Optional[OptimizationConfig] = None,
+    num_sobol_trials: int = 2,
+) -> Experiment:
+    x1_dist = ParameterDistribution(
+        parameters=["x1"], distribution_class="norm", distribution_parameters={}
+    )
+    search_space = RobustSearchSpace(
+        parameters=[
+            RangeParameter(
+                name="x1", parameter_type=ParameterType.FLOAT, lower=-5, upper=10
+            ),
+            RangeParameter(
+                name="x2", parameter_type=ParameterType.FLOAT, lower=0, upper=15
+            ),
+        ],
+        parameter_distributions=[x1_dist],
+        num_samples=16,
+    )
+
+    risk_measure = risk_measure or RiskMeasure(
+        risk_measure="CVaR",
+        options={"n_w": 16, "alpha": 0.8},
+    )
+
+    optimization_config = optimization_config or OptimizationConfig(
+        objective=Objective(
+            metric=BraninMetric(
+                name="branin_metric", param_names=["x1", "x2"], lower_is_better=True
+            ),
+            minimize=True,
+        ),
+        risk_measure=risk_measure,
+    )
+
+    exp = Experiment(
+        name="branin_experiment",
+        search_space=search_space,
+        optimization_config=optimization_config,
+        runner=SyntheticRunner(),
+    )
+
+    sobol = get_sobol(search_space=exp.search_space)
+    for _ in range(num_sobol_trials):
+        exp.new_trial(generator_run=sobol.gen(1)).run().mark_completed()
+    return exp
+
+
+def get_branin_experiment_with_timestamp_map_metric(
+    with_status_quo: bool = False,
+    rate: Optional[float] = None,
+) -> Experiment:
+    exp = Experiment(
         name="branin_with_timestamp_map_metric",
         search_space=get_branin_search_space(),
         optimization_config=OptimizationConfig(
             objective=Objective(
                 metric=BraninTimestampMapMetric(
-                    name="branin", param_names=["x1", "x2"], rate=rate
+                    name="branin_map",
+                    param_names=["x1", "x2"],
+                    rate=rate,
+                    lower_is_better=True,
                 ),
                 minimize=True,
             )
         ),
-        tracking_metrics=[BraninTimestampMapMetric(name="b", param_names=["x1", "x2"])],
+        tracking_metrics=[BraninMetric(name="branin", param_names=["x1", "x2"])],
         runner=SyntheticRunner(),
         default_data_type=DataType.MAP_DATA,
     )
 
+    if with_status_quo:
+        exp.status_quo = Arm(parameters={"x1": 0.0, "x2": 0.0})
+
+    return exp
+
 
 def get_multi_type_experiment(
-    add_trial_type: bool = True, add_trials: bool = False
+    add_trial_type: bool = True, add_trials: bool = False, num_arms: int = 10
 ) -> MultiTypeExperiment:
     oc = OptimizationConfig(Objective(BraninMetric("m1", ["x1", "x2"])))
     experiment = MultiTypeExperiment(
@@ -191,7 +288,7 @@ def get_multi_type_experiment(
 
     if add_trials and add_trial_type:
         generator = get_sobol(experiment.search_space)
-        gr = generator.gen(10)
+        gr = generator.gen(num_arms)
         t1 = experiment.new_batch_trial(generator_run=gr, trial_type="type1")
         t2 = experiment.new_batch_trial(generator_run=gr, trial_type="type2")
         t1.set_status_quo_with_weight(status_quo=t1.arms[0], weight=0.5)
@@ -267,32 +364,16 @@ def get_factorial_experiment(
     return exp
 
 
-def get_simple_experiment() -> SimpleExperiment:
-    experiment = SimpleExperiment(
-        name="test_branin",
-        search_space=get_branin_search_space(),
-        status_quo=Arm(parameters={"x1": 0.0, "x2": 0.0}),
-        objective_name="sum",
-    )
-
-    experiment.description = "foobar"
-
-    return experiment
-
-
-def get_simple_experiment_with_batch_trial() -> SimpleExperiment:
-    experiment = get_simple_experiment()
-    generator = get_sobol(experiment.search_space)
-    generator_run = generator.gen(10)
-    experiment.new_batch_trial(generator_run=generator_run)
-    return experiment
-
-
 def get_experiment_with_repeated_arms(num_repeated_arms: int) -> Experiment:
     batch_trial = get_batch_trial_with_repeated_arms(num_repeated_arms)
     return batch_trial.experiment
 
 
+def get_experiment_with_trial() -> Experiment:
+    trial = get_trial()
+    return trial.experiment
+
+
 def get_experiment_with_batch_trial() -> Experiment:
     batch_trial = get_batch_trial()
     return batch_trial.experiment
@@ -350,6 +431,7 @@ def get_branin_experiment_with_multi_objective(
     with_batch: bool = False,
     with_status_quo: bool = False,
     with_fidelity_parameter: bool = False,
+    num_objectives: int = 2,
 ) -> Experiment:
     exp = Experiment(
         name="branin_test_experiment",
@@ -357,7 +439,8 @@ def get_branin_experiment_with_multi_objective(
             with_fidelity_parameter=with_fidelity_parameter
         ),
         optimization_config=get_branin_multi_objective_optimization_config(
-            has_objective_thresholds=has_objective_thresholds
+            has_objective_thresholds=has_objective_thresholds,
+            num_objectives=num_objectives,
         )
         if has_optimization_config
         else None,
@@ -366,11 +449,12 @@ def get_branin_experiment_with_multi_objective(
     )
 
     if with_status_quo:
+        # Experiment chooses the name "status_quo" by default
         exp.status_quo = Arm(parameters={"x1": 0.0, "x2": 0.0})
 
     if with_batch:
-        sobol_generator = get_sobol(search_space=exp.search_space)
-        sobol_run = sobol_generator.gen(n=15)
+        sobol_generator = get_sobol(search_space=exp.search_space, seed=TEST_SOBOL_SEED)
+        sobol_run = sobol_generator.gen(n=5)
         exp.new_batch_trial(optimize_for_power=with_status_quo).add_generator_run(
             sobol_run
         )
@@ -378,6 +462,31 @@ def get_branin_experiment_with_multi_objective(
     return exp
 
 
+def get_branin_with_multi_task(with_multi_objective: bool = False) -> Experiment:
+    exp = Experiment(
+        name="branin_test_experiment",
+        search_space=get_branin_search_space(),
+        optimization_config=get_branin_multi_objective_optimization_config(
+            has_objective_thresholds=True,
+        )
+        if with_multi_objective
+        else get_branin_optimization_config(),
+        runner=SyntheticRunner(),
+        is_test=True,
+    )
+
+    exp.status_quo = Arm(parameters={"x1": 0.0, "x2": 0.0}, name="status_quo")
+
+    sobol_generator = get_sobol(search_space=exp.search_space, seed=TEST_SOBOL_SEED)
+    sobol_run = sobol_generator.gen(n=5)
+    exp.new_batch_trial(optimize_for_power=True).add_generator_run(sobol_run)
+    not_none(exp.trials.get(0)).run()
+    exp.new_batch_trial(optimize_for_power=True).add_generator_run(sobol_run)
+    not_none(exp.trials.get(1)).run()
+
+    return exp
+
+
 def get_experiment_with_scalarized_objective_and_outcome_constraint() -> Experiment:
     objective = get_scalarized_objective()
     outcome_constraints = [
@@ -398,6 +507,126 @@ def get_experiment_with_scalarized_objective_and_outcome_constraint() -> Experim
     )
 
 
+def get_hierarchical_search_space_experiment() -> Experiment:
+    return Experiment(
+        search_space=get_hierarchical_search_space(),
+        optimization_config=get_optimization_config(),
+    )
+
+
+def get_experiment_with_observations(
+    observations: List[List[float]],
+    minimize: bool = False,
+    scalarized: bool = False,
+    constrained: bool = False,
+) -> Experiment:
+    multi_objective = len(observations[0]) > 1
+    if multi_objective:
+        metrics = [
+            Metric(name="m1", lower_is_better=minimize),
+            Metric(name="m2", lower_is_better=False),
+        ]
+        if scalarized:
+            optimization_config = OptimizationConfig(
+                objective=ScalarizedObjective(metrics)
+            )
+            if constrained:  # pragma: no cover
+                raise NotImplementedError
+        else:
+            optimization_config = MultiObjectiveOptimizationConfig(
+                objective=MultiObjective(metrics=metrics),
+                objective_thresholds=[
+                    ObjectiveThreshold(metric=metrics[i], bound=0.0, relative=False)
+                    for i in [0, 1]
+                ],
+                outcome_constraints=[
+                    OutcomeConstraint(
+                        metric=Metric(name="m3"),
+                        op=ComparisonOp.GEQ,
+                        bound=0.0,
+                        relative=False,
+                    )
+                ]
+                if constrained
+                else None,
+            )
+    else:
+        if scalarized or constrained:  # pragma: no cover
+            raise NotImplementedError
+        optimization_config = OptimizationConfig(
+            objective=Objective(metric=Metric(name="m1"), minimize=minimize)
+        )
+    exp = Experiment(
+        search_space=get_search_space_for_range_values(min=0.0, max=1.0),
+        optimization_config=optimization_config,
+        runner=SyntheticRunner(),
+        is_test=True,
+    )
+    for i, obs in enumerate(observations):
+        # Create a dummy trial to add the observation.
+        trial = exp.new_trial()
+        trial.add_arm(Arm({"x": i / 100.0, "y": i / 100.0}))
+        data = Data(
+            df=pd.DataFrame.from_records(
+                [
+                    {
+                        "arm_name": f"{i}_0",
+                        "metric_name": f"m{j + 1}",
+                        "mean": o,
+                        "sem": None,
+                        "trial_index": i,
+                    }
+                    for j, o in enumerate(obs)
+                ]
+            )
+        )
+        exp.attach_data(data)
+        trial.run().complete()
+    return exp
+
+
+def get_high_dimensional_branin_experiment() -> Experiment:
+    search_space = SearchSpace(
+        # pyre-fixme[6]: In call `SearchSpace.__init__`, for 1st parameter `parameters`
+        # expected `List[Parameter]` but got `List[RangeParameter]`.
+        parameters=[
+            RangeParameter(
+                name=f"x{i}",
+                parameter_type=ParameterType.FLOAT,
+                lower=-5.0,
+                upper=10.0,
+            )
+            for i in range(25)
+        ]
+        + [
+            RangeParameter(
+                name=f"x{i + 25}",
+                parameter_type=ParameterType.FLOAT,
+                lower=0.0,
+                upper=15.0,
+            )
+            for i in range(25)
+        ],
+    )
+
+    optimization_config = OptimizationConfig(
+        objective=Objective(
+            metric=BraninMetric(
+                name="objective",
+                param_names=["x19", "x44"],
+            ),
+            minimize=True,
+        )
+    )
+
+    return Experiment(
+        name="high_dimensional_branin_experiment",
+        search_space=search_space,
+        optimization_config=optimization_config,
+        runner=SyntheticRunner(),
+    )
+
+
 ##############################
 # Search Spaces
 ##############################
@@ -421,7 +650,9 @@ def get_search_space() -> SearchSpace:
 
 
 def get_branin_search_space(
-    with_fidelity_parameter: bool = False, with_choice_parameter: bool = False
+    with_fidelity_parameter: bool = False,
+    with_choice_parameter: bool = False,
+    with_str_choice_param: bool = False,
 ) -> SearchSpace:
     parameters = [
         RangeParameter(
@@ -437,6 +668,14 @@ def get_branin_search_space(
             name="x2", parameter_type=ParameterType.FLOAT, lower=0, upper=15
         ),
     ]
+    if with_str_choice_param:
+        parameters.append(
+            ChoiceParameter(
+                name="str_param",
+                parameter_type=ParameterType.STRING,
+                values=["foo", "bar", "baz"],
+            )
+        )
     if with_fidelity_parameter:
         parameters.append(
             RangeParameter(
@@ -478,15 +717,36 @@ def get_large_factorial_search_space() -> SearchSpace:
     return SearchSpace(
         parameters=[
             ChoiceParameter(
-                name="factor1",
+                name=f"factor{j}",
                 parameter_type=ParameterType.STRING,
-                values=[f"level1{i}" for i in range(7)],
-            ),
+                values=[f"level1{i}" for i in range(10)],
+            )
+            for j in range(6)
+        ]
+    )
+
+
+def get_large_ordinal_search_space(
+    n_ordinal_choice_parameters: int,
+    n_continuous_range_parameters: int,
+) -> SearchSpace:
+    return SearchSpace(
+        parameters=[  # pyre-ignore[6]
+            RangeParameter(
+                name=f"x{i}",
+                parameter_type=ParameterType.FLOAT,
+                lower=0.0,
+                upper=1.0,
+            )
+            for i in range(n_continuous_range_parameters)
+        ]
+        + [
             ChoiceParameter(
-                name="factor2",
-                parameter_type=ParameterType.STRING,
-                values=[f"level2{i}" for i in range(10)],
-            ),
+                name=f"y{i}",
+                parameter_type=ParameterType.INT,
+                values=[2, 4, 8, 16],
+            )
+            for i in range(n_ordinal_choice_parameters)
         ]
     )
 
@@ -542,13 +802,125 @@ def get_discrete_search_space() -> SearchSpace:
     )
 
 
+def get_small_discrete_search_space() -> SearchSpace:
+    return SearchSpace(
+        [
+            RangeParameter("x", ParameterType.INT, 0, 1),
+            ChoiceParameter("y", ParameterType.STRING, ["red", "panda"]),
+        ]
+    )
+
+
+def get_hierarchical_search_space() -> HierarchicalSearchSpace:
+    return HierarchicalSearchSpace(
+        parameters=[
+            get_model_parameter(),
+            get_lr_parameter(),
+            get_l2_reg_weight_parameter(),
+            get_num_boost_rounds_parameter(),
+        ]
+    )
+
+
+def get_robust_search_space(
+    lb: float = 0.0,
+    ub: float = 5.0,
+    multivariate: bool = False,
+    use_discrete: bool = False,
+    num_samples: int = 4,  # dummy
+) -> RobustSearchSpace:
+
+    parameters = [
+        RangeParameter("x", ParameterType.FLOAT, lb, ub),
+        RangeParameter("y", ParameterType.FLOAT, lb, ub),
+        RangeParameter("z", ParameterType.INT, lb, ub),
+        ChoiceParameter("c", ParameterType.STRING, ["red", "panda"]),
+    ]
+    if multivariate:
+        if use_discrete:  # pragma: no cover
+            raise UserInputError(
+                "`multivariate` and `use_discrete` are not supported together."
+            )
+        distributions = [
+            ParameterDistribution(
+                parameters=["x", "y"],
+                distribution_class="multivariate_normal",
+                distribution_parameters={
+                    "mean": [0.1, 0.2],
+                    "cov": [[0.5, 0.1], [0.1, 0.3]],
+                },
+            )
+        ]
+    else:
+        distributions = []
+        distributions.append(
+            ParameterDistribution(
+                parameters=["x"],
+                distribution_class="norm",
+                distribution_parameters={"loc": 1.0, "scale": ub - lb},
+            )
+        )
+        if use_discrete:
+            distributions.append(
+                ParameterDistribution(
+                    parameters=["z"],
+                    distribution_class="binom",
+                    distribution_parameters={"n": 20, "p": 0.5},
+                )
+            )
+        else:
+            distributions.append(
+                ParameterDistribution(
+                    parameters=["y"],
+                    distribution_class="expon",
+                    distribution_parameters={},
+                )
+            )
+    return RobustSearchSpace(
+        # pyre-ignore Incompatible parameter type [6]
+        parameters=parameters,
+        parameter_distributions=distributions,
+        num_samples=num_samples,
+    )
+
+
+def get_robust_search_space_environmental(
+    lb: float = 0.0,
+    ub: float = 5.0,
+) -> RobustSearchSpace:
+    parameters = [
+        RangeParameter("x", ParameterType.FLOAT, lb, ub),
+        RangeParameter("y", ParameterType.FLOAT, lb, ub),
+    ]
+    environmental_variables = [
+        RangeParameter("z", ParameterType.INT, lb, ub),
+    ]
+    distributions = [
+        ParameterDistribution(
+            parameters=["z"],
+            distribution_class="binom",
+            distribution_parameters={"n": 5, "p": 0.5},
+        )
+    ]
+    return RobustSearchSpace(
+        # pyre-ignore Incompatible parameter type [6]
+        parameters=parameters,
+        parameter_distributions=distributions,
+        num_samples=4,
+        # pyre-ignore Incompatible parameter type [6]
+        environmental_variables=environmental_variables,
+    )
+
+
 ##############################
 # Trials
 ##############################
 
 
-def get_batch_trial(abandon_arm: bool = True) -> BatchTrial:
-    experiment = get_experiment()
+def get_batch_trial(
+    abandon_arm: bool = True, experiment: Optional[Experiment] = None
+) -> BatchTrial:
+    experiment = experiment or get_experiment()
     batch = experiment.new_batch_trial()
     arms = get_arms_from_dict(get_arm_weights1())
     weights = get_weights_from_dict(get_arm_weights1())
@@ -595,7 +967,6 @@ def get_batch_trial_with_repeated_arms(num_repeated_arms: int) -> BatchTrial:
 
     # Add num_repeated_arms to the new trial.
     arms = prev_arms + next_arms
-    # pyre-fixme[6]: Expected `List[int]` for 1st param but got `List[float]`.
     weights = prev_weights + next_weights
     batch = experiment.new_batch_trial()
     batch.add_arms_and_weights(arms=arms, weights=weights, multiplier=1)
@@ -638,7 +1009,7 @@ class TestTrial(BaseTrial):
         return self._arms
 
     @arms.setter
-    def arms(self, val: List[Arm]):
+    def arms(self, val: List[Arm]) -> None:
         self._arms = val
 
     def arms_by_name(self) -> str:
@@ -707,6 +1078,45 @@ def get_fixed_parameter() -> FixedParameter:
     return FixedParameter(name="z", parameter_type=ParameterType.BOOL, value=True)
 
 
+def get_model_parameter() -> ChoiceParameter:
+    return ChoiceParameter(
+        name="model",
+        parameter_type=ParameterType.STRING,
+        values=["Linear", "XGBoost"],
+        dependents={
+            "Linear": ["learning_rate", "l2_reg_weight"],
+            "XGBoost": ["num_boost_rounds"],
+        },
+    )
+
+
+def get_lr_parameter() -> RangeParameter:
+    return RangeParameter(
+        name="learning_rate",
+        parameter_type=ParameterType.FLOAT,
+        lower=0.001,
+        upper=0.1,
+    )
+
+
+def get_l2_reg_weight_parameter() -> RangeParameter:
+    return RangeParameter(
+        name="l2_reg_weight",
+        parameter_type=ParameterType.FLOAT,
+        lower=0.00001,
+        upper=0.001,
+    )
+
+
+def get_num_boost_rounds_parameter() -> RangeParameter:
+    return RangeParameter(
+        name="num_boost_rounds",
+        parameter_type=ParameterType.INT,
+        lower=10,
+        upper=20,
+    )
+
+
 ##############################
 # Parameter Constraints
 ##############################
@@ -718,8 +1128,10 @@ def get_order_constraint() -> OrderConstraint:
     return OrderConstraint(lower_parameter=x, upper_parameter=w)
 
 
-def get_parameter_constraint() -> ParameterConstraint:
-    return ParameterConstraint(constraint_dict={"x": 1.0, "w": -1.0}, bound=1.0)
+def get_parameter_constraint(
+    param_x: str = "x", param_y: str = "w"
+) -> ParameterConstraint:
+    return ParameterConstraint(constraint_dict={param_x: 1.0, param_y: -1.0}, bound=1.0)
 
 
 def get_sum_constraint1() -> SumConstraint:
@@ -743,22 +1155,24 @@ def get_metric() -> Metric:
     return Metric(name="m1", properties={"prop": "val"})
 
 
-def get_branin_metric(name="branin") -> BraninMetric:
+def get_branin_metric(name: str = "branin") -> BraninMetric:
     param_names = ["x1", "x2"]
     return BraninMetric(name=name, param_names=param_names, noise_sd=0.01)
 
 
-def get_augmented_branin_metric(name="aug_branin") -> AugmentedBraninMetric:
+def get_augmented_branin_metric(name: str = "aug_branin") -> AugmentedBraninMetric:
     param_names = ["x1", "x2", "fidelity"]
     return AugmentedBraninMetric(name=name, param_names=param_names, noise_sd=0.01)
 
 
-def get_hartmann_metric(name="hartmann") -> Hartmann6Metric:
+def get_hartmann_metric(name: str = "hartmann") -> Hartmann6Metric:
     param_names = [f"x{idx + 1}" for idx in range(6)]
     return Hartmann6Metric(name=name, param_names=param_names, noise_sd=0.01)
 
 
-def get_augmented_hartmann_metric(name="aug_hartmann") -> AugmentedHartmann6Metric:
+def get_augmented_hartmann_metric(
+    name: str = "aug_hartmann",
+) -> AugmentedHartmann6Metric:
     param_names = [f"x{idx + 1}" for idx in range(6)]
     param_names.append("fidelity")
     return AugmentedHartmann6Metric(name=name, param_names=param_names, noise_sd=0.01)
@@ -787,9 +1201,13 @@ def get_factorial_metric(name: str = "success_metric") -> FactorialMetric:
 ##############################
 
 
-def get_objective_threshold() -> ObjectiveThreshold:
+def get_objective_threshold(
+    metric_name: str = "m1",
+    bound: float = -0.25,
+    comparison_op: ComparisonOp = ComparisonOp.GEQ,
+) -> ObjectiveThreshold:
     return ObjectiveThreshold(
-        metric=Metric(name="m1"), bound=-0.25, op=ComparisonOp.GEQ
+        metric=Metric(name=metric_name), bound=bound, op=comparison_op
     )
 
 
@@ -844,13 +1262,15 @@ def get_branin_objective(minimize: bool = False) -> Objective:
     return Objective(metric=get_branin_metric(), minimize=minimize)
 
 
-def get_branin_multi_objective() -> Objective:
-    return MultiObjective(
-        objectives=[
-            Objective(metric=get_branin_metric(name="branin_a")),
-            Objective(metric=get_branin_metric(name="branin_b")),
-        ],
-    )
+def get_branin_multi_objective(num_objectives: int = 2) -> Objective:
+    _validate_num_objectives(num_objectives=num_objectives)
+    objectives = [
+        Objective(metric=get_branin_metric(name="branin_a")),
+        Objective(metric=get_branin_metric(name="branin_b")),
+    ]
+    if num_objectives == 3:
+        objectives.append(Objective(metric=get_branin_metric(name="branin_c")))
+    return MultiObjective(objectives=objectives)
 
 
 def get_augmented_branin_objective() -> Objective:
@@ -886,7 +1306,10 @@ def get_map_optimization_config() -> OptimizationConfig:
 def get_multi_objective_optimization_config() -> OptimizationConfig:
     objective = get_multi_objective()
     outcome_constraints = [get_outcome_constraint()]
-    objective_thresholds = [get_objective_threshold()]
+    objective_thresholds = [
+        get_objective_threshold(metric_name="m1"),
+        get_objective_threshold(metric_name="m3", comparison_op=ComparisonOp.LEQ),
+    ]
     return MultiObjectiveOptimizationConfig(
         objective=objective,
         outcome_constraints=outcome_constraints,
@@ -902,11 +1325,18 @@ def get_branin_optimization_config(minimize: bool = False) -> OptimizationConfig
     return OptimizationConfig(objective=get_branin_objective(minimize=minimize))
 
 
+def _validate_num_objectives(num_objectives: int) -> None:
+    if num_objectives not in (2, 3):
+        raise NotImplementedError("Only 2 and 3 objectives are supported.")
+
+
 def get_branin_multi_objective_optimization_config(
     has_objective_thresholds: bool = False,
-) -> OptimizationConfig:
-    objective_thresholds = (
-        [
+    num_objectives: int = 2,
+) -> MultiObjectiveOptimizationConfig:
+    _validate_num_objectives(num_objectives=num_objectives)
+    if has_objective_thresholds:
+        objective_thresholds = [
             ObjectiveThreshold(
                 metric=get_branin_metric(name="branin_a"),
                 bound=10,
@@ -920,11 +1350,19 @@ def get_branin_multi_objective_optimization_config(
                 relative=False,
             ),
         ]
-        if has_objective_thresholds
-        else None
-    )
+        if num_objectives == 3:
+            objective_thresholds.append(
+                ObjectiveThreshold(
+                    metric=get_branin_metric(name="branin_c"),
+                    bound=5.0,
+                    op=ComparisonOp.GEQ,
+                    relative=False,
+                )
+            )
+    else:
+        objective_thresholds = None
     return MultiObjectiveOptimizationConfig(
-        objective=get_branin_multi_objective(),
+        objective=get_branin_multi_objective(num_objectives=num_objectives),
         objective_thresholds=objective_thresholds,
     )
 
@@ -1087,6 +1525,52 @@ def get_data(
     return Data(df=pd.DataFrame.from_records(df_dict))
 
 
+def get_non_monolithic_branin_moo_data() -> Data:
+    now = datetime.now()
+    return Data(
+        df=pd.DataFrame.from_records(
+            [
+                {
+                    "arm_name": "status_quo",
+                    "trial_index": "0",
+                    "metric_name": "branin_a",  # Obj. metric for experiment.
+                    "mean": 2.0,
+                    "sem": 0.01,
+                    "start_time": now - timedelta(days=3),
+                    "end_time": now,
+                },
+                {
+                    "arm_name": "0_0",
+                    "trial_index": "0",
+                    "metric_name": "branin_a",  # Obj. metric for experiment.
+                    "mean": 1.0,
+                    "sem": 0.01,
+                    "start_time": now - timedelta(days=3),
+                    "end_time": now,
+                },
+                {
+                    "arm_name": "status_quo",
+                    "trial_index": "0",
+                    "metric_name": "branin_b",  # Obj. metric for experiment.
+                    "mean": 2.0,
+                    "sem": 0.01,
+                    "start_time": now - timedelta(days=2),
+                    "end_time": now - timedelta(days=1),
+                },
+                {
+                    "arm_name": "0_0",
+                    "trial_index": "0",
+                    "metric_name": "branin_b",  # Obj. metric for experiment.
+                    "mean": 1.0,
+                    "sem": 0.01,
+                    "start_time": now - timedelta(days=2),
+                    "end_time": now - timedelta(days=1),
+                },
+            ]
+        )
+    )
+
+
 def get_map_data(trial_index: int = 0) -> MapData:
     evaluations = {
         "status_quo": [
@@ -1111,10 +1595,15 @@ def get_map_data(trial_index: int = 0) -> MapData:
     return MapData.from_map_evaluations(
         evaluations=evaluations,  # pyre-ignore [6]: Spurious param type mismatch.
         trial_index=trial_index,
-        map_keys=["epoch"],
+        map_key_infos=[get_map_key_info()],
     )
 
 
+# pyre-fixme[24]: Generic type `MapKeyInfo` expects 1 type parameter.
+def get_map_key_info() -> MapKeyInfo:
+    return MapKeyInfo(key="epoch", default_value=0.0)
+
+
 def get_branin_data(
     trial_indices: Optional[Iterable[int]] = None,
     trials: Optional[Iterable[Trial]] = None,
@@ -1168,8 +1657,12 @@ def get_branin_data_batch(batch: BatchTrial) -> Data:
 
 
 def get_branin_data_multi_objective(
-    trial_indices: Optional[Iterable[int]] = None,
+    trial_indices: Optional[Iterable[int]] = None, num_objectives: int = 2
 ) -> Data:
+    _validate_num_objectives(num_objectives=num_objectives)
+    suffixes = ["a", "b"]
+    if num_objectives == 3:
+        suffixes.append("c")
     df_dicts = [
         {
             "trial_index": trial_index,
@@ -1179,11 +1672,98 @@ def get_branin_data_multi_objective(
             "sem": 0.0,
         }
         for trial_index in (trial_indices or [0])
-        for suffix in ["a", "b"]
+        for suffix in suffixes
     ]
     return Data(df=pd.DataFrame.from_records(df_dicts))
 
 
+def get_percentile_early_stopping_strategy() -> PercentileEarlyStoppingStrategy:
+    return PercentileEarlyStoppingStrategy(
+        percentile_threshold=0.25,
+        min_progression=0.2,
+        min_curves=10,
+        trial_indices_to_ignore=[0, 1, 2],
+        normalize_progressions=True,
+    )
+
+
+def get_percentile_early_stopping_strategy_with_true_objective_metric_name() -> PercentileEarlyStoppingStrategy:  # noqa
+    strategy = get_percentile_early_stopping_strategy()
+    strategy.true_objective_metric_name = "true_objective"
+    return strategy
+
+
+def get_percentile_early_stopping_strategy_with_non_objective_metric_name() -> PercentileEarlyStoppingStrategy:  # noqa
+    return PercentileEarlyStoppingStrategy(
+        metric_names=["foo"],
+        percentile_threshold=0.25,
+        min_progression=0.2,
+        min_curves=10,
+        trial_indices_to_ignore=[0, 1, 2],
+        normalize_progressions=True,
+    )
+
+
+def get_threshold_early_stopping_strategy() -> ThresholdEarlyStoppingStrategy:
+    return ThresholdEarlyStoppingStrategy(
+        true_objective_metric_name="true_objective",
+        metric_threshold=0.1,
+        min_progression=0.2,
+        trial_indices_to_ignore=[0, 1, 2],
+        normalize_progressions=True,
+    )
+
+
+def get_and_early_stopping_strategy() -> AndEarlyStoppingStrategy:
+    return AndEarlyStoppingStrategy(
+        left=get_percentile_early_stopping_strategy(),
+        right=get_threshold_early_stopping_strategy(),
+    )
+
+
+def get_or_early_stopping_strategy() -> OrEarlyStoppingStrategy:
+    return OrEarlyStoppingStrategy(
+        left=get_percentile_early_stopping_strategy(),
+        right=get_threshold_early_stopping_strategy(),
+    )
+
+
+class DummyEarlyStoppingStrategy(BaseEarlyStoppingStrategy):
+    def __init__(
+        self, early_stop_trials: Optional[Dict[int, Optional[str]]] = None
+    ) -> None:
+        self.early_stop_trials: Dict[int, Optional[str]] = early_stop_trials or {}
+
+    def should_stop_trials_early(
+        self,
+        trial_indices: Set[int],
+        experiment: Experiment,
+        **kwargs: Dict[str, Any],
+    ) -> Dict[int, Optional[str]]:
+        return self.early_stop_trials
+
+
+class DummyGlobalStoppingStrategy(BaseGlobalStoppingStrategy):
+    """
+    A dummy Global Stopping Strategy which stops the optimization after
+    a pre-specified number of trials are completed.
+    """
+
+    def __init__(self, min_trials: int, trial_to_stop: int) -> None:
+        super().__init__(min_trials=min_trials)
+        self.trial_to_stop = trial_to_stop
+
+    def should_stop_optimization(
+        self, experiment: Experiment, **kwargs: Dict[str, Any]
+    ) -> Tuple[bool, str]:
+        num_completed_trials = len(experiment.trials_by_status[TrialStatus.COMPLETED])
+
+        if num_completed_trials >= max([self.min_trials, self.trial_to_stop]):
+            return True, "Stop the optimization."
+        else:
+            return False, ""
+
+
 ##############################
 # Instances of types from core/types.py
 ##############################
@@ -1275,3 +1855,50 @@ def get_mll_type() -> Type[MarginalLogLikelihood]:
 
 def get_acquisition_function_type() -> Type[AcquisitionFunction]:
     return qExpectedImprovement
+
+
+def get_winsorization_config() -> WinsorizationConfig:
+    return WinsorizationConfig(
+        lower_quantile_margin=0.2,
+        upper_quantile_margin=0.3,
+        lower_boundary=20,
+        upper_boundary=50,
+    )
+
+
+def get_gamma_prior() -> GammaPrior:
+    return GammaPrior(concentration=0.9, rate=10.0)
+
+
+def get_interval() -> Interval:
+    return Interval(lower_bound=1e-6, upper_bound=0.1)
+
+
+##############################
+# Scheduler
+##############################
+
+
+def get_default_scheduler_options() -> SchedulerOptions:
+    return SchedulerOptions()
+
+
+def get_scheduler_options_batch_trial() -> SchedulerOptions:
+    return SchedulerOptions(trial_type=TrialType.BATCH_TRIAL)
+
+
+##############################
+# Other
+##############################
+
+
+def get_risk_measure() -> RiskMeasure:
+    return RiskMeasure(risk_measure="Expectation", options={"n_w": 8})
+
+
+def get_parameter_distribution() -> ParameterDistribution:
+    return ParameterDistribution(
+        parameters=["x"],
+        distribution_class="norm",
+        distribution_parameters={"loc": 1.0, "scale": 0.5},
+    )
